@@ -90,6 +90,11 @@ def get_properties(filename):
     return props
 
 
+def get_machineinfo():
+    if platform.uname()[4]:
+        return platform.uname()[4]
+    return "unknown"
+
 def get_cpuinfo():
     if platform.uname()[5]:
         return platform.uname()[5]
@@ -189,21 +194,26 @@ def get_platform_name():
     if sys.platform.find("openbsd")>=0:
         return "OpenBSD"
     if sys.platform.startswith("win"):
+        #TODO: use something a bit faster:
         try:
-            import wmi            #@UnresolvedImport
-            c = wmi.WMI()
-            for os in c.Win32_OperatingSystem():
-                eq = os.Caption.find("=")
-                if eq>0:
-                    return (os.Caption[:eq]).strip()
-                return os.Caption
+            o = subprocess.Popen('systeminfo', stdout=subprocess.PIPE).communicate()[0]
+            try:
+                o = str(o, "latin-1")  # Python 3+
+            except:
+                pass
+            return re.search("OS Name:\s*(.*)", o).group(1).strip()
         except:
             pass
         return "Microsoft Windows"
     if sys.platform.find("bsd")>=0:
         return "BSD"
-    if sys.platform.find("linux")>=0 and hasattr(platform, "linux_distribution"):
-        return "Linux %s" % (" ".join(platform.linux_distribution()))
+    try:
+        from xpra.os_util import get_linux_distribution
+        ld = get_linux_distribution()
+        if ld:
+            return "Linux %s" % (" ".join(ld))
+    except:
+        pass
     return sys.platform
 
 
@@ -220,6 +230,7 @@ def record_build_info(is_build=True):
         set_prop(props, "BUILT_ON", socket.gethostname())
         set_prop(props, "BUILD_DATE", datetime.date.today().isoformat())
         set_prop(props, "BUILD_TIME", datetime.datetime.now().strftime("%H:%M"))
+        set_prop(props, "BUILD_MACHINE", get_machineinfo())
         set_prop(props, "BUILD_CPU", get_cpuinfo())
         set_prop(props, "BUILD_BIT", platform.architecture()[0])
         set_prop(props, "BUILD_OS", get_platform_name())
@@ -236,9 +247,9 @@ def record_build_info(is_build=True):
         #record pkg-config versions:
         PKG_CONFIG = os.environ.get("PKG_CONFIG", "pkg-config")
         for pkg in ("libc",
-                    "vpx", "libvpx", "x264", "x265", "webp",
+                    "vpx", "libvpx", "x264", "x265",
                     "avcodec", "avutil", "swscale",
-                    "nvenc4", "nvenc5", "nvenc6",
+                    "nvenc",
                     "x11", "xrandr", "xtst", "xfixes", "xkbfile", "xcomposite", "xdamage", "xext",
                     "gtk+-3.0", "pycairo", "pygobject-2.0", "pygtk-2.0", ):
             #fugly magic for turning the package atom into a legal variable name:
