@@ -11,14 +11,13 @@ import numpy
 from xpra.log import Logger
 log = Logger()
 
-def roundup(n, m):
-    return (n + m - 1) & ~(m - 1)
-
+from xpra.util import roundup
 print("loading encoder class")
-from xpra.codecs.nvenc.encoder import cuda_check, get_BGRA2NV12
+from xpra.codecs.nvenc.encoder import get_BGRA2NV12
+from xpra.codecs.cuda_common.cuda_context import check_devices
 
 print("cuda check")
-cuda_check()
+check_devices()
 
 print("pycuda import")
 from pycuda import driver       #@UnresolvedImport
