@@ -120,7 +120,7 @@ def test_packets():
                    'build.on': 'desktop', 'rencode': True, 'root_window_size': (3840, 1080),
                    'window_configure': True, 'gtk.version': (2, 24, 22), 'window.raise': True,
                    'info-request': True, 'platform.name': 'Linux', 'zlib': True, 'build.revision': '4679',
-                   'encodings.lossless': ['rgb24', 'rgb32', 'png', 'png/L', 'png/P', 'webp'],
+                   'encodings.lossless': ['rgb24', 'rgb32', 'png', 'png/L', 'png/P'],
                    'modifier_keycodes': {'control': [('Control_L', 'Control_L'), ('Control_R', 'Control_R')],
                                          'mod1': [(64, 'Alt_L'), ('Alt_L', 'Alt_L'), ('Meta_L', 'Meta_L')],
                                          'mod2': [('Num_Lock', 'Num_Lock')], 'mod3': [],
@@ -131,12 +131,12 @@ def test_packets():
                    'sound.pulseaudio.server': '', 'build.by': 'root',
                    'machine_id': '7725dfc225d14958a625ddaaaea5962b', 'display': ':10',
                    'python.version': (2, 7, 5), 'uid': 1000, 'desktop_size': (3840, 1080),
-                   'encodings.with_quality': ['h264', 'webp', 'jpeg'], 'pid': 11810, 'sound_sequence': True,
+                   'encodings.with_quality': ['h264', 'jpeg'], 'pid': 11810, 'sound_sequence': True,
                    'change-speed': True, 'sound.pulseaudio.id': '', 'cursors': True,
-                   'encodings': ['rgb', 'vp8', 'h264', 'webp', 'png', 'png/L', 'png/P', 'jpeg'],
+                   'encodings': ['rgb', 'vp8', 'h264', 'png', 'png/L', 'png/P', 'jpeg'],
                    'sound.decoders': ['mp3', 'wavpack', 'wav', 'flac', 'speex'],
                    'rencode.version': '1.0.2', 'current_time': 1386771473, 'hostname': 'desktop',
-                   'encodings.with_lossless_mode': ['webp'], 'key_repeat': [500, 30], 'elapsed_time': 2,
+                   'encodings.with_lossless_mode': [], 'key_repeat': [500, 30], 'elapsed_time': 2,
                    'encoding.generic': True, 'version': '0.11.0', 'build.bit': '64bit', 'suspend-resume': True,
                    'platform.linux_distribution': ('Fedora', '19', 'Schr\xc3\xb6dinger\xe2\x80\x99s Cat'),
                    'max_desktop_size': (5120, 3200), 'sound.encoders': ['mp3', 'wavpack', 'wav', 'flac', 'speex'],
@@ -147,8 +147,8 @@ def test_packets():
                    'build.date': '2013-12-11',
                    'python.full_version': '2.7.5 (default, Nov 12 2013, 16:18:42) \n[GCC 4.8.2 20131017 (Red Hat 4.8.2-1)]',
                    'dbus_proxy': True, 'exit_server': True, 'toggle_cursors_bell_notify': True, 'notifications': False,
-                   'encodings.core': ['rgb24', 'rgb32', 'vp8', 'h264', 'webp', 'png', 'png/L', 'png/P', 'jpeg'],
-                   'encoding.webp.version': '0.2.2', 'key_repeat_modifiers': True, 'raw_packets': True,
+                   'encodings.core': ['rgb24', 'rgb32', 'vp8', 'h264', 'png', 'png/L', 'png/P', 'jpeg'],
+                   'key_repeat_modifiers': True, 'raw_packets': True,
                    'build.local_modifications': '5', 'platform.processor': 'x86_64', 'sound.gst.version': (0, 10, 36),
                    'encoding.swscale.version': 'SwS2.2.100', 'mmap_enabled': False})
     test_packet(hello)
