@@ -8,7 +8,7 @@ import gobject
 gobject.threads_init()
 import gtk
 
-from xpra.x11.gtk_x11 import gdk_display_source
+from xpra.x11.gtk2 import gdk_display_source
 assert gdk_display_source
 from xpra.x11.xroot_props import XRootPropWatcher
 
