@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2015 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2017 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -9,25 +9,24 @@
 # we reap the dead pids so that they don't become zombies,
 # also used for implementing --exit-with-children
 
-import os, sys
+import os
 import signal
 
-from xpra.util import updict
+from xpra.util import envint, envbool
+from xpra.os_util import POSIX
 from xpra.log import Logger
 log = Logger("server", "util")
 
 
-# use process polling with python versions older than 2.7 and 3.0, (because SIGCHLD support is broken)
-# or when the user requests it with the env var:
-BUGGY_PYTHON = sys.version_info<(2, 7) or sys.version_info[:2]==(3, 0)
-USE_PROCESS_POLLING = os.name!="posix" or os.environ.get("XPRA_USE_PROCESS_POLLING")=="1" or BUGGY_PYTHON
+USE_PROCESS_POLLING = not POSIX or envbool("XPRA_USE_PROCESS_POLLING")
+POLL_DELAY = envint("XPRA_POLL_DELAY", 2)
 
 
 singleton = None
-def getChildReaper(quit_cb=None):
+def getChildReaper():
     global singleton
     if singleton is None:
-        singleton = ChildReaper(quit_cb)
+        singleton = ChildReaper()
     return singleton
 
 
@@ -51,6 +50,9 @@ class ProcInfo(object):
 # thus before we even know the pid of the child).  So be careful:
 # We can also end up with multiple procinfo structures with the same pid,
 # and that should be fine too
+#
+# WNOHANG is a tricky beast, see:
+# https://github.com/gevent/gevent/issues/622
 class ChildReaper(object):
     #note: the quit callback will fire only once!
     def __init__(self, quit_cb=None):
@@ -60,12 +62,7 @@ class ChildReaper(object):
         self._quit = quit_cb
         self._proc_info = []
         if USE_PROCESS_POLLING:
-            POLL_DELAY = int(os.environ.get("XPRA_POLL_DELAY", 2))
-            if BUGGY_PYTHON:
-                log.warn("Warning: outdated/buggy version of Python: %s", ".".join(str(x) for x in sys.version_info))
-                log.warn(" switching to process polling every %s seconds to support 'exit-with-children'", POLL_DELAY)
-            else:
-                log("using process polling every %s seconds", POLL_DELAY)
+            log("using process polling every %s seconds", POLL_DELAY)
             self.glib.timeout_add(POLL_DELAY*1000, self.poll)
         else:
             #with a less buggy python, we can just check the list of pids
@@ -103,6 +100,7 @@ class ChildReaper(object):
         self._proc_info.append(procinfo)
         if procinfo.returncode is not None:
             self.add_dead_process(procinfo)
+        return procinfo
 
     def poll(self):
         #poll each process that is not dead yet:
@@ -113,14 +111,17 @@ class ChildReaper(object):
                 self.add_dead_process(procinfo)
         return True
 
+    def set_quit_callback(self, cb):
+        self._quit = cb
+
     def check(self):
         #see if we are meant to exit-with-children
         #see if we still have procinfos alive (and not meant to be ignored)
         self.poll()
         alive = [procinfo for procinfo in list(self._proc_info) if (not procinfo.ignore and not procinfo.dead)]
-        log("check() alive=%s", alive)
+        cb = self._quit
+        log("check() alive=%s, quit callback=%s", alive, cb)
         if len(alive)==0:
-            cb = self._quit
             if cb:
                 self._quit = None
                 cb()
@@ -173,7 +174,8 @@ class ChildReaper(object):
 
     def reap(self):
         self.poll()
-        while os.name=="posix":
+        while POSIX:
+            log("reap() calling os.waitpid%s", (-1, "WNOHANG"))
             try:
                 pid, _ = os.waitpid(-1, os.WNOHANG)
             except OSError:
@@ -185,11 +187,22 @@ class ChildReaper(object):
 
     def get_info(self):
         iv = list(self._proc_info)
-        info = {"children"          : len(iv),
-                "children.dead"     : len([x for x in iv if x.dead]),
-                "children.ignored"  : len([x for x in iv if x.ignore])}
+        info = {
+                "children"  : {
+                               "total"      : len(iv),
+                               "dead"       : len([x for x in iv if x.dead]),
+                               "ignored"    : len([x for x in iv if x.ignore]),
+                               }
+                }
         pi = sorted(self._proc_info, key=lambda x: x.pid, reverse=True)
+        cinfo = {}
         for i, procinfo in enumerate(pi):
-            d = dict((k,getattr(procinfo,k)) for k in ("name", "command", "ignore", "forget", "returncode", "dead", "pid"))
-            updict(info, "child[%i]" % i, d)
+            d = {}
+            for k in ("name", "command", "ignore", "forget", "returncode", "dead", "pid"):
+                v = getattr(procinfo, k)
+                if v is None:
+                    continue
+                d[k] = v
+            cinfo[i] = d
+        info["child"] = cinfo
         return info
