@@ -1,15 +1,16 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2015 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2017 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import sys, os
+import os
 import re
 
 from xpra.client.client_widget_base import ClientWidgetBase
-from xpra.util import typedict, bytestostr, WORKSPACE_UNSET, WORKSPACE_NAMES
+from xpra.os_util import bytestostr, PYTHON2, OSX
+from xpra.util import typedict, envbool, WORKSPACE_UNSET, WORKSPACE_NAMES
 from xpra.log import Logger
 log = Logger("window")
 plog = Logger("paint")
@@ -18,17 +19,18 @@ mouselog = Logger("mouse")
 workspacelog = Logger("workspace")
 keylog = Logger("keyboard")
 metalog = Logger("metadata")
+geomlog = Logger("geometry")
 
 
 REPAINT_ALL = os.environ.get("XPRA_REPAINT_ALL", "")
-SIMULATE_MOUSE_DOWN = os.environ.get("XPRA_SIMULATE_MOUSE_DOWN", "1")=="1"
+SIMULATE_MOUSE_DOWN = envbool("XPRA_SIMULATE_MOUSE_DOWN", True)
 PROPERTIES_DEBUG = [x.strip() for x in os.environ.get("XPRA_WINDOW_PROPERTIES_DEBUG", "").split(",")]
 
 
 class ClientWindowBase(ClientWidgetBase):
 
-    def __init__(self, client, group_leader, wid, x, y, ww, wh, bw, bh, metadata, override_redirect, client_properties, border, max_window_size):
-        log("%s%s", type(self), (client, group_leader, wid, x, y, ww, wh, bw, bh, metadata, override_redirect, client_properties, max_window_size))
+    def __init__(self, client, group_leader, wid, x, y, ww, wh, bw, bh, metadata, override_redirect, client_properties, border, max_window_size, default_cursor_data, pixel_depth):
+        log("%s%s", type(self), (client, group_leader, wid, x, y, ww, wh, bw, bh, metadata, override_redirect, client_properties, max_window_size, default_cursor_data, pixel_depth))
         ClientWidgetBase.__init__(self, client, wid, metadata.boolget("has-alpha"))
         self._override_redirect = override_redirect
         self.group_leader = group_leader
@@ -50,8 +52,11 @@ class ClientWindowBase(ClientWidgetBase):
         self._iconified = False
         self._focused = False
         self.border = border
+        self.cursor_data = None
+        self.default_cursor_data = default_cursor_data
         self.max_window_size = max_window_size
         self.button_state = {}
+        self.pixel_depth = pixel_depth      #0 for default
 
         self.init_window(metadata)
         self.setup_window(bw, bh)
@@ -89,6 +94,12 @@ class ClientWindowBase(ClientWidgetBase):
         return None
 
 
+    def set_cursor_data(self, cursor_data):
+        self.cursor_data = cursor_data
+        b = self._backing
+        if b:
+            b.set_cursor_data(cursor_data)
+
     def new_backing(self, bw, bh):
         backing_class = self.get_backing_class()
         log("new_backing(%s, %s) backing_class=%s", bw, bh, backing_class)
@@ -96,6 +107,8 @@ class ClientWindowBase(ClientWidgetBase):
         w, h = self._size
         self._backing = self.make_new_backing(backing_class, w, h, bw, bh)
         self._backing.border = self.border
+        self._backing.default_cursor_data = self.default_cursor_data
+        self._backing.set_cursor_data(self.cursor_data)
         return self._backing._backing
 
 
@@ -133,6 +146,10 @@ class ClientWindowBase(ClientWidgetBase):
     def send(self, *args):
         self._client.send(*args)
 
+    def reset_icon(self):
+        current_icon = self._current_icon
+        if current_icon:
+            self.update_icon(*current_icon)
 
     def update_icon(self, width, height, coding, data):
         raise Exception("override me!")
@@ -183,18 +200,21 @@ class ClientWindowBase(ClientWidgetBase):
                 if title.find("@")>=0:
                     #perform metadata variable substitutions:
                     #full of py3k unicode headaches that don't need to be
-                    default_values = {"title"           : "<untitled window>",
-                                      "client-machine"  : "<unknown machine>"}
+                    default_values = {
+                                      "title"           : "<untitled window>",
+                                      "client-machine"  : "<unknown machine>",
+                                      "windowid"        : str(self._id),
+                                      }
                     def metadata_replace(match):
                         atvar = match.group(0)          #ie: '@title@'
                         var = atvar[1:len(atvar)-1]     #ie: 'title'
                         default_value = default_values.get(var, "<unknown %s>" % var)
                         value = self._metadata.strget(var, default_value)
-                        if sys.version<'3':
+                        if PYTHON2:
                             value = value.decode("utf-8")
                         return value
                     title = re.sub("@[\w\-]*@", metadata_replace, title)
-                if sys.version<'3':
+                if PYTHON2:
                     utf8_title = title.encode("utf-8")
                 else:
                     utf8_title = title
@@ -206,6 +226,9 @@ class ClientWindowBase(ClientWidgetBase):
         if b"icon-title" in metadata:
             icon_title = metadata.strget("icon-title")
             self.set_icon_name(icon_title)
+            #the DE may have reset the icon now,
+            #force it to use the one we really want:
+            self.reset_icon()
 
         if b"size-constraints" in metadata:
             self.size_constraints = typedict(metadata.dictget("size-constraints"))
@@ -368,7 +391,7 @@ class ClientWindowBase(ClientWidgetBase):
 
 
     def set_size_constraints(self, size_constraints, max_window_size):
-        metalog("set_size_constraints(%s, %s)", size_constraints, max_window_size)
+        geomlog("set_size_constraints(%s, %s)", size_constraints, max_window_size)
         hints = typedict()
         for (a, h1, h2) in [
             (b"maximum-size", b"max_width", b"max_height"),
@@ -376,18 +399,27 @@ class ClientWindowBase(ClientWidgetBase):
             (b"base-size", b"base_width", b"base_height"),
             (b"increment", b"width_inc", b"height_inc"),
             ]:
+            if a in (b"base-size", b"increment"):
+                def closetoint(v):
+                    return abs(int(v)-v)<0.00001
+                int_scaling = closetoint(self._client.xscale) and closetoint(self._client.yscale)
+                if not int_scaling:
+                    #don't scale increment and base size constraints with non integer scaling values
+                    #as this would use rounding
+                    continue
             v = size_constraints.intpair(a)
             if v:
                 v1, v2 = v
                 hints[h1], hints[h2] = self._client.sp(v1, v2)
-        for (a, h) in [
-            (b"minimum-aspect-ratio", b"min_aspect"),
-            (b"maximum-aspect-ratio", b"max_aspect"),
-            ]:
-            v = size_constraints.intpair(a)
-            if v:
-                v1, v2 = v
-                hints[h] = (v1*self._client.xscale)/(v2*self._client.yscale)
+        if not OSX:
+            for (a, h) in [
+                (b"minimum-aspect-ratio", b"min_aspect"),
+                (b"maximum-aspect-ratio", b"max_aspect"),
+                ]:
+                v = size_constraints.intpair(a)
+                if v:
+                    v1, v2 = v
+                    hints[h] = float(v1*self._client.xscale)/float(v2*self._client.yscale)
         #apply max-size override if needed:
         w,h = max_window_size
         if w>0 and h>0 and not self._fullscreen:
@@ -413,7 +445,7 @@ class ClientWindowBase(ClientWidgetBase):
             #not honouring "base" + "inc", but honouring just "min" instead:
             maxw = max(minw, maxw)
             maxh = max(minh, maxh)
-            metalog("modified hints for max window size %s: %s (rw=%s, rh=%s) -> max=%sx%s", max_window_size, hints, rw, rh, maxw, maxh)
+            geomlog("modified hints for max window size %s: %s (rw=%s, rh=%s) -> max=%sx%s", max_window_size, hints, rw, rh, maxw, maxh)
             #ensure we don't have duplicates with bytes / strings,
             #and that keys are always "bytes":
             #(in practice this code should never fire, just here as a reminder)
@@ -425,12 +457,18 @@ class ClientWindowBase(ClientWidgetBase):
             hints[b"max_width"] = maxw
             hints[b"max_height"] = maxh
         try:
-            metalog("calling: %s(%s)", self.apply_geometry_hints, hints)
+            geomlog("calling: %s(%s)", self.apply_geometry_hints, hints)
             #save them so the window hooks can use the last value used:
             self.geometry_hints = hints
             self.apply_geometry_hints(hints)
         except:
-            metalog.error("with hints=%s", hints, exc_info=True)
+            geomlog("set_size_constraints%s", (size_constraints, max_window_size))
+            geomlog.error("Error setting window hints:")
+            for k,v in hints.items():
+                geomlog.error(" %s=%s", k, v)
+            geomlog.error(" from size constraints:")
+            for k,v in size_constraints.items():
+                geomlog.error(" %s=%s", k, v)
         #TODO: handle gravity
         #gravity = size_metadata.get("gravity")
 
@@ -456,6 +494,15 @@ class ClientWindowBase(ClientWidgetBase):
         pass
 
 
+    def toggle_debug(self, *args):
+        b = self._backing
+        if not b:
+            return
+        if b.paint_box_line_width>0:
+            b.paint_box_line_width = 0
+        else:
+            b.paint_box_line_width = b.default_paint_box_line_width
+
     def increase_quality(self, *args):
         if self._client.quality>0:
             #change fixed quality:
@@ -541,9 +588,10 @@ class ClientWindowBase(ClientWidgetBase):
             if backing and backing.draw_needs_refresh:
                 if REPAINT_ALL=="1" or self._client.xscale!=1 or self._client.yscale!=1:
                     w, h = self.get_size()
-                    self.queue_draw(0, 0, w, h)
+                    rect = 0, 0, w, h
                 else:
-                    self.queue_draw(*self._client.srect(x, y, width, height))
+                    rect = self._client.srect(x, y, width, height)
+                self.idle_add(self.queue_draw, *rect)
         #only register this callback if we actually need it:
         if backing.draw_needs_refresh:
             callbacks.append(after_draw_refresh)
@@ -616,19 +664,16 @@ class ClientWindowBase(ClientWidgetBase):
         return self._client.rpc_call("dbus", rpc_args, **kwargs)
 
 
-    def get_mouse_event_wid(self):
-        #on OSX, the mouse events are reported against the wrong window by GTK,
-        #so we have to use the currently focused window
-        if sys.platform.startswith("darwin"):
-            return self._client._focused or self._id
+    def get_mouse_event_wid(self, x, y):
+        #overriden in GTKClientWindowBase
         return self._id
 
     def do_motion_notify_event(self, event):
         if self._client.readonly:
             return
         pointer, modifiers, buttons = self._pointer_modifiers(event)
-        wid = self.get_mouse_event_wid()
-        mouselog("do_motion_notify_event(%s) wid=%s / focus=%s, device=%s, pointer=%s, modifiers=%s, buttons=%s", event, self._id, self._client._focused, self._device_info(event), pointer, modifiers, buttons)
+        wid = self.get_mouse_event_wid(*pointer)
+        mouselog("do_motion_notify_event(%s) wid=%s / focus=%s / window wid=%i, device=%s, pointer=%s, modifiers=%s, buttons=%s", event, wid, self._client._focused, self._id, self._device_info(event), pointer, modifiers, buttons)
         self._client.send_mouse_position(["pointer-position", wid,
                                           pointer, modifiers, buttons])
 
@@ -638,14 +683,28 @@ class ClientWindowBase(ClientWidgetBase):
         except:
             return ""
 
-    def _button_action(self, button, event, depressed):
+    def _button_action(self, button, event, depressed, *args):
         if self._client.readonly:
             return
         pointer, modifiers, buttons = self._pointer_modifiers(event)
-        wid = self.get_mouse_event_wid()
-        mouselog("_button_action(%s, %s, %s) wid=%s / focus=%s, device=%s, pointer=%s, modifiers=%s, buttons=%s", button, event, depressed, self._id, self._client._focused, self._device_info(event), pointer, modifiers, buttons)
+        wid = self.get_mouse_event_wid(*pointer)
+        mouselog("_button_action(%s, %s, %s) wid=%s / focus=%s / window wid=%i, device=%s, pointer=%s, modifiers=%s, buttons=%s", button, event, depressed, wid, self._client._focused, self._id, self._device_info(event), pointer, modifiers, buttons)
+        #map wheel buttons via translation table to support inverted axes:
+        server_button = button
+        if button>3:
+            server_button = self._client.wheel_map.get(button)
+            if not server_button:
+                return
+        server_buttons = []
+        for b in buttons:
+            if b>3:
+                sb = self._client.wheel_map.get(button)
+                if not sb:
+                    continue
+                b = sb
+            server_buttons.append(b)
         def send_button(pressed):
-            self._client.send_button(wid, button, pressed, pointer, modifiers, buttons)
+            self._client.send_button(wid, server_button, pressed, pointer, modifiers, server_buttons, *args)
         pressed_state = self.button_state.get(button, False)
         if SIMULATE_MOUSE_DOWN and pressed_state is False and depressed is False:
             mouselog("button action: simulating a missing mouse-down event for window %s before sending the mouse-up event", wid)
@@ -659,12 +718,3 @@ class ClientWindowBase(ClientWidgetBase):
 
     def do_button_release_event(self, event):
         self._button_action(event.button, event, False)
-
-    def do_scroll_event(self, event):
-        if self._client.readonly:
-            return
-        button_mapping = self.SCROLL_MAP.get(event.direction, -1)
-        mouselog("do_scroll_event device=%s, direction=%s, button_mapping=%s", self._device_info(event), event.direction, button_mapping)
-        if button_mapping>=0:
-            self._button_action(button_mapping, event, True)
-            self._button_action(button_mapping, event, False)
