@@ -1,29 +1,34 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
 # Copyright (C) 2012 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2012-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2016 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import sys, os
+import sys
 import logging
+from xpra.util import envbool
+from xpra.os_util import OSX, WIN32, PYTHON3
 from xpra.log import Logger, CaptureHandler
 log = Logger("opengl")
 
 required_extensions = ["GL_ARB_texture_rectangle", "GL_ARB_vertex_program"]
 
 
-WHITELIST = {}
-GREYLIST = {}
-VERSION_REQ = {"nouveau" : [3, 0],      #older versions have issues
+WHITELIST = {
+    "renderer"  : ["Skylake", "Kabylake"],
+    }
+GREYLIST = {
+            "vendor"    : ["Intel", "Humper"]
+            }
+VERSION_REQ = {
+               "nouveau" : [3, 0],      #older versions have issues
                }
-BLACKLIST = {"vendor" : ["Humper", "VMware, Inc."],
-             "renderer" : ["Software Rasterizer"]}
+BLACKLIST = {
+             "renderer" : ["Software Rasterizer", "Mesa DRI Intel(R) Ivybridge Desktop"],
+             "vendor"    : ["VMware, Inc."]
+             }
 
-#the Intel driver causes too many problems:
-GREYLIST.setdefault("vendor", []).append("Intel")
-GREYLIST.setdefault("vendor", []).append("Intel Inc.")
-GREYLIST.setdefault("vendor", []).append("Intel Open Source Technology Center")
 from xpra.os_util import getUbuntuVersion
 uv = getUbuntuVersion()
 if uv and uv<[15]:
@@ -31,55 +36,67 @@ if uv and uv<[15]:
     GREYLIST.setdefault("vendor", []).append("X.Org")
 if False:
     #for testing:
-    BLACKLIST["vendor"].append("NVIDIA Corporation")
+    GREYLIST["vendor"].append("NVIDIA Corporation")
     WHITELIST["renderer"] = ["GeForce GTX 760/PCIe/SSE2"]
 
-    if sys.platform.startswith("darwin"):
+    if OSX:
         #frequent crashes on osx with GT 650M: (see ticket #808)
         GREYLIST.setdefault("vendor", []).append("NVIDIA Corporation")
 
 
 #alpha requires gtk3 or *nix only for gtk2:
-DEFAULT_ALPHA = sys.version>'3' or (not sys.platform.startswith("win") and not sys.platform.startswith("darwin"))
-GL_ALPHA_SUPPORTED = os.environ.get("XPRA_ALPHA", DEFAULT_ALPHA) in (True, "1")
+DEFAULT_ALPHA = PYTHON3 or (not WIN32 and not OSX)
+GL_ALPHA_SUPPORTED = envbool("XPRA_ALPHA", DEFAULT_ALPHA)
 #not working with gtk3 yet?
-CAN_DOUBLE_BUFFER = True
+CAN_DOUBLE_BUFFER = not PYTHON3
 #needed on win32?:
-DEFAULT_DOUBLE_BUFFERED = sys.platform.startswith("win") and CAN_DOUBLE_BUFFER
-DOUBLE_BUFFERED = os.environ.get("XPRA_OPENGL_DOUBLE_BUFFERED", str(int(DEFAULT_DOUBLE_BUFFERED)))=="1"
+DEFAULT_DOUBLE_BUFFERED = WIN32 or CAN_DOUBLE_BUFFER
+DOUBLE_BUFFERED = envbool("XPRA_OPENGL_DOUBLE_BUFFERED", DEFAULT_DOUBLE_BUFFERED)
+
+from xpra.gtk_common.gtk_util import STATIC_GRAY, GRAYSCALE, STATIC_COLOR, PSEUDO_COLOR, TRUE_COLOR, DIRECT_COLOR
+VISUAL_NAMES = {
+                STATIC_GRAY      : "STATIC_GRAY",
+                GRAYSCALE        : "GRAYSCALE",
+                STATIC_COLOR     : "STATIC_COLOR",
+                PSEUDO_COLOR     : "PSEUDO_COLOR",
+                TRUE_COLOR       : "TRUE_COLOR",
+                DIRECT_COLOR     : "DIRECT_COLOR",
+                }
+
+from xpra.gtk_common.gtk_util import LSB_FIRST, MSB_FIRST
+
+VISUAL_TYPES = {
+                LSB_FIRST   : "LSB",
+                MSB_FIRST   : "MSB",
+                }
+
+from xpra.client.gl.gtk_compat import MODE_RGBA, MODE_ALPHA, MODE_RGB, MODE_DOUBLE, MODE_SINGLE, MODE_DEPTH
 
 
 def get_visual_name(visual):
-    from xpra.gtk_common.gtk_util import STATIC_GRAY, GRAYSCALE, STATIC_COLOR, PSEUDO_COLOR, TRUE_COLOR, DIRECT_COLOR
     if not visual:
         return ""
-    return {
-           STATIC_GRAY      : "STATIC_GRAY",
-           GRAYSCALE        : "GRAYSCALE",
-           STATIC_COLOR     : "STATIC_COLOR",
-           PSEUDO_COLOR     : "PSEUDO_COLOR",
-           TRUE_COLOR       : "TRUE_COLOR",
-           DIRECT_COLOR     : "DIRECT_COLOR"}.get(visual.type, "unknown")
+    global VISUAL_NAMES
+    return VISUAL_NAMES.get(visual.type, "unknown")
 
 def get_visual_byte_order(visual):
-    from xpra.gtk_common.gtk_util import LSB_FIRST, MSB_FIRST
     if not visual:
         return ""
-    return {
-            LSB_FIRST   : "LSB",
-            MSB_FIRST   : "MSB"}.get(visual.byte_order, "unknown")
+    global VISUAL_TYPES
+    return VISUAL_TYPES.get(visual.byte_order, "unknown")
 
 def visual_to_str(visual):
     if not visual:
         return ""
-    d = {"type"         : get_visual_name(visual),
-         "byte_order"   : get_visual_byte_order(visual)}
+    d = {
+         "type"         : get_visual_name(visual),
+         "byte_order"   : get_visual_byte_order(visual),
+         }
     for k in ("bits_per_rgb", "depth"):
         d[k] = getattr(visual, k)
     return str(d)
 
 def get_DISPLAY_MODE(want_alpha=GL_ALPHA_SUPPORTED):
-    from xpra.client.gl.gtk_compat import MODE_RGBA, MODE_ALPHA, MODE_RGB, MODE_DOUBLE, MODE_SINGLE
     #MODE_DEPTH
     if want_alpha:
         mode = MODE_RGBA | MODE_ALPHA
@@ -91,15 +108,18 @@ def get_DISPLAY_MODE(want_alpha=GL_ALPHA_SUPPORTED):
         mode = mode | MODE_SINGLE
     return mode
 
+FRIENDLY_MODE_NAMES = {
+                       MODE_RGB         : "RGB",
+                       MODE_RGBA        : "RGBA",
+                       MODE_ALPHA       : "ALPHA",
+                       MODE_DEPTH       : "DEPTH",
+                       MODE_DOUBLE      : "DOUBLE",
+                       MODE_SINGLE      : "SINGLE",
+                       }
+
 def get_MODE_names(mode):
-    from xpra.client.gl.gtk_compat import MODE_RGB, MODE_RGBA, MODE_ALPHA, MODE_DEPTH, MODE_DOUBLE, MODE_SINGLE
-    friendly_mode_names = {MODE_RGB         : "RGB",
-                           MODE_RGBA        : "RGBA",
-                           MODE_ALPHA       : "ALPHA",
-                           MODE_DEPTH       : "DEPTH",
-                           MODE_DOUBLE      : "DOUBLE",
-                           MODE_SINGLE      : "SINGLE"}
-    friendly_modes = [v for k,v in friendly_mode_names.items() if k>0 and (k&mode)==k]
+    global FRIENDLY_MODE_NAMES
+    friendly_modes = [v for k,v in FRIENDLY_MODE_NAMES.items() if k>0 and (k&mode)==k]
     #special case for single (value is zero!)
     if not (mode&MODE_DOUBLE==MODE_DOUBLE):
         friendly_modes.append("SINGLE")
@@ -118,9 +138,6 @@ def is_pyopengl_memoryview_safe(pyopengl_version, accel_version):
     if accel_version is not None and pyopengl_version!=accel_version:
         #mismatch is not safe!
         return False
-    if sys.version_info[:2]<(2,7):
-        #memoryview type requires python 2.7
-        return False
     vsplit = pyopengl_version.split('.')
     if vsplit[:2]<['3','1']:
         #requires PyOpenGL >= 3.1, earlier versions will not work
@@ -204,7 +221,7 @@ def do_check_GL_support(force_enable):
         from OpenGL import version as OpenGL_version
         pyopengl_version = OpenGL_version.__version__
         try:
-            import OpenGL_accelerate
+            import OpenGL_accelerate            #@UnresolvedImport
             accel_version = OpenGL_accelerate.__version__
             props["accelerate"] = accel_version
             log("OpenGL_accelerate version %s", accel_version)
@@ -221,10 +238,9 @@ def do_check_GL_support(force_enable):
                 _version_warning_shown = True
         vsplit = pyopengl_version.split('.')
         #we now require PyOpenGL 3.1 or later
-        if vsplit[:2]<['3','1'] and not force_enable:
-            gl_check_error("PyOpenGL version 3.1 or later is required (found version %s)" % pyopengl_version)
+        if vsplit[:3]<['3','1'] and not force_enable:
+            gl_check_error("PyOpenGL version %s is too old and buggy" % pyopengl_version)
             return {}
-
         props["zerocopy"] = bool(OpenGL_accelerate) and is_pyopengl_memoryview_safe(pyopengl_version, accel_version)
 
         try:
@@ -280,24 +296,18 @@ def do_check_GL_support(force_enable):
             log("%s: %s", d, v)
             props[d] = v
 
-        blacklisted = None
-        whitelisted = None
-        greylisted = None
-        for k,vlist in BLACKLIST.items():
-            v = props.get(k)
-            if v in vlist:
-                log("%s '%s' found in blacklist: %s", k, v, vlist)
-                blacklisted = k, v
-        for k,vlist in GREYLIST.items():
-            v = props.get(k)
-            if v in vlist:
-                log("%s '%s' found in greylist: %s", k, v, vlist)
-                greylisted = k, v
-        for k,vlist in WHITELIST.items():
-            v = props.get(k)
-            if v in vlist:
-                log("%s '%s' found in whitelist: %s", k, v, vlist)
-                whitelisted = k, v
+        def match_list(thelist, listname):
+            for k,vlist in thelist.items():
+                v = props.get(k)
+                matches = [x for x in vlist if v.find(x)>=0]
+                if matches:
+                    log("%s '%s' found in %s: %s", k, v, listname, vlist)
+                    return (k, v)
+                log("%s '%s' not found in %s: %s", k, v, listname, vlist)
+            return None
+        blacklisted = match_list(BLACKLIST, "blacklist")
+        greylisted = match_list(GREYLIST, "greylist")
+        whitelisted = match_list(WHITELIST, "whitelist")
         if blacklisted:
             if whitelisted:
                 log.info("%s '%s' enabled (found in both blacklist and whitelist)", *whitelisted)
@@ -305,10 +315,13 @@ def do_check_GL_support(force_enable):
                 log.warn("Warning: %s '%s' is blacklisted!", *blacklisted)
             else:
                 gl_check_error("%s '%s' is blacklisted!" % (blacklisted))
-        safe = bool(whitelisted) or not (bool(greylisted) or bool(blacklisted))
-        if safe and sys.version_info[0]>2:
+        safe = bool(whitelisted) or not bool(blacklisted)
+        if safe and PYTHON3:
             log.warn("Warning: OpenGL python3 support is not enabled by default")
             safe = False
+        if greylisted and not whitelisted:
+            log.warn("Warning: %s '%s' is greylisted,", *greylisted)
+            log.warn(" you may want to turn off OpenGL if you encounter bugs")
         props["safe"] = safe
 
         #check for specific functions we need:
@@ -409,9 +422,10 @@ def do_check_GL_support(force_enable):
             msg = x.getMessage().replace("No OpenGL_accelerate module loaded: ", "")
             if msg=="No module named OpenGL_accelerate":
                 msg = "missing accelerate module"
-            if msg!="OpenGL_accelerate module loaded":
-                msg = "PyOpenGL warning: %s" % msg
-            log.info(msg)
+            if msg=="OpenGL_accelerate module loaded":
+                log.info(msg)
+            else:
+                log.warn("PyOpenGL warning: %s", msg)
 
         #format handler messages:
         STRIP_LOG_MESSAGE = "Unable to load registered array format handler "
@@ -478,7 +492,7 @@ def check_support(force_enable=False, check_colormap=False):
     props = {}
     #this will import gtk.gtkgl / gdkgl or gi.repository.GtkGLExt / GdkGLExt:
     try:
-        from xpra.client.gl.gtk_compat import get_info, gdkgl, Config_new_by_mode, MODE_DOUBLE, GLDrawingArea
+        from xpra.client.gl.gtk_compat import get_info, gdkgl, Config_new_by_mode, GLDrawingArea
     except RuntimeError as e:
         gl_check_error(str(e))
         return {}
@@ -491,11 +505,38 @@ def check_support(force_enable=False, check_colormap=False):
         glconfig = Config_new_by_mode(display_mode)
     if not glconfig:
         gl_check_error("cannot setup an OpenGL context")
+        return {}
     props["display_mode"] = get_MODE_names(display_mode)
-    props["glconfig"] = glconfig
-    props["has_alpha"] = glconfig.has_alpha()
-    props["rgba"] = glconfig.is_rgba()
+    #on OSX, we had to patch out get_depth...
+    #so take extra precautions when querying properties:
+    for x,fn_name in {
+        "has_alpha"           : "has_alpha",
+        "rgba"                : "is_rgba",
+        "stereo"              : "is_stereo",
+        "double-buffered"     : "is_double_buffered",
+        "depth"               : "get_depth",
+        "has-depth-buffer"    : "has_depth_buffer",
+        "has-stencil-buffer"  : "has_stencil_buffer",
+        }.items():
+        fn = getattr(glconfig, fn_name, None)
+        if fn:
+            props[x] = fn()
+        else:
+            log("%s does not support %s()", glconfig, fn_name)
+    for x in ("RED_SIZE", "GREEN_SIZE", "BLUE_SIZE", "ALPHA_SIZE",
+              "AUX_BUFFERS", "DEPTH_SIZE", "STENCIL_SIZE",
+              "ACCUM_RED_SIZE", "ACCUM_GREEN_SIZE", "ACCUM_BLUE_SIZE",
+              "SAMPLE_BUFFERS", "SAMPLES"):
+        prop = getattr(gdkgl, x)
+        if not prop:
+            continue
+        try:
+            v = glconfig.get_attrib(prop)[0]
+            props[x.lower().replace("_", "-")] = v
+        except:
+            pass
     log("GL props=%s", props)
+
     from xpra.gtk_common.gtk_util import import_gtk, gdk_window_process_all_updates
     gtk = import_gtk()
     assert gdkgl.query_extension()
