@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2013 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2012-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2015 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -30,14 +30,19 @@ class GLPixmapBacking(GLWindowBackingBase):
     def get_gl_drawable(self):
         return gtkgl.widget_get_gl_drawable(self._backing)
 
-    def gl_expose_event(self, glarea, event):
+    def gl_expose_event(self, glarea=None, event=None):
         if not self.paint_screen:
             return
         context = self.gl_context()
-        log("%s.gl_expose_event(%s, %s) context=%s", self, glarea, event, context)
+        if event and event.area:
+            area = event.area
+            rect = (area.x, area.y, area.width, area.height)
+        else:
+            w, h = self.size
+            rect = (0, 0, w, h)
+        log("%s.gl_expose_event(%s, %s) context=%s, area=%s", self, glarea, event, context, area)
         if not context:
             return
         with context:
             self.gl_init()
-            w, h = self.size
-            self.present_fbo("expose", False, 0, 0, w, h)
+            self.present_fbo(*rect)
