@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2015 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2017 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -12,17 +12,19 @@ from xpra.log import Logger
 log = Logger("gobject", "client")
 
 import sys
-import re
-from xpra.util import nonl, DONE
-from xpra.os_util import bytestostr
-from xpra.client.client_base import XpraClientBase, EXTRA_TIMEOUT, \
-    EXIT_TIMEOUT, EXIT_OK, EXIT_UNSUPPORTED, EXIT_REMOTE_ERROR, EXIT_FILE_TOO_BIG
+from xpra.util import nonl, sorted_nicely, print_nested_dict, envbool, DONE
+from xpra.os_util import bytestostr, get_hex_uuid, PYTHON3
+from xpra.client.client_base import XpraClientBase, EXTRA_TIMEOUT
+from xpra.exit_codes import (EXIT_OK, EXIT_TIMEOUT, EXIT_INTERNAL_ERROR, EXIT_FAILURE, EXIT_UNSUPPORTED, EXIT_REMOTE_ERROR, EXIT_FILE_TOO_BIG)
+
+FLATTEN_INFO = envbool("XPRA_FLATTEN_INFO", True)
 
 
 class GObjectXpraClient(XpraClientBase, gobject.GObject):
     """
         Utility superclass for GObject clients
     """
+    INSTALL_SIGNAL_HANDLERS = True
 
     def __init__(self):
         gobject.GObject.__init__(self)
@@ -30,7 +32,8 @@ class GObjectXpraClient(XpraClientBase, gobject.GObject):
 
     def init(self, opts):
         XpraClientBase.init(self, opts)
-        self.install_signal_handlers()
+        if self.INSTALL_SIGNAL_HANDLERS:
+            self.install_signal_handlers()
         self.glib_init()
         self.gobject_init()
 
@@ -69,17 +72,7 @@ class GObjectXpraClient(XpraClientBase, gobject.GObject):
             self._packet_handlers[t] = noop
 
     def gobject_init(self):
-        try:
-            gobject.threads_init()
-        except AttributeError:
-            #old versions of gobject may not have this method
-            pass
-
-    def connect_with_timeout(self, conn):
-        self.setup_connection(conn)
-        if conn.timeout>0:
-            glib.timeout_add((conn.timeout + EXTRA_TIMEOUT) * 1000, self.timeout)
-        glib.idle_add(self.send_hello)
+        gobject.threads_init()
 
     def run(self):
         XpraClientBase.run(self)
@@ -106,24 +99,29 @@ class CommandConnectClient(GObjectXpraClient):
         Utility superclass for clients that only send one command
         via the hello packet.
     """
+    COMMAND_TIMEOUT = EXTRA_TIMEOUT
 
     def __init__(self, conn, opts):
         GObjectXpraClient.__init__(self)
         GObjectXpraClient.init(self, opts)
-        self.connect_with_timeout(conn)
+        connection, display_desc = conn
+        self.display_desc = display_desc
+        self.connect_with_timeout(connection)
         self._protocol._log_stats  = False
         #not used by command line clients,
         #so don't try probing for printers, etc
         self.file_transfer = False
         self.printing = False
-
-    def make_hello(self):
-        capabilities = GObjectXpraClient.make_hello(self)
         #don't bother with many of these things for one-off commands:
         for x in ("ui_client", "wants_aliases", "wants_encodings",
                   "wants_versions", "wants_features", "wants_sound", "windows"):
-            capabilities[x] = False
-        return capabilities
+            self.hello_extra[x] = False
+
+    def connect_with_timeout(self, conn):
+        self.setup_connection(conn)
+        if conn.timeout>0:
+            glib.timeout_add((conn.timeout + self.COMMAND_TIMEOUT) * 1000, self.timeout)
+        glib.idle_add(self.send_hello)
 
     def _process_connection_lost(self, packet):
         #override so we don't log a warning
@@ -137,6 +135,7 @@ class CommandConnectClient(GObjectXpraClient):
         log("server_capabilities: %s", self.server_capabilities)
         log("protocol state: %s", self._protocol.save_state())
         self.do_command()
+        return True
 
     def do_command(self):
         raise NotImplementedError()
@@ -152,7 +151,28 @@ class SendCommandConnectClient(CommandConnectClient):
     def server_connection_established(self):
         assert self.parse_encryption_capabilities(), "encryption failure"
         assert self.parse_network_capabilities(), "network capabilities failure"
-        CommandConnectClient.server_connection_established(self)
+        return CommandConnectClient.server_connection_established(self)
+
+
+class HelloRequestClient(SendCommandConnectClient):
+    """
+        Utility superclass for clients that send a server request
+        as part of the hello packet.
+    """
+
+    def make_hello_base(self):
+        caps = CommandConnectClient.make_hello_base(self)
+        caps.update(self.hello_request())
+        return caps
+
+    def timeout(self, *args):
+        self.warn_and_quit(EXIT_TIMEOUT, "timeout: server did not disconnect us")
+
+    def hello_request(self):
+        raise NotImplementedError()
+
+    def do_command(self):
+        self.quit(EXIT_OK)
 
 
 class ScreenshotXpraClient(CommandConnectClient):
@@ -164,13 +184,14 @@ class ScreenshotXpraClient(CommandConnectClient):
     def __init__(self, conn, opts, screenshot_filename):
         self.screenshot_filename = screenshot_filename
         CommandConnectClient.__init__(self, conn, opts)
+        self.hello_extra["screenshot_request"] = True
 
     def timeout(self, *args):
         self.warn_and_quit(EXIT_TIMEOUT, "timeout: did not receive the screenshot")
 
     def _process_screenshot(self, packet):
         (w, h, encoding, _, img_data) = packet[1:6]
-        assert encoding=="png"
+        assert encoding==b"png", "expected png screenshot data but got %s" % bytestostr(encoding)
         if len(img_data)==0:
             self.warn_and_quit(EXIT_OK, "screenshot is empty and has not been saved (maybe there are no windows or they are not currently shown)")
             return
@@ -182,52 +203,81 @@ class ScreenshotXpraClient(CommandConnectClient):
         GObjectXpraClient.init_packet_handlers(self)
         self._ui_packet_handlers["screenshot"] = self._process_screenshot
 
-    def make_hello(self):
-        capabilities = GObjectXpraClient.make_hello(self)
-        capabilities["screenshot_request"] = True
-        return capabilities
-
 
 class InfoXpraClient(CommandConnectClient):
     """ This client does one thing only:
         it queries the server with an 'info' request
     """
 
+    def __init__(self, *args):
+        CommandConnectClient.__init__(self, *args)
+        self.hello_extra["info_request"] = True
+        if FLATTEN_INFO!=1:
+            self.hello_extra["info-namespace"] = True
+
     def timeout(self, *args):
         self.warn_and_quit(EXIT_TIMEOUT, "timeout: did not receive the info")
 
     def do_command(self):
         if self.server_capabilities:
-            def sorted_nicely(l):
-                """ Sort the given iterable in the way that humans expect."""
-                def convert(text):
-                    if text.isdigit():
-                        return int(text)
-                    else:
-                        return text
-                alphanum_key = lambda key: [ convert(c) for c in re.split('([0-9]+)', bytestostr(key)) ]
-                return sorted(l, key = alphanum_key)
-            for k in sorted_nicely(self.server_capabilities.keys()):
-                v = self.server_capabilities.get(k)
-                if sys.version_info[0]>=3:
-                    #FIXME: this is a nasty and horrible python3 workaround (yet again)
-                    #we want to print bytes as strings without the ugly 'b' prefix..
-                    #it assumes that all the strings are raw or in (possibly nested) lists or tuples only
-                    def fixvalue(w):
-                        if type(w)==bytes:
-                            return bytestostr(w)
-                        elif type(w) in (tuple,list):
-                            return type(w)([fixvalue(x) for x in w])
-                        return w
-                    v = fixvalue(v)
-                log.info("%s=%s", bytestostr(k), nonl(v))
+            if FLATTEN_INFO<2:
+                #compatibility mode:
+                for k in sorted_nicely(self.server_capabilities.keys()):
+                    v = self.server_capabilities.get(k)
+                    if PYTHON3:
+                        #FIXME: this is a nasty and horrible python3 workaround (yet again)
+                        #we want to print bytes as strings without the ugly 'b' prefix..
+                        #it assumes that all the strings are raw or in (possibly nested) lists or tuples only
+                        def fixvalue(w):
+                            if type(w)==bytes:
+                                return bytestostr(w)
+                            elif type(w) in (tuple,list):
+                                return type(w)([fixvalue(x) for x in w])
+                            return w
+                        v = fixvalue(v)
+                    log.info("%s=%s", bytestostr(k), nonl(v))
+            else:
+                print_nested_dict(self.server_capabilities)
         self.quit(EXIT_OK)
 
-    def make_hello(self):
-        capabilities = GObjectXpraClient.make_hello(self)
-        log("make_hello() adding info_request to %s", capabilities)
-        capabilities["info_request"] = True
-        return capabilities
+
+class ConnectTestXpraClient(CommandConnectClient):
+    """ This client does one thing only:
+        it queries the server with an 'info' request
+    """
+
+    def __init__(self, *args, **kwargs):
+        CommandConnectClient.__init__(self, *args)
+        self.value = get_hex_uuid()
+        self.hello_extra.update({
+            "connect_test_request"      : self.value,
+            #tells proxy servers we don't want to connect to the real / new instance:
+            "connect"                   : False,
+            #older servers don't know about connect-test,
+            #pretend that we're interested in info:
+            "info_request"              : True,
+            "info-namespace"            : True,
+            })
+        self.hello_extra.update(kwargs)
+
+    def timeout(self, *args):
+        self.warn_and_quit(EXIT_TIMEOUT, "timeout: no server response")
+
+    def _process_connection_lost(self, packet):
+        #we should always receive a hello back and call do_command,
+        #which sets the correct exit code, landing here is an error:
+        self.quit(EXIT_FAILURE)
+
+    def do_command(self):
+        if self.server_capabilities:
+            ctr = self.server_capabilities.get("connect_test_response")
+            log("do_command() expected connect test response='%s', got '%s'", self.value, ctr)
+            if ctr==self.value:
+                self.quit(EXIT_OK)
+            else:
+                self.quit(EXIT_INTERNAL_ERROR)
+        else:
+            self.quit(EXIT_FAILURE)
 
 
 class MonitorXpraClient(SendCommandConnectClient):
@@ -236,14 +286,16 @@ class MonitorXpraClient(SendCommandConnectClient):
         If the server does not support this feature it exits with an error.
     """
 
+    def __init__(self, *args):
+        SendCommandConnectClient.__init__(self, *args)
+        for x in ("wants_features", "wants_events", "event_request"):
+            self.hello_extra[x] = True
+
     def timeout(self, *args):
         pass
         #self.warn_and_quit(EXIT_TIMEOUT, "timeout: did not receive the info")
 
     def do_command(self):
-        if not self.server_capabilities.get("server-events") or not self.server_capabilities.get("event_request"):
-            self.warn_and_quit(EXIT_UNSUPPORTED, "this server does not support event monitor mode")
-            return
         log.info("waiting for server events")
 
     def _process_server_event(self, packet):
@@ -254,35 +306,33 @@ class MonitorXpraClient(SendCommandConnectClient):
         self._packet_handlers["server-event"] = self._process_server_event
         self._packet_handlers["ping"] = self._process_ping
 
-    def make_hello(self):
-        capabilities = SendCommandConnectClient.make_hello(self)
-        log("make_hello() adding info_request to %s", capabilities)
-        capabilities["wants_features"]  = True      #so we can verify that the server supports monitor mode
-        capabilities["wants_events"]    = True      #tell the server we do support server events
-        capabilities["event_request"]   = True      #ask the server to enter this request mode (we're not a proper client)
-        return capabilities
-
     def _process_ping(self, packet):
         echotime = packet[1]
         self.send("ping_echo", echotime, 0, 0, 0, -1)
 
 
-class VersionXpraClient(CommandConnectClient):
+class VersionXpraClient(HelloRequestClient):
     """ This client does one thing only:
         it queries the server for version information and prints it out
     """
 
-    def timeout(self, *args):
-        self.warn_and_quit(EXIT_TIMEOUT, "timeout: did not receive the version")
+    def hello_request(self):
+        return {
+            "version_request"       : True,
+            "full-version-request"  : True,
+            }
+
+    def parse_network_capabilities(self, *args):
+        #don't bother checking anything - this could generate warnings
+        return True
 
     def do_command(self):
-        self.warn_and_quit(EXIT_OK, str(self.server_capabilities.get("version")))
-
-    def make_hello(self):
-        capabilities = GObjectXpraClient.make_hello(self)
-        log("make_hello() adding version_request to %s", capabilities)
-        capabilities["version_request"] = True
-        return capabilities
+        v = self.server_capabilities.get(b"version")
+        if not v:
+            self.warn_and_quit(EXIT_FAILURE, "server did not provide the version information")
+        else:
+            sys.stdout.write("%s\n" % (bytestostr(v),))
+            self.quit(EXIT_OK)
 
 
 class ControlXpraClient(CommandConnectClient):
@@ -323,8 +373,10 @@ class PrintClient(SendCommandConnectClient):
         #print command arguments:
         #filename, file_data, mimetype, source_uuid, title, printer, no_copies, print_options_str = packet[1:9]
         self.command = command[1:]
-        #FIXME: load as needed...
-        from xpra.os_util import load_binary_file
+        #TODO: load as needed...
+        def sizeerr(size):
+            self.warn_and_quit(EXIT_FILE_TOO_BIG, "the file is too large: %iMB (the file size limit is %iMB)" % (size//1024//1024, self.file_size_limit))
+            return
         if self.filename=="-":
             #replace with filename proposed
             self.filename = command[2]
@@ -332,10 +384,17 @@ class PrintClient(SendCommandConnectClient):
             self.file_data = sys.stdin.read()
             log("read %i bytes from stdin", len(self.file_data))
         else:
+            import os.path
+            size = os.path.getsize(self.filename)
+            if size>self.file_size_limit*1024*1024:
+                sizeerr(size)
+                return
+            from xpra.os_util import load_binary_file
             self.file_data = load_binary_file(self.filename)
             log("read %i bytes from %s", len(self.file_data), self.filename)
-        if len(self.file_data)>=self.file_size_limit*1024*1024:
-            self.warn_and_quit(EXIT_FILE_TOO_BIG, "the file is too large: %iMB (the file size limit is %iMB)" % (len(self.file_data)//1024//1024, self.file_size_limit))
+        size = len(self.file_data)
+        if size>self.file_size_limit*1024*1024:
+            sizeerr(size)
             return
         assert self.file_data, "no data found for '%s'" % self.filename
 
@@ -364,58 +423,78 @@ class PrintClient(SendCommandConnectClient):
         return capabilities
 
 
-class ExitXpraClient(SendCommandConnectClient):
+class ExitXpraClient(HelloRequestClient):
     """ This client does one thing only:
         it asks the server to terminate (like stop),
         but without killing the Xvfb or clients.
     """
 
-    def timeout(self, *args):
-        self.warn_and_quit(EXIT_TIMEOUT, "timeout: server did not disconnect us")
-
-    def make_hello(self):
-        capabilities = SendCommandConnectClient.make_hello(self)
-        capabilities["exit_request"] = True
-        return capabilities
+    def hello_request(self):
+        return {"exit_request" : True}
 
     def do_command(self):
         self.idle_add(self.send, "exit-server")
 
 
-class StopXpraClient(SendCommandConnectClient):
+class StopXpraClient(HelloRequestClient):
     """ stop a server """
 
-    def make_hello(self):
-        #used for telling the proxy server we want "stop"
-        #(as waiting for the hello back would be too late)
-        capabilities = SendCommandConnectClient.make_hello(self)
-        capabilities["stop_request"] = True
-        return capabilities
-
-    def timeout(self, *args):
-        self.warn_and_quit(EXIT_TIMEOUT, "timeout: server did not disconnect us")
+    def hello_request(self):
+        return {"stop_request"  : True}
 
     def do_command(self):
-        self.idle_add(self.send, "shutdown-server")
+        if not self.can_shutdown_server:
+            log.error("Error: cannot shutdown this server")
+            log.error(" the feature is disable on the server")
+            self.quit(EXIT_FAILURE)
+            return
+        self.idle_add(self.send_shutdown_server)
         #not exiting the client here,
         #the server should send us the shutdown disconnection message anyway
         #and if not, we will then hit the timeout to tell us something went wrong
 
-class DetachXpraClient(SendCommandConnectClient):
+
+class DetachXpraClient(HelloRequestClient):
     """ run the detach subcommand """
 
-    def make_hello(self):
-        #used for telling the proxy server we want "detach"
-        #(older versions ignore this flag and detach because this is a new valid connection
-        # but this breaks if sharing is enabled!)
-        capabilities = SendCommandConnectClient.make_hello(self)
-        capabilities["detach_request"] = True
-        return capabilities
-
-    def timeout(self, *args):
-        self.warn_and_quit(EXIT_TIMEOUT, "timeout: server did not disconnect us")
+    def hello_request(self):
+        return {"detach_request" : True}
 
     def do_command(self):
         self.idle_add(self.send, "disconnect", DONE, "detaching")
         #not exiting the client here,
         #the server should disconnect us with the response
+
+class WaitForDisconnectXpraClient(DetachXpraClient):
+    """ we just want the connection to close """
+
+    def _process_disconnect(self, packet):
+        self.quit(EXIT_OK)
+
+
+class RequestStartClient(HelloRequestClient):
+    """ request the system proxy server to start a new session for us """
+    #wait longer for this command to return:
+    from xpra.scripts.main import WAIT_SERVER_TIMEOUT
+    COMMAND_TIMEOUT = EXTRA_TIMEOUT+WAIT_SERVER_TIMEOUT
+
+    def hello_request(self):
+        return {
+            "start-new-session" : self.start_new_session,
+            #tells proxy servers we don't want to connect to the real / new instance:
+            "connect"                   : False,
+            }
+
+    def server_connection_established(self):
+        #the server should respond with the display chosen
+        log("server_connection_established() exit_code=%s", self.exit_code)
+        display = self.server_capabilities.get("display")
+        if display:
+            try:
+                sys.stderr.write("session now available on display %s\n" % display)
+                sys.stderr.flush()
+            except:
+                pass
+        if not self.exit_code:
+            self.quit(0)
+        return True
