@@ -6,21 +6,16 @@
 # later version. See the file COPYING for details.
 
 import gobject
-try:
-    #we *have to* do this as early as possible on win32..
-    gobject.threads_init()
-except:
-    pass
+gobject.threads_init()
 import gtk
 from gtk import gdk
-gdk.threads_init()
 
 
 from xpra.gtk_common.gtk_util import gtk_main, color_parse
 from xpra.client.gtk_base.gtk_client_base import GTKXpraClient
 from xpra.client.gtk2.tray_menu import GTK2TrayMenu
 from xpra.client.window_border import WindowBorder
-from xpra.net.compression import Uncompressed
+from xpra.net.compression import Compressible
 from xpra.log import Logger
 
 log = Logger("gtk", "client")
@@ -135,12 +130,6 @@ class XpraClient(GTKXpraClient):
             log.warn("cannot load GTK2 notifier: %s", e)
         return ncs
 
-    def make_new_window(self, *args):
-        w = GTKXpraClient.make_new_window(self, *args)
-        if w:
-            w.border = self.border
-        return w
-
     def do_get_core_encodings(self):
         encodings = GTKXpraClient.do_get_core_encodings(self)
         #we can handle rgb32 format (but not necessarily transparency!)
@@ -151,9 +140,9 @@ class XpraClient(GTKXpraClient):
         return encodings
 
 
-    def _startup_complete(self, *args):
-        GTKXpraClient._startup_complete(self, *args)
-        gtk.gdk.notify_startup_complete()
+    def _process_startup_complete(self, *args):
+        GTKXpraClient._process_startup_complete(self, *args)
+        gdk.notify_startup_complete()
 
 
     def get_tray_menu_helper_classes(self):
@@ -165,9 +154,8 @@ class XpraClient(GTKXpraClient):
     def process_clipboard_packet(self, packet):
         clipboardlog("process_clipboard_packet(%s) level=%s", packet, gtk.main_level())
         #check for clipboard loops:
-        if gtk.main_level()>=40:
-            clipboardlog.warn("loop nesting too deep: %s", gtk.main_level())
-            clipboardlog.warn("you may have a clipboard forwarding loop, disabling the clipboard")
+        from xpra.clipboard.clipboard_base import nesting_check
+        if not nesting_check():
             self.clipboard_enabled = False
             self.emit("clipboard-toggled")
             return
@@ -206,8 +194,9 @@ class XpraClient(GTKXpraClient):
             try:
                 parts = co.split(".")
                 mod = ".".join(parts[:-1])
-                __import__(mod, {}, {}, [parts[-1]])
-                loadable.append(co)
+                module = __import__(mod, {}, {}, [parts[-1]])
+                helperclass = getattr(module, parts[-1])
+                loadable.append(helperclass)
             except ImportError as e:
                 clipboardlog("cannot load %s: %s", co, e)
                 continue
@@ -221,39 +210,26 @@ class XpraClient(GTKXpraClient):
         """
         clipboard_options = self.get_clipboard_helper_classes()
         clipboardlog("make_clipboard_helper() options=%s", clipboard_options)
-        for classname in clipboard_options:
-            module_name, _class = classname.rsplit(".", 1)
-            c = self.try_load_clipboard_helper(module_name, _class)
-            if c:
-                return c
-        return None
-
-    def try_load_clipboard_helper(self, module, classname):
-        try:
-            m = __import__(module, {}, {}, classname)
-            if m:
-                if not hasattr(m, classname):
-                    log.warn("cannot load %s from %s, odd", classname, m)
-                    return None
-                c = getattr(m, classname)
-                if c:
-                    return self.setup_clipboard_helper(c)
-        except ImportError as e:
-            clipboardlog.error("Error: cannot load %s.%s:", module, classname)
-            clipboardlog.error(" %s", e)
-            return None
-        except:
-            clipboardlog.error("cannot load %s.%s", module, classname, exc_info=True)
-            return None
-        clipboardlog.error("cannot load %s.%s", module, classname)
+        for helperclass in clipboard_options:
+            try:
+                return self.setup_clipboard_helper(helperclass)
+            except ImportError as e:
+                clipboardlog.error("Error: cannot instantiate %s:", helperclass)
+                clipboardlog.error(" %s", e)
+            except:
+                clipboardlog.error("cannot instantiate %s", helperclass, exc_info=True)
         return None
 
     def setup_clipboard_helper(self, helperClass):
         clipboardlog("setup_clipboard_helper(%s)", helperClass)
         #first add the platform specific one, (may be None):
         from xpra.platform.features import CLIPBOARDS
-        kwargs= {"clipboards.local"     : CLIPBOARDS,                   #all the local clipboards supported
-                 "clipboards.remote"    : self.server_clipboards}       #all the remote clipboards supported
+        kwargs= {
+                 "clipboards.local"     : CLIPBOARDS,                   #all the local clipboards supported
+                 "clipboards.remote"    : self.server_clipboards,       #all the remote clipboards supported
+                 "can-send"             : self.client_clipboard_direction in ("to-server", "both"),
+                 "can-receive"          : self.client_clipboard_direction in ("to-client", "both"),
+                 }
         #only allow translation overrides if we have a way of telling the server about them:
         if self.server_supports_clipboard_enable_selections:
             kwargs.update({
@@ -268,7 +244,7 @@ class XpraClient(GTKXpraClient):
             packet = list(parts)
             for i in range(len(packet)):
                 v = packet[i]
-                if type(v)==Uncompressed:
+                if type(v)==Compressible:
                     #register the compressor which will fire in protocol.encode:
                     def compress_clipboard():
                         clipboardlog("compress_clipboard() compressing %s", v)
@@ -343,14 +319,14 @@ class XpraClient(GTKXpraClient):
 
 
     def window_grab(self, window):
-        mask = gtk.gdk.BUTTON_PRESS_MASK | gtk.gdk.BUTTON_RELEASE_MASK | gtk.gdk.POINTER_MOTION_MASK  | gtk.gdk.POINTER_MOTION_HINT_MASK | gtk.gdk.ENTER_NOTIFY_MASK | gtk.gdk.LEAVE_NOTIFY_MASK
-        gtk.gdk.pointer_grab(window.get_window(), owner_events=True, event_mask=mask)
+        mask = gdk.BUTTON_PRESS_MASK | gdk.BUTTON_RELEASE_MASK | gdk.POINTER_MOTION_MASK  | gdk.POINTER_MOTION_HINT_MASK | gdk.ENTER_NOTIFY_MASK | gdk.LEAVE_NOTIFY_MASK
+        gdk.pointer_grab(window.get_window(), owner_events=True, event_mask=mask)
         #also grab the keyboard so the user won't Alt-Tab away:
-        gtk.gdk.keyboard_grab(window.get_window(), owner_events=False)
+        gdk.keyboard_grab(window.get_window(), owner_events=False)
 
     def window_ungrab(self):
-        gtk.gdk.pointer_ungrab()
-        gtk.gdk.keyboard_ungrab()
+        gdk.pointer_ungrab()
+        gdk.keyboard_ungrab()
 
 
 gobject.type_register(XpraClient)
