@@ -5,7 +5,6 @@
 
 from xpra.client.gtk_base.gtk_client_base import GTKXpraClient
 
-import sys
 from gi.repository import GObject               #@UnresolvedImport
 from gi.repository import Gtk                   #@UnresolvedImport
 
@@ -15,11 +14,10 @@ from gi.repository import Gtk                   #@UnresolvedImport
 
 from xpra.client.gtk3.client_window import ClientWindow
 from xpra.client.gtk3.tray_menu import GTK3TrayMenu
+from xpra.os_util import WIN32
 from xpra.log import Logger
 log = Logger("gtk", "client")
 
-WIN32 = sys.platform.startswith("win")
-
 
 class XpraClient(GTKXpraClient):
 
@@ -55,11 +53,8 @@ class XpraClient(GTKXpraClient):
 
     def do_get_core_encodings(self):
         cencs = GTKXpraClient.do_get_core_encodings(self)
-        for x in ("webp", ):
-            if x in cencs:
-                cencs.remove(x)
-        #for some reason, the cairo_workaround does not work for ARGB32
-        #cencs.append("rgb32")
+        #we can now paint with alpha via pixbufs:
+        cencs.append("rgb32")
         return cencs
 
 
@@ -73,7 +68,7 @@ class XpraClient(GTKXpraClient):
 
     def set_windows_cursor(self, windows, cursor_data):
         #avoid buggy win32 version:
-        if not sys.platform.startswith("win"):
+        if not WIN32:
             GTKXpraClient.set_windows_cursor(self, windows, cursor_data)
 
     def get_mouse_position(self):
