@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2012-2015 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -11,6 +11,7 @@ cairo           = import_cairo()
 PixbufLoader    = import_pixbufloader()
 
 from xpra.gtk_common.gtk_util import cairo_set_source_pixbuf, gdk_cairo_context
+from xpra.gtk_common.paint_colors import get_paint_box_color
 from xpra.client.gtk_base.gtk_window_backing_base import GTKWindowBacking
 from xpra.codecs.loader import get_codec
 from xpra.os_util import BytesIOClass, memoryview_to_bytes, strtobytes
@@ -29,7 +30,6 @@ Superclass for gtk2 and gtk3 cairo implementations.
 """
 class CairoBackingBase(GTKWindowBacking):
 
-
     def init(self, ww, wh, w, h):
         self.size = w, h
         self.render_size = ww, wh
@@ -76,26 +76,47 @@ class CairoBackingBase(GTKWindowBacking):
         GTKWindowBacking.close(self)
 
 
-    def cairo_paint_pixbuf(self, pixbuf, x, y):
+    def cairo_paint_pixbuf(self, pixbuf, x, y, options):
         """ must be called from UI thread """
-        log("cairo_paint_pixbuf(%s, %s, %s) backing=%s", pixbuf, x, y, self._backing)
-        #now use it to paint:
-        gc = gdk_cairo_context(cairo.Context(self._backing))
-        gc.set_operator(cairo.OPERATOR_SOURCE)
-        cairo_set_source_pixbuf(gc, pixbuf, x, y)
-        gc.paint()
+        log("source pixbuf: %s", pixbuf)
+        w, h = pixbuf.get_width(), pixbuf.get_height()
+        self.cairo_paint_from_source(cairo_set_source_pixbuf, pixbuf, x, y, w, h, options)
 
-    def cairo_paint_surface(self, img_surface, x, y):
+    def cairo_paint_surface(self, img_surface, x, y, options={}):
+        w, h = img_surface.get_width(), img_surface.get_height()
+        log("source image surface: %s", (img_surface.get_format(), w, h, img_surface.get_stride(), img_surface.get_content(), ))
+        def set_source_surface(gc, surface, sx, sy):
+            gc.set_source_surface(surface, sx, sy)
+        self.cairo_paint_from_source(set_source_surface, img_surface, x, y, w, h, options)
+
+    def cairo_paint_from_source(self, set_source_fn, source, x, y, w, h, options):
         """ must be called from UI thread """
-        log("cairo_paint_surface(%s, %s, %s)", img_surface, x, y)
-        log("source image surface: %s", (img_surface.get_format(), img_surface.get_width(), img_surface.get_height(), img_surface.get_stride(), img_surface.get_content(), ))
+        log("cairo_paint_surface(%s, %s, %s, %s, %s, %s, %s) backing=%s, paint box line width=%i", set_source_fn, source, x, y, w, h, options, self._backing, self.paint_box_line_width)
         gc = gdk_cairo_context(cairo.Context(self._backing))
+        if self.paint_box_line_width:
+            gc.save()
+
+        gc.rectangle(x, y, w, h)
+        gc.clip()
+
         gc.set_operator(cairo.OPERATOR_CLEAR)
-        gc.rectangle(x, y, img_surface.get_width(), img_surface.get_height())
+        gc.rectangle(x, y, w, h)
         gc.fill()
+
         gc.set_operator(cairo.OPERATOR_SOURCE)
-        gc.set_source_surface(img_surface, x, y)
+        gc.translate(x, y)
+        gc.rectangle(0, 0, w, h)
+        set_source_fn(gc, source, 0, 0)
         gc.paint()
+        if self.paint_box_line_width and options:
+            gc.restore()
+            encoding = options.get("encoding")
+            if encoding:
+                color = get_paint_box_color(encoding)
+                gc.set_line_width(self.paint_box_line_width)
+                gc.set_source_rgba(*color)
+                gc.rectangle(x, y, w, h)
+                gc.stroke()
 
 
     def _do_paint_rgb24(self, img_data, x, y, width, height, rowstride, options):
@@ -109,7 +130,7 @@ class CairoBackingBase(GTKWindowBacking):
 
 
     def nasty_rgb_via_png_paint(self, cairo_format, has_alpha, img_data, x, y, width, height, rowstride, rgb_format):
-        log("nasty_rgb_via_png_paint%s", (cairo_format, has_alpha, len(img_data), x, y, width, height, rowstride, rgb_format))
+        log.warn("nasty_rgb_via_png_paint%s", (cairo_format, has_alpha, len(img_data), x, y, width, height, rowstride, rgb_format))
         #PIL fallback
         PIL = get_codec("PIL")
         if has_alpha:
@@ -119,10 +140,12 @@ class CairoBackingBase(GTKWindowBacking):
         #use frombytes rather than frombuffer to be compatible with python3 new-style buffers
         #this is slower, but since this codepath is already dreadfully slow, we don't care
         bdata = strtobytes(memoryview_to_bytes(img_data))
+        src_format = rgb_format.replace("X", "A")
         try:
-            img = PIL.Image.frombytes(oformat, (width,height), bdata, "raw", rgb_format.replace("X", "A"), rowstride, 1)
+            img = PIL.Image.frombytes(oformat, (width,height), bdata, "raw", src_format, rowstride, 1)
         except ValueError as e:
-            raise Exception("failed to parse raw %s data to %s: %s" % (rgb_format, oformat, e))
+            log("PIL Image frombytes:", exc_info=True)
+            raise Exception("failed to parse raw %s data as %s to %s: %s" % (rgb_format, src_format, oformat, e))
         #This is insane, the code below should work, but it doesn't:
         # img_data = bytearray(img.tostring('raw', oformat, 0, 1))
         # pixbuf = pixbuf_new_from_data(img_data, COLORSPACE_RGB, True, 8, width, height, rowstride)
@@ -138,14 +161,21 @@ class CairoBackingBase(GTKWindowBacking):
 
 
     def cairo_draw(self, context):
-        log("cairo_draw(%s) backing=%s", context, self._backing)
+        log("cairo_draw: size=%s, render-size=%s, offsets=%s", self.size, self.render_size, self.offsets)
         if self._backing is None:
             return False
+        #try:
+        #    log("clip rectangles=%s", context.copy_clip_rectangle_list())
+        #except:
+        #    log.error("clip:", exc_info=True)
         try:
             if self.render_size!=self.size:
                 ww, wh = self.render_size
                 w, h = self.size
                 context.scale(float(ww)/w, float(wh)/h)
+            x, y = self.offsets[:2]
+            if x!=0 or y!=0:
+                context.translate(x, y)
             context.set_source_surface(self._backing, 0, 0)
             context.set_operator(cairo.OPERATOR_SOURCE)
             context.paint()
