@@ -1,14 +1,14 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2015 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2017 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import os, sys
+import os
 import weakref
 from xpra.gtk_common.gobject_compat import import_gobject, import_gtk, import_gdk, is_gtk3
-from xpra.client.gtk_base.gtk_client_window_base import HAS_X11_BINDINGS
+from xpra.client.gtk_base.gtk_client_window_base import HAS_X11_BINDINGS, XSHAPE
 gobject = import_gobject()
 gtk = import_gtk()
 gdk = import_gdk()
@@ -25,24 +25,26 @@ filelog = Logger("gtk", "client", "file")
 
 from xpra.gtk_common.quit import (gtk_main_quit_really,
                            gtk_main_quit_on_fatal_exceptions_enable)
-from xpra.util import bytestostr, updict, pver, iround, DEFAULT_METADATA_SUPPORTED
+from xpra.util import updict, pver, iround, flatten_dict, envbool, typedict, DEFAULT_METADATA_SUPPORTED
+from xpra.os_util import bytestostr, WIN32, OSX, POSIX
+from xpra.simple_stats import std_unit
 from xpra.gtk_common.cursor_names import cursor_types
 from xpra.gtk_common.gtk_util import get_gtk_version_info, scaled_image, get_default_cursor, \
             new_Cursor_for_display, new_Cursor_from_pixbuf, icon_theme_get_default, \
             pixbuf_new_from_file, display_get_default, screen_get_default, get_pixbuf_from_data, \
             get_default_root_window, get_root_size, get_xwindow, \
-            INTERP_BILINEAR, WINDOW_TOPLEVEL
+            INTERP_BILINEAR, WINDOW_TOPLEVEL, DIALOG_DESTROY_WITH_PARENT, MESSAGE_INFO, BUTTONS_CLOSE
 from xpra.client.ui_client_base import UIXpraClient
 from xpra.client.gobject_client_base import GObjectXpraClient
 from xpra.client.gtk_base.gtk_keyboard_helper import GTKKeyboardHelper
 from xpra.client.gtk_base.session_info import SessionInfo
 from xpra.platform.paths import get_icon_filename
-from xpra.platform.gui import get_window_frame_sizes, get_window_frame_size, system_bell, get_workarea, get_workareas, get_fixed_cursor_size, get_menu_support_function
+from xpra.platform.gui import get_window_frame_sizes, get_window_frame_size, system_bell, get_fixed_cursor_size, get_menu_support_function, get_wm_name
 
 missing_cursor_names = set()
 
 METADATA_SUPPORTED = os.environ.get("XPRA_METADATA_SUPPORTED")
-USE_LOCAL_CURSORS = os.environ.get("XPRA_USE_LOCAL_CURSORS", "1")=="1"
+USE_LOCAL_CURSORS = envbool("XPRA_USE_LOCAL_CURSORS", True)
 
 
 class GTKXpraClient(UIXpraClient, GObjectXpraClient):
@@ -56,7 +58,10 @@ class GTKXpraClient(UIXpraClient, GObjectXpraClient):
         UIXpraClient.__init__(self)
         self.session_info = None
         self.bug_report = None
+        self.file_size_dialog = None
+        self.file_dialog = None
         self.start_new_command = None
+        self.keyboard_helper_class = GTKKeyboardHelper
         #opengl bits:
         self.client_supports_opengl = False
         self.opengl_enabled = False
@@ -133,6 +138,8 @@ class GTKXpraClient(UIXpraClient, GObjectXpraClient):
         if self.bug_report:
             self.bug_report.destroy()
             self.bug_report = None
+        self.close_file_size_warning()
+        self.close_file_upload_dialog()
         if self.start_new_command:
             self.start_new_command.destroy()
             self.start_new_command = None
@@ -140,6 +147,10 @@ class GTKXpraClient(UIXpraClient, GObjectXpraClient):
 
 
     def show_start_new_command(self, *args):
+        if not self.start_new_commands:
+            log.warn("Warning: cannot start new commands")
+            log.warn(" the feature is currently disabled on the server")
+            return
         log("show_start_new_command%s current start_new_command=%s, flag=%s", args, self.start_new_command, self.start_new_commands)
         if self.start_new_command is None:
             from xpra.client.gtk_base.start_new_command import getStartNewCommand
@@ -149,29 +160,88 @@ class GTKXpraClient(UIXpraClient, GObjectXpraClient):
         self.start_new_command.show()
         return self.start_new_command
 
+
+    def file_size_warning(self, action, location, basefilename, filesize, limit):
+        if self.file_size_dialog:
+            #close previous warning
+            self.file_size_dialog.destroy()
+            self.file_size_dialog = None
+        parent = None
+        msgs = (
+                "Warning: cannot %s the file '%s'" % (action, basefilename),
+                "this file is too large: %sB" % std_unit(filesize, unit=1024),
+                "the %s file size limit is %iMB" % (location, limit),
+                )
+        self.file_size_dialog = gtk.MessageDialog(parent, DIALOG_DESTROY_WITH_PARENT, MESSAGE_INFO,
+                                                  BUTTONS_CLOSE, "\n".join(msgs))
+        try:
+            image = gtk.image_new_from_stock(gtk.STOCK_DIALOG_WARNING, 64)
+            self.file_size_dialog.set_image(image)
+        except Exception as e:
+            log.warn("failed to set dialog image: %s", e)
+        self.file_size_dialog.connect("response", self.close_file_size_warning)
+        self.file_size_dialog.show()
+
+    def close_file_size_warning(self, *args):
+        fsd = self.file_size_dialog
+        if fsd:
+            self.file_size_dialog = None
+            fsd.destroy()
+
     def show_file_upload(self, *args):
-        filelog("show_file_upload%s can open=%s", args, self.server_open_files)
+        if self.file_dialog:
+            self.file_dialog.present()
+            return
+        filelog("show_file_upload%s can open=%s", args, self.remote_open_files)
         buttons = [gtk.STOCK_CANCEL,    gtk.RESPONSE_CANCEL]
-        if self.server_open_files:
+        if self.remote_open_files:
             buttons += [gtk.STOCK_OPEN,      gtk.RESPONSE_ACCEPT]
         buttons += [gtk.STOCK_OK,        gtk.RESPONSE_OK]
-        dialog = gtk.FileChooserDialog("File to upload", parent=None, action=gtk.FILE_CHOOSER_ACTION_OPEN, buttons=tuple(buttons))
-        dialog.set_default_response(gtk.RESPONSE_OK)
-        v = dialog.run()
+        self.file_dialog = gtk.FileChooserDialog("File to upload", parent=None, action=gtk.FILE_CHOOSER_ACTION_OPEN, buttons=tuple(buttons))
+        self.file_dialog.set_default_response(gtk.RESPONSE_OK)
+        self.file_dialog.connect("response", self.file_upload_dialog_response)
+        self.file_dialog.show()
+
+    def close_file_upload_dialog(self):
+        fd = self.file_dialog
+        if fd:
+            fd.destroy()
+            self.file_dialog = None
+
+    def file_upload_dialog_response(self, dialog, v):
         if v not in (gtk.RESPONSE_OK, gtk.RESPONSE_ACCEPT):
             filelog("dialog response code %s", v)
-            dialog.destroy()
+            self.close_file_upload_dialog()
             return
         filename = dialog.get_filename()
+        filelog("file_upload_dialog_response: filename=%s", filename)
+        try:
+            filesize = os.stat(filename).st_size
+        except:
+            pass
+        else:
+            if not self.check_file_size("upload", filename, filesize):
+                self.close_file_upload_dialog()
+                return
         gfile = dialog.get_file()
-        data, filesize, entity = gfile.load_contents()
-        filelog("load_contents: filename=%s, %i bytes, entity=%s, response=%s", filename, filesize, entity, v)
-        dialog.destroy()
-        self.send_file(filename, data, filesize, openit=(v==gtk.RESPONSE_ACCEPT))
+        self.close_file_upload_dialog()
+        filelog("load_contents: filename=%s, response=%s", filename, v)
+        gfile.load_contents_async(self.file_upload_ready, user_data=(filename, v==gtk.RESPONSE_ACCEPT))
+
+    def file_upload_ready(self, gfile, result, user_data):
+        filelog("file_upload_ready%s", (gfile, result, user_data))
+        filename, openit = user_data
+        data, filesize, entity = gfile.load_contents_finish(result)
+        filelog("load_contents_finish(%s)=%s", result, (type(data), filesize, entity))
+        if not data:
+            log.warn("Warning: failed to load file '%s'", filename)
+            return
+        filelog("load_contents: filename=%s, %i bytes, entity=%s, openit=%s", filename, filesize, entity, openit)
+        self.send_file(filename, "", data, filesize=filesize, openit=openit)
 
 
     def show_about(self, *args):
-        from xpra.client.gtk_base.about import about
+        from xpra.gtk_common.about import about
         about()
 
     def show_session_info(self, *args):
@@ -183,7 +253,11 @@ class GTKXpraClient(UIXpraClient, GObjectXpraClient):
         pixbuf = self.get_pixbuf("statistics.png")
         if not pixbuf:
             pixbuf = self.get_pixbuf("xpra.png")
-        self.session_info = SessionInfo(self, self.session_name, pixbuf, self._protocol._conn, self.get_pixbuf)
+        conn = None
+        p = self._protocol
+        if p:
+            conn = p._conn
+        self.session_info = SessionInfo(self, self.session_name, pixbuf, conn, self.get_pixbuf)
         self.session_info.set_args(*args)
         self.session_info.show_all()
 
@@ -268,17 +342,28 @@ class GTKXpraClient(UIXpraClient, GObjectXpraClient):
         if self.frame_request_window:
             v = self.get_frame_extents(self.frame_request_window)
             if v:
-                l, r, t, b = v
-                wfs["frame"] = (l, r, t, b)
-                wfs["offset"] = (l, t)
+                try:
+                    wm_name = get_wm_name()
+                except:
+                    wm_name = None
+                try:
+                    if len(v)==8:
+                        framelog.warn("Warning: invalid frame extents value '%s'", v)
+                        if wm_name:
+                            framelog.warn(" this is probably a bug in '%s'", wm_name)
+                        v = v[4:]
+                        framelog.warn(" using '%s' instead", v)
+                    l, r, t, b = v
+                    wfs["frame"] = (l, r, t, b)
+                    wfs["offset"] = (l, t)
+                except Exception as e:
+                    framelog.warn("Warning: invalid frame extents value '%s'", v)
+                    framelog.warn(" %s", e)
+                    framelog.warn(" this is probably a bug in '%s'", wm_name)
         framelog("get_window_frame_sizes()=%s", wfs)
         return wfs
 
 
-    def make_keyboard_helper(self, keyboard_sync, key_shortcuts):
-        return GTKKeyboardHelper(self.send, keyboard_sync, key_shortcuts)
-
-
     def _add_statusicon_tray(self, tray_list):
         #add gtk.StatusIcon tray:
         try:
@@ -336,7 +421,7 @@ class GTKXpraClient(UIXpraClient, GObjectXpraClient):
     def make_hello(self):
         capabilities = UIXpraClient.make_hello(self)
         capabilities["named_cursors"] = len(cursor_types)>0
-        capabilities.update(get_gtk_version_info())
+        capabilities.update(flatten_dict(get_gtk_version_info()))
         #tell the server which icons GTK can use
         #so it knows when it should supply one as fallback
         it = icon_theme_get_default()
@@ -363,17 +448,18 @@ class GTKXpraClient(UIXpraClient, GObjectXpraClient):
             capabilities["window.states"] = ["fullscreen", "maximized", "sticky", "above", "below", "shaded", "iconified", "skip-taskbar", "skip-pager"]
             ms = list(DEFAULT_METADATA_SUPPORTED)
             #added in 0.15:
-            ms += ["command", "workspace", "above", "below", "sticky"]
-        if os.name=="posix":
+            ms += ["command", "workspace", "above", "below", "sticky",
+                   "set-initial-position"]  #0.17
+        if POSIX:
             #this is only really supported on X11, but posix is easier to check for..
             #"strut" and maybe even "fullscreen-monitors" could also be supported on other platforms I guess
             ms += ["shaded", "bypass-compositor", "strut", "fullscreen-monitors"]
-        if HAS_X11_BINDINGS:
+        if HAS_X11_BINDINGS and XSHAPE:
             ms += ["shape"]
         if self._set_window_menu:
             ms += ["menu"]
         #figure out if we can handle the "global menu" stuff:
-        if os.name=="posix" and not sys.platform.startswith("darwin"):
+        if POSIX and not OSX:
             try:
                 from xpra.dbus.helper import DBusHelper
                 assert DBusHelper
@@ -381,9 +467,8 @@ class GTKXpraClient(UIXpraClient, GObjectXpraClient):
                 pass
         log("metadata.supported: %s", ms)
         capabilities["metadata.supported"] = ms
-        #we need the bindings to support initiate-moveresize (posix only for now):
         updict(capabilities, "window", {
-               "initiate-moveresize"    : HAS_X11_BINDINGS,
+               "initiate-moveresize"    : True,
                "configure.pointer"      : True,
                "frame_sizes"            : self.get_window_frame_sizes()
                })
@@ -402,64 +487,8 @@ class GTKXpraClient(UIXpraClient, GObjectXpraClient):
 
 
     def get_screen_sizes(self, xscale=1, yscale=1):
-        def xs(v):
-            return iround(v/xscale)
-        def ys(v):
-            return iround(v/yscale)
-        def swork(*workarea):
-            return xs(workarea[0]), ys(workarea[1]), xs(workarea[2]), ys(workarea[3])
-        display = display_get_default()
-        i=0
-        screen_sizes = []
-        n_screens = display.get_n_screens()
-        screenlog("get_screen_sizes(%f, %f) found %s screens", xscale, yscale, n_screens)
-        while i<n_screens:
-            screen = display.get_screen(i)
-            j = 0
-            monitors = []
-            workareas = []
-            #native "get_workareas()" is only valid for a single screen (but describes all the monitors)
-            #and it is only implemented on win32 right now
-            #other platforms only implement "get_workarea()" instead, which is reported against the screen
-            n_monitors = screen.get_n_monitors()
-            screenlog(" screen %s has %s monitors", i, n_monitors)
-            if n_screens==1:
-                workareas = get_workareas()
-                if len(workareas)!=n_monitors:
-                    screenlog(" workareas: %s", workareas)
-                    screenlog(" number of monitors does not match number of workareas!")
-                    workareas = []
-            while j<screen.get_n_monitors():
-                geom = screen.get_monitor_geometry(j)
-                plug_name = ""
-                if hasattr(screen, "get_monitor_plug_name"):
-                    plug_name = screen.get_monitor_plug_name(j) or ""
-                wmm = -1
-                if hasattr(screen, "get_monitor_width_mm"):
-                    wmm = screen.get_monitor_width_mm(j)
-                hmm = -1
-                if hasattr(screen, "get_monitor_height_mm"):
-                    hmm = screen.get_monitor_height_mm(j)
-                monitor = [plug_name, xs(geom.x), ys(geom.y), xs(geom.width), ys(geom.height), wmm, hmm]
-                screenlog(" monitor %s: %s", j, monitor)
-                if workareas:
-                    w = workareas[j]
-                    monitor += list(swork(*w))
-                monitors.append(tuple(monitor))
-                j += 1
-            work_x, work_y, work_width, work_height = swork(0, 0, screen.get_width(), screen.get_height())
-            workarea = get_workarea()
-            if workarea:
-                work_x, work_y, work_width, work_height = swork(*workarea)
-            screenlog(" workarea=%s", workarea)
-            item = (screen.make_display_name(), xs(screen.get_width()), ys(screen.get_height()),
-                        screen.get_width_mm(), screen.get_height_mm(),
-                        monitors,
-                        work_x, work_y, work_width, work_height)
-            screenlog(" screen %s: %s", i, item)
-            screen_sizes.append(item)
-            i += 1
-        return screen_sizes
+        from xpra.gtk_common.gtk_util import get_screen_sizes
+        return get_screen_sizes(xscale, yscale)
 
 
     def reset_windows_cursors(self, *args):
@@ -469,7 +498,7 @@ class GTKXpraClient(UIXpraClient, GObjectXpraClient):
 
 
     def set_windows_cursor(self, windows, cursor_data):
-        cursorlog("set_windows_cursor(%s, ..)", windows)
+        cursorlog("set_windows_cursor(%s, args[%i])", windows, len(cursor_data))
         cursor = None
         if cursor_data:
             try:
@@ -481,6 +510,7 @@ class GTKXpraClient(UIXpraClient, GObjectXpraClient):
                 #use default:
                 cursor = get_default_cursor()
         for w in windows:
+            w.set_cursor_data(cursor_data)
             gdkwin = w.get_window()
             #trays don't have a gdk window
             if gdkwin:
@@ -490,10 +520,10 @@ class GTKXpraClient(UIXpraClient, GObjectXpraClient):
     def make_cursor(self, cursor_data):
         #if present, try cursor ny name:
         display = display_get_default()
-        cursorlog("make_cursor: has-name=%s, has-cursor-types=%s, xscale=%s, yscale=%s, USE_LOCAL_CURSORS=%s", len(cursor_data)>=9, bool(cursor_types), self.xscale, self.yscale, USE_LOCAL_CURSORS)
+        cursorlog("make_cursor: has-name=%s, has-cursor-types=%s, xscale=%s, yscale=%s, USE_LOCAL_CURSORS=%s", len(cursor_data)>=10, bool(cursor_types), self.xscale, self.yscale, USE_LOCAL_CURSORS)
         #named cursors cannot be scaled (round to 10 to compare so 0.95 and 1.05 are considered the same as 1.0, no scaling):
-        if len(cursor_data)>=9 and cursor_types and iround(self.xscale*10)==10 and iround(self.yscale*10)==10:
-            cursor_name = bytestostr(cursor_data[8])
+        if len(cursor_data)>=10 and cursor_types and iround(self.xscale*10)==10 and iround(self.yscale*10)==10:
+            cursor_name = bytestostr(cursor_data[9])
             if cursor_name and USE_LOCAL_CURSORS:
                 gdk_cursor = cursor_types.get(cursor_name.upper())
                 if gdk_cursor is not None:
@@ -505,21 +535,26 @@ class GTKXpraClient(UIXpraClient, GObjectXpraClient):
                         cursorlog("cursor name '%s' not found", cursor_name)
                         missing_cursor_names.add(cursor_name)
         #create cursor from the pixel data:
-        w, h, xhot, yhot, serial, pixels = cursor_data[2:8]
+        encoding, _, _, w, h, xhot, yhot, serial, pixels = cursor_data[0:9]
+        if encoding!=b"raw":
+            cursorlog.warn("Warning: invalid cursor encoding: %s", encoding)
+            return None
         if len(pixels)<w*h*4:
             import binascii
-            cursorlog.warn("not enough pixels provided in cursor data: %s needed and only %s bytes found (%s)", w*h*4, len(pixels), binascii.hexlify(pixels)[:100])
-            return
+            cursorlog.warn("Warning: not enough pixels provided in cursor data")
+            cursorlog.warn(" %s needed and only %s bytes found:", w*h*4, len(pixels))
+            cursorlog.warn(" '%s')", binascii.hexlify(pixels)[:100])
+            return None
         pixbuf = get_pixbuf_from_data(pixels, True, w, h, w*4)
         x = max(0, min(xhot, w-1))
         y = max(0, min(yhot, h-1))
         csize = display.get_default_cursor_size()
         cmaxw, cmaxh = display.get_maximal_cursor_size()
-        if len(cursor_data)>=11:
-            ssize = cursor_data[9]
-            smax = cursor_data[10]
+        if len(cursor_data)>=12:
+            ssize = cursor_data[10]
+            smax = cursor_data[11]
             cursorlog("server cursor sizes: default=%s, max=%s", ssize, smax)
-        cursorlog("new cursor at %s,%s with serial=%s, dimensions: %sx%s, len(pixels)=%s, default cursor size is %s, maximum=%s", xhot,yhot, serial, w,h, len(pixels), csize, (cmaxw, cmaxh))
+        cursorlog("new %s cursor at %s,%s with serial=%s, dimensions: %sx%s, len(pixels)=%s, default cursor size is %s, maximum=%s", encoding, xhot, yhot, serial, w, h, len(pixels), csize, (cmaxw, cmaxh))
         fw, fh = get_fixed_cursor_size()
         if fw>0 and fh>0 and (w!=fw or h!=fh):
             #OS wants a fixed cursor size! (win32 does, and GTK doesn't do this for us)
@@ -609,6 +644,16 @@ class GTKXpraClient(UIXpraClient, GObjectXpraClient):
             if warning:
                 warnings.append(warning)
         self.opengl_props["info"] = ""
+
+        def err(msg, e):
+            opengllog("OpenGL initialization error", exc_info=True)
+            self.GLClientWindowClass = None
+            self.client_supports_opengl = False
+            opengllog.error("%s", msg)
+            for x in str(e).split("\n"):
+                opengllog.error(" %s", x)
+            self.opengl_props["info"] = str(e)
+
         if warnings:
             if enable_opengl is True:
                 opengllog.warn("OpenGL safety warning (enabled at your own risk):")
@@ -648,8 +693,6 @@ class GTKXpraClient(UIXpraClient, GObjectXpraClient):
                 self.opengl_enabled = False
             self.GLClientWindowClass.MAX_VIEWPORT_DIMS = self.gl_max_viewport_dims
             self.GLClientWindowClass.MAX_BACKING_DIMS = self.gl_texture_size_limit, self.gl_texture_size_limit
-            self.GLClientWindowClass.MAX_VIEWPORT_DIMS = 8192, 8192
-            self.GLClientWindowClass.MAX_BACKING_DIMS = 4096, 4096
             mww, mwh = self.max_window_size
             opengllog("OpenGL: enabled=%s, texture-size-limit=%s, max-window-size=%s", self.opengl_enabled, self.gl_texture_size_limit, self.max_window_size)
             if self.opengl_enabled and self.gl_texture_size_limit<16*1024 and (mww==0 or mwh==0 or self.gl_texture_size_limit<mww or self.gl_texture_size_limit<mwh):
@@ -657,27 +700,60 @@ class GTKXpraClient(UIXpraClient, GObjectXpraClient):
                 #(if we're likely to hit it - if the screen is as big or bigger)
                 w, h = self.get_root_size()
                 l = opengllog.info
+                if w*2<=self.gl_texture_size_limit and h*2<=self.gl_texture_size_limit:
+                    l = opengllog
                 if w>=self.gl_texture_size_limit or h>=self.gl_texture_size_limit:
-                    l = log.warn
+                    l = opengllog.warn
                 l("Warning: OpenGL windows will be clamped to the maximum texture size %ix%i", self.gl_texture_size_limit, self.gl_texture_size_limit)
                 l(" for OpenGL %s renderer '%s'", pver(self.opengl_props.get("opengl", "")), self.opengl_props.get("renderer", "unknown"))
+            if self.opengl_enabled:
+                #try to render using a temporary window:
+                draw_result = {}
+                window = None
+                try:
+                    w, h = 50, 50
+                    window = self.GLClientWindowClass(self, None, 2**32-1, -100, -100, w, h, w, h, typedict({}), False, typedict({}), self.border, self.max_window_size, self.default_cursor_data, self.pixel_depth)
+                    window.realize()
+                    pixel_format = "BGRX"
+                    bpp = len(pixel_format)
+                    options = typedict({"pixel_format" : pixel_format})
+                    stride = bpp*w
+                    img_data = "\0"*stride*h
+                    coding = "rgb32"
+                    #we have to suspend idle_add to make this synchronous
+                    #we can do this because this method must be running in the UI thread already:
+                    def no_idle_add(*args, **kwargs):
+                        args[0](*args[1:], **kwargs)
+                    window._backing.idle_add = no_idle_add
+                    widget = window._backing._backing
+                    widget.realize()
+                    def paint_callback(success, message):
+                        opengllog("paint_callback(%s, %s)", success, message)
+                        draw_result.update({
+                            "success"   : success,
+                            "message"   : message,
+                            })
+                    opengllog("OpenGL: testing draw on %s widget %s with %s : %s", window, widget, coding, pixel_format)
+                    window.draw_region(0, 0, w, h, coding, img_data, stride, 1, options, [paint_callback])
+                finally:
+                    if window:
+                        window.destroy()
+                if not draw_result.get("success"):
+                    err("OpenGL test rendering failed:", draw_result.get("message", "unknown error"))
+                    return
+                log("OpenGL test rendering succeeded")
             driver_info = self.opengl_props.get("renderer") or self.opengl_props.get("vendor") or "unknown card"
             if self.opengl_enabled:
                 opengllog.info("OpenGL enabled with %s", driver_info)
             elif self.client_supports_opengl:
                 opengllog("OpenGL supported with %s, but not enabled", driver_info)
         except ImportError as e:
-            opengllog.warn("OpenGL support is missing:")
-            opengllog.warn(" %s", e)
-            self.opengl_props["info"] = str(e)
+            err("OpenGL support is missing:", e)
         except RuntimeError as e:
-            opengllog.warn("OpenGL support could not be enabled on this hardware:")
-            opengllog.warn(" %s", e)
-            self.opengl_props["info"] = str(e)
+            err("OpenGL support could not be enabled on this hardware:", e)
         except Exception as e:
-            opengllog.error("Error loading OpenGL support:")
-            opengllog.error(" %s", e, exc_info=True)
-            self.opengl_props["info"] = str(e)
+            err("Error loading OpenGL support:", e)
+            opengllog("init_opengl(%s)", enable_opengl, exc_info=True)
 
     def get_client_window_classes(self, w, h, metadata, override_redirect):
         log("get_client_window_class(%i, %i, %s, %s) GLClientWindowClass=%s, opengl_enabled=%s, mmap_enabled=%s, encoding=%s", w, h, metadata, override_redirect, self.GLClientWindowClass, self.opengl_enabled, self.mmap_enabled, self.encoding)
@@ -687,7 +763,6 @@ class GTKXpraClient(UIXpraClient, GObjectXpraClient):
         return [self.GLClientWindowClass, self.ClientWindowClass]
 
     def toggle_opengl(self, *args):
-        assert self.window_unmap, "server support for 'window_unmap' is required for toggling opengl at runtime"
         self.opengl_enabled = not self.opengl_enabled
         opengllog("opengl_toggled: %s", self.opengl_enabled)
         def fake_send(*args):
@@ -707,6 +782,7 @@ class GTKXpraClient(UIXpraClient, GObjectXpraClient):
                 bw, bh = window._backing.size
             except:
                 bw, bh = ww, wh
+            icon = window._current_icon
             client_properties = window._client_properties
             metadata = window._metadata
             override_redirect = window._override_redirect
@@ -747,10 +823,12 @@ class GTKXpraClient(UIXpraClient, GObjectXpraClient):
                     backing._video_decoder = video_decoder
                     backing._csc_decoder = csc_decoder
                     backing._decoder_lock = decoder_lock
+                window._current_icon = icon
             finally:
                 if decoder_lock:
                     decoder_lock.release()
         opengllog("replaced all the windows with opengl=%s: %s", self.opengl_enabled, self._id_to_window)
+        self.reinit_window_icons()
 
 
     def get_group_leader(self, wid, metadata, override_redirect):
@@ -817,7 +895,7 @@ class GTKXpraClient(UIXpraClient, GObjectXpraClient):
             group_leader_window = gdk.Window(None, 1, 1, gdk.WINDOW_TOPLEVEL, 0, gdk.INPUT_ONLY, title)
         self._ref_to_group_leader[refkey] = group_leader_window
         #avoid warning on win32...
-        if not sys.platform.startswith("win"):
+        if not WIN32:
             #X11 spec says window should point to itself:
             group_leader_window.set_group(group_leader_window)
         log("new hidden group leader window %s for ref=%s", group_leader_window, refkey)
