@@ -1,26 +1,29 @@
 # coding=utf8
 # This file is part of Xpra.
-# Copyright (C) 2011-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2016 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import sys
-import os
 from xpra.gtk_common.gobject_compat import import_gtk, import_glib
 gtk = import_gtk()
 glib = import_glib()
 
-from xpra.util import CLIENT_EXIT, iround
-from xpra.os_util import bytestostr
-from xpra.gtk_common.gtk_util import ensure_item_selected, menuitem, popup_menu_workaround, CheckMenuItem
+from xpra.util import CLIENT_EXIT, iround, envbool
+from xpra.os_util import bytestostr, OSX
+from xpra.gtk_common.gtk_util import ensure_item_selected, menuitem, popup_menu_workaround, CheckMenuItem, MESSAGE_QUESTION, BUTTONS_NONE
 from xpra.client.client_base import EXIT_OK
 from xpra.gtk_common.about import about, close_about
 from xpra.codecs.loader import PREFERED_ENCODING_ORDER, ENCODINGS_HELP, ENCODINGS_TO_NAME
 from xpra.platform.gui import get_icon_size
 try:
     from xpra.clipboard.translated_clipboard import TranslatedClipboardProtocolHelper
-except:
+except ImportError:
     TranslatedClipboardProtocolHelper = None
+try:
+    from xpra import clipboard
+    HAS_CLIPBOARD = bool(clipboard)
+except ImportError:
+    HAS_CLIPBOARD = False
 
 from xpra.log import Logger
 log = Logger("menu")
@@ -28,12 +31,14 @@ clipboardlog = Logger("menu", "clipboard")
 webcamlog = Logger("menu", "webcam")
 
 
-HIDE_DISABLED_MENU_ENTRIES = sys.platform.startswith("darwin")
+HIDE_DISABLED_MENU_ENTRIES = OSX
 
-#compression is fine with default value (3), no need to clutter the UI
-SHOW_COMPRESSION_MENU = False
-STARTSTOP_SOUND_MENU = os.environ.get("XPRA_SHOW_SOUND_MENU", "1")=="1"
-WEBCAM_MENU = os.environ.get("XPRA_SHOW_WEBCAM_MENU", "1")=="1"
+SHOW_UPLOAD = envbool("XPRA_SHOW_UPLOAD_MENU", True)
+STARTSTOP_SOUND_MENU = envbool("XPRA_SHOW_SOUND_MENU", True)
+WEBCAM_MENU = envbool("XPRA_SHOW_WEBCAM_MENU", True)
+RUNCOMMAND_MENU = envbool("XPRA_SHOW_RUNCOMMAND_MENU", True)
+SHOW_CLIPBOARD_MENU = envbool("XPRA_SHOW_CLIPBOARD_MENU", HAS_CLIPBOARD)
+SHOW_SHUTDOWN = envbool("XPRA_SHOW_SHUTDOWN", True)
 
 LOSSLESS = "Lossless"
 QUALITY_OPTIONS_COMMON = {
@@ -42,6 +47,7 @@ QUALITY_OPTIONS_COMMON = {
                 }
 MIN_QUALITY_OPTIONS = QUALITY_OPTIONS_COMMON.copy()
 MIN_QUALITY_OPTIONS[0] = "None"
+MIN_QUALITY_OPTIONS[75] = "High"
 QUALITY_OPTIONS = QUALITY_OPTIONS_COMMON.copy()
 QUALITY_OPTIONS[0]  = "Auto"
 QUALITY_OPTIONS[1]  = "Lowest"
@@ -60,11 +66,23 @@ SPEED_OPTIONS[0]    = "Auto"
 SPEED_OPTIONS[1]    = "Lowest Bandwidth"
 SPEED_OPTIONS[100]  = "Lowest Latency"
 
-CLIPBOARD_LABELS = ["Disabled", "Clipboard", "Primary", "Secondary"]
-CLIPBOARD_LABEL_TO_NAME = {"Disabled"   : None,
+CLIPBOARD_LABELS = ["Clipboard", "Primary", "Secondary"]
+CLIPBOARD_LABEL_TO_NAME = {
                            "Clipboard"  : "CLIPBOARD",
                            "Primary"    : "PRIMARY",
-                           "Secondary"  : "SECONDARY"}
+                           "Secondary"  : "SECONDARY"
+                           }
+CLIPBOARD_NAME_TO_LABEL  = dict((v,k) for k,v in CLIPBOARD_LABEL_TO_NAME.items())
+
+CLIPBOARD_DIRECTION_LABELS = ["Client to server only", "Server to client only", "Both directions", "Disabled"]
+CLIPBOARD_DIRECTION_LABEL_TO_NAME = {
+                                     "Client to server only"    : "to-server",
+                                     "Server to client only"    : "to-client",
+                                     "Both directions"          : "both",
+                                     "Disabled"                 : "disabled",
+                                     }
+CLIPBOARD_DIRECTION_NAME_TO_LABEL = dict((v,k) for k,v in CLIPBOARD_DIRECTION_LABEL_TO_NAME.items())
+
 
 def ll(m):
     try:
@@ -73,7 +91,7 @@ def ll(m):
         return str(m)
 
 def set_sensitive(widget, sensitive):
-    if sys.platform.startswith("darwin"):
+    if OSX:
         if sensitive:
             widget.show()
         else:
@@ -216,10 +234,11 @@ class GTKTrayMenuBase(object):
         self.session_info = None
         self.menu = None
         self.menu_shown = False
+        self.menu_icon_size = 0
 
     def build(self):
         if self.menu is None:
-            show_close = True #or sys.platform.startswith("win")
+            show_close = True #or WIN32
             self.menu = self.setup_menu(show_close)
         return self.menu
 
@@ -235,28 +254,32 @@ class GTKTrayMenuBase(object):
 
     def setup_menu(self, show_close=True):
         self.menu_shown = False
+        self.menu_icon_size = get_icon_size()
         menu = gtk.Menu()
         menu.set_title(self.client.session_name or "Xpra")
         def set_menu_title(*args):
             #set the real name when available:
-            self.menu.set_title(self.client.session_name)
+            m = self.menu
+            if m:
+                m.set_title(self.client.session_name or "Xpra")
         self.client.after_handshake(set_menu_title)
 
         menu.append(self.make_aboutmenuitem())
         menu.append(self.make_sessioninfomenuitem())
         menu.append(self.make_bugreportmenuitem())
         menu.append(gtk.SeparatorMenuItem())
+        menu.append(self.make_readonlymenuitem())
         menu.append(self.make_bellmenuitem())
         menu.append(self.make_notificationsmenuitem())
         if self.client.windows_enabled:
             menu.append(self.make_cursorsmenuitem())
         if self.client.client_supports_opengl:
             menu.append(self.make_openglmenuitem())
-        if self.client.windows_enabled and not self.client.readonly:
+        if self.client.windows_enabled:
             menu.append(self.make_keyboardsyncmenuitem())
-        if not self.client.readonly and self.client.keyboard_helper:
+        if self.client.keyboard_helper:
             menu.append(self.make_layoutsmenuitem())
-        if not self.client.readonly:
+        if SHOW_CLIPBOARD_MENU:
             menu.append(self.make_clipboardmenuitem())
         if self.client.windows_enabled and len(self.client.get_encodings())>1:
             menu.append(self.make_encodingsmenuitem())
@@ -266,25 +289,27 @@ class GTKTrayMenuBase(object):
         menu.append(self.make_speedmenuitem())
         if STARTSTOP_SOUND_MENU:
             menu.append(self.make_speakermenuitem())
-        if STARTSTOP_SOUND_MENU:
             menu.append(self.make_microphonemenuitem())
         if WEBCAM_MENU:
             menu.append(self.make_webcammenuitem())
-        if SHOW_COMPRESSION_MENU:
-            menu.append(self.make_compressionmenu())
         if self.client.windows_enabled:
             menu.append(self.make_refreshmenuitem())
             menu.append(self.make_raisewindowsmenuitem())
         #menu.append(item("Options", "configure", None, self.options))
         menu.append(gtk.SeparatorMenuItem())
-        menu.append(self.make_startnewcommandmenuitem())
-        menu.append(self.make_uploadmenuitem())
+        if RUNCOMMAND_MENU:
+            menu.append(self.make_runcommandmenu())
+        if SHOW_UPLOAD:
+            menu.append(self.make_uploadmenuitem())
+        if SHOW_SHUTDOWN:
+            menu.append(self.make_shutdownmenuitem())
         menu.append(self.make_disconnectmenuitem())
         if show_close:
             menu.append(self.make_closemenuitem())
         self.popup_menu_workaround(menu)
         menu.connect("deactivate", self.menu_deactivated)
         menu.show_all()
+        self.menu_icon_size = 0
         return menu
 
     def cleanup(self):
@@ -303,9 +328,9 @@ class GTKTrayMenuBase(object):
     def menu_deactivated(self, *args):
         self.menu_shown = False
 
-    def activate(self):
-        log("activate()")
-        self.show_menu(1, 0)
+    def activate(self, button=1, time=0):
+        log("activate(%s, %s)", button, time)
+        self.show_menu(button, time)
 
     def popup(self, button, time):
         log("popup(%s, %s)", button, time)
@@ -332,7 +357,7 @@ class GTKTrayMenuBase(object):
         """ Utility method for easily creating an ImageMenuItem """
         image = None
         if icon_name:
-            icon_size = get_icon_size()
+            icon_size = self.menu_icon_size or get_icon_size()
             image = self.get_image(icon_name, icon_size)
         return menuitem(title, image, tooltip, cb)
 
@@ -364,9 +389,30 @@ class GTKTrayMenuBase(object):
             self.show_bug_report()
         return  self.handshake_menuitem("Bug Report", "bugs.png", None, show_bug_report_cb)
 
+    def make_readonlymenuitem(self):
+        def readonly_toggled(*args):
+            v = self.readonly_menuitem.get_active()
+            self.client.readonly = v
+            log("readonly_toggled(%s) readonly=%s", args, self.client.readonly)
+        self.readonly_menuitem = self.checkitem("Read-only", readonly_toggled)
+        set_sensitive(self.readonly_menuitem, False)
+        def set_readonly_menuitem(*args):
+            log("set_readonly_menuitem%s enabled=%s", args, self.client.readonly)
+            self.bell_menuitem.set_active(self.client.readonly)
+            set_sensitive(self.readonly_menuitem, not self.client.server_readonly)
+            if not self.client.server_readonly:
+                self.bell_menuitem.set_tooltip_text("Disable all mouse and keyboard input")
+            else:
+                self.bell_menuitem.set_tooltip_text("Cannot disable readonly mode: the server has locked the session to read only")
+        self.client.after_handshake(set_readonly_menuitem)
+        return self.readonly_menuitem
 
     def make_bellmenuitem(self):
+        c = self.client
         def bell_toggled(*args):
+            can_toggle_bell = c.server_supports_bell and c.client_supports_bell
+            if not can_toggle_bell:
+                return
             v = self.bell_menuitem.get_active()
             changed = self.client.bell_enabled != v
             self.client.bell_enabled = v
@@ -378,7 +424,6 @@ class GTKTrayMenuBase(object):
         def set_bell_menuitem(*args):
             log("set_bell_menuitem%s enabled=%s", args, self.client.bell_enabled)
             self.bell_menuitem.set_active(self.client.bell_enabled)
-            c = self.client
             can_toggle_bell = c.server_supports_bell and c.client_supports_bell
             set_sensitive(self.bell_menuitem, can_toggle_bell)
             if can_toggle_bell:
@@ -426,8 +471,7 @@ class GTKTrayMenuBase(object):
         def set_notifications_menuitem(*args):
             log("set_notifications_menuitem%s enabled=%s", args, self.client.notifications_enabled)
             self.notifications_menuitem.set_active(self.client.notifications_enabled)
-            c = self.client
-            can_notify = c.server_supports_notifications and c.client_supports_notifications
+            can_notify = self.client.client_supports_notifications
             set_sensitive(self.notifications_menuitem, can_notify)
             if can_notify:
                 self.notifications_menuitem.set_tooltip_text("Forward system notifications")
@@ -436,132 +480,117 @@ class GTKTrayMenuBase(object):
         self.client.after_handshake(set_notifications_menuitem)
         return self.notifications_menuitem
 
-    def make_clipboard_togglemenuitem(self):
-        clipboardlog("make_clipboard_togglemenuitem()")
-        def menu_clipboard_toggled(*args):
-            new_state = self.clipboard_menuitem.get_active()
-            clipboardlog("clipboard_toggled(%s) clipboard_enabled=%s, new_state=%s", args, self.client.clipboard_enabled, new_state)
-            if self.client.clipboard_enabled!=new_state:
-                self.client.clipboard_enabled = new_state
-                self.client.emit("clipboard-toggled")
-        self.clipboard_menuitem = self.checkitem("Clipboard", menu_clipboard_toggled)
-        set_sensitive(self.clipboard_menuitem, False)
-        def set_clipboard_menuitem(*args):
-            clipboardlog("set_clipboard_menuitem%s enabled=%s", args, self.client.clipboard_enabled)
-            self.clipboard_menuitem.set_active(self.client.clipboard_enabled)
-            c = self.client
-            can_clipboard = c.server_supports_clipboard and c.client_supports_clipboard
-            set_sensitive(self.clipboard_menuitem, can_clipboard)
-            if can_clipboard:
-                self.clipboard_menuitem.set_tooltip_text("Enable clipboard synchronization")
-            else:
-                self.clipboard_menuitem.set_tooltip_text("Clipboard synchronization cannot be enabled: disabled by server")
-        self.client.after_handshake(set_clipboard_menuitem)
-        def clipboard_toggled(*args):
-            #keep menu in sync with actual "clipboard_enabled" flag:
-            if self.client.clipboard_enabled != self.clipboard_menuitem.get_active():
-                self.clipboard_menuitem.set_active(self.client.clipboard_enabled)
-        self.client.connect("clipboard-toggled", clipboard_toggled)
-        return self.clipboard_menuitem
-
-    def _can_handle_clipboard(self):
-        c = self.client
-        return c.server_supports_clipboard and c.client_supports_clipboard
-
-    def get_clipboard_helper_class(self):
-        return TranslatedClipboardProtocolHelper
 
     def remote_clipboard_changed(self, item, clipboard_submenu):
-        if not self._can_handle_clipboard():
-            item = None
-        if item is None:
-            item = ([x for x in clipboard_submenu.get_children() if x.get_label()=="Disabled"]+[None])[0]
-            if item is None:
-                return
+        c = self.client
+        if not c or not c.server_supports_clipboard or not c.client_supports_clipboard:
+            return
         #prevent infinite recursion where ensure_item_selected
         #ends up calling here again
-        ich = getattr(clipboard_submenu, "_in_change_handler_", False)
+        key = "_in_remote_clipboard_changed"
+        ich = getattr(clipboard_submenu, key, False)
         clipboardlog("remote_clipboard_changed%s already in change handler: %s, visible=%s", (ll(item), clipboard_submenu), ich, clipboard_submenu.get_visible())
         if ich: # or not clipboard_submenu.get_visible():
             return
         try:
-            setattr(clipboard_submenu, "_in_change_handler_", True)
+            setattr(clipboard_submenu, key, True)
             selected_item = ensure_item_selected(clipboard_submenu, item)
             selected = selected_item.get_label()
             remote_clipboard = CLIPBOARD_LABEL_TO_NAME.get(selected)
             self.set_new_remote_clipboard(remote_clipboard)
         finally:
-            setattr(clipboard_submenu, "_in_change_handler_", False)
+            setattr(clipboard_submenu, key, False)
 
     def set_new_remote_clipboard(self, remote_clipboard):
         clipboardlog("set_new_remote_clipboard(%s)", remote_clipboard)
-        old_state = self.client.clipboard_enabled
-        send_tokens = False
-        if remote_clipboard is not None:
-            #clipboard is not disabled
-            if self.client.clipboard_helper is None:
-                self.client.clipboard_helper = self.client.setup_clipboard_helper(self.get_clipboard_helper_class())
-            self.client.clipboard_helper.remote_clipboard = remote_clipboard
-            self.client.clipboard_helper.remote_clipboards = [remote_clipboard]
-            send_tokens = True
-            new_state = True
-            selections = [remote_clipboard]
-        else:
-            self.client.clipboard_helper = None
-            send_tokens = False
-            new_state = False
-            selections = []
-        clipboardlog("set_new_remote_clipboard(%s) old_state=%s, new_state=%s", remote_clipboard, old_state, new_state)
-        if new_state!=old_state:
-            self.client.clipboard_enabled = new_state
-            self.client.emit("clipboard-toggled")
-            send_tokens = True
+        ch = self.client.clipboard_helper
+        ch.remote_clipboard = remote_clipboard
+        ch.remote_clipboards = [remote_clipboard]
+        selections = [remote_clipboard]
+        clipboardlog.info("server clipboard synchronization changed to %s selection", remote_clipboard)
         #tell the server what to look for:
         #(now that "clipboard-toggled" has re-enabled clipboard if necessary)
         self.client.send_clipboard_selections(selections)
-        if send_tokens and self.client.clipboard_helper:
-            self.client.clipboard_helper.send_all_tokens()
+        ch.send_all_tokens()
 
     def make_translatedclipboard_optionsmenuitem(self):
         clipboardlog("make_translatedclipboard_optionsmenuitem()")
-        clipboard_menu = self.menuitem("Clipboard", "clipboard.png", "Choose which remote clipboard to connect to", None)
-        set_sensitive(clipboard_menu, False)
-        def set_clipboard_menu(*args):
-            clipboard_submenu = gtk.Menu()
-            clipboard_menu.set_submenu(clipboard_submenu)
-            self.popup_menu_workaround(clipboard_submenu)
-            c = self.client
-            can_clipboard = self._can_handle_clipboard()
-            clipboardlog("set_clipboard_menu(%s) can_clipboard=%s, server=%s, client=%s", args, can_clipboard, c.server_supports_clipboard, c.client_supports_clipboard)
-            set_sensitive(clipboard_menu, can_clipboard)
-            for label in CLIPBOARD_LABELS:
-                remote_clipboard = CLIPBOARD_LABEL_TO_NAME[label]
-                clipboard_item = CheckMenuItem(label)
-                ch = self.client.clipboard_helper
-                active = bool(ch) and isinstance(ch, TranslatedClipboardProtocolHelper) \
-                            and ch.remote_clipboard==remote_clipboard
-                clipboard_item.set_active(active)
-                set_sensitive(clipboard_item, can_clipboard)
-                clipboard_item.set_draw_as_radio(True)
-                def remote_clipboard_changed(item):
-                    self.remote_clipboard_changed(item, clipboard_submenu)
-                clipboard_item.connect("toggled", remote_clipboard_changed)
-                clipboard_submenu.append(clipboard_item)
-            clipboard_submenu.show_all()
-        self.client.after_handshake(set_clipboard_menu)
-        return clipboard_menu
+        ch = self.client.clipboard_helper
+        selection_menu = self.menuitem("Selection", None, "Choose which remote clipboard to connect to")
+        selection_submenu = gtk.Menu()
+        selection_menu.set_submenu(selection_submenu)
+        self.popup_menu_workaround(selection_submenu)
+        for label in CLIPBOARD_LABELS:
+            remote_clipboard = CLIPBOARD_LABEL_TO_NAME[label]
+            selection_item = CheckMenuItem(label)
+            active = getattr(ch, "remote_clipboard", "CLIPBOARD")==remote_clipboard
+            selection_item.set_active(active)
+            selection_item.set_draw_as_radio(True)
+            def remote_clipboard_changed(item):
+                self.remote_clipboard_changed(item, selection_submenu)
+            selection_item.connect("toggled", remote_clipboard_changed)
+            selection_submenu.append(selection_item)
+        selection_submenu.show_all()
+        return selection_menu
+
+    def clipboard_direction_changed(self, item, submenu):
+        log("clipboard_direction_changed(%s, %s)", item, submenu)
+        sel = ensure_item_selected(submenu, item, recurse=False)
+        if not sel:
+            return
+        self.do_clipboard_direction_changed(sel.get_label() or "")
+
+    def do_clipboard_direction_changed(self, label):
+        #find the value matching this item label:
+        d = CLIPBOARD_DIRECTION_LABEL_TO_NAME.get(label)
+        if d and d!=self.client.client_clipboard_direction:
+            log.info("clipboard synchronization direction changed to: %s", label.lower())
+            self.client.client_clipboard_direction = d
+            can_send = d in ("to-server", "both")
+            can_receive = d in ("to-client", "both")
+            self.client.clipboard_helper.set_direction(can_send, can_receive)
+            #will send new tokens and may help reset things:
+            self.client.emit("clipboard-toggled")
 
     def make_clipboardmenuitem(self):
-        try:
-            copts = self.client.get_clipboard_helper_classes()
-            #ugly alert: the helper does not exist yet.. we just check the helper classnames:
-            if len(copts)>0:
-                chname = copts[0].lower()
-                if chname.find("translated_clipboard")>=0 or chname.find("osxclipboard")>=0:
-                    return self.make_translatedclipboard_optionsmenuitem()
-        except:
-            clipboardlog.error("make_clipboardmenuitem()", exc_info=True)
-        return self.make_clipboard_togglemenuitem()
+        clipboardlog("make_clipboardmenuitem()")
+        self.clipboard_menuitem = self.menuitem("Clipboard", "clipboard.png")
+        set_sensitive(self.clipboard_menuitem, False)
+        def set_clipboard_menu(*args):
+            c = self.client
+            if not c.server_supports_clipboard:
+                self.clipboard_menuitem.set_tooltip_text("Server does not support clipboard synchronization")
+                return
+            if not c.client_supports_clipboard:
+                self.client_menuitem.set_tooltip_text("Client does not support clipboard synchronization")
+                return
+            #add a submenu:
+            set_sensitive(self.clipboard_menuitem, True)
+            c = self.client
+            ch = self.client.clipboard_helper
+            clipboard_submenu = gtk.Menu()
+            self.clipboard_menuitem.set_submenu(clipboard_submenu)
+            self.popup_menu_workaround(clipboard_submenu)
+            #figure out if this is a translated clipboard (win32 or osx)
+            #and if so, add a submenu to change the selection we synchronize with:
+            try:
+                from xpra.clipboard.translated_clipboard import TranslatedClipboardProtocolHelper
+                assert TranslatedClipboardProtocolHelper
+                clipboardlog("set_clipboard_menu(%s) helper=%s, server=%s, client=%s", args, ch, c.server_supports_clipboard, c.client_supports_clipboard)
+                if issubclass(type(ch), TranslatedClipboardProtocolHelper):
+                    clipboard_submenu.append(self.make_translatedclipboard_optionsmenuitem())
+                    clipboard_submenu.append(gtk.SeparatorMenuItem())
+            except:
+                clipboardlog.error("make_clipboardmenuitem()", exc_info=True)
+            for label in CLIPBOARD_DIRECTION_LABELS:
+                direction_item = CheckMenuItem(label)
+                d = CLIPBOARD_DIRECTION_LABEL_TO_NAME.get(label)
+                direction_item.set_active(d==self.client.client_clipboard_direction)
+                direction_item.connect("toggled", self.clipboard_direction_changed, clipboard_submenu)
+                clipboard_submenu.append(direction_item)
+            clipboard_submenu.show_all()
+        self.client.after_handshake(set_clipboard_menu)
+        return self.clipboard_menuitem
 
 
     def make_keyboardsyncmenuitem(self):
@@ -619,12 +648,16 @@ class GTKTrayMenuBase(object):
         return encodings
 
     def make_encodingssubmenu(self, handshake_complete=True):
+        server_encodings = list(self.client.server_encodings)
         all_encodings = [x for x in PREFERED_ENCODING_ORDER if x in self.client.get_encodings()]
         encodings = [x for x in all_encodings if x not in self.client.server_encodings_problematic]
         if not encodings:
             #all we have, show the "bad" hidden ones then!
             encodings = all_encodings
-        encodings_submenu = make_encodingsmenu(self.get_current_encoding, self.set_current_encoding, encodings, self.client.server_encodings)
+        if self.client.server_auto_video_encoding:
+            encodings.insert(0, "auto")
+            server_encodings.insert(0, "auto")
+        encodings_submenu = make_encodingsmenu(self.get_current_encoding, self.set_current_encoding, encodings, server_encodings)
         self.popup_menu_workaround(encodings_submenu)
         return encodings_submenu
 
@@ -727,9 +760,12 @@ class GTKTrayMenuBase(object):
 
     def set_qualitymenu(self, *args):
         if self.quality:
-            can_use = not self.client.mmap_enabled and self.client.encoding in self.client.server_encodings_with_quality
+            can_use = not self.client.mmap_enabled and (self.client.encoding in self.client.server_encodings_with_quality or self.client.encoding=="auto")
             set_sensitive(self.quality, can_use)
-            if not can_use:
+            if self.client.mmap_enabled:
+                self.quality.set_tooltip_text("Speed is always 100% with mmap")
+                return
+            elif not can_use:
                 self.quality.set_tooltip_text("Not supported with %s encoding" % self.client.encoding)
                 return
             self.quality.set_tooltip_text("Minimum picture quality")
@@ -773,7 +809,7 @@ class GTKTrayMenuBase(object):
 
     def set_speedmenu(self, *args):
         if self.speed:
-            can_use = not self.client.mmap_enabled and self.client.encoding in self.client.server_encodings_with_speed
+            can_use = not self.client.mmap_enabled and (self.client.encoding in self.client.server_encodings_with_speed or self.client.encoding=="auto")
             set_sensitive(self.speed, can_use)
             if self.client.mmap_enabled:
                 self.speed.set_tooltip_text("Quality is always 100% with mmap")
@@ -876,60 +912,101 @@ class GTKTrayMenuBase(object):
         return menu
 
     def make_webcammenuitem(self):
+        webcam = self.menuitem("Webcam", "webcam.png", "Forward webcam pictures to the server", None)
+        if not self.client.webcam_forwarding:
+            webcam.set_tooltip_text("Webcam forwarding is disabled")
+            set_sensitive(webcam, False)
+            return webcam
+        from xpra.platform.webcam import get_all_video_devices, get_virtual_video_devices, add_video_device_change_callback
+        #TODO: register remove_video_device_change_callback for cleanup
         menu = gtk.Menu()
+        self.popup_menu_workaround(menu)
         #so we can toggle the menu items without causing yet more events and infinite loops:
         menu.ignore_events = False
-        def onoffitem(label, active, cb):
+        def deviceitem(label, cb, device_no=0):
             c = CheckMenuItem(label)
             c.set_draw_as_radio(True)
-            c.set_active(active)
+            c.set_active(get_active_device_no()==device_no)
+            c.device_no = device_no
             def activate_cb(item, *args):
                 webcamlog("activate_cb(%s, %s) ignore_events=%s", item, menu, menu.ignore_events)
                 if not menu.ignore_events:
                     try:
                         menu.ignore_events = True
                         ensure_item_selected(menu, item)
-                        cb()
+                        cb(device_no)
                     finally:
                         menu.ignore_events = False
             c.connect("toggled", activate_cb, menu)
             return c
-        webcam = self.menuitem("Webcam", "webcam.png", "Forward webcam pictures to the server", None)
-        if not self.client.webcam_forwarding:
-            webcam.set_tooltip_text("Webcam forwarding is disabled")
-            set_sensitive(webcam, False)
-            return webcam
-            return c
-        def start_webcam(*args):
-            webcamlog("start_webcam%s", args)
-            self.client.start_sending_webcam()
-        def stop_webcam(*args):
-            webcamlog("stop_webcam%s", args)
+        def start_webcam(device_no=0):
+            webcamlog("start_webcam(%s)", device_no)
+            self.client.do_start_sending_webcam(device_no)
+        def stop_webcam(device_no=0):
+            webcamlog("stop_webcam(%s)", device_no)
             self.client.stop_sending_webcam()
-        on = onoffitem("On", False, start_webcam)
-        off = onoffitem("Off", True, stop_webcam)
-        menu.append(on)
-        menu.append(off)
+
+        def get_active_device_no():
+            if self.client.webcam_device is None:
+                return -1
+            return self.client.webcam_device_no
+
+        def populate_webcam_menu():
+            menu.ignore_events = True
+            webcamlog("populate_webcam_menu()")
+            for x in menu.get_children():
+                menu.remove(x)
+            all_video_devices = get_all_video_devices()
+            off_label = "Off"
+            if all_video_devices is None:
+                #None means that this platform cannot give us the device names,
+                #so we just use a single "On" menu item and hope for the best
+                on = deviceitem("On", start_webcam)
+                menu.append(on)
+            else:
+                on = None
+                virt_devices = get_virtual_video_devices()
+                non_virtual = dict([(k,v) for k,v in all_video_devices.items() if k not in virt_devices])
+                for device_no,info in non_virtual.items():
+                    label = info.get("card", info.get("device", str(device_no)))
+                    item = deviceitem(label, start_webcam, device_no)
+                    menu.append(item)
+                if len(non_virtual)==0:
+                    off_label = "No devices found"
+            off = deviceitem(off_label, stop_webcam, -1)
+            set_sensitive(off, off_label=="Off")
+            menu.append(off)
+            menu.show_all()
+            menu.ignore_events = False
+        populate_webcam_menu()
+
+        def video_devices_changed(added=None, device=None):
+            if added is not None and device:
+                log.info("video device %s: %s", ["removed", "added"][added], device)
+            else:
+                log("video_devices_changed")
+            #this callback runs in another thread,
+            #and we want to wait for the devices to settle
+            #so that the file permissions are correct when we try to access it:
+            glib.timeout_add(1000, populate_webcam_menu)
+        add_video_device_change_callback(video_devices_changed)
 
         webcam.set_submenu(menu)
         def webcam_changed(*args):
             webcamlog("webcam_changed%s webcam_device=%s", args, self.client.webcam_device)
             if not self.client.webcam_forwarding:
-                for x in (webcam, on, off):
-                    set_sensitive(x, False)
+                set_sensitive(webcam, False)
                 webcam.set_tooltip_text("Webcam forwarding is disabled")
                 return
             if self.client.server_virtual_video_devices<=0:
-                for x in (webcam, on, off):
-                    set_sensitive(x, False)
+                set_sensitive(webcam, False)
                 webcam.set_tooltip_text("Server does not support webcam forwarding")
                 return
             set_sensitive(webcam, True)
-            active = self.client.webcam_device is not None
-            webcamlog("webcam_changed%s active=%s", args, active)
+            webcamlog("webcam_changed%s active device no=%s", args, get_active_device_no())
             menu.ignore_events = True
-            on.set_active(active)
-            off.set_active(not active)
+            for x in menu.get_children():
+                x.set_active(x.device_no==get_active_device_no())
             menu.ignore_events = False
         self.client.connect("webcam-changed", webcam_changed)
         set_sensitive(webcam, False)
@@ -950,19 +1027,19 @@ class GTKTrayMenuBase(object):
                 variant = item.keyboard_variant
                 kh = self.client.keyboard_helper
                 kh.locked = layout!="Auto"
-                if layout!=kh.xkbmap_layout or variant!=kh.xkbmap_variant:
+                if layout!=kh.layout_option or variant!=kh.variant_option:
                     if layout=="Auto":
                         #re-detect everything:
-                        kh.update()
-                        log.info("keyboard automatic mode: %s", kh.layout_str())
-                        kh.send_layout()
-                        kh.send_keymap()
+                        msg = "keyboard automatic mode"
                     else:
                         #use layout specified and send it:
-                        kh.xkbmap_layout = layout
-                        kh.xkbmap_variant = variant
-                        log.info("new keyboard layout selected: %s", kh.layout_str())
-                        kh.send_layout()
+                        kh.layout_option = layout
+                        kh.variant_option = variant
+                        msg = "new keyboard layout selected"
+                    kh.update()
+                    kh.send_layout()
+                    kh.send_keymap()
+                    log.info("%s: %s", msg, kh.layout_str())
             l = self.checkitem(title, set_layout, active)
             l.set_draw_as_radio(True)
             l.keyboard_layout = layout
@@ -971,7 +1048,7 @@ class GTKTrayMenuBase(object):
         def keysort(key):
             c,l = key
             return c.lower()+l.lower()
-        layout,layouts,variant,variants = self.client.keyboard_helper.keyboard.get_layout_spec()
+        layout,layouts,variant,variants = self.client.keyboard_helper.get_layout_spec()
         full_layout_list = False
         if len(layouts)>1:
             log("keyboard layouts: %s", u",".join(bytestostr(x) for x in layouts))
@@ -1030,31 +1107,6 @@ class GTKTrayMenuBase(object):
         self.client.after_handshake(set_layout_enabled)
         return keyboard
 
-    def make_compressionmenu(self):
-        self.compression = self.menuitem("Compression", "compressed.png", "Network packet compression", None)
-        set_sensitive(self.compression, False)
-        self.compression_submenu = gtk.Menu()
-        self.compression.set_submenu(self.compression_submenu)
-        self.popup_menu_workaround(self.compression_submenu)
-        compression_options = {0 : "None"}
-        def set_compression(item):
-            ensure_item_selected(self.compression_submenu, item)
-            c = int(item.get_label().replace("None", "0"))
-            if c!=self.client.compression_level:
-                log("setting compression level to %s", c)
-                self.client.set_deflate_level(c)
-        for i in range(0, 10):
-            c = CheckMenuItem(str(compression_options.get(i, i)))
-            c.set_draw_as_radio(True)
-            c.set_active(i==self.client.compression_level)
-            c.connect('activate', set_compression)
-            self.compression_submenu.append(c)
-        def enable_compressionmenu(self):
-            set_sensitive(self.compression, True)
-            self.compression_submenu.show_all()
-        self.client.after_handshake(enable_compressionmenu)
-        return self.compression
-
 
     def make_refreshmenuitem(self):
         def force_refresh(*args):
@@ -1069,7 +1121,7 @@ class GTKTrayMenuBase(object):
                     win.present()
         return self.handshake_menuitem("Raise Windows", "raise.png", None, raise_windows)
 
-    def make_startnewcommandmenuitem(self):
+    def make_runcommandmenu(self):
         self.startnewcommand = self.menuitem("Run Command", "forward.png", "Run a new command on the server", self.client.show_start_new_command)
         def enable_start_new_command(*args):
             log("enable_start_new_command%s start_new_command=%s", args, self.client.start_new_commands)
@@ -1082,14 +1134,35 @@ class GTKTrayMenuBase(object):
     def make_uploadmenuitem(self):
         self.upload = self.menuitem("Upload File", "upload.png", "Send a file to the server", self.client.show_file_upload)
         def enable_upload(*args):
-            log("enable_upload%s server_file_transfer=%s", args, self.client.server_file_transfer)
-            set_sensitive(self.upload, self.client.server_file_transfer)
-            if not self.client.server_file_transfer:
+            log("enable_upload%s server_file_transfer=%s", args, self.client.remote_file_transfer)
+            set_sensitive(self.upload, self.client.remote_file_transfer)
+            if not self.client.remote_file_transfer:
                 self.upload.set_tooltip_text("Not supported by the server")
         self.client.after_handshake(enable_upload)
         return self.upload
 
 
+    def make_shutdownmenuitem(self):
+        def ask_shutdown_confirm(*args):
+            dialog = gtk.MessageDialog (None, 0, MESSAGE_QUESTION,
+                                    BUTTONS_NONE,
+                                    "Shutting down this session may cause data loss,\nare you sure you want to proceed?")
+            dialog.add_button(gtk.STOCK_CANCEL, 0)
+            SHUTDOWN = 1
+            dialog.add_button("Shutdown", SHUTDOWN)
+            response = dialog.run()
+            dialog.destroy()
+            if response == SHUTDOWN:
+                self.client.send_shutdown_server()
+        self.shutdown = self.menuitem("Shutdown Server", "shutdown.png", "Shutdown this server session", ask_shutdown_confirm)
+        def enable_shutdown(*args):
+            log("enable_shutdown%s can_shutdown_server=%s", args, self.client.can_shutdown_server)
+            set_sensitive(self.shutdown, self.client.can_shutdown_server)
+            if not self.client.can_shutdown_server:
+                self.shutdown.set_tooltip_text("Disabled by the server")
+        self.client.after_handshake(enable_shutdown)
+        return self.shutdown
+
     def make_disconnectmenuitem(self):
         def menu_quit(*args):
             self.client.disconnect_and_quit(EXIT_OK, CLIENT_EXIT)
