@@ -1,13 +1,14 @@
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 # A tray implemented using gtk.StatusIcon
 
 import os
-import sys
+from xpra.os_util import WIN32, OSX, POSIX
+from xpra.util import envbool
 from xpra.gtk_common.gobject_compat import import_gtk, import_gdk, is_gtk3
 gtk = import_gtk()
 gdk = import_gdk()
@@ -21,8 +22,8 @@ if not is_gtk3():
     ORIENTATION[gtk.ORIENTATION_HORIZONTAL] = "HORIZONTAL"
     ORIENTATION[gtk.ORIENTATION_VERTICAL]   = "VERTICAL"
 
-GUESS_GEOMETRY = str(int(sys.platform.startswith("win") or sys.platform.startswith("darwin")))
-GUESS_GEOMETRY = os.environ.get("XPRA_GUESS_ICON_GEOMETRY", GUESS_GEOMETRY)=="1"
+GUESS_GEOMETRY = WIN32 or OSX
+GUESS_GEOMETRY = envbool("XPRA_GUESS_ICON_GEOMETRY", GUESS_GEOMETRY)
 log("tray GUESS_GEOMETRY=%s", GUESS_GEOMETRY)
 
 
@@ -36,9 +37,8 @@ class GTKStatusIconTray(TrayBase):
         self.tray_widget.connect('popup-menu', self.popup_menu)
         if self.size_changed_cb:
             self.tray_widget.connect('size-changed', self.size_changed_cb)
-        filename = self.get_tray_icon_filename(self.default_icon_filename)
-        if filename:
-            self.set_icon_from_file(filename)
+        if self.default_icon_filename:
+            self.set_icon()
         self.tray_widget.set_visible(True)
 
     def may_guess(self):
@@ -51,14 +51,16 @@ class GTKStatusIconTray(TrayBase):
     def activate_menu(self, widget, *args):
         log("activate_menu(%s, %s)", widget, args)
         self.may_guess()
-        self.click_cb(1, 1)
-        self.click_cb(1, 0)
+        if self.click_cb:
+            self.click_cb(1, 1)
+            self.click_cb(1, 0)
 
     def popup_menu(self, widget, button, time, *args):
         log("popup_menu(%s, %s, %s, %s)", widget, button, time, args)
         self.may_guess()
-        self.click_cb(button, 1, 0)
-        self.click_cb(button, 0, 0)
+        if self.click_cb:
+            self.click_cb(button, 1, time)
+            self.click_cb(button, 0, time)
 
 
     def hide(self, *args):
@@ -96,7 +98,7 @@ class GTKStatusIconTray(TrayBase):
         assert self.tray_widget
         #on X11, if we don't have an xid, don't bother querying its geometry,
         #as this would trigger some ugly GTK warnings we can do nothing about
-        if os.name=="posix" and os.environ.get("DISPLAY") and self.tray_widget.get_x11_window_id()==0:
+        if POSIX and os.environ.get("DISPLAY") and self.tray_widget.get_x11_window_id()==0:
             ag = None
         else:
             ag = self.tray_widget.get_geometry()
@@ -129,7 +131,7 @@ class GTKStatusIconTray(TrayBase):
             self.tray_widget.set_blinking(on)
 
 
-    def set_icon_from_data(self, pixels, has_alpha, w, h, rowstride):
+    def set_icon_from_data(self, pixels, has_alpha, w, h, rowstride, options={}):
         tray_icon = get_pixbuf_from_data(pixels, has_alpha, w, h, rowstride)
         self.set_icon_from_pixbuf(tray_icon)
 
@@ -166,10 +168,11 @@ def main():
     log.enable_debug()
     from xpra.gtk_common.gobject_compat import import_glib
     glib = import_glib()
-    s = GTKStatusIconTray(None, "test", "xpra.png", None, None, None, gtk.main_quit)
+    log.enable_debug()
+    s = GTKStatusIconTray(None, None, "test", "xpra.png", None, None, None, gtk.main_quit)
     glib.timeout_add(1000*2, s.set_blinking, True)
     glib.timeout_add(1000*5, s.set_blinking, False)
-    glib.timeout_add(1000*10, gtk.main_quit)
+    glib.timeout_add(1000*30, gtk.main_quit)
     gtk.main()
 
 
