@@ -8,21 +8,30 @@
 from xpra.log import Logger
 log = Logger("keyboard")
 
+from xpra.keyboard.layouts import xkbmap_query_tostring
 from xpra.keyboard.mask import DEFAULT_MODIFIER_MEANINGS, DEFAULT_MODIFIER_NUISANCE
-from xpra.util import nonl
+from xpra.util import nonl, csv, std, print_nested_dict
 
 
 class KeyboardHelper(object):
 
-    def __init__(self, net_send, keyboard_sync, key_shortcuts):
+    def __init__(self, net_send, keyboard_sync, key_shortcuts, raw, layout, layouts, variant, variants, options):
         self.reset_state()
         self.send = net_send
         self.locked = False
         self.keyboard_sync = keyboard_sync
         self.key_shortcuts = self.parse_shortcuts(key_shortcuts)
+        #command line overrides:
+        self.xkbmap_raw = raw
+        self.layout_option = layout
+        self.variant_option = variant
+        self.layouts_option = layouts
+        self.variants_option = variants
+        self.options = options
+        #the platform class which allows us to map the keys:
         from xpra.platform.keyboard import Keyboard
         self.keyboard = Keyboard()
-        log("KeyboardHelper.__init__(%s, %s, %s) keyboard=%s", net_send, keyboard_sync, key_shortcuts, self.keyboard)
+        log("KeyboardHelper(%s) keyboard=%s", (net_send, keyboard_sync, key_shortcuts, raw, layout, layouts, variant, variants, options), self.keyboard)
 
     def mask_to_names(self, mask):
         return self.keyboard.mask_to_names(mask)
@@ -37,6 +46,7 @@ class KeyboardHelper(object):
         self.xkbmap_mod_managed = []
         self.xkbmap_mod_pointermissing = []
         self.xkbmap_layout = ""
+        self.xkbmap_layouts = []
         self.xkbmap_variant = ""
         self.xkbmap_variants = []
         self.xkbmap_print = ""
@@ -58,6 +68,9 @@ class KeyboardHelper(object):
             pass
         self.send = nosend
 
+    def keymap_changed(self, *args):
+        pass
+
 
     def parse_shortcuts(self, strs):
         #TODO: maybe parse with re instead?
@@ -95,10 +108,10 @@ class KeyboardHelper(object):
             #example for s: Control+F8:some_action()
             parts = s.split(":", 1)
             if len(parts)!=2:
-                log.error("invalid shortcut: %s" % s)
+                log.error("Error: invalid key shortcut '%s'", s)
                 continue
             #example for action: "quit"
-            action = parts[1]
+            action = parts[1].strip()
             args = ()
             if action.find("(")>0 and action.endswith(")"):
                 try:
@@ -120,6 +133,8 @@ class KeyboardHelper(object):
                 except Exception as e:
                     log.warn("failed to parse arguments of shortcut '%s': %s", s, e)
                     continue
+            action = action.replace("-", "_")       #must be an object attribute
+            log("action(%s)=%s%s", s, action, args)
 
             #example for keyspec: ["Alt", "F8"]
             keyspec = parts[0].split("+")
@@ -131,40 +146,80 @@ class KeyboardHelper(object):
                     #ie: "alt_l" -> "mod1"
                     imod = modifier_names.get(mod.lower())
                     if not imod:
-                        log.error("invalid modifier: %s, valid modifiers are: %s", mod, modifier_names.keys())
+                        log.error("Error: invalid modifier '%s' in keyboard shortcut '%s'", mod, s)
+                        log.error(" the modifiers must be one of: %s", csv(modifier_names.keys()))
                         valid = False
                         break
                     modifiers.append(imod)
                 if not valid:
                     continue
+            #TODO: validate keyname
             keyname = keyspec[len(keyspec)-1]
-            shortcuts[keyname] = (modifiers, action, args)
+            shortcuts.setdefault(keyname, []).append((modifiers, action, args))
+            log("shortcut(%s)=%s", keyname, (modifiers, action, args))
         log("parse_shortcuts(%s)=%s" % (str(strs), shortcuts))
+        print_nested_dict(shortcuts, print_fn=log)
         return  shortcuts
 
     def key_handled_as_shortcut(self, window, key_name, modifiers, depressed):
-        shortcut = self.key_shortcuts.get(key_name)
-        if not shortcut:
-            return  False
+        #find the shortcuts that may match this key:
+        shortcuts = self.key_shortcuts.get(key_name)
+        if not shortcuts:
+            return False
+        if len(shortcuts)>1:
+            #sort shortcuts based on how many modifiers are required,
+            #so that if multiple shortcuts use the same key,
+            #we will try to match the one with the most modifiers first.
+            #ie: Num_Lock+Menu will be tested before Menu
+            #(this is needed because Num_Lock is then discarded when comparing the list of required modifiers!)
+            shortcuts = sorted(shortcuts, key=lambda x : len(x[0]), reverse=True)
+        for shortcut in shortcuts:
+            if self._check_shortcut(window, key_name, modifiers, depressed, shortcut):
+                return True
+        return False
+
+    def _check_shortcut(self, window, key_name, modifiers, depressed, shortcut):
         (req_mods, action, args) = shortcut
+        extra_modifiers = list(modifiers)
         for rm in req_mods:
             if rm not in modifiers:
                 #modifier is missing, bail out
                 return False
+            try:
+                extra_modifiers.remove(rm)
+            except:
+                pass        #same modifier listed twice?
+        kmod = self.keyboard.get_keymap_modifiers()[0]
+        if not kmod and self.keyboard.modifier_keys:
+            #fallback to server supplied map:
+            kmod = self.keyboard.modifier_keys
+        #ie: {'ISO_Level3_Shift': 'mod5', 'Meta_L': 'mod1', ...}
+        log("keymap modifiers: %s", kmod)
+        ignoremod = ("Caps_Lock", "Num_Lock")
+        for x in ignoremod:
+            mod = kmod.get(x)
+            if mod in extra_modifiers:
+                extra_modifiers.remove(mod)
+        if extra_modifiers:
+            log("skipping partial shortcut match %s, modifiers unmatched: %s", shortcut, extra_modifiers)
+            return False
+        log("matched shortcut %s", shortcut)
         if not depressed:
             """ when the key is released, just ignore it - do NOT send it to the server! """
-            return  True
+            return True
         try:
             method = getattr(window, action)
             log("key_handled_as_shortcut(%s,%s,%s,%s) found shortcut=%s, will call %s%s", window, key_name, modifiers, depressed, shortcut, method, args)
         except AttributeError as e:
             log.error("key dropped, invalid method name in shortcut %s: %s", action, e)
-            return  True
+            return True
         try:
             method(*args)
             log("key_handled_as_shortcut(%s,%s,%s,%s) has been handled: %s", window, key_name, modifiers, depressed, method)
         except Exception as e:
-            log.error("key_handled_as_shortcut(%s,%s,%s,%s) failed to execute shortcut=%s: %s", window, key_name, modifiers, depressed, shortcut, e)
+            log.error("key_handled_as_shortcut(%s,%s,%s,%s)", window, key_name, modifiers, depressed)
+            log.error(" failed to execute shortcut=%s", shortcut)
+            log.error("", exc_info=True)
         return  True
 
 
@@ -244,14 +299,54 @@ class KeyboardHelper(object):
         self.keys_pressed = {}
 
 
+    def get_layout_spec(self):
+        """ add / honour overrides """
+        layout, layouts, variant, variants = self.keyboard.get_layout_spec()
+        log("%s.get_layout_spec()=%s", self.keyboard, (layout, layouts, variant, variants))
+        def inl(v, l):
+            try:
+                if v in l or v is None:
+                    return l
+                return [v]+list(l)
+            except:
+                if v is not None:
+                    return [v]
+                return []
+        layout   = self.layout_option or layout
+        layouts  = inl(layout, self.layouts_option or layouts)
+        variant  = self.variant_option or variant
+        variants = inl(variant, self.variants_option or variants)
+        val = (layout, layouts, self.variant_option or variant, self.variants_option or variants)
+        log("get_layout_spec()=%s", val)
+        return val
+
+    def get_keymap_spec(self):
+        _print, query, query_struct = self.keyboard.get_keymap_spec()
+        if self.layout_option:
+            query_struct["layout"] = self.layout_option
+        if self.layouts_option:
+            query_struct["layouts"] = csv(self.layouts_option)
+        if self.variant_option:
+            query_struct["variant"] = self.variant_option
+        if self.variants_option:
+            query_struct["variants"] = csv(self.variants_option)
+        if self.options:
+            if self.options.lower()=="none":
+                query_struct["options"] = ""
+            else:
+                query_struct["options"] = self.options
+        if self.layout_option or self.layouts_option or self.variant_option or self.variants_option or self.options:
+            query = xkbmap_query_tostring(query_struct)
+        return _print, query, query_struct
+
     def query_xkbmap(self):
-        self.xkbmap_layout, _, self.xkbmap_variant, self.xkbmap_variants = self.keyboard.get_layout_spec()
-        self.xkbmap_print, self.xkbmap_query, self.xkbmap_query_struct = self.keyboard.get_keymap_spec()
+        self.xkbmap_layout, self.xkbmap_layouts, self.xkbmap_variant, self.xkbmap_variants = self.get_layout_spec()
+        self.xkbmap_print, self.xkbmap_query, self.xkbmap_query_struct = self.get_keymap_spec()
         self.xkbmap_keycodes = self.get_full_keymap()
         self.xkbmap_x11_keycodes = self.keyboard.get_x11_keymap()
         self.xkbmap_mod_meanings, self.xkbmap_mod_managed, self.xkbmap_mod_pointermissing = self.keyboard.get_keymap_modifiers()
         self.update_hash()
-        log("layout=%s, variant=%s, variants=%s", self.xkbmap_layout, self.xkbmap_variant, self.xkbmap_variants)
+        log("layout=%s, layouts=%s, variant=%s, variants=%s", self.xkbmap_layout, self.xkbmap_layouts, self.xkbmap_variant, self.xkbmap_variants)
         log("print=%s, query=%s, struct=%s", nonl(self.xkbmap_print), nonl(self.xkbmap_query), nonl(self.xkbmap_query_struct))
         log("keycodes=%s", str(self.xkbmap_keycodes)[:80]+"...")
         log("x11 keycodes=%s", str(self.xkbmap_x11_keycodes)[:80]+"...")
@@ -265,12 +360,12 @@ class KeyboardHelper(object):
             self.query_xkbmap()
 
     def layout_str(self):
-        return " / ".join([x for x in (self.xkbmap_layout, self.xkbmap_variant) if bool(x)])
+        return " / ".join([x for x in (self.layout_option or self.xkbmap_layout, self.variant_option or self.xkbmap_variant) if bool(x)])
 
 
     def send_layout(self):
-        log("send_layout()")
-        self.send("layout-changed", self.xkbmap_layout or "", self.xkbmap_variant or "")
+        log("send_layout() layout_option=%s, xkbmap_layout=%s, variant_option=%s, xkbmap_variant=%s", self.layout_option, self.xkbmap_layout, self.variant_option, self.xkbmap_variant)
+        self.send("layout-changed", self.layout_option or self.xkbmap_layout or "", self.variant_option or self.xkbmap_variant or "")
 
     def send_keymap(self):
         log("send_keymap()")
@@ -298,7 +393,9 @@ class KeyboardHelper(object):
 
     def get_keymap_properties(self):
         props = {}
-        for x in ("layout", "variant", "print", "query", "query_struct", "mod_meanings",
+        for x in ("layout", "layouts", "variant", "variants",
+                  "raw",
+                  "print", "query", "query_struct", "mod_meanings",
                   "mod_managed", "mod_pointermissing", "keycodes", "x11_keycodes"):
             p = "xkbmap_%s" % x
             v = getattr(self, p)
@@ -307,3 +404,24 @@ class KeyboardHelper(object):
                 v = ""
             props[p] = v
         return  props
+
+
+    def log_keyboard_info(self):
+        #show the user a summary of what we have detected:
+        kb_info = {}
+        if self.xkbmap_query_struct or self.xkbmap_query:
+            xkbqs = self.xkbmap_query_struct
+            if xkbqs:
+                #parse query into a dict
+                from xpra.keyboard.layouts import parse_xkbmap_query
+                xkbqs = parse_xkbmap_query(self.xkbmap_query)
+            for x in ["rules", "model", "layout"]:
+                v = xkbqs.get(x)
+                if v:
+                    kb_info[x] = v
+        if self.xkbmap_layout:
+            kb_info["layout"] = self.xkbmap_layout
+        if len(kb_info)==0:
+            log.info(" using default keyboard settings")
+        else:
+            log.info(" keyboard settings: %s", ", ".join(["%s=%s" % (std(k), std(v)) for k,v in kb_info.items()]))
