@@ -1,10 +1,9 @@
 # This file is part of Xpra.
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2012-2015 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import os
 import hashlib
 from xpra.log import Logger
 log = Logger("paint")
@@ -13,19 +12,20 @@ deltalog = Logger("delta")
 from threading import Lock
 from xpra.net.mmap_pipe import mmap_read
 from xpra.net import compression
-from xpra.util import typedict, csv
+from xpra.util import typedict, csv, envint, envbool, repr_ellipsized
 from xpra.codecs.loader import get_codec
 from xpra.codecs.video_helper import getVideoHelper
 from xpra.os_util import BytesIOClass, bytestostr, _buffer
 from xpra.codecs.xor.cyxor import xor_str   #@UnresolvedImport
 from xpra.codecs.argb.argb import unpremultiply_argb, unpremultiply_argb_in_place   #@UnresolvedImport
 
-DELTA_BUCKETS = int(os.environ.get("XPRA_DELTA_BUCKETS", "5"))
-INTEGRITY_HASH = os.environ.get("XPRA_INTEGRITY_HASH", "0")=="1"
-WEBP_PILLOW = os.environ.get("XPRA_WEBP_PILLOW", "0")=="1"
+DELTA_BUCKETS = envint("XPRA_DELTA_BUCKETS", 5)
+INTEGRITY_HASH = envbool("XPRA_INTEGRITY_HASH", False)
+PAINT_BOX = envint("XPRA_PAINT_BOX", 0) or envint("XPRA_OPENGL_PAINT_BOX", 0)
+
 
 #ie:
-#CSC_OPTIONS = { "YUV420P" : {"RGBX" : [opencl.spec, swscale.spec], "BGRX" : ...} }
+#CSC_OPTIONS = { "YUV420P" : {"RGBX" : [swscale.spec], "BGRX" : ...} }
 CSC_OPTIONS = None
 def load_csc_options():
     global CSC_OPTIONS
@@ -46,7 +46,7 @@ def load_video_decoders():
         for encoding in vh.get_decodings():
             specs = vh.get_decoder_specs(encoding)
             for colorspace, decoders in specs.items():
-                log("%-4s decoders for %7s: %s", encoding, colorspace, csv([d.get_type() for _,d in decoders]))
+                log("%-5s decoders for %7s: %s", encoding, colorspace, csv([d.get_type() for _,d in decoders]))
                 assert len(decoders)>0
                 #use the first one:
                 _, decoder_module = decoders[0]
@@ -55,7 +55,7 @@ def load_video_decoders():
     return VIDEO_DECODERS
 
 
-def fire_paint_callbacks(callbacks, success, message=""):
+def fire_paint_callbacks(callbacks, success=True, message=""):
     for x in callbacks:
         try:
             x(success, message)
@@ -75,6 +75,7 @@ class WindowBackingBase(object):
         load_video_decoders()
         self.wid = wid
         self.size = 0, 0
+        self.offsets = 0, 0, 0, 0       #top,left,bottom,right
         self.idle_add = idle_add
         self._alpha_enabled = window_alpha
         self._backing = None
@@ -83,12 +84,18 @@ class WindowBackingBase(object):
         self._csc_decoder = None
         self._decoder_lock = Lock()
         self._PIL_encodings = []
+        self.default_paint_box_line_width = PAINT_BOX or 1
+        self.paint_box_line_width = PAINT_BOX
+        self.pointer_overlay = None
+        self.cursor_data = None
+        self.default_cursor_data = None
         PIL = get_codec("dec_pillow")
         if PIL:
             self._PIL_encodings = PIL.get_encodings()
         self.draw_needs_refresh = True
         self.mmap = None
         self.mmap_enabled = False
+        self.jpeg_decoder = get_codec("dec_jpeg")
 
     def enable_mmap(self, mmap_area):
         self.mmap = mmap_area
@@ -104,8 +111,10 @@ class WindowBackingBase(object):
         self.close_decoder(False)
 
     def close_decoder(self, blocking=False):
+        log("close_decoder(%s)", blocking)
         dl = self._decoder_lock
         if dl is None or not dl.acquire(blocking):
+            log("close_decoder(%s) lock %s not acquired", blocking, dl)
             return False
         try:
             self.do_clean_video_decoder()
@@ -154,6 +163,10 @@ class WindowBackingBase(object):
         return unpremultiply_argb(img_data)
 
 
+    def set_cursor_data(self, cursor_data):
+        self.cursor_data = cursor_data
+
+
     def process_delta(self, raw_data, width, height, rowstride, options):
         """
             Can be called from any thread, decompresses and xors the rgb raw_data,
@@ -167,12 +180,12 @@ class WindowBackingBase(object):
                 assert len(comp)==1, "more than one compressor specified: %s" % str(comp)
                 img_data = compression.decompress_by_name(raw_data, algo=comp[0])
         if len(img_data)!=rowstride * height:
-            deltalog.error("invalid img data length: expected %s but got %s (%s: %s)", rowstride * height, len(img_data), type(img_data), str(img_data)[:256])
+            deltalog.error("invalid img data length: expected %s but got %s (%s: %s)", rowstride * height, len(img_data), type(img_data), repr_ellipsized(img_data))
             raise Exception("expected %s bytes for %sx%s with rowstride=%s but received %s (%s compressed)" %
                                 (rowstride * height, width, height, rowstride, len(img_data), len(raw_data)))
-        delta = options.intget("delta", -1)
-        bucket = options.intget("bucket", 0)
-        rgb_format = options.strget("rgb_format")
+        delta = options.intget(b"delta", -1)
+        bucket = options.intget(b"bucket", 0)
+        rgb_format = options.strget(b"rgb_format")
         rgb_data = img_data
         if delta>=0:
             assert bucket>=0 and bucket<DELTA_BUCKETS, "invalid delta bucket number: %s" % bucket
@@ -192,6 +205,16 @@ class WindowBackingBase(object):
         return rgb_data
 
 
+    def paint_jpeg(self, img_data, x, y, width, height, options, callbacks):
+        img = self.jpeg_decoder.decompress_to_rgb("RGBX", img_data, width, height, options)
+        rgb_format = img.get_pixel_format()
+        img_data = img.get_pixels()
+        rowstride = img.get_rowstride()
+        w = img.get_width()
+        h = img.get_height()
+        self.idle_add(self.paint_rgb, rgb_format, img_data, x, y, w, h, rowstride, options, callbacks)
+
+
     def paint_image(self, coding, img_data, x, y, width, height, options, callbacks):
         """ can be called from any thread """
         #log("paint_image(%s, %s bytes, %s, %s, %s, %s, %s, %s)", coding, len(img_data), x, y, width, height, options, callbacks)
@@ -226,59 +249,46 @@ class WindowBackingBase(object):
             else:
                 img = img.convert("RGB")
 
-        #use tobytes() if present, fallback to tostring():
-        data_fn = getattr(img, "tobytes", getattr(img, "tostring", None))
-        raw_data = data_fn("raw", img.mode)
+        raw_data = img.tobytes("raw", img.mode)
         paint_options = typedict(options)
-        if img.mode=="RGB":
+        rgb_format = img.mode
+        if rgb_format=="RGB":
             #PIL flattens the data to a continuous straightforward RGB format:
             rowstride = width*3
-            paint_options["rgb_format"] = "RGB"
             img_data = self.process_delta(raw_data, width, height, rowstride, options)
-            self.idle_add(self.do_paint_rgb24, img_data, x, y, width, height, rowstride, paint_options, callbacks)
-        elif img.mode=="RGBA":
+        elif rgb_format=="RGBA":
             rowstride = width*4
-            paint_options["rgb_format"] = "RGBA"
             img_data = self.process_delta(raw_data, width, height, rowstride, options)
-            self.idle_add(self.do_paint_rgb32, img_data, x, y, width, height, rowstride, paint_options, callbacks)
+        else:
+            raise Exception("invalid image mode: %s" % img.mode)
+        self.idle_add(self.do_paint_rgb, rgb_format, img_data, x, y, width, height, rowstride, paint_options, callbacks)
         return False
 
-    def paint_webp(self, img_data, x, y, width, height, options, callbacks):
-        dec_webp = get_codec("dec_webp")
-        if not dec_webp or WEBP_PILLOW:
-            #if webp is enabled, then Pillow should be able to take care of it:
-            return self.paint_image("webp", img_data, x, y, width, height, options, callbacks)
-        has_alpha = options.get("has_alpha", False)
-        buffer_wrapper, width, height, stride, has_alpha, rgb_format = dec_webp.decompress(img_data, has_alpha, options.get("rgb_format"))
-        #replace with the actual rgb format we get from the decoder:
-        options["rgb_format"] = rgb_format
-        def free_buffer(*args):
-            buffer_wrapper.free()
-        callbacks.append(free_buffer)
-        data = buffer_wrapper.get_pixels()
-        if len(rgb_format)==4:
-            return self.paint_rgb32(data, x, y, width, height, stride, options, callbacks)
-        else:
-            return self.paint_rgb24(data, x, y, width, height, stride, options, callbacks)
-
-    def paint_rgb24(self, raw_data, x, y, width, height, rowstride, options, callbacks):
-        """ called from non-UI thread
-            this method calls process_delta before calling do_paint_rgb24 from the UI thread via idle_add
+    def paint_rgb(self, rgb_format, raw_data, x, y, width, height, rowstride, options, callbacks):
+        """ can be called from a non-UI thread
+            this method calls process_delta
+            before calling _do_paint_rgb from the UI thread via idle_add
         """
-        rgb24_data = self.process_delta(raw_data, width, height, rowstride, options)
-        self.idle_add(self.do_paint_rgb24, rgb24_data, x, y, width, height, rowstride, options, callbacks)
-        return  False
+        rgb_data = self.process_delta(raw_data, width, height, rowstride, options)
+        self.idle_add(self.do_paint_rgb, rgb_format, rgb_data, x, y, width, height, rowstride, options, callbacks)
 
-    def do_paint_rgb24(self, img_data, x, y, width, height, rowstride, options, callbacks):
-        """ must be called from UI thread
+    def do_paint_rgb(self, rgb_format, img_data, x, y, width, height, rowstride, options, callbacks):
+        """ must be called from the UI thread
             this method is only here to ensure that we always fire the callbacks,
-            the actual paint code is in _do_paint_rgb24
+            the actual paint code is in _do_paint_rgb[24|32]
         """
         try:
+            if not options.get("paint", True):
+                fire_paint_callbacks(callbacks)
+                return
             if self._backing is None:
                 fire_paint_callbacks(callbacks, -1, "no backing")
                 return
-            success = self._do_paint_rgb24(img_data, x, y, width, height, rowstride, options)
+            bpp = len(rgb_format)*8
+            assert bpp in (24, 32), "invalid rgb format %s" % rgb_format
+            paint_fn = getattr(self, "_do_paint_rgb%i" % bpp)
+            options[b"rgb_format"] = rgb_format
+            success = paint_fn(img_data, x, y, width, height, rowstride, options)
             fire_paint_callbacks(callbacks, success)
         except KeyboardInterrupt:
             raise
@@ -286,42 +296,13 @@ class WindowBackingBase(object):
             if not self._backing:
                 fire_paint_callbacks(callbacks, -1, "paint error on closed backing ignored")
             else:
-                log.error("do_paint_rgb24 error", exc_info=True)
-                message = "do_paint_rgb24 error: %s" % e
+                log.error("Error painting rgb%s", bpp, exc_info=True)
+                message = "paint rgb%s error: %s" % (bpp, e)
                 fire_paint_callbacks(callbacks, False, message)
 
     def _do_paint_rgb24(self, img_data, x, y, width, height, rowstride, options):
         raise Exception("override me!")
 
-
-    def paint_rgb32(self, raw_data, x, y, width, height, rowstride, options, callbacks):
-        """ called from non-UI thread
-            this method calls process_delta before calling do_paint_rgb32 from the UI thread via idle_add
-        """
-        rgb32_data = self.process_delta(raw_data, width, height, rowstride, options)
-        self.idle_add(self.do_paint_rgb32, rgb32_data, x, y, width, height, rowstride, options, callbacks)
-        return  False
-
-    def do_paint_rgb32(self, img_data, x, y, width, height, rowstride, options, callbacks):
-        """ must be called from UI thread
-            this method is only here to ensure that we always fire the callbacks,
-            the actual paint code is in _do_paint_rgb32
-        """
-        try:
-            if self._backing is None:
-                fire_paint_callbacks(callbacks, -1, "no backing")
-                return
-            success = self._do_paint_rgb32(img_data, x, y, width, height, rowstride, options)
-            fire_paint_callbacks(callbacks, success)
-        except KeyboardInterrupt:
-            raise
-        except Exception as e:
-            if not self._backing:
-                fire_paint_callbacks(callbacks, -1, "paint error on closed backing ignored")
-            else:
-                log.error("do_paint_rgb32 error", exc_info=True)
-                fire_paint_callbacks(callbacks, False, "do_paint_rgb32 error: %s" % e)
-
     def _do_paint_rgb32(self, img_data, x, y, width, height, rowstride, options):
         raise Exception("override me!")
 
@@ -337,21 +318,55 @@ class WindowBackingBase(object):
             if not specs:
                 continue
             for spec in specs:
-                if spec.min_w>src_width or spec.min_w>dst_width or \
-                   spec.max_w<src_width or spec.max_w<dst_width:
-                    log("csc module %s cannot cope with dimensions %sx%s to %sx%s", spec.codec_class, src_width, src_height, dst_width, dst_height)
-                    continue
-                if not spec.can_scale and (src_width!=dst_width or src_height!=dst_height):
-                    log("csc module %s cannot scale")
+                v = self.validate_csc_size(spec, src_width, src_height, dst_width, dst_height)
+                if v:
                     continue
                 try:
                     csc = spec.make_instance()
                     csc.init_context(src_width, src_height, src_format,
                                dst_width, dst_height, dst_format, speed)
                     return csc
-                except:
-                    log.error("failed to create csc instance of %s for %s to %s", spec.codec_class, src_format, dst_format, exc_info=True)
-        raise Exception("no csc module found for %s(%sx%s) to %s(%sx%s) in %s" % (src_format, src_width, src_height, " or ".join(dst_format_options), dst_width, dst_height, CSC_OPTIONS))
+                except Exception as e:
+                    log("make_csc%s", (src_width, src_height, src_format, dst_width, dst_height, dst_format_options, speed), exc_info=True)
+                    log.error("Error: failed to create csc instance %s", spec.codec_class)
+                    log.error(" for %s to %s: %s", src_format, dst_format, e)
+        log.error("Error: no matching CSC module found")
+        log.error(" for %ix%i %s source format,", src_width, src_height, src_format)
+        log.error(" to %ix%i %s", dst_width, dst_height, " or ".join(dst_format_options))
+        log.error(" with options=%s, speed=%i", dst_format_options, speed)
+        log.error(" tested:")
+        for dst_format in dst_format_options:
+            specs = in_options.get(dst_format)
+            if not specs:
+                continue
+            log.error(" * %s:", dst_format)
+            for spec in specs:
+                log.error("   - %s:", spec)
+                v = self.validate_csc_size(spec, src_width, src_height, dst_width, dst_height)
+                if v:
+                    log.error("       "+v[0], *v[1:])
+        raise Exception("no csc module found for %s(%sx%s) to %s(%sx%s)" % (src_format, src_width, src_height, " or ".join(dst_format_options), dst_width, dst_height, CSC_OPTIONS))
+
+    def validate_csc_size(self, spec, src_width, src_height, dst_width, dst_height):
+        if not spec.can_scale and (src_width!=dst_width or src_height!=dst_height):
+            return "scaling not suported"
+        elif src_width<spec.min_w:
+            return "source width %i is out of range: minimum is %i", src_width, spec.min_w
+        elif src_height<spec.min_h:
+            return "source height %i is out of range: minimum is %i", src_height, spec.min_h
+        elif dst_width<spec.min_w:
+            return "target width %i is out of range: minimum is %i", dst_width, spec.min_w
+        elif dst_height<spec.min_h:
+            return "target height %i is out of range: minimum is %i", dst_height, spec.min_h
+        elif src_width>spec.max_w:
+            return "source width %i is out of range: maximum is %i", src_width, spec.max_w
+        elif src_height>spec.max_h:
+            return "source height %i is out of range: maximum is %i", src_height, spec.max_h
+        elif dst_width>spec.max_w:
+            return "target width %i is out of range: maximum is %i", dst_width, spec.max_w
+        elif dst_height>spec.max_h:
+            return "target height %i is out of range: maximum is %i", dst_height, spec.max_h
+        return None
 
     def paint_with_video_decoder(self, decoder_module, coding, img_data, x, y, width, height, options, callbacks):
         #log("paint_with_video_decoder%s", (decoder_module, coding, "%s bytes" % len(img_data), x, y, width, height, options, callbacks))
@@ -365,35 +380,32 @@ class WindowBackingBase(object):
                 message = "window %s is already gone!" % self.wid
                 log(message)
                 fire_paint_callbacks(callbacks, -1, message)
-                return  False
+                return
             enc_width, enc_height = options.intpair("scaled_size", (width, height))
             input_colorspace = options.strget("csc")
             if not input_colorspace:
                 message = "csc mode is missing from the video options!"
                 log.error(message)
                 fire_paint_callbacks(callbacks, False, message)
-                return  False
+                return
             #do we need a prep step for decoders that cannot handle the input_colorspace directly?
             decoder_colorspaces = decoder_module.get_input_colorspaces(coding)
             assert input_colorspace in decoder_colorspaces, "decoder does not support %s for %s" % (input_colorspace, coding)
 
             vd = self._video_decoder
             if vd:
-                if vd.get_encoding()!=coding:
+                if options.get("frame", -1)==0:
+                    log("paint_with_video_decoder: first frame of new stream")
+                    self.do_clean_video_decoder()
+                elif vd.get_encoding()!=coding:
                     log("paint_with_video_decoder: encoding changed from %s to %s", vd.get_encoding(), coding)
                     self.do_clean_video_decoder()
                 elif vd.get_width()!=enc_width or vd.get_height()!=enc_height:
-                    log("paint_with_video_decoder: window dimensions have changed from %s to %s", (vd.get_width(), vd.get_height()), (enc_width, enc_height))
+                    log("paint_with_video_decoder: video dimensions have changed from %s to %s", (vd.get_width(), vd.get_height()), (enc_width, enc_height))
                     self.do_clean_video_decoder()
                 elif vd.get_colorspace()!=input_colorspace:
                     #this should only happen on encoder restart, which means this should be the first frame:
-                    l = log
-                    if options.get("frame", 0)>1:
-                        l = log.warn
-                    l("paint_with_video_decoder: colorspace changed from %s to %s", vd.get_colorspace(), input_colorspace)
-                    self.do_clean_video_decoder()
-                elif options.get("frame")==0:
-                    log("paint_with_video_decoder: first frame of new stream")
+                    log.warn("Warning: colorspace unexpectedly changed from %s to %s", vd.get_colorspace(), input_colorspace)
                     self.do_clean_video_decoder()
             if self._video_decoder is None:
                 log("paint_with_video_decoder: new %s(%s,%s,%s)", decoder_module.Decoder, width, height, input_colorspace)
@@ -404,18 +416,23 @@ class WindowBackingBase(object):
 
             img = vd.decompress_image(img_data, options)
             if not img:
-                fire_paint_callbacks(callbacks, False, "video decoder %s failed to decode %i bytes of %s data" % (vd.get_type(), len(img_data), coding))
-                log.error("Error: decode failed on %s bytes of %s data", len(img_data), coding)
-                log.error(" %sx%s pixels using %s", width, height, vd.get_type())
-                log.error(" decoding options=%s", options)
-                return False
+                if options.get("delayed", 0)>0:
+                    #there are further frames queued up,
+                    #and this frame references those, so assume all is well:
+                    fire_paint_callbacks(callbacks)
+                else:
+                    fire_paint_callbacks(callbacks, False, "video decoder %s failed to decode %i bytes of %s data" % (vd.get_type(), len(img_data), coding))
+                    log.error("Error: decode failed on %s bytes of %s data", len(img_data), coding)
+                    log.error(" %sx%s pixels using %s", width, height, vd.get_type())
+                    log.error(" frame options:")
+                    for k,v in options.items():
+                        log.error("   %s=%s", k, v)
+                return
             self.do_video_paint(img, x, y, enc_width, enc_height, width, height, options, callbacks)
         if self._backing is None:
             self.close_decoder(True)
-        return  False
 
     def do_video_paint(self, img, x, y, enc_width, enc_height, width, height, options, callbacks):
-        #try 24 bit first (paint_rgb24), then 32 bit (paint_rgb32):
         target_rgb_formats = self.RGB_MODES
         #as some video formats like vpx can forward transparency
         #also we could skip the csc step in some cases:
@@ -453,36 +470,27 @@ class WindowBackingBase(object):
         assert rgb.get_planes()==0, "invalid number of planes for %s: %s" % (rgb_format, rgb.get_planes())
         #make a new options dict and set the rgb format:
         paint_options = typedict(options)
-        paint_options["rgb_format"] = rgb_format
         #this will also take care of firing callbacks (from the UI thread):
         def paint():
             data = rgb.get_pixels()
             rowstride = rgb.get_rowstride()
-            if len(rgb_format)==3:
-                self.do_paint_rgb24(data, x, y, width, height, rowstride, paint_options, callbacks)
-            else:
-                assert len(rgb_format)==4
-                self.do_paint_rgb32(data, x, y, width, height, rowstride, paint_options, callbacks)
-            rgb.free()
+            try:
+                self.do_paint_rgb(rgb_format, data, x, y, width, height, rowstride, paint_options, callbacks)
+            finally:
+                rgb.free()
         self.idle_add(paint)
 
     def paint_mmap(self, img_data, x, y, width, height, rowstride, options, callbacks):
-        """ must be called from UI thread """
-        #we could run just paint_rgb24 from the UI thread,
-        #but this would not make much of a difference
-        #and would complicate the code (add a callback to free mmap area)
-        """ see _mmap_send() in server.py for details """
+        """ must be called from UI thread
+            see _mmap_send() in server.py for details """
         assert self.mmap_enabled
-        data = mmap_read(self.mmap, img_data)
-        rgb_format = options.strget("rgb_format", "RGB")
+        data = mmap_read(self.mmap, *img_data)
+        rgb_format = options.strget(b"rgb_format", b"RGB")
         #Note: BGR(A) is only handled by gl_window_backing
-        if rgb_format in ("RGB", "BGR"):
-            self.do_paint_rgb24(data, x, y, width, height, rowstride, options, callbacks)
-        elif rgb_format in ("RGBA", "BGRA", "BGRX", "RGBX"):
-            self.do_paint_rgb32(data, x, y, width, height, rowstride, options, callbacks)
-        else:
-            raise Exception("invalid rgb format: %s" % rgb_format)
-        return  False
+        self.do_paint_rgb(rgb_format, data, x, y, width, height, rowstride, options, callbacks)
+
+    def paint_scroll(self, *args):
+        raise NotImplementedError("no paint scroll on %s" % type(self))
 
 
     def draw_region(self, x, y, width, height, coding, img_data, rowstride, options, callbacks):
@@ -506,25 +514,26 @@ class WindowBackingBase(object):
                 self.idle_add(self.paint_mmap, img_data, x, y, width, height, rowstride, options, callbacks)
             elif coding == "rgb24" or coding == "rgb32":
                 #avoid confusion over how many bytes-per-pixel we may have:
-                rgb_format = options.get("rgb_format")
+                rgb_format = options.get(b"rgb_format")
                 if rgb_format:
                     Bpp = len(rgb_format)
                 elif coding=="rgb24":
-                    Bpp = 3
+                    #legacy:
+                    rgb_format = "RGB"
                 else:
-                    Bpp = 4
+                    #legacy:
+                    rgb_format = "RGBX"
                 if rowstride==0:
                     rowstride = width * Bpp
-                if Bpp==3:
-                    self.paint_rgb24(img_data, x, y, width, height, rowstride, options, callbacks)
-                else:
-                    self.paint_rgb32(img_data, x, y, width, height, rowstride, options, callbacks)
+                self.paint_rgb(rgb_format, img_data, x, y, width, height, rowstride, options, callbacks)
             elif coding in VIDEO_DECODERS:
                 self.paint_with_video_decoder(VIDEO_DECODERS.get(coding), coding, img_data, x, y, width, height, options, callbacks)
-            elif coding == "webp":
-                self.paint_webp(img_data, x, y, width, height, options, callbacks)
+            elif self.jpeg_decoder and coding=="jpeg":
+                self.paint_jpeg(img_data, x, y, width, height, options, callbacks)
             elif coding in self._PIL_encodings:
                 self.paint_image(coding, img_data, x, y, width, height, options, callbacks)
+            elif coding == "scroll":
+                self.paint_scroll(x, y, width, height, img_data, options, callbacks)
             else:
                 self.do_draw_region(x, y, width, height, coding, img_data, rowstride, options, callbacks)
         except Exception:
@@ -534,6 +543,6 @@ class WindowBackingBase(object):
                 raise
 
     def do_draw_region(self, x, y, width, height, coding, img_data, rowstride, options, callbacks):
-        msg = "invalid encoding: %s" % coding
+        msg = "invalid encoding: '%s'" % coding
         log.error("Error: %s", msg)
         fire_paint_callbacks(callbacks, False, msg)
