@@ -1,10 +1,9 @@
 # This file is part of Xpra.
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2012-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import sys
 import os
 import struct
 import re
@@ -19,11 +18,13 @@ gdk = import_gdk()
 from xpra.log import Logger
 log = Logger("clipboard")
 
-from xpra.gtk_common.gobject_util import n_arg_signal
+from xpra.gtk_common.gobject_util import no_arg_signal, SIGNAL_RUN_LAST
 from xpra.gtk_common.gtk_util import GetClipboard, PROPERTY_CHANGE_MASK
 from xpra.gtk_common.nested_main import NestedMainLoop
-from xpra.net.compression import Uncompressed
-from xpra.util import csv
+from xpra.net.compression import Compressible
+from xpra.os_util import WIN32, POSIX, monotonic_time
+from xpra.util import csv, envint, envbool, repr_ellipsized
+from xpra.platform.features import CLIPBOARD_GREEDY
 
 
 MIN_CLIPBOARD_COMPRESSION_SIZE = 512
@@ -38,8 +39,9 @@ if CLIPBOARDS_ENV is not None:
     CLIPBOARDS = [x.upper().strip() for x in CLIPBOARDS]
 
 
-TEST_DROP_CLIPBOARD_REQUESTS = int(os.environ.get("XPRA_TEST_DROP_CLIPBOARD", "0"))
-STORE_ON_EXIT = os.environ.get("XPRA_CLIPBOARD_STORE_ON_EXIT", "1")=="1"
+TEST_DROP_CLIPBOARD_REQUESTS = envint("XPRA_TEST_DROP_CLIPBOARD")
+STORE_ON_EXIT = envbool("XPRA_CLIPBOARD_STORE_ON_EXIT", True)
+DELAY_SEND_TOKEN = envint("XPRA_DELAY_SEND_TOKEN", 100)
 
 _discard_target_strs_ = os.environ.get("XPRA_DISCARD_TARGETS")
 if _discard_target_strs_ is not None:
@@ -58,11 +60,44 @@ DISCARD_TARGETS = [re.compile(x) for x in DISCARD_TARGETS]
 
 TEXT_TARGETS = ("UTF8_STRING", "TEXT", "STRING", "text/plain")
 
+TRANSLATED_TARGETS = {
+    "application/x-moz-nativehtml" : "UTF8_STRING"
+    }
+
+
+MAX_NESTING = 20
+
+def nesting_check():
+    l = gtk.main_level()
+    if l>=MAX_NESTING:
+        log.warn("Warning: clipboard loop nesting too deep: %s", l)
+        log.warn(" your setup may have a clipboard forwarding loop,")
+        log.warn(" disabling the clipboard")
+        return False
+    return True
+
+
+#may get overriden
+def nosanitize_gtkselectiondata(selectiondata):
+    return False
+sanitize_gtkselectiondata = nosanitize_gtkselectiondata
+
+
+def must_discard(target):
+    return any(x for x in DISCARD_TARGETS if x.match(target))
+
+def _filter_targets(targets):
+    f = [target for target in targets if not must_discard(target)]
+    log("_filter_targets(%s)=%s", targets, f)
+    return f
+
 
 class ClipboardProtocolHelperBase(object):
     def __init__(self, send_packet_cb, progress_cb=None, **kwargs):
         self.send = send_packet_cb
         self.progress_cb = progress_cb
+        self.can_send = kwargs.get("can-send", True)
+        self.can_receive = kwargs.get("can-receive", True)
         self.max_clipboard_packet_size = MAX_CLIPBOARD_PACKET_SIZE
         self.filter_res = []
         filter_res = kwargs.get("filters")
@@ -83,16 +118,19 @@ class ClipboardProtocolHelperBase(object):
         return "ClipboardProtocolHelperBase"
 
     def get_info(self):
-        info = {"type"      : str(self),
+        info = {
+                "type"      : str(self),
                 "max_size"  : self.max_clipboard_packet_size,
                 "filters"   : [x.pattern for x in self.filter_res],
                 "requests"  : self._clipboard_request_counter,
                 "pending"   : self._clipboard_outstanding_requests.keys(),
+                "can-send"      : self.can_send,
+                "can-receive"   : self.can_receive,
                 "want_targets"  : self._want_targets,
+                "sanitize-gtkselectiondata" : sanitize_gtkselectiondata!=nosanitize_gtkselectiondata,
                 }
         for clipboard, proxy in self._clipboard_proxies.items():
-            for k,v in proxy.get_info().items():
-                info["%s.%s" % (clipboard, k)] = v
+            info[clipboard] = proxy.get_info()
         return info
 
     def cleanup(self):
@@ -103,6 +141,12 @@ class ClipboardProtocolHelperBase(object):
             x.cleanup()
         self._clipboard_proxies = {}
 
+    def set_direction(self, can_send, can_receive):
+        self.can_send = can_send
+        self.can_receive = can_receive
+        for proxy in self._clipboard_proxies.values():
+            proxy.set_direction(can_send, can_receive)
+
     def enable_selections(self, selections):
         #when clients first connect or later through the "clipboard-enable-selections" packet,
         #they can tell us which clipboard selections they want enabled
@@ -136,6 +180,7 @@ class ClipboardProtocolHelperBase(object):
         self._clipboard_proxies = {}
         for clipboard in clipboards:
             proxy = self.make_proxy(clipboard)
+            proxy.set_direction(self.can_send, self.can_receive)
             proxy.connect("send-clipboard-token", self._send_clipboard_token_handler)
             proxy.connect("get-clipboard-from-remote", self._get_clipboard_from_remote_handler)
             proxy.show()
@@ -151,8 +196,9 @@ class ClipboardProtocolHelperBase(object):
 
     # Used by the client during startup:
     def send_all_tokens(self):
-        for selection, proxy in self._clipboard_proxies.items():
-            self._send_clipboard_token_handler(proxy, selection)
+        for proxy in self._clipboard_proxies.values():
+            proxy._have_token = False
+            self._send_clipboard_token_handler(proxy)
 
     def _process_clipboard_token(self, packet):
         selection = packet[1]
@@ -179,13 +225,21 @@ class ClipboardProtocolHelperBase(object):
             target, dtype, dformat, wire_encoding, wire_data = packet[3:8]
             raw_data = self._munge_wire_selection_to_raw(wire_encoding, dtype, dformat, wire_data)
             target_data[target] = raw_data
-        proxy.got_token(targets, target_data)
+        #older versions always claimed the selection when the token is received:
+        claim = True
+        if len(packet)>=10:
+            claim = bool(packet[8])
+            #clients can now also change the greedy flag on the fly,
+            #this is needed for clipboard direction restrictions:
+            #the client may want to be notified of clipboard changes, just like a greedy client
+            proxy._greedy_client = bool(packet[9])
+        synchronous_client = len(packet)>=11 and bool(packet[10])
+        proxy.got_token(targets, target_data, claim, synchronous_client)
 
     def _get_clipboard_from_remote_handler(self, proxy, selection, target):
-        for x in DISCARD_TARGETS:
-            if x.match(target):
-                log("invalid target '%s'", target)
-                return None
+        if must_discard(target):
+            log("invalid target '%s'", target)
+            return None
         request_id = self._clipboard_request_counter
         self._clipboard_request_counter += 1
         log("get clipboard from remote handler id=%s", request_id)
@@ -210,10 +264,19 @@ class ClipboardProtocolHelperBase(object):
             return
         loop.done({"type": dtype, "format": dformat, "data": data})
 
-    def _send_clipboard_token_handler(self, proxy, selection):
+    def _send_clipboard_token_handler(self, proxy):
+        selection = proxy._selection
         log("send clipboard token: %s", selection)
         rsel = self.local_to_remote(selection)
+        def send_token(*args):
+            self.send("clipboard-token", *args)
+        if not proxy._can_send:
+            #send the token without data,
+            #and with claim flag set to False, greedy set to True
+            send_token(rsel, [], "NOTARGET", "UTF8_STRING", 8, "bytes", "", False, True)
+            return
         if self._want_targets:
+            log("client wants targets with the token, querying TARGETS")
             #send the token with the target and data once we get them:
             #first get the targets, then get the contents for targets we want to send (if any)
             def got_targets(dtype, dformat, targets):
@@ -221,13 +284,13 @@ class ClipboardProtocolHelperBase(object):
                 #if there is a text target, send that too (just the first one that matches for now..)
                 send_now = [x for x in targets if x in TEXT_TARGETS]
                 def send_targets_only():
-                    self.send("clipboard-token", rsel, targets)
+                    send_token(rsel, targets)
                 if len(send_now)==0:
                     send_targets_only()
                     return
                 target = send_now[0]
                 def got_contents(dtype, dformat, data):
-                    log("got_contents for selection %s: %s, %s, %s", selection, dtype, dformat, data)
+                    log("got_contents for selection %s: %s, %s, %s", selection, dtype, dformat, repr_ellipsized(data))
                     #code mostly duplicated from _process_clipboard_request
                     #see there for details
                     if dtype is None or data is None:
@@ -241,13 +304,13 @@ class ClipboardProtocolHelperBase(object):
                     if not wire_data:
                         send_targets_only()
                         return
-                    target_data = (target, dtype, dformat, wire_encoding, wire_data)
+                    target_data = (target, dtype, dformat, wire_encoding, wire_data, True, CLIPBOARD_GREEDY)
                     log("sending token with target data: %s", target_data)
-                    self.send("clipboard-token", rsel, targets, *target_data)
+                    send_token(rsel, targets, *target_data)
                 proxy.get_contents(target, got_contents)
             proxy.get_contents("TARGETS", got_targets)
             return
-        self.send("clipboard-token", rsel)
+        send_token(rsel)
 
     def _munge_raw_selection_to_wire(self, target, dtype, dformat, data):
         # Some types just cannot be marshalled:
@@ -259,22 +322,9 @@ class ClipboardProtocolHelperBase(object):
             #targets is special cased here
             #because we get the values in wire format already (not atoms)
             #thanks to the request_targets() function (required on win32)
-            return "atoms", self._filter_targets(data)
+            return "atoms", _filter_targets(data)
         return self._do_munge_raw_selection_to_wire(target, dtype, dformat, data)
 
-    def _filter_targets(self, targets):
-        remove = []
-        for target in targets:
-            for x in DISCARD_TARGETS:
-                if x.match(target):
-                    remove.append(target)
-                    break
-        f = list(targets)
-        for t in remove:
-            f.remove(t)
-        log("_filter_targets(%s)=%s", targets, f)
-        return f
-
     def _do_munge_raw_selection_to_wire(self, target, dtype, dformat, data):
         """ this method is overriden in xclipboard to parse X11 atoms """
         # Other types need special handling, and all types need to be
@@ -282,7 +332,7 @@ class ClipboardProtocolHelperBase(object):
         log("_do_munge_raw_selection_to_wire(%s, %s, %s, %s:%s)", target, dtype, dformat, type(data), len(data or ""))
         if dformat == 32:
             #you should be using gdk_clipboard for atom support!
-            if dtype in ("ATOM", "ATOM_PAIR") and os.name=="posix":
+            if dtype in ("ATOM", "ATOM_PAIR") and POSIX:
                 #we cannot handle gdk atoms here (but gdk_clipboard does)
                 return None, None
             #important note: on 64 bits, format=32 means 8 bytes, not 4
@@ -333,21 +383,20 @@ class ClipboardProtocolHelperBase(object):
         request_id, selection, target = packet[1:4]
         def no_contents():
             self.send("clipboard-contents-none", request_id, selection)
-        for x in DISCARD_TARGETS:
-            if x.match(target):
-                log("invalid target '%s'", target)
-                no_contents()
-                return
+        if must_discard(target):
+            log("invalid target '%s'", target)
+            no_contents()
+            return
         name = self.remote_to_local(selection)
         log("process clipboard request, request_id=%s, selection=%s, local name=%s, target=%s", request_id, selection, name, target)
         proxy = self._clipboard_proxies.get(name)
         if proxy is None:
             #err, we were asked about a clipboard we don't handle..
-            log.warn("ignoring clipboard request for '%s' (no proxy)", name)
+            log.error("Error: clipboard request for '%s' (no proxy, ignored)", name)
             no_contents()
             return
         if not proxy.is_enabled():
-            log.warn("ignoring clipboard request for '%s' (disabled)", name)
+            log.warn("Warning: ignoring clipboard request for '%s' (disabled)", name)
             no_contents()
             return
         if TEST_DROP_CLIPBOARD_REQUESTS>0 and (request_id % TEST_DROP_CLIPBOARD_REQUESTS)==0:
@@ -373,11 +422,11 @@ class ClipboardProtocolHelperBase(object):
 
     def _may_compress(self, dtype, dformat, wire_data):
         if len(wire_data)>self.max_clipboard_packet_size:
-            log.warn("clipboard contents are too big and have not been sent:"
-                     " %s compressed bytes dropped (maximum is %s)", len(wire_data), self.max_clipboard_packet_size)
+            log.warn("Warning: clipboard contents are too big and have not been sent")
+            log.warn(" %s compressed bytes dropped (maximum is %s)", len(wire_data), self.max_clipboard_packet_size)
             return  None
         if type(wire_data)==str and len(wire_data)>=MIN_CLIPBOARD_COMPRESSION_SIZE:
-            return Uncompressed("clipboard: %s / %s" % (dtype, dformat), wire_data)
+            return Compressible("clipboard: %s / %s" % (dtype, dformat), wire_data)
         return wire_data
 
     def _process_clipboard_contents(self, packet):
@@ -419,12 +468,12 @@ class DefaultClipboardProtocolHelper(ClipboardProtocolHelperBase):
 class ClipboardProxy(gtk.Invisible):
     __gsignals__ = {
         # arguments: (selection, target)
-        "get-clipboard-from-remote": (gobject.SIGNAL_RUN_LAST,
+        "get-clipboard-from-remote": (SIGNAL_RUN_LAST,
                                       gobject.TYPE_PYOBJECT,
                                       (gobject.TYPE_PYOBJECT,) * 2,
                                       ),
         # arguments: (selection,)
-        "send-clipboard-token": n_arg_signal(1),
+        "send-clipboard-token": no_arg_signal,
         }
 
     def __init__(self, selection):
@@ -435,12 +484,14 @@ class ClipboardProxy(gtk.Invisible):
         self._enabled = True
         self._have_token = False
         #this workaround is only needed on win32 AFAIK:
-        self._strip_nullbyte = sys.platform.startswith("win")
+        self._strip_nullbyte = WIN32
         #clients that need a new token for every owner-change: (ie: win32 and osx)
         #(forces the client to request new contents - prevents stale clipboard data)
         self._greedy_client = False
         #semaphore to block the sending of the token when we change the owner ourselves:
         self._block_owner_change = False
+        self._last_emit_token = 0
+        self._emit_token_timer = 0
         #counters for info:
         self._selection_request_events = 0
         self._selection_get_events = 0
@@ -449,6 +500,7 @@ class ClipboardProxy(gtk.Invisible):
         self._got_token_events = 0
         self._get_contents_events = 0
         self._request_contents_events = 0
+        self._last_targets = ()
 
         try:
             from xpra.x11.gtk_x11.prop import prop_get
@@ -458,23 +510,33 @@ class ClipboardProxy(gtk.Invisible):
 
         self._clipboard.connect("owner-change", self.do_owner_changed)
 
+    def set_direction(self, can_send, can_receive):
+        self._can_send = can_send
+        self._can_receive = can_receive
+
     def get_info(self):
-        info = {"have_token"    : self._have_token,
-                "enabled"       : self._enabled,
-                "greedy_client" : self._greedy_client,
-                "blocked_owner_change" : self._block_owner_change,
-                "event.selection_request"   : self._selection_request_events,
-                "event.selection_get"       : self._selection_get_events,
-                "event.selection_clear"     : self._selection_clear_events,
-                "event.got_token"           : self._got_token_events,
-                "event.sent_token"          : self._sent_token_events,
-                "event.get_contents"        : self._get_contents_events,
-                "event.request_contents"    : self._request_contents_events,
+        info = {
+                "have_token"            : self._have_token,
+                "enabled"               : self._enabled,
+                "greedy_client"         : self._greedy_client,
+                "blocked_owner_change"  : self._block_owner_change,
+                "last-targets"          : self._last_targets,
+                "event"         : {
+                                   "selection_request"     : self._selection_request_events,
+                                   "selection_get"         : self._selection_get_events,
+                                   "selection_clear"       : self._selection_clear_events,
+                                   "got_token"             : self._got_token_events,
+                                   "sent_token"            : self._sent_token_events,
+                                   "get_contents"          : self._get_contents_events,
+                                   "request_contents"      : self._request_contents_events,
+                                   },
                 }
         return info
 
     def cleanup(self):
-        if not self._have_token and STORE_ON_EXIT:
+        self._enabled = False
+        self.cancel_emit_token()
+        if self._can_receive and not self._have_token and STORE_ON_EXIT:
             self._clipboard.store()
         self.destroy()
 
@@ -493,19 +555,52 @@ class ClipboardProxy(gtk.Invisible):
         return  "ClipboardProxy(%s)" % self._selection
 
     def do_owner_changed(self, *args):
-        #log("do_owner_changed(%s) greedy_client=%s, block_owner_change=%s", args, self._greedy_client, self._block_owner_change)
-        log("clipboard: %s owner_changed greedy_client=%s, block_owner_change=%s", self._selection, self._greedy_client, self._block_owner_change)
-        if self._enabled and self._greedy_client and not self._block_owner_change:
-            self._block_owner_change = True
-            self._have_token = False
-            self.emit("send-clipboard-token", self._selection)
-            self._sent_token_events += 1
+        #an application on our side owns the clipboard selection
+        #(they are ready to provide something via the clipboard)
+        log("clipboard: %s owner_changed, enabled=%s, can-send=%s, can-receive=%s, have_token=%s, greedy_client=%s, block_owner_change=%s", self._selection, self._enabled, self._can_send, self._can_receive, self._have_token, self._greedy_client, self._block_owner_change)
+        if not self._enabled or self._block_owner_change:
+            return
+        if self._have_token or self._greedy_client:
+            if self._have_token or DELAY_SEND_TOKEN<0:
+                #token ownership will change or told not to wait
+                glib.idle_add(self.emit_token)
+            elif not self._emit_token_timer:
+                #we had it already, this can wait:
+                #TODO: don't throttle clients without "want-targets" attribute
+                # (sending the token is only expensive for those)
+                self.schedule_emit_token()
+
+    def schedule_emit_token(self):
+        now = monotonic_time()
+        elapsed = int((now-self._last_emit_token)*1000)
+        log("schedule_emit_token() elapsed=%i (max=%i)", elapsed, DELAY_SEND_TOKEN)
+        if elapsed>=DELAY_SEND_TOKEN:
+            #enough time has passed
+            self.emit_token()
+        else:
+            self._emit_token_timer = glib.timeout_add(DELAY_SEND_TOKEN-elapsed, self.emit_token)
+
+    def emit_token(self):
+        self._emit_token_timer = None
+        boc = self._block_owner_change
+        self._block_owner_change = True
+        self._have_token = False
+        self._last_emit_token = monotonic_time()
+        self.emit("send-clipboard-token")
+        self._sent_token_events += 1
+        if boc is False:
             glib.idle_add(self.remove_block)
 
+    def cancel_emit_token(self):
+        ett = self._emit_token_timer
+        if ett:
+            self._emit_token_timer = None
+            glib.source_remove(ett)
+
     def do_selection_request_event(self, event):
         log("do_selection_request_event(%s)", event)
         self._selection_request_events += 1
-        if not self._enabled:
+        if not self._enabled or not self._can_receive:
             gtk.Invisible.do_selection_request_event(self, event)
             return
         # Black magic: the superclass default handler for this signal
@@ -565,7 +660,10 @@ class ClipboardProxy(gtk.Invisible):
         # Either call selection_data.set() or don't, and then return.
         # In practice, send a call across the wire, then block in a recursive
         # main loop.
-        if not self._enabled:
+        def nodata():
+            selection_data.set("STRING", 8, "")
+        if not self._enabled or not self._can_receive:
+            nodata()
             return
         log("do_selection_get(%s, %s, %s) selection=%s", selection_data, info, time, selection_data.selection)
         self._selection_get_events += 1
@@ -575,20 +673,24 @@ class ClipboardProxy(gtk.Invisible):
         result = self.emit("get-clipboard-from-remote", self._selection, target)
         if result is None or result["type"] is None:
             log("remote selection fetch timed out or empty")
-            selection_data.set("STRING", 8, "")
+            nodata()
             return
         data = result["data"]
         dformat = result["format"]
         dtype = result["type"]
         log("do_selection_get(%s,%s,%s) calling selection_data.set(%s, %s, %s:%s)",
               selection_data, info, time, dtype, dformat, type(data), len(data or ""))
+        boc = self._block_owner_change
+        self._block_owner_change = True
         selection_data.set(dtype, dformat, data)
+        if boc is False:
+            glib.idle_add(self.remove_block)
 
     def do_selection_clear_event(self, event):
         # Someone else on our side has the selection
         log("do_selection_clear_event(%s) have_token=%s, block_owner_change=%s selection=%s", event, self._have_token, self._block_owner_change, self._selection)
         self._selection_clear_events += 1
-        if self._enabled:
+        if self._enabled and not self._block_owner_change:
             #if greedy_client is set, do_owner_changed will fire the token
             #so don't bother sending it now (same if we don't have it)
             send = ((self._greedy_client and not self._block_owner_change) or self._have_token)
@@ -598,33 +700,52 @@ class ClipboardProxy(gtk.Invisible):
             # ours now"
             # Send off the anti-token.
             if send:
-                boc = self._block_owner_change
-                self._block_owner_change = True
-                self.emit("send-clipboard-token", self._selection)
-                if boc is False:
-                    glib.idle_add(self.remove_block)
+                self.emit_token()
         gtk.Invisible.do_selection_clear_event(self, event)
 
-    def got_token(self, targets, target_data):
+    def got_token(self, targets, target_data, claim=True, synchronous_client=False):
         # We got the anti-token.
-        log("got token, selection=%s, targets=%s, target_data=%s", self._selection, targets, target_data)
+        self.cancel_emit_token()
         if not self._enabled:
             return
         self._got_token_events += 1
-        self._have_token = True
-        if self._greedy_client:
+        log("got token, selection=%s, targets=%s, target data=%s, claim=%s, can-receive=%s", self._selection, targets, target_data, claim, self._can_receive)
+        if self._greedy_client or CLIPBOARD_GREEDY:
             self._block_owner_change = True
-        self.claim()
-        if self._block_owner_change:
             #re-enable the flag via idle_add so events like do_owner_changed
             #get a chance to run first.
             glib.idle_add(self.remove_block)
+        if (CLIPBOARD_GREEDY or synchronous_client) and self._can_receive:
+            if targets:
+                for target in targets:
+                    self.selection_add_target(self._selection, target, 0)
+                self.selection_owner_set(self._selection)
+            if target_data:
+                for text_target in TEXT_TARGETS:
+                    if text_target in target_data:
+                        text_data = target_data.get(text_target)
+                        log("clipboard %s set to '%s'", self._selection, repr_ellipsized(text_data))
+                        self._clipboard.set_text(text_data, len=len(text_data))
+        if not claim:
+            log("token packet without claim, not setting the token flag")
+            #the other end is just telling us to send the token again next time something changes,
+            #not that they want to own the clipboard selection
+            return
+        self._have_token = True
+        if self._can_receive:
+            if not self._block_owner_change:
+                #if we don't claim the selection (can-receive=False),
+                #we will have to send the token back on owner-change!
+                self._block_owner_change = True
+                glib.idle_add(self.remove_block)
+            self.claim()
 
     def remove_block(self, *args):
         log("remove_block: %s", self._selection)
         self._block_owner_change = False
 
     def claim(self):
+        log("claim() selection=%s, enabled=%s", self._selection, self._enabled)
         if self._enabled and not self.selection_owner_set(self._selection):
             # I don't know how this can actually fail, given that we pass
             # CurrentTime, but just in case:
@@ -637,8 +758,8 @@ class ClipboardProxy(gtk.Invisible):
     # This function is called by the xpra core when the peer has requested the
     # contents of this clipboard:
     def get_contents(self, target, cb):
-        log("get_contents(%s,%s) selection=%s", target, cb, self._selection)
-        if not self._enabled:
+        log("get_contents(%s,%s) selection=%s, enabled=%s, can-send=%s", target, cb, self._selection, self._enabled, self._can_send)
+        if not self._enabled or not self._can_send:
             cb(None, None, None)
             return
         self._get_contents_events += 1
@@ -652,11 +773,13 @@ class ClipboardProxy(gtk.Invisible):
             def got_targets(c, targets, *args):
                 log("got_targets(%s, %s, %s)", c, targets, args)
                 cb("ATOM", 32, targets)
+                self._last_targets = targets or ()
             self._clipboard.request_targets(got_targets)
             return
         def unpack(clipboard, selection_data, user_data):
             log("unpack %s: %s", clipboard, type(selection_data))
-            if selection_data is None:
+            global sanitize_gtkselectiondata
+            if sanitize_gtkselectiondata(selection_data):
                 cb(None, None, None)
                 return
             log("unpack: %s", selection_data)
@@ -669,6 +792,13 @@ class ClipboardProxy(gtk.Invisible):
                     log("stripping end of string null byte")
                     data = data[:-1]
             cb(str(selection_data.type), selection_data.format, data)
+        #some applications (ie: firefox, thunderbird) can request invalid targets,
+        #when that happens, translate it to something the application can handle (if any)
+        translated_target = TRANSLATED_TARGETS.get(target)
+        if (translated_target is not None) and self._last_targets and (target not in self._last_targets) and \
+            (translated_target in self._last_targets) and (not must_discard(translated_target)):
+            log("invalid target %s, replaced with %s", target, translated_target)
+            target = translated_target
         self._clipboard.request_contents(target, unpack)
 
 gobject.type_register(ClipboardProxy)
