@@ -28,16 +28,23 @@ def get_subsampling_divs(pixel_format):
     return PIXEL_SUBSAMPLING.get(pixel_format)
 
 
-RGB_FORMATS = ("XRGB",
+RGB_FORMATS = (
+               "XRGB",
                "BGRX",
                "ARGB",
                "BGRA",
-               "RGB")
+               "RGB",
+               "BGR",
+               "r210",
+               )
 
 
 class TransientCodecException(Exception):
     pass
 
+class CodecStateException(Exception):
+    pass
+
 
 class _codec_spec(object):
 
@@ -46,7 +53,8 @@ class _codec_spec(object):
     WARN_LIMIT = 25
 
     def __init__(self, codec_class, codec_type="",
-                    quality=100, speed=100,
+                    quality=50, speed=50,
+                    size_efficiency=50,
                     setup_cost=50, cpu_cost=100, gpu_cost=0,
                     min_w=1, min_h=1, max_w=4*1024, max_h=4*1024,
                     can_scale=False,
@@ -56,6 +64,7 @@ class _codec_spec(object):
         self.codec_type = codec_type            #ie: "nvenc"
         self.quality = quality
         self.speed = speed
+        self.size_efficiency = size_efficiency
         self.setup_cost = setup_cost
         self.cpu_cost = cpu_cost
         self.gpu_cost = gpu_cost
@@ -161,18 +170,15 @@ class csc_spec(_codec_spec):
 
 
 def main():
-    from xpra.platform import init, clean
-    try:
-        init("Codec-Constants", "Codec Constants Info")
+    from xpra.platform import program_context
+    with program_context("Codec-Constants", "Codec Constants Info"):
         import sys
         if "-v" in sys.argv or "--verbose" in sys.argv:
             log.enable_debug()
         log.info("LOSSY_PIXEL_FORMATS=%s", LOSSY_PIXEL_FORMATS)
         log.info("PIXEL_SUBSAMPLING=%s", PIXEL_SUBSAMPLING)
         log.info("RGB_FORMATS=%s", RGB_FORMATS)
-    finally:
-        #this will wait for input on win32:
-        clean()
+
 
 if __name__ == "__main__":
     main()
