@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2013, 2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2013-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -12,15 +12,13 @@ from xpra.log import Logger
 log = Logger("csc", "cuda")
 
 import os
-import time
 import pycuda
 from pycuda import driver
 
-from xpra.util import engs
-
-CUDA_DEVICE_ID = int(os.environ.get("XPRA_CUDA_DEVICE", "-1"))
-CUDA_DEVICE_NAME = os.environ.get("XPRA_CUDA_DEVICE_NAME", "")
+from xpra.util import engs, print_nested_dict, envbool
+from xpra.os_util import monotonic_time, bytestostr
 
+IGNORE_BLACKLIST = envbool("XPRA_CUDA_IGNORE_BLACKLIST", False)
 
 #record when we get failures/success:
 DEVICE_STATE = {}
@@ -70,20 +68,71 @@ def get_device_info(i):
     global DEVICE_INFO
     return DEVICE_INFO.get(i, None)
 
+
+PREFS = None
+def get_prefs():
+    global PREFS
+    if PREFS is None:
+        PREFS = {}
+        from xpra.platform.paths import get_default_conf_dirs, get_system_conf_dirs, get_user_conf_dirs
+        dirs = get_default_conf_dirs() + get_system_conf_dirs() + get_user_conf_dirs()
+        log("get_prefs() will try to load cuda.conf from: %s", dirs)
+        for d in dirs:
+            conf_file = os.path.join(os.path.expanduser(d), "cuda.conf")
+            if not os.path.exists(conf_file):
+                log("get_prefs() '%s' does not exist!", conf_file)
+                continue
+            if not os.path.isfile(conf_file):
+                log("get_prefs() '%s' is not a file!", conf_file)
+                continue
+            try:
+                with open(conf_file, "rb") as f:
+                    for line in f:
+                        sline = bytestostr(line.strip().rstrip(b'\r\n').strip())
+                        props = sline.split("=", 1)
+                        if len(props)!=2:
+                            continue
+                        name = props[0].strip()
+                        value = props[1].strip()
+                        if name in ("blacklist", ):
+                            PREFS.setdefault(name, []).append(value)
+                        elif name in ("device-id", "device-name"):
+                            PREFS[name] = value
+            except Exception as e:
+                log.error("Error: cannot read cuda configuration file '%s':", conf_file)
+                log.error(" %s", e)
+    return PREFS
+
+def get_pref(name):
+    assert name in ("blacklist", "device-id", "device-name")
+    #ie: env_name("device-id")="XPRA_CUDA_DEVICE_ID"
+    env_name = "XPRA_CUDA_%s" % str(name).upper().replace("-", "_")
+    env_value = os.environ.get(env_name)
+    if env_value is not None:
+        return env_value
+    return get_prefs().get(name)
+
+
 DEVICES = None
 def init_all_devices():
     global DEVICES, DEVICE_INFO
     if DEVICES is not None:
-        return  DEVICES
+        return DEVICES
     log.info("CUDA initialization (this may take a few seconds)")
-    driver.init()
     DEVICES = []
     DEVICE_INFO = {}
+    try:
+        driver.init()
+    except Exception as e:
+        log.error("Error: cannot initialize CUDA")
+        log.error(" %s", e)
+        return DEVICES
     log("CUDA driver version=%s", driver.get_driver_version())
     ngpus = driver.Device.count()
     if ngpus==0:
         log.info("CUDA %s / PyCUDA %s, no devices found", ".".join([str(x) for x in driver.get_version()]), pycuda.VERSION_TEXT)
         return DEVICES
+    cuda_device_blacklist = get_pref("blacklist")
     da = driver.device_attribute
     cf = driver.ctx_flags
     for i in range(ngpus):
@@ -93,6 +142,12 @@ def init_all_devices():
         try:
             device = driver.Device(i)
             devinfo = device_info(device)
+            if cuda_device_blacklist and not IGNORE_BLACKLIST:
+                blacklisted = [x for x in cuda_device_blacklist if x and devinfo.find(x)>=0]
+                log("blacklisted(%s / %s)=%s", devinfo, cuda_device_blacklist, blacklisted)
+                if blacklisted:
+                    log.warn("Warning: device '%s' is blacklisted and will not be used", devinfo)
+                    continue
             log(" + testing device %s: %s", i, devinfo)
             DEVICE_INFO[i] = devinfo
             host_mem = device.get_attribute(da.CAN_MAP_HOST_MEMORY)
@@ -104,7 +159,7 @@ def init_all_devices():
                 log("   created context=%s", context)
                 log("   api version=%s", context.get_api_version())
                 free, total = driver.mem_get_info()
-                log("   memory: free=%sMB, total=%sMB",  int(free/1024/1024), int(total/1024/1024))
+                log("   memory: free=%sMB, total=%sMB",  int(free//1024//1024), int(total//1024//1024))
                 log("   multi-processors: %s, clock rate: %s", device.get_attribute(da.MULTIPROCESSOR_COUNT), device.get_attribute(da.CLOCK_RATE))
                 log("   max block sizes: (%s, %s, %s)", device.get_attribute(da.MAX_BLOCK_DIM_X), device.get_attribute(da.MAX_BLOCK_DIM_Y), device.get_attribute(da.MAX_BLOCK_DIM_Z))
                 log("   max grid sizes: (%s, %s, %s)", device.get_attribute(da.MAX_GRID_DIM_X), device.get_attribute(da.MAX_GRID_DIM_Y), device.get_attribute(da.MAX_GRID_DIM_Z))
@@ -120,8 +175,11 @@ def init_all_devices():
                     #so that the log output is bunched up together
                     log.info("CUDA %s / PyCUDA %s, found %s device%s:",
                              ".".join([str(x) for x in driver.get_version()]), pycuda.VERSION_TEXT, ngpus, engs(ngpus))
-                DEVICES.append(i)
-                log.info("  + %s (memory: %s%% free, compute: %s.%s)", device_info(device), 100*free/total, SMmajor, SMminor)
+                if SMmajor>=2:
+                    DEVICES.append(i)
+                else:
+                    log.info("  this device is too old!")
+                log.info("  + %s (memory: %s%% free, compute: %s.%s)", device_info(device), 100*free//total, SMmajor, SMminor)
             finally:
                 context.pop()
         except Exception as e:
@@ -143,9 +201,13 @@ def reset_state():
     DEVICES = None
 
 
-def select_device(preferred_device_id=-1, preferred_device_name=CUDA_DEVICE_NAME, min_compute=0):
-    if preferred_device_id<0 and CUDA_DEVICE_ID>=0:
-        preferred_device_id = CUDA_DEVICE_ID
+def select_device(preferred_device_id=-1, preferred_device_name=None, min_compute=0):
+    if preferred_device_name is None:
+        preferred_device_name = get_pref("device-name")
+    if preferred_device_id<0:
+        device_id = get_pref("device-id")
+        if device_id is not None and device_id>=0:
+            preferred_device_id = device_id
     devices = init_all_devices()
     global DEVICE_STATE
     free_pct = 0
@@ -278,19 +340,25 @@ def get_CUDA_function(device_id, function_name):
         log("get_CUDA_function(%s, %s) cubin file=%s", device_id, function_name, cubin_file)
         data = load_binary_file(cubin_file)
         if not data:
-            log.error("failed to load CUDA bin file %s", cubin_file)
+            log.error("Error: failed to load CUDA bin file '%s'", cubin_file)
             return None
         log(" loaded %s bytes", len(data))
         KERNELS[function_name] = data
     #now load from cubin:
-    start = time.time()
-    mod = driver.module_from_buffer(data)
+    start = monotonic_time()
+    try:
+        mod = driver.module_from_buffer(data)
+    except Exception as e:
+        log("module_from_buffer(%s)", data, exc_info=True)
+        log.error("Error: failed to load module from buffer for '%s'", function_name)
+        log.error(" %s", e)
+        return None
     log("get_CUDA_function(%s, %s) module=%s", device_id, function_name, mod)
     try:
         CUDA_function = mod.get_function(function_name)
     except driver.LogicError as e:
         raise Exception("failed to load '%s' from %s: %s" % (function_name, mod, e))
-    end = time.time()
+    end = monotonic_time()
     log("loading function %s from pre-compiled cubin took %.1fms", function_name, 1000.0*(end-start))
     return CUDA_function
 
@@ -300,13 +368,14 @@ def main():
     if "-v" in sys.argv or "--verbose" in sys.argv:
         log.enable_debug()
 
-    log.info("pycuda_info: %s" % get_pycuda_info())
-    log.info("cuda_info: %s" % get_cuda_info())
-
-    if sys.platform.startswith("win"):
-        print("\nPress Enter to close")
-        sys.stdin.readline()
-
+    from xpra.platform import program_context
+    with program_context("CUDA-Info", "CUDA Info"):
+        log.info("pycuda_info")
+        print_nested_dict(get_pycuda_info(), print_fn=log.info)
+        log.info("cuda_info")
+        print_nested_dict(get_cuda_info(), print_fn=log.info)
+        log.info("preferences:")
+        print_nested_dict(get_prefs(), print_fn=log.info)
 
 if __name__ == "__main__":
     main()
