@@ -1,14 +1,13 @@
 # This file is part of Xpra.
-# Copyright (C) 2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2014-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import time
-
 from xpra.log import Logger
 log = Logger("encoder", "proxy")
 
 from xpra.codecs.image_wrapper import ImageWrapper
+from xpra.os_util import memoryview_to_bytes, monotonic_time
 from collections import deque
 
 
@@ -67,7 +66,7 @@ class Encoder(object):
         if self.scaling!=(1,1):
             info["scaling"] = self.scaling
         #calculate fps:
-        now = time.time()
+        now = monotonic_time()
         last_time = now
         cut_off = now-10.0
         f = 0
@@ -123,7 +122,8 @@ class Encoder(object):
         assert image.get_planes()==ImageWrapper.PACKED, "invalid number of planes: %s" % image.get_planes()
         self.quality = quality
         self.speed = speed
-        pixels = str(image.get_pixels())
+        pixels = image.get_pixels()
+        assert pixels, "failed to get pixels from %s" % image
         #info used by proxy encoder:
         client_options = {
                 "proxy"     : True,
@@ -149,6 +149,6 @@ class Encoder(object):
         if self.scaling!=(1,1):
             client_options["scaling"] = self.scaling
         log("compress_image(%s, %s) returning %s bytes and options=%s", image, options, len(pixels), client_options)
-        self.last_frame_times.append(time.time())
+        self.last_frame_times.append(monotonic_time())
         self.frames += 1
-        return  pixels, client_options
+        return  memoryview_to_bytes(pixels[:]), client_options
