@@ -1,11 +1,11 @@
 #!/usr/bin/env python
 # coding=utf8
 # This file is part of Xpra.
-# Copyright (C) 2013, 2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2013 - 2016 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import sys, os
+import sys
 from threading import Lock
 from xpra.log import Logger
 log = Logger("codec", "video")
@@ -14,18 +14,18 @@ from xpra.codecs.loader import get_codec, get_codec_error
 from xpra.util import csv, engs
 
 
-NVENC_OPTIONS = [("nvenc%s" % v) for v in os.environ.get("XPRA_NVENC_VERSIONS", "4,5").split(",")]
-
 #the codec loader uses the names...
 #but we need the module name to be able to probe without loading the codec:
-CODEC_TO_MODULE = {"vpx"        : ["vpx"],
+CODEC_TO_MODULE = {
+                   "vpx"        : ["vpx"],
                    "x264"       : ["enc_x264"],
                    "x265"       : ["enc_x265"],
-                   "nvenc"      : NVENC_OPTIONS,
+                   "nvenc"      : ["nvenc"],
                    "swscale"    : ["csc_swscale"],
-                   "cython"     : ["csc_cython"],
-                   "opencl"     : ["csc_opencl"],
-                   "avcodec2"   : ["dec_avcodec2"]}
+                   "libyuv"     : ["csc_libyuv"],
+                   "avcodec2"   : ["dec_avcodec2"],
+                   "ffmpeg"     : ["enc_ffmpeg"],
+                   }
 
 def has_codec_module(module_name):
     top_module = "xpra.codecs.%s" % module_name
@@ -49,9 +49,9 @@ def try_import_modules(codec_names):
 
 #all the codecs we know about:
 #try to import the module that contains them (cheap check):
-ALL_VIDEO_ENCODER_OPTIONS = try_import_modules(["x264", "vpx", "x265", "nvenc"])
-ALL_CSC_MODULE_OPTIONS = try_import_modules(["swscale", "cython", "opencl"])
-NO_GFX_CSC_OPTIONS = [x for x in ALL_CSC_MODULE_OPTIONS if x not in ("opencl", )]
+ALL_VIDEO_ENCODER_OPTIONS = try_import_modules(["x264", "vpx", "x265", "nvenc", "ffmpeg"])
+ALL_CSC_MODULE_OPTIONS = try_import_modules(["swscale", "libyuv"])
+NO_GFX_CSC_OPTIONS = []
 ALL_VIDEO_DECODER_OPTIONS = try_import_modules(["avcodec2", "vpx"])
 
 PREFERRED_ENCODER_ORDER = ["nvenc", "x264", "vpx", "x265"]
@@ -67,7 +67,7 @@ log("video_helper: ALL_VIDEO_DECODER_OPTIONS=%s", ALL_VIDEO_DECODER_OPTIONS)
 
 def get_encoder_module_names(x):
     if x=="nvenc":
-        return NVENC_OPTIONS
+        return ["nvenc"]
     elif x.find("enc")>=0:
         return [x]              #ie: "nvenc" or "enc_vpx"
     return ["enc_"+x]           #ie: "enc_x264"
@@ -139,7 +139,7 @@ class VideoHelper(object):
     def set_modules(self, video_encoders=[], csc_modules=[], video_decoders=[]):
         assert not self._initialized, "too late to set modules, the helper is already initialized!"
         def filt(name, inlist, all_list):
-            notfound = [x for x in inlist if x not in all_list]
+            notfound = [x for x in inlist if x and x not in all_list]
             if notfound:
                 log.warn("ignoring unknown %s: %s", name, ", ".join(notfound))
             return [x for x in inlist if x in all_list]
@@ -187,18 +187,21 @@ class VideoHelper(object):
 
     def get_info(self):
         d = {}
+        einfo = d.setdefault("encoding", {})
+        dinfo = d.setdefault("decoding", {})
+        cinfo = d.setdefault("csc", {})
         for encoding, encoder_specs in self._video_encoder_specs.items():
             for in_csc, specs in encoder_specs.items():
                 for spec in specs:
-                    d.setdefault("encoding.%s_to_%s" % (in_csc, encoding), []).append(spec.codec_type)
+                    einfo.setdefault("%s_to_%s" % (in_csc, encoding), []).append(spec.codec_type)
         for in_csc, specs in self._csc_encoder_specs.items():
             for out_csc, specs in specs.items():
-                d["csc.%s_to_%s" % (in_csc, out_csc)] = [spec.codec_type for spec in specs]
+                cinfo["%s_to_%s" % (in_csc, out_csc)] = [spec.codec_type for spec in specs]
         for encoding, decoder_specs in self._video_decoder_specs.items():
             for out_csc, decoders in decoder_specs.items():
                 for decoder in decoders:
                     decoder_name, _ = decoder
-                    d.setdefault("decoding.%s_to_%s" % (encoding, out_csc), []).append(decoder_name)
+                    dinfo.setdefault("%s_to_%s" % (encoding, out_csc), []).append(decoder_name)
         def modstatus(x, def_list, active_list):
             #the module is present
             if x in active_list:
@@ -207,10 +210,12 @@ class VideoHelper(object):
                 return "disabled"
             else:
                 return "not found"
+        venc = einfo.setdefault("video-encoder", {})
         for x in ALL_VIDEO_ENCODER_OPTIONS:
-            d["encoding.video-encoder.%s" % x] = modstatus(x, get_DEFAULT_VIDEO_ENCODERS(), self.video_encoders)
+            venc["%s" % x] = modstatus(x, get_DEFAULT_VIDEO_ENCODERS(), self.video_encoders)
+        cscm = einfo.setdefault("csc-module", {})
         for x in ALL_CSC_MODULE_OPTIONS:
-            d["encoding.csc-module.%s" % x] = modstatus(x, get_DEFAULT_CSC_MODULES(), self.csc_modules)
+            cscm["%s" % x] = modstatus(x, get_DEFAULT_CSC_MODULES(), self.csc_modules)
         return d
 
     def init(self):
@@ -427,10 +432,10 @@ def getVideoHelper():
 
 def main():
     from xpra.codecs.loader import log as loader_log, load_codecs
+    from xpra.util import print_nested_dict
     from xpra.log import enable_color
-    from xpra.platform import init, clean
-    try:
-        init("Video Helper")
+    from xpra.platform import program_context
+    with program_context("Video Helper"):
         enable_color()
         if "-v" in sys.argv or "--verbose" in sys.argv:
             loader_log.enable_debug()
@@ -439,15 +444,8 @@ def main():
         vh = getVideoHelper()
         vh.set_modules(ALL_VIDEO_ENCODER_OPTIONS, ALL_CSC_MODULE_OPTIONS, ALL_VIDEO_DECODER_OPTIONS)
         vh.init()
-        log.info("VideoHelper.get_info():")
         info = vh.get_info()
-        for k in sorted(info.keys()):
-            v = info.get(k)
-            if type(v) in (list, tuple):
-                v = csv(v)
-            log.info("%s=%s", k, v)
-    finally:
-        clean()
+        print_nested_dict(info)
 
 
 if __name__ == "__main__":
