@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2012-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -9,7 +9,7 @@ with python2, we try to import pygtk (gobject/gtk/gdk) before trying gobject int
 Once we have imported something, stick to that version from then on for all other imports.
 """
 
-import sys, os
+import sys
 
 __all__ = ["is_gtk3", "get_xid", "import_gobject", "import_gtk", "import_gdk", "import_pango", "import_glib", "import_pixbufloader"]
 
@@ -47,7 +47,8 @@ def _try_import(import_method_gtk3, import_method_gtk2):
     return imported
 
 def try_import_GdkX11():
-    if os.name=="posix" and not sys.platform.startswith("darwin"):
+    from xpra.os_util import OSX, POSIX
+    if POSIX and not OSX:
         #try to ensure that we can call get_xid() on Gdk windows later,
         #this is a workaround for this GTK bug:
         #https://bugzilla.gnome.org/show_bug.cgi?id=656314
@@ -71,6 +72,12 @@ def import_gobject2():
     return gobject
 def import_gobject3():
     from gi.repository import GObject               #@UnresolvedImport
+    #silence a GTK3 warning about threads_init not beeing needed:
+    v = getattr(GObject, "pygobject_version", (0))
+    if v>=(3,10):
+        def noop(*args):
+            pass
+        GObject.threads_init = noop
     return GObject
 def import_gobject():
     return  _try_import(import_gobject3, import_gobject2)
@@ -102,12 +109,25 @@ def import_gdk2():
     from gtk import gdk
     return gdk
 def import_gdk3():
+    import gi
+    gi.require_version('Gdk', '3.0')
     from gi.repository import Gdk                   #@UnresolvedImport
     try_import_GdkX11()
     return Gdk
 def import_gdk():
     return  _try_import(import_gdk3, import_gdk2)
 
+def import_pixbuf2():
+    from gtk.gdk import Pixbuf
+    return Pixbuf
+def import_pixbuf3():
+    import gi
+    gi.require_version('GdkPixbuf', '2.0')
+    from gi.repository import GdkPixbuf             #@UnresolvedImport
+    return GdkPixbuf
+def import_pixbuf():
+    return  _try_import(import_pixbuf3, import_pixbuf2)
+
 def import_pixbufloader2():
     from gtk.gdk import PixbufLoader
     return PixbufLoader
