@@ -1,74 +1,103 @@
 # coding=utf8
 # This file is part of Xpra.
-# Copyright (C) 2011-2015 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import sys
 import os.path
-
 import array
-from xpra.os_util import strtobytes
-from xpra.gtk_common.gobject_compat import import_gtk, import_gdk, import_pixbufloader, import_pango, import_cairo, import_gobject, is_gtk3
+from xpra.util import iround
+from xpra.os_util import strtobytes, WIN32
+from xpra.gtk_common.gobject_compat import import_gtk, import_gdk, import_glib, import_pixbufloader, import_pango, import_cairo, import_gobject, import_pixbuf, is_gtk3
 gtk     = import_gtk()
 gdk     = import_gdk()
 pango   = import_pango()
 cairo   = import_cairo()
 gobject = import_gobject()
 PixbufLoader = import_pixbufloader()
+Pixbuf = import_pixbuf()
 
 from xpra.log import Logger
 log = Logger("gtk", "util")
+traylog = Logger("gtk", "tray")
+screenlog = Logger("gtk", "screen")
 
 
-WIN32 = sys.platform.startswith("win")
+SHOW_ALL_VISUALS = False
 
 GTK_VERSION_INFO = {}
 def get_gtk_version_info():
     #update props given:
     global GTK_VERSION_INFO
+    def av(k, v):
+        GTK_VERSION_INFO.setdefault(k, {})["version"] = v
+    def V(k, module, *fields):
+        for field in fields:
+            v = getattr(module, field, None)
+            if v is not None:
+                av(k, v)
+                return True
+        return False
+
     if not GTK_VERSION_INFO:
-        if hasattr(gtk, "pygtk_version"):
-            GTK_VERSION_INFO["pygtk.version"] = gtk.pygtk_version
-        if hasattr(gtk, "gtk_version"):
-            #GTK2:
-            GTK_VERSION_INFO["gtk.version"] = gtk.gtk_version
-        elif hasattr(gtk, "_version"):
-            #GTK3:
-            GTK_VERSION_INFO["gtk.version"] = gtk._version
-        if hasattr(gdk, "__version__"):
-            #GTK2:
-            GTK_VERSION_INFO["gdk.version"] = gdk.__version__
-        elif hasattr(gdk, "_version"):
-            #GTK3:
-            GTK_VERSION_INFO["gdk.version"] = gdk._version
+        V("gobject",    gobject,    "pygobject_version")
+
         if is_gtk3():
+            #this isn't the actual version, (only shows as "3.0")
+            #but still better than nothing:
+            import gi
+            V("gi",         gi,         "__version__")
+            V("gtk",        gtk,        "_version")
+            V("gdk",        gdk,        "_version")
+            V("gobject",    gobject,    "_version")
+            V("pixbuf",     Pixbuf,     "_version")
+
+            av("pygtk", "n/a")
+            V("pixbuf",     Pixbuf,     "PIXBUF_VERSION")
             try:
-                import gi
-                GTK_VERSION_INFO["gi.version"] = gi.__version__
+                v = [getattr(gtk, x) for x in ["MAJOR_VERSION", "MICRO_VERSION", "MINOR_VERSION"]]
+                av("gtk", ".".join(str(x) for x in v))
             except:
                 pass
-        if hasattr(gobject, "pygobject_version"):
-            GTK_VERSION_INFO["gobject.version"] = gobject.pygobject_version
-        elif hasattr(gobject, "_version"):
-            GTK_VERSION_INFO["gobject.version"] = gobject._version
-        if hasattr(cairo, "version"):
-            GTK_VERSION_INFO["cairo.version"] = cairo.version
-        if hasattr(pango, "version_string"):
-            GTK_VERSION_INFO["pango.version"] = pango.version_string()
-        try:
-            import glib
-            GTK_VERSION_INFO["glib.version"] = glib.glib_version
-            GTK_VERSION_INFO["pyglib.version"] = glib.pyglib_version
-        except:
-            pass
+        else:
+            V("pygtk",      gtk,        "pygtk_version")
+            V("gtk",        gtk,        "gtk_version")
+            V("gdk",        gdk,        "__version__")
+
+        #from here on, the code is the same for both GTK2 and GTK3, hooray:
+        vfn = getattr(cairo, "cairo_version_string", None)
+        if vfn:
+            av("cairo", vfn())
+        vfn = getattr(pango, "version_string")
+        if vfn:
+            av("pango", vfn())
+        glib = import_glib()
+        V("glib",       glib,       "glib_version")
+        V("pyglib",     glib,       "pyglib_version")
     return GTK_VERSION_INFO.copy()
 
 
+def popup_menu_workaround(*args):
+    #only implemented with GTK2 on win32 below
+    pass
+
+
 if is_gtk3():
     def is_realized(widget):
         return widget.get_realized()
 
+    def get_pixbuf_from_data(rgb_data, has_alpha, w, h, rowstride):
+        data = array.array('B', strtobytes(rgb_data))
+        return GdkPixbuf.Pixbuf.new_from_data(data, GdkPixbuf.Colorspace.RGB,
+                                         True, 8, w, h, rowstride,
+                                         None, None)
+
+    def get_preferred_size(widget):
+        #ignore "min", we only care about "natural":
+        _, w = widget.get_preferred_width()
+        _, h = widget.get_preferred_height()
+        return w, h
+
     def color_parse(*args):
         ok, v = gdk.Color.parse(*args)
         if not ok:
@@ -145,7 +174,9 @@ if is_gtk3():
     POINTER_MOTION_MASK         = gdk.EventMask.POINTER_MOTION_MASK
     POINTER_MOTION_HINT_MASK    = gdk.EventMask.POINTER_MOTION_HINT_MASK
     MESSAGE_INFO    = gtk.MessageType.INFO
+    MESSAGE_QUESTION= gtk.MessageType.QUESTION
     BUTTONS_CLOSE   = gtk.ButtonsType.CLOSE
+    BUTTONS_NONE    = gtk.ButtonsType.NONE
     DIALOG_DESTROY_WITH_PARENT  = 0
 
     WINDOW_POPUP    = gtk.WindowType.POPUP
@@ -160,6 +191,11 @@ if is_gtk3():
     TRUE_COLOR      = gdk.VisualType.TRUE_COLOR
     DIRECT_COLOR    = gdk.VisualType.DIRECT_COLOR
 
+    SCROLL_UP       = gdk.ScrollDirection.UP
+    SCROLL_DOWN     = gdk.ScrollDirection.DOWN
+    SCROLL_LEFT     = gdk.ScrollDirection.LEFT
+    SCROLL_RIGHT    = gdk.ScrollDirection.RIGHT
+
     mt = gdk.ModifierType
     SHIFT_MASK      = mt.SHIFT_MASK
     LOCK_MASK       = mt.LOCK_MASK
@@ -179,6 +215,7 @@ if is_gtk3():
     del mt
 
     em = gdk.EventMask
+    KEY_PRESS_MASK = em.KEY_PRESS_MASK
     WINDOW_EVENT_MASK = em.STRUCTURE_MASK | em.KEY_PRESS_MASK | em.KEY_RELEASE_MASK \
             | em.POINTER_MOTION_MASK | em.BUTTON_PRESS_MASK | em.BUTTON_RELEASE_MASK \
             | em.PROPERTY_CHANGE_MASK | em.SCROLL_MASK
@@ -201,6 +238,12 @@ if is_gtk3():
                 "DND"           : gdk.WindowTypeHint.DND
                 }
 
+    GRAB_SUCCESS        = gdk.GrabStatus.SUCCESS
+    ALREADY_GRABBED     = gdk.GrabStatus.ALREADY_GRABBED
+    GRAB_INVALID_TIME   = gdk.GrabStatus.INVALID_TIME
+    GRAB_NOT_VIEWABLE   = gdk.GrabStatus.NOT_VIEWABLE
+    GRAB_FROZEN         = gdk.GrabStatus.FROZEN
+
     from gi.repository.Gtk import Clipboard     #@UnresolvedImport
     CLIPBOARD_SELECTION = {}
     #gtk2: uses strings:
@@ -233,10 +276,18 @@ if is_gtk3():
             gdk.threads_leave()
 
 else:
+    def get_pixbuf_from_data(rgb_data, has_alpha, w, h, rowstride):
+        return gdk.pixbuf_new_from_data(rgb_data, gdk.COLORSPACE_RGB, has_alpha, 8, w, h, rowstride)
+
+    def get_preferred_size(widget):
+        return widget.size_request()
+
     def get_xwindow(w):
         return w.xid
     #gtk2:
     if gtk.gtk_version<(2,18):
+        raise Exception("your version of GTK is too old: %s" % str(gtk.gtk_version))
+    if gtk.pygtk_version<(2,16):
         raise Exception("your version of PyGTK is too old: %s" % str(gtk.pygtk_version))
 
     if not hasattr(gtk.Widget, "get_realized"):
@@ -292,7 +343,9 @@ else:
     POINTER_MOTION_MASK         = gdk.POINTER_MOTION_MASK
     POINTER_MOTION_HINT_MASK    = gdk.POINTER_MOTION_HINT_MASK
     MESSAGE_INFO    = gtk.MESSAGE_INFO
+    MESSAGE_QUESTION= gtk.MESSAGE_QUESTION
     BUTTONS_CLOSE   = gtk.BUTTONS_CLOSE
+    BUTTONS_NONE    = gtk.BUTTONS_NONE
     DIALOG_DESTROY_WITH_PARENT = gtk.DIALOG_DESTROY_WITH_PARENT
 
     WINDOW_POPUP    = gtk.WINDOW_POPUP
@@ -307,6 +360,11 @@ else:
     TRUE_COLOR      = gdk.VISUAL_TRUE_COLOR
     DIRECT_COLOR    = gdk.VISUAL_DIRECT_COLOR
 
+    SCROLL_UP       = gdk.SCROLL_UP
+    SCROLL_DOWN     = gdk.SCROLL_DOWN
+    SCROLL_LEFT     = gdk.SCROLL_LEFT
+    SCROLL_RIGHT    = gdk.SCROLL_RIGHT
+
     SHIFT_MASK      = gdk.SHIFT_MASK
     LOCK_MASK       = gdk.LOCK_MASK
     META_MASK       = gdk.META_MASK
@@ -323,6 +381,7 @@ else:
                    gdk.BUTTON4_MASK : 4,
                    gdk.BUTTON5_MASK : 5}
 
+    KEY_PRESS_MASK = gdk.KEY_PRESS_MASK
     WINDOW_EVENT_MASK = gdk.STRUCTURE_MASK | gdk.KEY_PRESS_MASK | gdk.KEY_RELEASE_MASK \
             | gdk.POINTER_MOTION_MASK | gdk.BUTTON_PRESS_MASK | gdk.BUTTON_RELEASE_MASK \
             | gdk.PROPERTY_CHANGE_MASK
@@ -344,6 +403,12 @@ else:
                 "DND"           : gdk.WINDOW_TYPE_HINT_DND
                 }
 
+    GRAB_SUCCESS        = gdk.GRAB_SUCCESS
+    ALREADY_GRABBED     = gdk.GRAB_ALREADY_GRABBED
+    GRAB_INVALID_TIME   = gdk.GRAB_INVALID_TIME
+    GRAB_NOT_VIEWABLE   = gdk.GRAB_NOT_VIEWABLE
+    GRAB_FROZEN         = gdk.GRAB_FROZEN
+
     OptionMenu  = gtk.OptionMenu
 
     def GetClipboard(selection):
@@ -359,6 +424,277 @@ else:
             finally:
                 gdk.threads_leave()
 
+    if WIN32:
+        mouse_in_tray_menu_counter = 0
+        mouse_in_tray_menu = False
+        OUTSIDE_TRAY_TIMEOUT = 500
+        def popup_menu_workaround(menu, close_cb):
+            """ MS Windows does not automatically close the popup menu when we click outside it
+                so we workaround it by using a timer and closing the menu when the mouse
+                has stayed outside it for more than OUTSIDE_TRAY_TIMEOUT (0.5s).
+                This code must be added to all the sub-menus of the popup menu too!
+            """
+            global mouse_in_tray_menu, mouse_in_tray_menu_counter
+            def enter_menu(*args):
+                global mouse_in_tray_menu, mouse_in_tray_menu_counter
+                traylog("mouse_in_tray_menu=%s", mouse_in_tray_menu)
+                mouse_in_tray_menu_counter += 1
+                mouse_in_tray_menu = True
+            def leave_menu(*args):
+                global mouse_in_tray_menu, mouse_in_tray_menu_counter
+                traylog("mouse_in_tray_menu=%s", mouse_in_tray_menu)
+                mouse_in_tray_menu_counter += 1
+                mouse_in_tray_menu = False
+                def check_menu_left(expected_counter):
+                    if mouse_in_tray_menu:
+                        return    False
+                    if expected_counter!=mouse_in_tray_menu_counter:
+                        return    False            #counter has changed
+                    close_cb()
+                gobject.timeout_add(OUTSIDE_TRAY_TIMEOUT, check_menu_left, mouse_in_tray_menu_counter)
+            mouse_in_tray_menu_counter = 0
+            mouse_in_tray_menu = False
+            traylog("popup_menu_workaround: adding events callbacks")
+            menu.connect("enter-notify-event", enter_menu)
+            menu.connect("leave-notify-event", leave_menu)
+        def gtk_main():
+            gtk.main()
+
+GRAB_STATUS_STRING = {
+                      GRAB_SUCCESS          : "SUCCESS",
+                      ALREADY_GRABBED       : "ALREADY_GRABBED",
+                      GRAB_INVALID_TIME     : "INVALID_TIME",
+                      GRAB_NOT_VIEWABLE     : "NOT_VIEWABLE",
+                      GRAB_FROZEN           : "FROZEN",
+                      }
+
+
+class TrayCheckMenuItem(gtk.CheckMenuItem):
+    """ We add a button handler to catch clicks that somehow do not
+        trigger the "toggled" signal on some platforms (win32?) when we
+        show the tray menu with a right click and click on the item with the left click.
+        (or the other way around?)
+    """
+    def __init__(self, label, tooltip=None):
+        gtk.CheckMenuItem.__init__(self, label)
+        self.label = label
+        if tooltip:
+            self.set_tooltip_text(tooltip)
+        self.add_events(BUTTON_PRESS_MASK)
+        self.connect("button-release-event", self.on_button_release_event)
+
+    def on_button_release_event(self, *args):
+        traylog("TrayCheckMenuItem.on_button_release_event(%s) label=%s", args, self.label)
+        self.active_state = self.get_active()
+        def recheck():
+            traylog("TrayCheckMenuItem: recheck() active_state=%s, get_active()=%s", self.active_state, self.get_active())
+            state = self.active_state
+            self.active_state = None
+            if state is not None and state==self.get_active():
+                #toggle did not fire after the button release, so force it:
+                self.set_active(not state)
+        gobject.idle_add(recheck)
+
+CheckMenuItemClass = gtk.CheckMenuItem
+def CheckMenuItem(*args, **kwargs):
+    global CheckMenuItemClass
+    return CheckMenuItemClass(*args, **kwargs)
+
+def set_use_tray_workaround(enabled):
+    global CheckMenuItemClass
+    if enabled:
+        CheckMenuItemClass = TrayCheckMenuItem
+    else:
+        CheckMenuItemClass = gtk.CheckMenuItem
+set_use_tray_workaround(True)
+
+
+
+def get_screens_info():
+    display = display_get_default()
+    info = {}
+    for i in range(display.get_n_screens()):
+        screen = display.get_screen(i)
+        info[i] = get_screen_info(display, screen)
+    return info
+
+def get_screen_sizes(xscale=1, yscale=1):
+    from xpra.platform.gui import get_workarea, get_workareas
+    def xs(v):
+        return iround(v/xscale)
+    def ys(v):
+        return iround(v/yscale)
+    def swork(*workarea):
+        return xs(workarea[0]), ys(workarea[1]), xs(workarea[2]), ys(workarea[3])
+    display = display_get_default()
+    i=0
+    screen_sizes = []
+    n_screens = display.get_n_screens()
+    screenlog("get_screen_sizes(%f, %f) found %s screens", xscale, yscale, n_screens)
+    while i<n_screens:
+        screen = display.get_screen(i)
+        j = 0
+        monitors = []
+        workareas = []
+        #native "get_workareas()" is only valid for a single screen (but describes all the monitors)
+        #and it is only implemented on win32 right now
+        #other platforms only implement "get_workarea()" instead, which is reported against the screen
+        n_monitors = screen.get_n_monitors()
+        screenlog(" screen %s has %s monitors", i, n_monitors)
+        if n_screens==1:
+            workareas = get_workareas()
+            if workareas and len(workareas)!=n_monitors:
+                screenlog(" workareas: %s", workareas)
+                screenlog(" number of monitors does not match number of workareas!")
+                workareas = []
+        while j<screen.get_n_monitors():
+            geom = screen.get_monitor_geometry(j)
+            plug_name = ""
+            if hasattr(screen, "get_monitor_plug_name"):
+                plug_name = screen.get_monitor_plug_name(j) or ""
+            wmm = -1
+            if hasattr(screen, "get_monitor_width_mm"):
+                wmm = screen.get_monitor_width_mm(j)
+            hmm = -1
+            if hasattr(screen, "get_monitor_height_mm"):
+                hmm = screen.get_monitor_height_mm(j)
+            monitor = [plug_name, xs(geom.x), ys(geom.y), xs(geom.width), ys(geom.height), wmm, hmm]
+            screenlog(" monitor %s: %s", j, monitor)
+            if workareas:
+                w = workareas[j]
+                monitor += list(swork(*w))
+            monitors.append(tuple(monitor))
+            j += 1
+        work_x, work_y, work_width, work_height = swork(0, 0, screen.get_width(), screen.get_height())
+        workarea = get_workarea()
+        if workarea:
+            work_x, work_y, work_width, work_height = swork(*workarea)
+        screenlog(" workarea=%s", workarea)
+        item = (screen.make_display_name(), xs(screen.get_width()), ys(screen.get_height()),
+                    screen.get_width_mm(), screen.get_height_mm(),
+                    monitors,
+                    work_x, work_y, work_width, work_height)
+        screenlog(" screen %s: %s", i, item)
+        screen_sizes.append(item)
+        i += 1
+    return screen_sizes
+
+def get_screen_info(display, screen):
+    info = {}
+    try:
+        w = screen.get_root_window()
+        info["root"] = w.get_geometry()
+    except:
+        pass
+    info["name"] = screen.make_display_name()
+    for x in ("width", "height", "width_mm", "height_mm", "resolution", "primary_monitor"):
+        fn = getattr(screen, "get_"+x)
+        try:
+            info[x] = int(fn())
+        except:
+            pass
+    info["monitors"] = screen.get_n_monitors()
+    m_info = info.setdefault("monitor", {})
+    for i in range(screen.get_n_monitors()):
+        m_info[i] = get_monitor_info(display, screen, i)
+    try:
+        import cairo
+        fo = screen.get_font_options()
+        #win32 and osx return nothing here...
+        if fo:
+            fontoptions = info.setdefault("fontoptions", {})
+            for x,vdict in {
+                            "antialias"     : {cairo.ANTIALIAS_DEFAULT      : "default", cairo.ANTIALIAS_NONE       : "none",   cairo.ANTIALIAS_GRAY        : "gray",   cairo.ANTIALIAS_SUBPIXEL    : "subpixel"},
+                            "hint_metrics"  : {cairo.HINT_METRICS_DEFAULT   : "default", cairo.HINT_METRICS_OFF     : "off",    cairo.HINT_METRICS_ON       : "on"},
+                            "hint_style"    : {cairo.HINT_STYLE_DEFAULT     : "default", cairo.HINT_STYLE_NONE      : "none",   cairo.HINT_STYLE_SLIGHT     : "slight", cairo.HINT_STYLE_MEDIUM     : "medium", cairo.HINT_STYLE_FULL       : "full"},
+                            "subpixel_order": {cairo.SUBPIXEL_ORDER_DEFAULT : "default", cairo.SUBPIXEL_ORDER_RGB   : "RGB",    cairo.SUBPIXEL_ORDER_BGR    : "BGR",    cairo.SUBPIXEL_ORDER_VRGB   : "VRGB",   cairo.SUBPIXEL_ORDER_VBGR   : "VBGR"},
+                            }.items():
+                fn = getattr(fo, "get_"+x)
+                val = fn()
+                fontoptions[x] = vdict.get(val, val)
+    except:
+        pass
+    vinfo = info.setdefault("visual", {})
+    def visual(name, v):
+        if not v:
+            return
+        for x, vdict in {"bits_per_rgb" : {},
+                         "byte_order"   : {LSB_FIRST    : "LSB", MSB_FIRST  : "MSB"},
+                         "colormap_size": {},
+                         "depth"        : {},
+                         "red_pixel_details"    : {},
+                         "green_pixel_details"  : {},
+                         "blue_pixel_details"   : {},
+                         "visual_type"  : {STATIC_GRAY : "STATIC_GRAY", GRAYSCALE : "GRAYSCALE",  STATIC_COLOR : "STATIC_COLOR", PSEUDO_COLOR : "PSEUDO_COLOR", TRUE_COLOR : "TRUE_COLOR", DIRECT_COLOR : "DIRECT_COLOR"},
+                         }.items():
+            val = None
+            try:
+                val = getattr(v, x.replace("visual_"))  #ugly workaround for "visual_type" -> "type" for GTK2...
+            except:
+                try:
+                    fn = getattr(v, "get_"+x)
+                    val = fn()
+                except:
+                    pass
+            if val is not None:
+                vinfo.setdefault(name, {})[x] = vdict.get(val, val)
+    try:
+        visual("rgb", screen.get_rgb_visual())
+    except:
+        pass    #not in gtk3?
+    visual("rgba", screen.get_rgba_visual())
+    visual("system_visual", screen.get_system_visual())
+    if SHOW_ALL_VISUALS:
+        for i, v in enumerate(screen.list_visuals()):
+            visual(i, v)
+    #gtk.settings
+    if is_gtk3():
+        def get_setting(key):
+            #try string first, then int
+            for t in (gobject.TYPE_STRING, gobject.TYPE_INT):
+                v = gobject.Value()
+                v.init(t)
+                if screen.get_setting(key, v):
+                    return v.get_value()
+            return None
+    else:
+        settings = gtk.settings_get_for_screen(screen)
+        def get_setting(key):
+            return settings.get_property(key)
+    sinfo = info.setdefault("settings", {})
+    try:
+        for x in ("antialias", "dpi", "hinting", "hintstyle", "rgba"):
+            try:
+                v = get_setting("gtk-xft-"+x)
+            except:
+                continue
+            if v is None:
+                v = ""
+            sinfo[x] = v
+    except:
+        pass
+    return info
+
+def get_monitor_info(display, screen, i):
+    info = {}
+    geom = screen.get_monitor_geometry(i)
+    for x in ("x", "y", "width", "height"):
+        info[x] = getattr(geom, x)
+    if hasattr(screen, "get_monitor_plug_name"):
+        info["plug_name"] = screen.get_monitor_plug_name(i) or ""
+    for x in ("scale_factor", "width_mm", "height_mm"):
+        try:
+            fn = getattr(screen, "get_monitor_"+x)
+            info[x] = int(fn(i))
+        except:
+            pass
+    if hasattr(screen, "get_monitor_workarea"): #GTK3.4:
+        rectangle = screen.get_monitor_workarea(i)
+        workarea_info = info.setdefault("workarea", {})
+        for x in ("x", "y", "width", "height"):
+            workarea_info[x] = getattr(rectangle, x)
+    return info
+
 
 def get_display_info():
     display = display_get_default()
@@ -373,159 +709,94 @@ def get_display_info():
             "maximal_cursor_size"   : display.get_maximal_cursor_size(),
             "pointer_is_grabbed"    : display.pointer_is_grabbed(),
             }
+    sinfo = info.setdefault("supports", {})
     for x in ("composite", "cursor_alpha", "cursor_color", "selection_notification", "clipboard_persistence", "shapes"):
         f = "supports_"+x
         if hasattr(display, f):
             fn = getattr(display, f)
-            info[x]  = fn()
-    for i in range(display.get_n_screens()):
-        screen = display.get_screen(i)
-        sk = "screen[%s]" % i
-        try:
-            w = screen.get_root_window()
-            info[sk+".root"] = w.get_geometry()
-        except:
-            pass
-        info[sk+".name"] = screen.make_display_name()
-        for x in ("width", "height", "width_mm", "height_mm", "resolution", "primary_monitor"):
-            fn = getattr(screen, "get_"+x)
-            try:
-                info[sk+"."+x] = int(fn())
-            except:
-                pass
-        info[sk+".monitors"] = screen.get_n_monitors()
-        for j in range(screen.get_n_monitors()):
-            mk = "screen[%s].monitor[%s]" % (i, j)
-            geom = screen.get_monitor_geometry(j)
-            for x in ("x", "y", "width", "height"):
-                info[mk+"."+x] = getattr(geom, x)
-            if hasattr(screen, "get_monitor_plug_name"):
-                info[mk+".plug_name"] = screen.get_monitor_plug_name(j) or ""
-            for x in ("scale_factor", "width_mm", "height_mm"):
-                try:
-                    fn = getattr(screen, "get_monitor_"+x)
-                    info[mk+"."+x] = int(fn(j))
-                except:
-                    pass
-            if hasattr(screen, "get_monitor_workarea"): #GTK3.4:
-                rectangle = screen.get_monitor_workarea(j)
-                for x in ("x", "y", "width", "height"):
-                    info[mk+".workarea."+x] = getattr(rectangle, x)
-        try:
-            import cairo
-            fo = screen.get_font_options()
-            #win32 and osx return nothing here...
-            if fo:
-                fk = sk+".fontoptions"
-                for x,vdict in {
-                                "antialias"     : {cairo.ANTIALIAS_DEFAULT      : "default", cairo.ANTIALIAS_NONE       : "none",   cairo.ANTIALIAS_GRAY        : "gray",   cairo.ANTIALIAS_SUBPIXEL    : "subpixel"},
-                                "hint_metrics"  : {cairo.HINT_METRICS_DEFAULT   : "default", cairo.HINT_METRICS_OFF     : "off",    cairo.HINT_METRICS_ON       : "on"},
-                                "hint_style"    : {cairo.HINT_STYLE_DEFAULT     : "default", cairo.HINT_STYLE_NONE      : "none",   cairo.HINT_STYLE_SLIGHT     : "slight", cairo.HINT_STYLE_MEDIUM     : "medium", cairo.HINT_STYLE_FULL       : "full"},
-                                "subpixel_order": {cairo.SUBPIXEL_ORDER_DEFAULT : "default", cairo.SUBPIXEL_ORDER_RGB   : "RGB",    cairo.SUBPIXEL_ORDER_BGR    : "BGR",    cairo.SUBPIXEL_ORDER_VRGB   : "VRGB",   cairo.SUBPIXEL_ORDER_VBGR   : "VBGR"},
-                                }.items():
-                    fn = getattr(fo, "get_"+x)
-                    val = fn()
-                    info[fk+"."+x] = vdict.get(val, val)
-        except:
-            pass
-        def visual(k, v):
-            if not v:
-                return
-            vk = sk+"."+k       #ie: screen[0].rgb
-            for x, vdict in {"bits_per_rgb" : {},
-                             "byte_order"   : {LSB_FIRST    : "LSB", MSB_FIRST  : "MSB"},
-                             "colormap_size": {},
-                             "depth"        : {},
-                             "red_pixel_details"    : {},
-                             "green_pixel_details"  : {},
-                             "blue_pixel_details"   : {},
-                             "visual_type"  : {STATIC_GRAY : "STATIC_GRAY", GRAYSCALE : "GRAYSCALE",  STATIC_COLOR : "STATIC_COLOR", PSEUDO_COLOR : "PSEUDO_COLOR", TRUE_COLOR : "TRUE_COLOR", DIRECT_COLOR : "DIRECT_COLOR"},
-                             }.items():
-                val = None
-                try:
-                    val = getattr(v, x.replace("visual_"))  #ugly workaround for "visual_type" -> "type" for GTK2...
-                except:
-                    try:
-                        fn = getattr(v, "get_"+x)
-                        val = fn()
-                    except:
-                        pass
-                if val is not None:
-                    info[vk+"."+x] = vdict.get(val, val)
-        try:
-            visual("rgb", screen.get_rgb_visual())
-        except:
-            pass    #not in gtk3?
-        visual("rgba", screen.get_rgba_visual())
-        visual("system_visual", screen.get_system_visual())
-        visuals = screen.list_visuals()
-        info[sk+".visuals"] = len(visuals)
-        #gtk.settings
-        if is_gtk3():
-            def get_setting(key):
-                #try string first, then int
-                for t in (gobject.TYPE_STRING, gobject.TYPE_INT):
-                    v = gobject.Value()
-                    v.init(t)
-                    if screen.get_setting(key, v):
-                        return v.get_value()
-                return None
-        else:
-            settings = gtk.settings_get_for_screen(screen)
-            def get_setting(key):
-                return settings.get_property(key)
-        try:
-            ssk = "%s.settings" % sk
-            for x in ("antialias", "dpi", "hinting", "hintstyle", "rgba"):
-                try:
-                    v = get_setting("gtk-xft-"+x)
-                except:
-                    continue
-                if v is None:
-                    v = ""
-                info[ssk+"."+x] = v
-        except:
-            pass
+            sinfo[x]  = fn()
+    info["screens"] = get_screens_info()
     if is_gtk3():
         dm = display.get_device_manager()
         for dt, name in {gdk.DeviceType.MASTER  : "master",
                          gdk.DeviceType.SLAVE   : "slave",
                          gdk.DeviceType.FLOATING: "floating"}.items():
-            dk = "devices.%s" % name
+            dinfo = info.setdefault("device", {})
+            dtinfo = dinfo.setdefault(name, {})
             devices = dm.list_devices(dt)
-            info[dk] = len(devices)
             for i, d in enumerate(devices):
-                info[dk+"[%s]" % i] = d.get_name()
+                dtinfo[i] = d.get_name()
     else:
         devices = display.list_devices()
-        info["devices"] = len(devices)
         for i, d in enumerate(devices):
-            info["device[%s]" % i] = d.get_name()
+            dinfo = info.setdefault("device", {}).setdefault(i, {})
+            dinfo[""] = d.get_name()
+            AXES_STR = {
+                        gdk.AXIS_IGNORE         : "IGNORE",
+                        gdk.AXIS_X              : "X",
+                        gdk.AXIS_Y              : "Y",
+                        gdk.AXIS_PRESSURE       : "PRESSURE",
+                        gdk.AXIS_XTILT          : "XTILT",
+                        gdk.AXIS_YTILT          : "YTILT",
+                        gdk.AXIS_WHEEL          : "WHEEL",
+                        gdk.AXIS_LAST           : "LAST",
+                        }
+            MODE_STR = {
+                        gdk.MODE_DISABLED       : "DISABLED",
+                        gdk.MODE_SCREEN         : "SCREEN",
+                        gdk.MODE_WINDOW         : "WINDOW",
+                        }
+            SOURCE_STR = {
+                        gdk.SOURCE_MOUSE        : "MOUSE",
+                        gdk.SOURCE_PEN          : "PEN",
+                        gdk.SOURCE_ERASER       : "ERASER",
+                        gdk.SOURCE_CURSOR       : "CURSOR",
+                        }
+            def notrans(v, d):
+                return v
+            MOD_STR = {
+                        gdk.SHIFT_MASK          : "SHIFT",
+                        gdk.LOCK_MASK           : "LOCK",
+                        gdk.CONTROL_MASK        : "CONTROL",
+                        gdk.META_MASK           : "META",
+                        gdk.MOD1_MASK           : "MOD1",
+                        gdk.MOD2_MASK           : "MOD2",
+                        gdk.MOD3_MASK           : "MOD3",
+                        gdk.MOD4_MASK           : "MOD4",
+                        gdk.MOD5_MASK           : "MOD5",
+                        gdk.BUTTON1_MASK        : "BUTTON1",
+                        gdk.BUTTON2_MASK        : "BUTTON2",
+                        gdk.BUTTON3_MASK        : "BUTTON3",
+                        gdk.BUTTON4_MASK        : "BUTTON4",
+                        gdk.BUTTON5_MASK        : "BUTTON5",
+                        gdk.RELEASE_MASK        : "RELEASE",
+                       }
+            def modtrans(mod):
+                return [v for k,v in MOD_STR.items() if k&mod]
+            def keys_trans(l, d):
+                #GdkModifierType can be converted to an int
+                return [(k,modtrans(v)) for (k,v) in l]
+            for name, trans in {"axes"          : AXES_STR.get,
+                                "has_cursor"    : notrans,
+                                "keys"          : keys_trans,
+                                "mode"          : MODE_STR.get,
+                                "num_axes"      : notrans,
+                                "num_keys"      : notrans,
+                                "source"        : SOURCE_STR.get}.items():
+                try:
+                    v = getattr(d, name)
+                    dinfo[name] = trans(v, v)
+                except:
+                    pass
     return info
 
 
-def get_preferred_size(widget):
-    if is_gtk3():
-        #ignore "min", we only care about "natural":
-        _, w = widget.get_preferred_width()
-        _, h = widget.get_preferred_height()
-        return w, h
-    return widget.size_request()
-
 def scaled_image(pixbuf, icon_size=None):
     if icon_size:
         pixbuf = pixbuf.scale_simple(icon_size, icon_size, INTERP_BILINEAR)
     return image_new_from_pixbuf(pixbuf)
 
 
-def get_pixbuf_from_data(rgb_data, has_alpha, w, h, rowstride):
-    if is_gtk3():
-        data = array.array('B', strtobytes(rgb_data))
-        return GdkPixbuf.Pixbuf.new_from_data(data, GdkPixbuf.Colorspace.RGB,
-                                         True, 8, w, h, rowstride,
-                                         None, None)
-    return gdk.pixbuf_new_from_data(rgb_data, gdk.COLORSPACE_RGB, has_alpha, 8, w, h, rowstride)
-
 
 def get_icon_from_file(filename):
     try:
@@ -538,8 +809,10 @@ def get_icon_from_file(filename):
         loader.write(data)
         loader.close()
     except Exception as e:
-        log.error("get_icon_from_file(%s) %s", filename, e)
-        return    None
+        log("get_icon_from_file(%s)", filename, exc_info=True)
+        log.error("Error: failed to load '%s'", filename)
+        log.error(" %s", e)
+        return None
     pixbuf = loader.get_pixbuf()
     return pixbuf
 
@@ -630,7 +903,7 @@ def title_box(label_str):
 
 #utility method to ensure there is always only one CheckMenuItem
 #selected in a submenu:
-def ensure_item_selected(submenu, item):
+def ensure_item_selected(submenu, item, recurse=True):
     if not isinstance(item, gtk.CheckMenuItem):
         return
     if item.get_active():
@@ -641,7 +914,7 @@ def ensure_item_selected(submenu, item):
                     continue
                 if isinstance(x, gtk.MenuItem):
                     submenu = x.get_submenu()
-                    if submenu:
+                    if submenu and recurse:
                         deactivate(submenu.get_children(), skip)
                 if isinstance(x, gtk.CheckMenuItem):
                     if x!=item and x.get_active():
@@ -669,33 +942,19 @@ def ensure_item_selected(submenu, item):
     return item
 
 
-def CheckMenuItem(label, tooltip=None):
-    """ adds a get_label() method for older versions of gtk which do not have it
-        beware that this label is not mutable!
-    """
-    cmi = gtk.CheckMenuItem(label)
-    if not hasattr(cmi, "get_label"):
-        def get_label():
-            return  label
-        cmi.get_label = get_label
-    if tooltip:
-        cmi.set_tooltip_text(tooltip)
-    return cmi
-
-
 class TableBuilder(object):
 
-    def __init__(self, rows=1, columns=2, homogeneous=False):
+    def __init__(self, rows=1, columns=2, homogeneous=False, col_spacings=0, row_spacings=0):
         self.table = gtk.Table(rows, columns, homogeneous)
-        self.table.set_col_spacings(0)
-        self.table.set_row_spacings(0)
+        self.table.set_col_spacings(col_spacings)
+        self.table.set_row_spacings(row_spacings)
         self.row = 0
         self.widget_xalign = 0.0
 
     def get_table(self):
         return self.table
 
-    def add_row(self, label, *widgets):
+    def add_row(self, label, *widgets, **kwargs):
         if label:
             l_al = gtk.Alignment(xalign=1.0, yalign=0.5, xscale=0.0, yscale=0.0)
             l_al.add(label)
@@ -706,19 +965,19 @@ class TableBuilder(object):
                 if w:
                     w_al = gtk.Alignment(xalign=self.widget_xalign, yalign=0.5, xscale=0.0, yscale=0.0)
                     w_al.add(w)
-                    self.attach(w_al, i)
+                    self.attach(w_al, i, **kwargs)
                 i += 1
         self.inc()
 
-    def attach(self, widget, i, count=1, xoptions=FILL, xpadding=10):
-        self.table.attach(widget, i, i+count, self.row, self.row+1, xoptions=xoptions, xpadding=xpadding)
+    def attach(self, widget, i, count=1, xoptions=FILL, yoptions=FILL, xpadding=10, ypadding=0):
+        self.table.attach(widget, i, i+count, self.row, self.row+1, xoptions=xoptions, yoptions=yoptions, xpadding=xpadding, ypadding=ypadding)
 
     def inc(self):
         self.row += 1
 
-    def new_row(self, row_label_str, value1, value2=None, label_tooltip=None):
+    def new_row(self, row_label_str, value1, value2=None, label_tooltip=None, **kwargs):
         row_label = label(row_label_str, label_tooltip)
-        self.add_row(row_label, value1, value2)
+        self.add_row(row_label, value1, value2, **kwargs)
 
 
 def choose_file(parent_window, title, action, action_button, callback, file_filter=None):
