@@ -119,15 +119,12 @@ class KeyboardStateInfoWindow:
 
 
 def main():
-    from xpra.platform import init, clean
+    from xpra.platform import program_context
     from xpra.log import enable_color
-    try:
-        init("Keyboard-Test", "Keyboard Test Tool")
+    with program_context("Keyboard-Test", "Keyboard Test Tool"):
         enable_color()
         KeyboardStateInfoWindow()
         gtk.main()
-    finally:
-        clean()
 
 
 if __name__ == "__main__":
