@@ -23,10 +23,11 @@ def get_gtk_keymap(ignore_keys=[None, "VoidSymbol", "0xffffff"]):
     from xpra.gtk_common.gtk_util import get_default_keymap, import_gdk, is_gtk3
     gdk = import_gdk()
     keymap = get_default_keymap()
+    log("get_default_keymap()=%s, direction=%s, bidirectional layouts: %s", keymap, keymap.get_direction(), keymap.have_bidi_layouts())
     keycodes=[]
     for i in range(0, 2**8):
         entries = keymap.get_entries_for_keycode(i)
-        log("%s.get_entries_for_keycode(%s)=%s", keymap, i, entries)
+        log("get_entries_for_keycode(%s)=%s", i, entries)
         if not entries:
             continue
         if is_gtk3():
@@ -55,10 +56,9 @@ def get_gtk_keymap(ignore_keys=[None, "VoidSymbol", "0xffffff"]):
 
 def main():
     import sys
-    from xpra.platform import init, clean
+    from xpra.platform import program_context
     from xpra.log import enable_color
-    try:
-        init("Keymap-Tool", "Keymap Information Tool")
+    with program_context("Keymap-Tool", "Keymap Information Tool"):
         enable_color()
         if "-v" in sys.argv or "--verbose" in sys.argv:
             log.enable_debug()
@@ -69,8 +69,6 @@ def main():
         pkey("keyval", "name", "keycode", "group", "level")
         for x in gtk_keymap:
             pkey(*x)
-    finally:
-        clean()
 
 
 if __name__ == "__main__":
