@@ -1,13 +1,13 @@
 # coding=utf8
 # This file is part of Xpra.
-# Copyright (C) 2010-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2017 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import gtk.gdk
-import time
+from gtk import gdk
 
+from xpra.os_util import monotonic_time
 from xpra.log import Logger
 log = Logger("gtk")
 
@@ -18,7 +18,7 @@ def get_rgb_rawdata(pixmap, x, y, width, height, logger=None):
     """
         Extracts pixels from the given pixmap
     """
-    start = time.time()
+    start = monotonic_time()
     pixmap_w, pixmap_h = pixmap.get_size()
     # Just in case we somehow end up with damage larger than the pixmap,
     # we don't want to start requesting random chunks of memory (this
@@ -36,11 +36,11 @@ def get_rgb_rawdata(pixmap, x, y, width, height, logger=None):
     if not colormap:
         log.error("get_rgb_rawdata(..) no colormap for RGB pixbuf %sx%s", width, height)
         return None
-    pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, False, 8, width, height)
+    pixbuf = gdk.Pixbuf(gdk.COLORSPACE_RGB, False, 8, width, height)
     pixbuf.get_from_drawable(pixmap, colormap, x, y, 0, 0, width, height)
     if logger:
         logger("get_rgb_rawdata(..) pixbuf.get_from_drawable took %s ms, visual depth=%s",
-             int(1000*(time.time()-start)), colormap.get_visual().depth)
+             int(1000*(monotonic_time()-start)), colormap.get_visual().depth)
     raw_data = pixbuf.get_pixels()
     rowstride = pixbuf.get_rowstride()
-    return (x, y, width, height, raw_data, "RGB", 24, rowstride)
+    return (x, y, width, height, raw_data, "RGB", 24, rowstride, 3)
