@@ -28,32 +28,30 @@ def gtk_main_quit_really():
         # that on every user of this function.
         from xpra.gtk_common.gobject_compat import import_gtk
         gtk = import_gtk()
-        gtk.main_quit()
         # So long as there are more nested main loops, re-register ourselves
         # to be called again:
-        if gtk.main_level() > 1:
+        if gtk.main_level() > 0:
+            gtk.main_quit()
             return True
         else:
             # But when we've just quit the outermost main loop, then
             # unregister ourselves so that it's possible to start the
             # main-loop again if desired:
             return False
-    from xpra.gtk_common.gobject_compat import import_gobject
-    gobject = import_gobject()
-    gobject.timeout_add(0, gtk_main_quit_forever)
+    from xpra.gtk_common.gobject_compat import import_glib
+    glib = import_glib()
+    glib.timeout_add(0, gtk_main_quit_forever)
 
 # If a user hits control-C, and we are currently executing Python code below
 # the main loop, then the exception will get swallowed up. (If we're just
 # idling in the main loop, then it will pass the exception along, but it won't
 # propagate it from Python code. Sigh.) But sys.excepthook will still get
 # called with such exceptions.
-_hooked = False
 _oldhook = None
 def gtk_main_quit_on_fatal_exceptions_enable():
-    global _hooked, _oldhook
-    if _hooked:
+    global _oldhook
+    if _oldhook:
         return
-    _hooked = True
     _oldhook = sys.excepthook
     def gtk_main_quit_on_fatal_exception(etype, val, tb):
         if issubclass(etype, (KeyboardInterrupt, SystemExit)):
@@ -71,7 +69,7 @@ def gtk_main_quit_on_fatal_exceptions_enable():
             return _oldhook(etype, val, tb)
     sys.excepthook = gtk_main_quit_on_fatal_exception
 
-def gtk_main_quit_on_fatal_exceptions_disable():    
+def gtk_main_quit_on_fatal_exceptions_disable():
     global _oldhook
     oh = _oldhook
     if oh:
