@@ -19,7 +19,8 @@ MODIFIER_MAP = {
 
 
 DEFAULT_MODIFIER_NAMES = ["shift", "control"]
-DEFAULT_MODIFIER_NUISANCE = ["lock", "num", "scroll"]
+DEFAULT_MODIFIER_NUISANCE_KEYNAMES = ["Num_Lock", "Caps_Lock", "Scroll_Lock"]
+DEFAULT_MODIFIER_NUISANCE = ["lock"]
 DEFAULT_ALL_MODIFIER_NAMES = DEFAULT_MODIFIER_NAMES+DEFAULT_MODIFIER_NUISANCE+["mod1", "mod2", "mod3", "mod4", "mod5"]
 
 DEFAULT_MODIFIER_MEANINGS = {
