@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2008, 2009 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2012-2015 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -74,12 +74,14 @@ def remove_disabled_category(*cat):
 def get_loggers_for_categories(*cat):
     if not cat:
         return  []
+    if "all" in cat:
+        return get_all_loggers()
     cset = set(cat)
     matches = set()
     for l in get_all_loggers():
         if set(l.categories).issuperset(cset):
             matches.add(l)
-    return list(matches or [])
+    return list(matches)
 
 def enable_debug_for(*cat):
     loggers = get_loggers_for_categories(*cat)
@@ -122,28 +124,29 @@ def setloghandler(lh):
     logging.root.handlers = []
     logging.root.addHandler(lh)
 
-def enable_color(to=sys.stdout, formatter=NOPREFIX_FORMAT):
+def enable_color(to=sys.stdout, format_string=NOPREFIX_FORMAT):
     if not hasattr(to, "fileno"):
         #on win32 sys.stdout can be a "Blackhole",
         #which does not have a fileno
         return
     from xpra.colorstreamhandler import ColorStreamHandler
-    from logging import Formatter
     csh = ColorStreamHandler(to)
-    csh.setFormatter(Formatter(formatter))
+    csh.setFormatter(logging.Formatter(format_string))
     setloghandler(csh)
 
+def enable_format(format_string):
+    try:
+        logging.root.handlers[0].formatter = logging.Formatter(format_string)
+    except:
+        pass
 
-#the order will be preserved with Python 2.7 onwards:
-try:
-    from collections import OrderedDict
-except:
-    OrderedDict = dict
 
+from collections import OrderedDict
 STRUCT_KNOWN_FILTERS = OrderedDict([
     ("Client", OrderedDict([
                 ("client"       , "All client code"),
                 ("paint"        , "Client window paint code"),
+                ("draw"         , "Client draw packets"),
                 ("cairo"        , "Cairo paint code used with the GTK3 client"),
                 ("opengl"       , "Client OpenGL rendering"),
                 ("info"         , "About and Session info dialogs"),
@@ -166,6 +169,7 @@ STRUCT_KNOWN_FILTERS = OrderedDict([
                 ])),
     ("Window", OrderedDict([
                 ("window"       , "All window code"),
+                ("damage"       , "Window X11 repaint events"),
                 ("geometry"     , "Window geometry"),
                 ("shape"        , "Window shape forwarding (XShape)"),
                 ("focus"        , "Window focus"),
@@ -173,7 +177,9 @@ STRUCT_KNOWN_FILTERS = OrderedDict([
                 ("metadata"     , "Window metadata"),
                 ("state"        , "Window state"),
                 ("icon"         , "Window icons"),
-                ("frame"        , "Window frame")
+                ("frame"        , "Window frame"),
+                ("grab"         , "Window grabs (both keyboard and mouse)"),
+                ("dragndrop"    , "Window drag-n-drop events"),
                 ])),
     ("Encoding", OrderedDict([
                 ("codec"        , "Codec loader and video helper"),
@@ -183,6 +189,7 @@ STRUCT_KNOWN_FILTERS = OrderedDict([
                 ("encoding"     , "Server side encoding selection and compression"),
                 ("scaling"      , "Picture scaling"),
                 ("delta"        , "Delta pre-compression"),
+                ("scroll"       , "Scrolling detection and compression"),
                 ("xor"          , "XOR delta pre-compression"),
                 ("subregion"    , "Video subregion processing"),
                 ("regiondetect" , "Video region detection"),
@@ -195,21 +202,20 @@ STRUCT_KNOWN_FILTERS = OrderedDict([
                 ("csc"          , "Colourspace conversion codecs"),
                 ("cuda"         , "CUDA device access (nvenc)"),
                 ("cython"       , "Cython CSC module"),
-                ("opencl"       , "OpenCL CSC module"),
-                ("opencv"       , "OpenCV CSC module"),
                 ("swscale"      , "swscale CSC module"),
                 ("libyuv"       , "libyuv CSC module"),
                 ("decoder"      , "All decoders"),
                 ("encoder"      , "All encoders"),
                 ("avcodec"      , "avcodec decoder"),
-                ("libav"        , "libav common code (swscale and avcodec)"),
+                ("libav"        , "libav common code (used by swscale, avcodec and ffmpeg)"),
+                ("ffmpeg"       , "ffmpeg encoder"),
                 ("pillow"       , "Pillow encoder and decoder"),
+                ("jpeg"         , "JPEG codec"),
                 ("vpx"          , "libvpx encoder and decoder"),
-                ("nvenc"        , "nvenc encoder (all versions)"),
+                ("nvenc"        , "nvenc hardware encoder"),
+                ("nvfbc"        , "nfbc screen capture"),
                 ("x264"         , "libx264 encoder"),
                 ("x265"         , "libx265 encoder"),
-                ("xvid"         , "Xvid encoder"),
-                ("webp"         , "libwebp encoder and decoder"),
                 ("webcam"       , "webcam access"),
                 ])),
     ("Pointer", OrderedDict([
@@ -229,8 +235,11 @@ STRUCT_KNOWN_FILTERS = OrderedDict([
     ("Network", OrderedDict([
                 #internal / network:
                 ("network"      , "All network code"),
+                ("http"         , "HTTP requests"),
                 ("mmap"         , "mmap transfers"),
                 ("protocol"     , "Packet input and output (formatting, parsing, sending and receiving)"),
+                ("websocket"    , "WebSocket layer"),
+                ("named-pipe"   , "Named pipe"),
                 ("crypto"       , "Encryption"),
                 ("auth"         , "Authentication"),
                 ])),
@@ -246,7 +255,6 @@ STRUCT_KNOWN_FILTERS = OrderedDict([
                 ("mdns"         , "mDNS session publishing"),
                 #server internals:
                 ("stats"        , "Server statistics"),
-                ("grab"         , "Window grabs"),
                 ("xshm"         , "XShm pixel capture"),
                 ])),
     ("Sound", OrderedDict([
@@ -256,6 +264,7 @@ STRUCT_KNOWN_FILTERS = OrderedDict([
                 ])),
     ("X11", OrderedDict([
                 ("x11"          , "All X11 code"),
+                ("xinput"       , "XInput bindings"),
                 ("bindings"     , "X11 Cython bindings"),
                 ("core"         , "X11 core bindings"),
                 ("randr"        , "X11 RandR bindings"),
@@ -299,6 +308,7 @@ def isenvdebug(category):
 
 class Logger(object):
     def __init__(self, *categories):
+        global default_level, debug_disabled_categories, KNOWN_FILTERS
         self.categories = list(categories)
         caller = sys._getframe(1).f_globals["__name__"]
         if caller!="__main__":
@@ -319,6 +329,13 @@ class Logger(object):
             if x not in KNOWN_FILTERS:
                 self.warn("unknown logging category: %s", x)
 
+    def get_info(self):
+        return {
+            "categories"    : self.categories,
+            "debug"         : self.debug_enabled,
+            "level"         : self.logger.getEffectiveLevel(),
+            }
+
     def __repr__(self):
         return "Logger(%s)" % ", ".join(self.categories)
 
@@ -334,7 +351,9 @@ class Logger(object):
 
     def log(self, level, msg, *args, **kwargs):
         if kwargs.get("exc_info") is True:
-            kwargs["exc_info"] = sys.exc_info()
+            ei = sys.exc_info()
+            if ei!=(None, None, None):
+                kwargs["exc_info"] = sys.exc_info()
         global global_logging_handler
         if LOG_PREFIX:
             msg = LOG_PREFIX+msg
