@@ -1,9 +1,4 @@
 # This file is part of Xpra.
-# Copyright (C) 2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2013-2016 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
-
-class ConnectionClosedException(Exception):
-    pass
-
-XPRA_MDNS_TYPE = '_xpra._tcp.'
