@@ -11,55 +11,54 @@ import zlib
 from xpra.log import Logger
 log = Logger("network", "protocol")
 from xpra.net.header import LZ4_FLAG, ZLIB_FLAG, LZO_FLAG
-from xpra.os_util import _memoryview
+
+
+MAX_SIZE = 256*1024*1024
 
 
 python_lz4_version = None
 lz4_version = None
+has_lz4 = False
+def lz4_compress(packet, level):
+    raise Exception("lz4 is not supported!")
 try:
     import lz4
+    from lz4 import VERSION as python_lz4_version   #@UnresolvedImport
     try:
-        from lz4 import VERSION as lz4_VERSION                          #@UnresolvedImport
+        #using unicode for version numbers is dumb:
+        python_lz4_version = python_lz4_version.encode("latin1")
     except:
-        #older versions (0.7 and older):
-        import pkg_resources
-        lz4_VERSION = pkg_resources.get_distribution("lz4").version
-    from lz4 import LZ4_compress, compressHC, LZ4_uncompress        #@UnresolvedImport
-    has_lz4 = True
-    if hasattr(lz4, "LZ4_compress_fast"):
-        from lz4 import LZ4_compress_fast                           #@UnresolvedImport
+        pass
+    from lz4 import LZ4_VERSION as lz4_version   #@UnresolvedImport
+    if hasattr(lz4, "block"):
+        from lz4.block import compress, decompress
+        LZ4_uncompress = decompress
+        has_lz4 = True
         def lz4_compress(packet, level):
-            if level>=9:
-                return level | LZ4_FLAG, compressHC(packet)
-            #clamp it: 0->17, 1->12, 2->7, 3->2, >=4->1
-            accel = max(1, 17-level*5)
-            return level | LZ4_FLAG, LZ4_compress_fast(packet, accel)
+            flag = min(15, level) | LZ4_FLAG
+            if level>=7:
+                return flag, compress(packet, mode="high_compression", compression=level)
+            elif level<=3:
+                return flag, compress(packet, mode="fast", acceleration=8-level*2)
+            return flag, compress(packet)
     else:
-        #v0.7.0 and earlier
-        def lz4_compress(packet, level):
-            if level>=9:
-                return level | LZ4_FLAG, compressHC(packet)
-            return level | LZ4_FLAG, LZ4_compress(packet)
-    #try to figure out the version number:
-    python_lz4_version = lz4_VERSION
-    assert python_lz4_version>="0.7", "python-lz4 version %s is older than 0.7.0 are vulnerable and should not be used, see CVE-2014-4715" % lz4_version
-    #now try to check the underlying "liblz4" version
-    #which is only available with python-lz4 0.8.0 onwards:
-    if hasattr(lz4, "LZ4_VERSION"):
-        try:
-            from distutils.version import LooseVersion
-        except ImportError:
-            pass
-        else:
-            #last known security issue:
-            assert LooseVersion(lz4.LZ4_VERSION)>=LooseVersion("r119"), "lz4 version %s is vulnerable and should not be used, see CVE-2014-4715" % lz4.LZ4_VERSION
-        lz4_version = lz4.LZ4_VERSION
+        #LZ4_compress_fast is 0.8 or later
+        from lz4 import LZ4_compress, LZ4_uncompress, compressHC        #@UnresolvedImport
+        if hasattr(lz4, "LZ4_compress_fast"):
+            from lz4 import LZ4_compress_fast     #@UnresolvedImport
+            has_lz4 = True
+            def lz4_compress(packet, level):
+                if level>=9:
+                    return level | LZ4_FLAG, compressHC(packet)
+                #clamp it: 0->17, 1->12, 2->7, 3->2, >=4->1
+                if level<=2:
+                    #clamp it: 0->17, 1->12, 2->7, 3->2, >=4->1
+                    accel = max(1, 17-level*5)
+                    return level | LZ4_FLAG, LZ4_compress_fast(packet, accel)
+                return level | LZ4_FLAG, LZ4_compress(packet)
 except Exception as e:
     log("lz4 not found: %s", e)
     LZ4_uncompress = None
-    has_lz4 = False
-    def lz4_compress(packet, level):
-        raise Exception("lz4 is not supported!")
 
 python_lzo_version = None
 lzo_version = None
@@ -169,7 +168,8 @@ def sanity_checks():
     if not use_lzo and not use_lz4:
         if not use_zlib:
             log.warn("Warning: all the compressors are disabled,")
-            log.warn(" unless you have a gigabit connection or better, performance will suffer")
+            log.warn(" unless you use mmap or have a gigabit connection or better")
+            log.warn(" performance will suffer")
         else:
             log.warn("Warning: zlib is the only compressor enabled")
             log.warn(" install and enable lzo or lz4 support for better performance")
@@ -177,6 +177,7 @@ def sanity_checks():
 
 class Compressed(object):
     def __init__(self, datatype, data, can_inline=False):
+        assert data is not None, "compressed data cannot be set to None"
         self.datatype = datatype
         self.data = data
         self.can_inline = can_inline
@@ -195,20 +196,30 @@ class LevelCompressed(Compressed):
         return  "LevelCompressed(%s: %i bytes as %s/%i)" % (self.datatype, len(self.data), self.algorithm, self.level)
 
 
-class Uncompressed(object):
+class LargeStructure(object):
     def __init__(self, datatype, data):
         self.datatype = datatype
         self.data = data
     def __len__(self):
         return len(self.data)
     def __repr__(self):
-        return  "Uncompressed(%s: %i bytes)" % (self.datatype, len(self.data))
+        return  "LargeStructure(%s: %i bytes)" % (self.datatype, len(self.data))
+
+class Compressible(LargeStructure):
+    #wrapper for data that should be compressed at some point,
+    #to use this class, you must override compress()
+    def __repr__(self):
+        return  "Compressible(%s: %i bytes)" % (self.datatype, len(self.data))
     def compress(self):
         raise Exception("compress() not defined on %s" % self)
 
+
 def compressed_wrapper(datatype, data, level=5, zlib=False, lz4=False, lzo=False, can_inline=True):
-    if _memoryview and isinstance(data, _memoryview):
+    if isinstance(data, memoryview):
         data = data.tobytes()
+    size = len(data)
+    if size>MAX_SIZE:
+        raise Exception("uncompressed data is too large: %iMB, limit is %iMB" % (size//1024//1024, MAX_SIZE//1024//1024))
     if lz4:
         assert use_lz4, "cannot use lz4"
         algo = "lz4"
@@ -218,7 +229,7 @@ def compressed_wrapper(datatype, data, level=5, zlib=False, lz4=False, lzo=False
         algo = "lzo"
         cl, cdata = lzo_compress(data, level)
     else:
-        assert use_zlib, "cannot use zlib"
+        assert zlib and use_zlib, "cannot use zlib"
         algo = "zlib"
         cl, cdata = zcompress(data, level)
     return LevelCompressed(datatype, cdata, cl, algo, can_inline=can_inline)
@@ -236,6 +247,7 @@ def get_compression_type(level):
     else:
         return "zlib"
 
+
 import struct
 LZ4_HEADER = struct.Struct('<L')
 def decompress(data, level):
@@ -248,7 +260,8 @@ def decompress(data, level):
         size = LZ4_HEADER.unpack_from(data[:4])[0]
         #TODO: it would be better to use the max_size we have in protocol,
         #but this hardcoded value will do for now
-        assert size<=(256*1024*1024)
+        if size>MAX_SIZE:
+            raise Exception("uncompressed data is too large: %iMB, limit is %iMB" % (size//1024//1024, MAX_SIZE//1024//1024))
         return LZ4_uncompress(data)
     elif level & LZO_FLAG:
         if not has_lzo:
