@@ -1,20 +1,22 @@
 # This file is part of Xpra.
-# Copyright (C) 2011-2015 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008, 2009, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
+from xpra.util import envint, envbool, csv, xor
+from xpra.os_util import strtobytes, memoryview_to_bytes
 from xpra.log import Logger
 log = Logger("network", "crypto")
 
-ENABLE_CRYPTO = os.environ.get("XPRA_ENABLE_CRYPTO", "1")=="1"
-ENCRYPT_FIRST_PACKET = os.environ.get("XPRA_ENCRYPT_FIRST_PACKET", "0")=="1"
+ENABLE_CRYPTO = envbool("XPRA_ENABLE_CRYPTO", True)
+ENCRYPT_FIRST_PACKET = envbool("XPRA_ENCRYPT_FIRST_PACKET", False)
 
 DEFAULT_IV = os.environ.get("XPRA_CRYPTO_DEFAULT_IV", "0000000000000000")
 DEFAULT_SALT = os.environ.get("XPRA_CRYPTO_DEFAULT_SALT", "0000000000000000")
-DEFAULT_ITERATIONS = int(os.environ.get("XPRA_CRYPTO_DEFAULT_ITERATIONS", "1000"))
-DEFAULT_BLOCKSIZE = int(os.environ.get("XPRA_CRYPTO_BLOCKSIZE", "32"))      #fixme: can we derive this?
+DEFAULT_ITERATIONS = envint("XPRA_CRYPTO_DEFAULT_ITERATIONS", 1000)
+DEFAULT_BLOCKSIZE = envint("XPRA_CRYPTO_BLOCKSIZE", 32)      #fixme: can we derive this?
 
 #other option "PKCS#7", "legacy"
 PADDING_LEGACY = "legacy"
@@ -30,7 +32,6 @@ PADDING_OPTIONS = [PREFERRED_PADDING]
 for x in ALL_PADDING_OPTIONS:
     if x not in PADDING_OPTIONS:
         PADDING_OPTIONS.append(x)
-CRYPTO_LIBRARY = os.environ.get("XPRA_CRYPTO_BACKEND", "python-cryptography")    #pycrypto
 
 
 ENCRYPTION_CIPHERS = []
@@ -40,49 +41,22 @@ def crypto_backend_init():
     log("crypto_backend_init() backend=%s", backend)
     if backend is not False:
         return
-    try_backends = []
-    if CRYPTO_LIBRARY=="python-cryptography":
-        try_backends = ["python-cryptography", "pycrypto"]
-    elif CRYPTO_LIBRARY=="pycrypto":
-        try_backends = ["pycrypto", "python-cryptography"]
-    else:
-        raise ImportError("invalid crypto library specified: '%s'" % CRYPTO_LIBRARY)
-    errors = {}
-    for tb in try_backends:
-        try:
-            if tb=="python-cryptography":
-                from xpra.net import pycryptography_backend
-                try_backend = pycryptography_backend
-            else:
-                assert tb=="pycrypto"
-                from xpra.net import pycrypto_backend
-                try_backend = pycrypto_backend
-
-            #validate it:
-            validate_backend(try_backend)
-            ENCRYPTION_CIPHERS[:] = try_backend.ENCRYPTION_CIPHERS[:]
-            backend = try_backend
-            break
-        except ImportError as e:
-            errors[tb] = e
-            log("%s import failure", tb, exc_info=True)
-        except Exception as e:
-            errors[tb] = e
-            log("%s validation failure", tb, exc_info=True)
-    if errors:
-        if backend:
-            log.warn("Warning: using fallback encryption library %s", tb)
-            for k,e in errors.items():
-                log.warn(" %s is not available: %s", k, e)
-        else:
-            log.error("Error: no encryption libraries could be loaded")
-            for k,e in errors.items():
-                log.error(" %s is not available: %s", k, e)
-    log("crypto_backend_init() backend=%s, ENCRYPTION_CIPHERS=%s", backend, ENCRYPTION_CIPHERS)
+    try:
+        from xpra.net import pycryptography_backend
+        #validate it:
+        validate_backend(pycryptography_backend)
+        ENCRYPTION_CIPHERS[:] = pycryptography_backend.ENCRYPTION_CIPHERS[:]
+        backend = pycryptography_backend
+        return
+    except ImportError:
+        log("crypto backend init failure", exc_info=True)
+        log.error("Error: cannot import python-cryptography")
+    except Exception:
+        log.error("Error: cannot initialize python-cryptography", exc_info=True)
+    backend = None
 
 def validate_backend(try_backend):
     import binascii
-    from xpra.util import strtobytes
     try_backend.init()
     message = b"some message1234"
     password = "this is our secret"
@@ -107,6 +81,53 @@ def validate_backend(try_backend):
     log("validate_backend(%s) passed", try_backend)
 
 
+def get_digest_module(digest):
+    log("get_digest_module(%s)", digest)
+    if not digest or not digest.startswith("hmac"):
+        return None
+    import hashlib
+    try:
+        digest_module = digest.split("+")[1]        #ie: "hmac+sha512" -> "sha512"
+    except:
+        digest_module = "md5"
+    try:
+        return getattr(hashlib, digest_module)
+    except AttributeError:
+        return None
+
+def choose_digest(options):
+    assert len(options)>0, "no digest options"
+    log("choose_digest(%s)", options)
+    #prefer stronger hashes:
+    for h in ("sha512", "sha384", "sha256", "sha224", "sha1", "md5"):
+        hname = "hmac+%s" % h
+        if hname in options:
+            return hname
+    #legacy name for "hmac+md5":
+    if "hmac" in options:
+        return "hmac"
+    if "xor" in options:
+        return "xor"
+    raise Exception("no known digest options found in '%s'" % csv(options))
+
+
+def gendigest(digest, password, salt):
+    assert digest and password and salt
+    salt = memoryview_to_bytes(salt)
+    password = strtobytes(password)
+    if digest=="xor":
+        salt = salt.ljust(16, "\x00")[:len(password)]
+        return memoryview_to_bytes(xor(password, salt))
+    digestmod = get_digest_module(digest)
+    if not digestmod:
+        log("invalid digest module '%s': %s", digest)
+        return None
+        #warn_server_and_exit(EXIT_UNSUPPORTED, "server requested digest '%s' but it is not supported" % digest, "invalid digest")
+    import hmac
+    v = hmac.HMAC(strtobytes(password), strtobytes(salt), digestmod=digestmod).hexdigest()
+    return v
+
+
 def pad(padding, size):
     if padding==PADDING_LEGACY:
         return " "*size
@@ -131,7 +152,7 @@ def get_iv():
     #IV = "0000000000000000"
     return IV or get_hex_uuid()[:16]
 
-def get_salt(l=64):
+def get_rand_str(l):
     #too short: we would not feed enough random data to HMAC
     assert l>=32, "salt is too short: only %i bytes" % l
     #too long: limit the amount of random data we request from the system
@@ -143,6 +164,9 @@ def get_salt(l=64):
         salt += get_hex_uuid()
     return salt[:l]
 
+def get_salt(l=64):
+    return get_rand_str(l)
+
 def get_iterations():
     return DEFAULT_ITERATIONS
 
@@ -185,7 +209,7 @@ def get_encryptor(ciphername, iv, password, key_salt, iterations):
     return backend.get_encryptor(key, iv), block_size
 
 def get_decryptor(ciphername, iv, password, key_salt, iterations):
-    log("get_encryptor(%s, %s, %s, %s, %s)", ciphername, iv, password, key_salt, iterations)
+    log("get_decryptor(%s, %s, %s, %s, %s)", ciphername, iv, password, key_salt, iterations)
     if not ciphername:
         return None, 0
     assert iterations>=100
