@@ -1,11 +1,12 @@
 # This file is part of Xpra.
-# Copyright (C) 2011-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
 import ctypes
-from xpra.os_util import memoryview_to_bytes
+from xpra.util import roundup
+from xpra.os_util import memoryview_to_bytes, WIN32, POSIX
 from xpra.simple_stats import to_std_unit
 from xpra.log import Logger
 log = Logger("mmap")
@@ -14,119 +15,157 @@ log = Logger("mmap")
 Utility functions for communicating via mmap
 """
 
-def can_use_mmap():
-    return hasattr(ctypes.c_ubyte, "from_buffer")
 
-def roundup(n, m):
-    return (n + m - 1) & ~(m - 1)
-
-
-def init_client_mmap(token, mmap_group=None, socket_filename=None, size=128*1024*1024):
+def init_client_mmap(mmap_group=None, socket_filename=None, size=128*1024*1024, filename=None):
     """
         Initializes an mmap area, writes the token in it and returns:
             (success flag, mmap_area, mmap_size, temp_file, mmap_filename)
         The caller must keep hold of temp_file to ensure it does not get deleted!
         This is used by the client.
     """
-    if not can_use_mmap():
-        log.error("cannot use mmap: python version is too old?")
-        return False, None, 0, None, None
-    log("init_mmap(%s, %s, %s)", token, mmap_group, socket_filename)
+    def rerr():
+        return False, False, None, 0, None, None
+    log("init_mmap%s", (mmap_group, socket_filename, size, filename))
+    mmap_filename = filename
+    mmap_temp_file = None
+    delete = True
     try:
         import mmap
-        import tempfile
-        from stat import S_IRUSR,S_IWUSR,S_IRGRP,S_IWGRP
-        mmap_dir = os.getenv("TMPDIR", "/tmp")
-        if not os.path.exists(mmap_dir):
-            raise Exception("TMPDIR %s does not exist!" % mmap_dir)
-        #create the mmap file, the mkstemp that is called via NamedTemporaryFile ensures
-        #that the file is readable and writable only by the creating user ID
-        temp = tempfile.NamedTemporaryFile(prefix="xpra.", suffix=".mmap", dir=mmap_dir)
-        #keep a reference to it so it does not disappear!
-        mmap_temp_file = temp
-        mmap_filename = temp.name
-        fd = temp.file.fileno()
-        #set the group permissions and gid if the mmap-group option is specified
-        if mmap_group and type(socket_filename)==str and os.path.exists(socket_filename):
-            s = os.stat(socket_filename)
-            os.fchown(fd, -1, s.st_gid)
-            os.fchmod(fd, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP)
-        assert size>=1024*1024, "mmap size is too small: %s (minimum is 1MB)" % to_std_unit(size)
-        assert size<=1024*1024*1024, "mmap is too big: %s (maximum is 1GB)" % to_std_unit(size)
         unit = max(4096, mmap.PAGESIZE)
         #add 8 bytes for the mmap area control header zone:
         mmap_size = roundup(size + 8, unit)
-        log("using mmap file %s, fd=%s, size=%s", mmap_filename, fd, mmap_size)
-        os.lseek(fd, mmap_size-1, os.SEEK_SET)
-        assert os.write(fd, b'\x00')
-        os.lseek(fd, 0, os.SEEK_SET)
-        mmap_area = mmap.mmap(fd, length=mmap_size)
-        write_mmap_token(mmap_area, token)
-        return True, mmap_area, mmap_size, mmap_temp_file, mmap_filename
+        if WIN32:
+            if not filename:
+                from xpra.net.crypto import get_hex_uuid
+                filename = "xpra-%s" % get_hex_uuid()
+            mmap_filename = filename
+            mmap_area = mmap.mmap(0, mmap_size, filename)
+            #not a real file:
+            delete = False
+            mmap_temp_file = None
+        else:
+            assert POSIX
+            if filename:
+                if os.path.exists(filename):
+                    fd = os.open(filename, os.O_EXCL | os.O_RDWR)
+                    mmap_size = os.path.getsize(mmap_filename)
+                    #mmap_size = 4*1024*1024    #size restriction needed with ivshmem
+                    delete = False
+                    log.info("Using existing mmap file '%s': %sMB", mmap_filename, mmap_size//1024//1024)
+                else:
+                    import errno
+                    flags = os.O_CREAT | os.O_EXCL | os.O_RDWR
+                    try:
+                        fd = os.open(filename, flags)
+                        mmap_temp_file = None   #os.fdopen(fd, 'w')
+                        mmap_filename = filename
+                    except OSError as e:
+                        if e.errno == errno.EEXIST:
+                            log.error("Error: the mmap file '%s' already exists", filename)
+                            return rerr()
+                        raise
+            else:
+                import tempfile
+                mmap_dir = os.getenv("TMPDIR", "/tmp")
+                if not os.path.exists(mmap_dir):
+                    raise Exception("TMPDIR %s does not exist!" % mmap_dir)
+                #create the mmap file, the mkstemp that is called via NamedTemporaryFile ensures
+                #that the file is readable and writable only by the creating user ID
+                try:
+                    temp = tempfile.NamedTemporaryFile(prefix="xpra.", suffix=".mmap", dir=mmap_dir)
+                except OSError as e:
+                    log.error("Error: cannot create mmap file:")
+                    log.error(" %s", e)
+                    return rerr()
+                #keep a reference to it so it does not disappear!
+                mmap_temp_file = temp
+                mmap_filename = temp.name
+                fd = temp.file.fileno()
+            #set the group permissions and gid if the mmap-group option is specified
+            if mmap_group and type(socket_filename)==str and os.path.exists(socket_filename):
+                from stat import S_IRUSR,S_IWUSR,S_IRGRP,S_IWGRP
+                s = os.stat(socket_filename)
+                os.fchown(fd, -1, s.st_gid)
+                os.fchmod(fd, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP)
+            assert mmap_size>=1024*1024, "mmap size is too small: %s (minimum is 1MB)" % to_std_unit(mmap_size)
+            assert mmap_size<=1024*1024*1024, "mmap is too big: %s (maximum is 1GB)" % to_std_unit(mmap_size)
+            log("using mmap file %s, fd=%s, size=%s", mmap_filename, fd, mmap_size)
+            os.lseek(fd, mmap_size-1, os.SEEK_SET)
+            assert os.write(fd, b'\x00')
+            os.lseek(fd, 0, os.SEEK_SET)
+            mmap_area = mmap.mmap(fd, length=mmap_size)
+        return True, delete, mmap_area, mmap_size, mmap_temp_file, mmap_filename
     except Exception as e:
-        log.error("failed to setup mmap: %s", e, exc_info=True)
+        log("failed to setup mmap: %s", e, exc_info=True)
+        log.error("Error: mmap setup failed:")
+        log.error(" %s", e)
         clean_mmap(mmap_filename)
-        return False, None, 0, None, None
+        return rerr()
 
 def clean_mmap(mmap_filename):
     log("clean_mmap(%s)", mmap_filename)
     if mmap_filename and os.path.exists(mmap_filename):
-        os.unlink(mmap_filename)
+        try:
+            os.unlink(mmap_filename)
+        except OSError as e:
+            log.error("Error: failed to remove the mmap file '%s':", mmap_filename)
+            log.error(" %s", e)
 
-MAX_TOKEN_BYTES = 128
+DEFAULT_TOKEN_INDEX = 512
+DEFAULT_TOKEN_BYTES = 128
 
-def write_mmap_token(mmap_area, token, index=512):
-    #write the 16 byte token one byte at a time - no endianness
-    log("mmap_token=%s", token)
+def write_mmap_token(mmap_area, token, index=DEFAULT_TOKEN_INDEX, count=DEFAULT_TOKEN_BYTES):
+    assert count>0
+    #write the token one byte at a time - no endianness
+    log("write_mmap_token(%s, %#x, %#x, %#x)", mmap_area, token, index, count)
     v = token
-    for i in range(0, MAX_TOKEN_BYTES):
-        poke = ctypes.c_ubyte.from_buffer(mmap_area, 512+i)
+    for i in range(0, count):
+        poke = ctypes.c_ubyte.from_buffer(mmap_area, index+i)
         poke.value = v % 256
         v = v>>8
     assert v==0, "token value is too big"
 
-def read_mmap_token(mmap_area, index=512):
+def read_mmap_token(mmap_area, index=DEFAULT_TOKEN_INDEX, count=DEFAULT_TOKEN_BYTES):
+    assert count>0
     v = 0
-    for i in range(0, MAX_TOKEN_BYTES):
+    for i in range(0, count):
         v = v<<8
-        peek = ctypes.c_ubyte.from_buffer(mmap_area, 512+MAX_TOKEN_BYTES-1-i)
+        peek = ctypes.c_ubyte.from_buffer(mmap_area, index+count-1-i)
         v += peek.value
+    log("read_mmap_token(%s, %#x, %#x)=%#x", mmap_area, index, count, v)
     return v
 
 
-def init_server_mmap(mmap_filename, mmap_token=None, new_mmap_token=None):
+def init_server_mmap(mmap_filename, mmap_size=0):
     """
         Reads the mmap file provided by the client
         and verifies the token if supplied.
         Returns the mmap object and its size: (mmap, size)
     """
-    if not can_use_mmap():
-        log.error("cannot use mmap: python version is too old?")
-        return None, 0
-    try:
-        f = open(mmap_filename, "r+b")
-    except Exception as e:
-        log.error("cannot access mmap file '%s' (see mmap-group option?)", mmap_filename)
-        log.error("  %s", e)
-        return None, 0
+    if not WIN32:
+        try:
+            f = open(mmap_filename, "r+b")
+        except Exception as e:
+            log.error("Error: cannot access mmap file '%s':", mmap_filename)
+            log.error("  %s", e)
+            log.error(" see mmap-group option?")
+            return None, 0
 
     mmap_area = None
     try:
         import mmap
-        mmap_size = os.path.getsize(mmap_filename)
-        mmap_area = mmap.mmap(f.fileno(), mmap_size)
-        if mmap_token:
-            #verify the token:
-            v = read_mmap_token(mmap_area)
-            log("mmap_token=%s, verification=%s", mmap_token, v)
-            if v!=mmap_token:
-                log.warn("WARNING: mmap token verification failed, not using mmap area!")
-                log.warn("expected '%s', found '%s'", mmap_token, v)
-                mmap_area.close()
-                return None, 0
-            if new_mmap_token:
-                write_mmap_token(mmap_area, new_mmap_token)
-        return mmap_area, mmap_size
+        if not WIN32:
+            actual_mmap_size = os.path.getsize(mmap_filename)
+            if mmap_size and actual_mmap_size!=mmap_size:
+                log.warn("Warning: expected mmap file '%s' of size %i but got %i", mmap_filename, mmap_size, actual_mmap_size)
+            mmap_area = mmap.mmap(f.fileno(), mmap_size)
+        else:
+            if mmap_size==0:
+                log.error("Error: client did not supply the mmap area size")
+                log.error(" try updating your client version?")
+            mmap_area = mmap.mmap(0, mmap_size, mmap_filename)
+            actual_mmap_size = mmap_size
+        return mmap_area, actual_mmap_size
     except Exception as e:
         log.error("cannot use mmap file '%s': %s", mmap_filename, e, exc_info=True)
         if mmap_area:
@@ -139,7 +178,7 @@ def int_from_buffer(mmap_area, pos):
 
 #descr_data is a list of (offset, length)
 #areas from the mmap region
-def mmap_read(mmap_area, descr_data):
+def mmap_read(mmap_area, *descr_data):
     """
         Reads data from the mmap_area as written by 'mmap_write'.
         The descr_data is the list of mmap chunks used.
