@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2013-2016 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2013-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -18,7 +18,7 @@ netifaces_version = None
 try:
 	import netifaces				#@UnresolvedImport
 	log("netifaces loaded sucessfully")
-	netifaces_version = netifaces.version
+	netifaces_version = netifaces.version		#@UndefinedVariable
 except:
 	has_netifaces = False
 	log.warn("python netifaces package is missing")
@@ -26,20 +26,28 @@ iface_ipmasks = {}
 bind_IPs = None
 
 
+def get_free_tcp_port():
+	s = socket.socket()
+	s.bind(('', 0))
+	port = s.getsockname()[1]
+	s.close()
+	return port
+
+
 def get_interfaces():
 	if not has_netifaces:
 		return	[]
-	return	netifaces.interfaces()
+	return	netifaces.interfaces()			#@UndefinedVariable
 
 def get_gateways():
 	if not has_netifaces:
 		return	{}
 	#versions older than 0.10.5 can crash when calling gateways()
 	#https://bitbucket.org/al45tair/netifaces/issues/15/gateways-function-crash-segmentation-fault
-	if netifaces.version<'0.10.5':
+	if netifaces.version<'0.10.5':			#@UndefinedVariable
 		return {}
 	try:
-		d =	netifaces.gateways()
+		d =	netifaces.gateways()			#@UndefinedVariable
 		AF_NAMES = {}
 		for k in dir(netifaces):
 			if k.startswith("AF_"):
@@ -67,7 +75,7 @@ def get_bind_IPs():
 def do_get_bind_IPs():
 	global iface_ipmasks
 	ips = []
-	ifaces = netifaces.interfaces()
+	ifaces = netifaces.interfaces()			#@UndefinedVariable
 	log("ifaces=%s", ifaces)
 	for iface in ifaces:
 		if_ipmasks = []
@@ -80,14 +88,16 @@ def do_get_bind_IPs():
 				if ipmask not in if_ipmasks:
 					if_ipmasks.append(ipmask)
 		except Exception as e:
-			log.error("error on %s: %s", iface, e)
+			log("do_get_bind_IPs()", exc_info=True)
+			log.error("Error parsing network interface '%s':", iface)
+			log.error(" %s", iface, e)
 		iface_ipmasks[iface] = if_ipmasks
 	log("do_get_bind_IPs()=%s", ips)
 	return ips
 
 def do_get_bind_ifacemask(iface):
 	ipmasks = []
-	address_types = netifaces.ifaddresses(iface)
+	address_types = netifaces.ifaddresses(iface)	#@UndefinedVariable
 	for addresses in address_types.values():
 		for address in addresses:
 			if 'netmask' in address and 'addr' in address:
@@ -103,14 +113,32 @@ def do_get_bind_ifacemask(iface):
 	return ipmasks
 
 def get_iface(ip):
+	log("get_iface(%s)", ip)
 	if not ip:
 		return	None
+	if ip.find("%")>=0:
+		return ip.split("%", 1)[1]
 	if ip.find(":")>=0:
 		#ipv6?
 		return None
+	if any(x for x in ip if (".:0123456789").find(x)<0):
+		#extra characters, assume this is a hostname:
+		try:
+			v = socket.getaddrinfo(ip, None)
+			assert len(v)>0
+		except Exception as e:
+			log.error("Error: cannot revolve '%s'", ip)
+			return None
+		for i, x in enumerate(v):
+			family, socktype, proto, canonname, sockaddr = x
+			log("get_iface(%s) [%i]=%s", ip, i, (family, socktype, proto, canonname, sockaddr))
+			if family==socket.AF_INET:
+				break
+		log("get_iface(%s) sockaddr=%s", ip, sockaddr)
+		ip = sockaddr[0]
+
 	ip_parts = ip.split(".")
 	if len(ip_parts)!=4:
-		log.error("Error: invalid IPv4 format, %d parts in '%s'", len(ip_parts), ip_parts)
 		return	None
 
 	best_match = None
@@ -146,9 +174,10 @@ def get_iface(ip):
 # http://code.activestate.com/recipes/442490/
 if_nametoindex = None
 if_indextoname = None
-if not sys.platform.startswith("win"):
+from xpra.os_util import WIN32, OSX
+if not WIN32:
 	library = "libc.so.6"
-	if sys.platform.startswith("darwin"):
+	if OSX:
 		library = "/usr/lib/libc.dylib"
 	elif sys.platform.startswith("sunos"):
 		library = "libsocket.so.1"
@@ -174,8 +203,17 @@ if not sys.platform.startswith("win"):
 		def if_nametoindex(interfaceName):
 			return _libc.if_nametoindex(create_string_buffer(interfaceName.encode()))
 		def if_indextoname(index):
-			s = create_string_buffer('\000' * 256)
+			s = create_string_buffer(b'\000' * 256)
 			return _libc.if_indextoname(c_uint(index), s)
+else:
+	def if_nametoindex(iface):
+		#IPv6 addresses give us the interface as a string:
+		#fe80:....%11, so try to convert "11" into 11
+		try:
+			return int(iface)
+		except:
+			return None
+
 
 net_sys_config = None
 def get_net_sys_config():
@@ -243,15 +281,65 @@ def get_net_sys_config():
 				addproc("/proc/sys/net/ipv4/%s" % k, 	"ipv4", k, int)
 	return net_sys_config
 
+def get_net_config():
+	try:
+		from xpra.net.bytestreams import VSOCK_TIMEOUT, SOCKET_TIMEOUT, TCP_NODELAY
+		return {
+				"vsocket.timeout"	: VSOCK_TIMEOUT,
+				"socket.timeout"	: SOCKET_TIMEOUT,
+				"tcp.nodelay"		: TCP_NODELAY,
+				}
+	except:
+		return {}
+
+def get_ssl_info():
+	try:
+		import ssl
+	except ImportError as e:
+		log("no ssl: %s", e)
+		return {}
+	protocols = dict((k,getattr(ssl, k)) for k in dir(ssl) if k.startswith("PROTOCOL_"))
+	ops = dict((k,getattr(ssl, k)) for k in dir(ssl) if k.startswith("OP_"))
+	vers = dict((k,getattr(ssl, k)) for k in dir(ssl) if k.startswith("VERIFY_"))
+	info = {
+			"protocols"	: protocols,
+			"options"	: ops,
+			"verify"	: vers,
+			}
+	for k,name in {
+					"HAS_ALPN"				: "alpn",
+					"HAS_ECDH"				: "ecdh",
+					"HAS_SNI"				: "sni",
+					"HAS_NPN"				: "npn",
+					"CHANNEL_BINDING_TYPES"	: "channel-binding-types",
+					}.items():
+		v = getattr(ssl, k, None)
+		if v is not None:
+			info[name] = v
+	for k,name in {
+					""			: "version",
+					"_INFO"		: "version-info",
+					"_NUMBER"	: "version-number",
+					}.items():
+		v = getattr(ssl, "OPENSSL_VERSION%s" % k, None)
+		if v is not None:
+			info.setdefault("openssl", {})[name] = v
+	return info
+
 def get_info():
 	from xpra.net.protocol import get_network_caps
 	i = get_network_caps()
 	if has_netifaces:
 		i["interfaces"] = get_interfaces()
 		i["gateways"] = get_gateways()
+	if "ssl" in sys.modules:
+		ssli = get_ssl_info()
+		ssli[""] = True
+		i["ssl"] = ssli
 	s = get_net_sys_config()
 	if s:
 		i["system"] = s
+	i["config"] = get_net_config()
 	return i
 
 
@@ -285,7 +373,7 @@ def main():
 					s += str(v[i])
 				return s
 			if type(v)==bytes:
-				from xpra.util import bytestostr
+				from xpra.os_util import bytestostr
 				v = bytestostr(v)
 			if type(v)==str and v.startswith("v"):
 				return v[1:]
@@ -304,19 +392,32 @@ def main():
 		netcaps["netifaces"] = netif
 		print_nested_dict(netcaps)
 
+		print("")
+		print("Network Config:")
+		print_nested_dict(get_net_config())
+
 		net_sys = get_net_sys_config()
 		if net_sys:
+			print("")
 			print("Network System Config:")
 			print_nested_dict(net_sys)
 
-		from xpra.net.crypto import crypto_backend_init, get_crypto_caps
-		crypto_backend_init()
-		ccaps = get_crypto_caps()
-		if ccaps:
-			print("")
-			print("Crypto Capabilities:")
-			print_nested_dict(ccaps)
-			
+		print("")
+		print("SSL:")
+		print_nested_dict(get_ssl_info())
+
+		try:
+			from xpra.net.crypto import crypto_backend_init, get_crypto_caps
+			crypto_backend_init()
+			ccaps = get_crypto_caps()
+			if ccaps:
+				print("")
+				print("Crypto Capabilities:")
+				print_nested_dict(ccaps)
+		except Exception as e:
+			print("No Crypto:")
+			print(" %s" % e)
+
 
 if __name__ == "__main__":
 	main()
