@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2011-2016 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008, 2009, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -8,9 +8,8 @@
 
 # but it works on win32, for whatever that's worth.
 
-import sys
-from socket import error as socket_error
 import os
+from socket import error as socket_error
 import binascii
 from threading import Lock, Event
 
@@ -19,14 +18,15 @@ from xpra.log import Logger
 log = Logger("network", "protocol")
 cryptolog = Logger("network", "crypto")
 
-from xpra.os_util import Queue, strtobytes
-from xpra.util import repr_ellipsized, csv
-from xpra.net import ConnectionClosedException
+from xpra.os_util import PYTHON3, Queue, memoryview_to_bytes, strtobytes
+from xpra.util import repr_ellipsized, csv, envint, envbool
+from xpra.make_thread import make_thread, start_thread
+from xpra.net.common import ConnectionClosedException          #@UndefinedVariable (pydev false positive)
 from xpra.net.bytestreams import ABORT
 from xpra.net import compression
 from xpra.net import packet_encoding
 from xpra.net.compression import get_compression_caps, decompress, sanity_checks as compression_sanity_checks,\
-        InvalidCompressionException, Compressed, LevelCompressed, Uncompressed
+        InvalidCompressionException, Compressed, LevelCompressed, Compressible, LargeStructure
 from xpra.net.packet_encoding import get_packet_encoding_caps, decode, sanity_checks as packet_encoding_sanity_checks, InvalidPacketEncodingException
 from xpra.net.header import unpack_header, pack_header, FLAGS_CIPHER, FLAGS_NOHEADER
 from xpra.net.crypto import get_crypto_caps, get_encryptor, get_decryptor, pad, INITIAL_PADDING
@@ -34,48 +34,43 @@ from xpra.net.crypto import get_crypto_caps, get_encryptor, get_decryptor, pad,
 
 #stupid python version breakage:
 JOIN_TYPES = (str, bytes)
-if sys.version > '3':
+if PYTHON3:
     long = int              #@ReservedAssignment
     unicode = str           #@ReservedAssignment
     JOIN_TYPES = (bytes, )
 
 
-FAULT_RATE = int(os.environ.get("XPRA_PROTOCOL_FAULT_INJECTION_RATE", "0"))
-if FAULT_RATE>0:
-    _counter = 0
-    def INJECT_FAULT(p):
-        global _counter
-        _counter += 1
-        if (_counter % FAULT_RATE)==0:
-            log.warn("injecting fault in %s", p)
-            p.raw_write("Protocol JUNK! added by fault injection code")
-else:
-    def INJECT_FAULT(p):
-        pass
-
-USE_ALIASES = os.environ.get("XPRA_USE_ALIASES", "1")=="1"
-
-READ_BUFFER_SIZE = int(os.environ.get("XPRA_READ_BUFFER_SIZE", 65536))
+USE_ALIASES = envbool("XPRA_USE_ALIASES", True)
+READ_BUFFER_SIZE = envint("XPRA_READ_BUFFER_SIZE", 65536)
 #merge header and packet if packet is smaller than:
-PACKET_JOIN_SIZE = int(os.environ.get("XPRA_PACKET_JOIN_SIZE", READ_BUFFER_SIZE))
-LARGE_PACKET_SIZE = int(os.environ.get("XPRA_LARGE_PACKET_SIZE", 4096))
+PACKET_JOIN_SIZE = envint("XPRA_PACKET_JOIN_SIZE", READ_BUFFER_SIZE)
+LARGE_PACKET_SIZE = envint("XPRA_LARGE_PACKET_SIZE", 4096)
 #inline compressed data in packet if smaller than:
-INLINE_SIZE = int(os.environ.get("XPRA_INLINE_SIZE", 32768))
-FAKE_JITTER = int(os.environ.get("XPRA_FAKE_JITTER", "0"))
-MIN_COMPRESS_SIZE = int(os.environ.get("XPRA_MIN_COMPRESS_SIZE", 378))
+INLINE_SIZE = envint("XPRA_INLINE_SIZE", 32768)
+FAKE_JITTER = envint("XPRA_FAKE_JITTER", 0)
+MIN_COMPRESS_SIZE = envint("XPRA_MIN_COMPRESS_SIZE", 378)
+SEND_INVALID_PACKET = envint("XPRA_SEND_INVALID_PACKET", 0)
+SEND_INVALID_PACKET_DATA = strtobytes(os.environ.get("XPRA_SEND_INVALID_PACKET_DATA", b"ZZinvalid-packetZZ"))
 
 
+def get_digests():
+    import hashlib
+    return ["hmac", "xor"] + ["hmac+%s" % x for x in list(reversed(sorted(hashlib.algorithms_available)))]
+
 def get_network_caps():
     try:
-        from xpra.net.mmap_pipe import can_use_mmap
-        mmap = can_use_mmap()
+        from xpra.platform.features import MMAP_SUPPORTED
     except:
-        mmap = False
+        MMAP_SUPPORTED = False
+    digests = get_digests()
+    #"hmac" is the legacy name, "xor" and "des" should not be used for salt:
+    salt_digests = [x for x in digests if x not in ("hmac", "xor", "des")]
     caps = {
-                "digest"                : ("hmac", "xor"),
+                "digest"                : digests,
+                "salt-digest"           : salt_digests,
                 "compressors"           : compression.get_enabled_compressors(),
                 "encoders"              : packet_encoding.get_enabled_encoders(),
-                "mmap"                  : mmap,
+                "mmap"                  : MMAP_SUPPORTED,
                }
     caps.update(get_crypto_caps())
     caps.update(get_compression_caps())
@@ -123,7 +118,7 @@ class Protocol(object):
         #initial value which may get increased by client/server after handshake:
         self.max_packet_size = 256*1024
         self.abs_max_packet_size = 256*1024*1024
-        self.large_packets = ["hello", "window-metadata"]
+        self.large_packets = ["hello", "window-metadata", "sound-data"]
         self.send_aliases = {}
         self.receive_aliases = {}
         self._log_stats = None          #None here means auto-detect
@@ -142,8 +137,7 @@ class Protocol(object):
         self.cipher_out_block_size = 0
         self.cipher_out_padding = INITIAL_PADDING
         self._write_lock = Lock()
-        from xpra.make_thread import make_thread
-        self._write_thread = make_thread(self._write_thread_loop, "write", daemon=True)
+        self._write_thread = None
         self._read_thread = make_thread(self._read_thread_loop, "read", daemon=True)
         self._read_parser_thread = None         #started when needed
         self._write_format_thread = None        #started when needed
@@ -173,11 +167,11 @@ class Protocol(object):
             if t and t.isAlive():
                 t.join(timeout)
         exited = True
+        cinfo = self._conn or "cleared connection"
         for t in (self._read_thread, self._write_thread):
             if t and t.isAlive():
-                log.warn("%s thread of %s has not yet exited (timeout=%s)", t.name, self._conn, timeout)
+                log.warn("Warning: %s thread of %s is still alive (timeout=%s)", t.name, cinfo, timeout)
                 exited = False
-                break
         return exited
 
     def set_packet_source(self, get_packet_cb):
@@ -213,7 +207,9 @@ class Protocol(object):
             "large_packets"         : self.large_packets,
             "compression_level"     : self.compression_level,
             "max_packet_size"       : self.max_packet_size,
+            "aliases"               : USE_ALIASES,
             "input" : {
+                       "buffer-size"            : READ_BUFFER_SIZE,
                        "packetcount"            : self.input_packetcount,
                        "raw_packetcount"        : self.input_raw_packetcount,
                        "count"                  : self.input_stats,
@@ -222,6 +218,10 @@ class Protocol(object):
                                                    },
                         },
             "output" : {
+                        "packet-join-size"      : PACKET_JOIN_SIZE,
+                        "large-packet-size"     : LARGE_PACKET_SIZE,
+                        "inline-size"           : INLINE_SIZE,
+                        "min-compress-size"     : MIN_COMPRESS_SIZE,
                         "packetcount"           : self.output_packetcount,
                         "raw_packetcount"       : self.output_raw_packetcount,
                         "count"                 : self.output_stats,
@@ -256,11 +256,12 @@ class Protocol(object):
 
 
     def start(self):
-        def do_start():
+        def start_network_read_thread():
             if not self._closed:
-                self._write_thread.start()
                 self._read_thread.start()
-        self.idle_add(do_start)
+        self.idle_add(start_network_read_thread)
+        if SEND_INVALID_PACKET:
+            self.timeout_add(SEND_INVALID_PACKET*1000, self.raw_write, SEND_INVALID_PACKET_DATA)
 
     def send_now(self, packet):
         if self._closed:
@@ -268,8 +269,12 @@ class Protocol(object):
             return
         log("send_now(%s ...)", packet[0])
         assert self._get_packet_cb==None, "cannot use send_now when a packet source exists! (set to %s)" % self._get_packet_cb
+        tmp_queue = [packet]
         def packet_cb():
             self._get_packet_cb = None
+            if not tmp_queue:
+                raise Exception("packet callback used more than once!")
+            packet = tmp_queue.pop()
             return (packet, )
         self._get_packet_cb = packet_cb
         self.source_has_more()
@@ -278,10 +283,10 @@ class Protocol(object):
         self._source_has_more.set()
         #start the format thread:
         if not self._write_format_thread and not self._closed:
-            from xpra.make_thread import make_thread
             self._write_format_thread = make_thread(self._write_format_thread_loop, "format", daemon=True)
             self._write_format_thread.start()
-        INJECT_FAULT(self)
+        #from now on, take shortcut:
+        self.source_has_more = self._source_has_more.set
 
     def _write_format_thread_loop(self):
         log("write_format_thread_loop starting")
@@ -308,7 +313,12 @@ class Protocol(object):
         with self._write_lock:
             if self._closed:
                 return
-            self._add_chunks_to_queue(chunks, start_send_cb, end_send_cb)
+            try:
+                self._add_chunks_to_queue(chunks, start_send_cb, end_send_cb)
+            except:
+                log.error("Error: failed to queue '%s' packet", packet[0])
+                log("add_chunks_to_queue%s", (chunks, start_send_cb, end_send_cb), exc_info=True)
+                raise
 
     def _add_chunks_to_queue(self, chunks, start_send_cb=None, end_send_cb=None):
         """ the write_lock must be held when calling this function """
@@ -332,7 +342,7 @@ class Protocol(object):
                     padded = data
                 else:
                     # pad byte value is number of padding bytes added
-                    padded = data + pad(self.cipher_out_padding, padding_size)
+                    padded = memoryview_to_bytes(data) + pad(self.cipher_out_padding, padding_size)
                     actual_size += padding_size
                 assert len(padded)==actual_size, "expected padded size to be %i, but got %i" % (len(padded), actual_size)
                 data = self.cipher_out.encrypt(padded)
@@ -345,19 +355,26 @@ class Protocol(object):
                 items.append((data, scb, ecb))
             elif actual_size<PACKET_JOIN_SIZE:
                 if type(data) not in JOIN_TYPES:
-                    data = bytes(data)
+                    data = memoryview_to_bytes(data)
                 header_and_data = pack_header(proto_flags, level, index, payload_size) + data
                 items.append((header_and_data, scb, ecb))
             else:
                 header = pack_header(proto_flags, level, index, payload_size)
                 items.append((header, scb, None))
-                items.append((strtobytes(data), None, ecb))
+                items.append((data, None, ecb))
             counter += 1
+        if self._write_thread is None:
+            self.start_write_thread()
         self._write_queue.put(items)
         self.output_packetcount += 1
 
+    def start_write_thread(self):
+        self._write_thread = start_thread(self._write_thread_loop, "write", daemon=True)
+
     def raw_write(self, contents, start_cb=None, end_cb=None):
         """ Warning: this bypasses the compression and packet encoder! """
+        if self._write_thread is None:
+            self.start_write_thread()
         self._write_queue.put(((contents, start_cb, end_cb), ))
 
     def verify_packet(self, packet):
@@ -435,7 +452,7 @@ class Protocol(object):
 
     def noencode(self, data):
         #just send data as a string for clients that don't understand xpra packet format:
-        if sys.version_info[0] >= 3:
+        if PYTHON3:
             import codecs
             def b(x):
                 if type(x)==bytes:
@@ -477,7 +494,12 @@ class Protocol(object):
                 l = len(item)
             except TypeError as e:
                 raise TypeError("invalid type %s in %s packet at index %s: %s" % (ti, packet[0], i, e))
-            if ti==Uncompressed:
+            if ti==LargeStructure:
+                item = item.data
+                packet[i] = item
+                ti = type(item)
+                continue
+            elif ti==Compressible:
                 #this is a marker used to tell us we should compress it now
                 #(used by the client for clipboard data)
                 item = item.compress()
@@ -544,10 +566,11 @@ class Protocol(object):
     def _io_thread_loop(self, name, callback):
         try:
             log("io_thread_loop(%s, %s) loop starting", name, callback)
-            while not self._closed:
-                callback()
+            while not self._closed and callback():
+                pass
             log("io_thread_loop(%s, %s) loop ended, closed=%s", name, callback, self._closed)
         except ConnectionClosedException as e:
+            log("%s closed", self._conn, exc_info=True)
             if not self._closed:
                 #ConnectionClosedException means the warning has been logged already
                 self._connection_lost("%s connection %s closed" % (name, self._conn))
@@ -568,19 +591,23 @@ class Protocol(object):
         if items is None:
             log("write thread: empty marker, exiting")
             self.close()
-            return
+            return False
         for buf, start_cb, end_cb in items:
             con = self._conn
             if not con:
-                return
+                return False
             if start_cb:
                 try:
                     start_cb(con.output_bytecount)
                 except:
                     if not self._closed:
-                        log.error("error on %s", start_cb, exc_info=True)
+                        log.error("Error on write start callback %s", start_cb, exc_info=True)
             while buf and not self._closed:
                 written = con.write(buf)
+                #example test code, for sending small chunks very slowly:
+                #written = con.write(buf[:1024])
+                #import time
+                #time.sleep(0.05)
                 if written:
                     buf = buf[written:]
                     self.output_raw_packetcount += 1
@@ -589,7 +616,8 @@ class Protocol(object):
                     end_cb(self._conn.output_bytecount)
                 except:
                     if not self._closed:
-                        log.error("error on %s", end_cb, exc_info=True)
+                        log.error("Error on write end callback %s", end_cb, exc_info=True)
+        return True
 
     def _read_thread_loop(self):
         self._io_thread_loop("read", self._read)
@@ -603,8 +631,9 @@ class Protocol(object):
             #give time to the parse thread to call close itself
             #so it has time to parse and process the last packet received
             self.timeout_add(1000, self.close)
-            return
+            return False
         self.input_raw_packetcount += 1
+        return True
 
     def _internal_error(self, message="", exc=None, exc_info=False):
         #log exception info with last log message
@@ -638,23 +667,29 @@ class Protocol(object):
     #delegates to invalid_header()
     #(so this can more easily be intercepted and overriden
     # see tcp-proxy)
-    def _invalid_header(self, data):
-        self.invalid_header(self, data)
+    def _invalid_header(self, data, msg=""):
+        self.invalid_header(self, data, msg)
 
-    def invalid_header(self, proto, data):
-        err = "invalid packet header: '%s'" % binascii.hexlify(data[:8])
+    def invalid_header(self, proto, data, msg="invalid packet header"):
+        err = "%s: '%s'" % (msg, binascii.hexlify(data[:8]))
         if len(data)>1:
-            err += " read buffer=%s" % repr_ellipsized(data)
+            err += " read buffer=%s (%i bytes)" % (repr_ellipsized(data), len(data))
         self.gibberish(err, data)
 
 
     def read_queue_put(self, data):
         #start the parse thread if needed:
         if not self._read_parser_thread and not self._closed:
-            from xpra.make_thread import make_thread
+            if data is None:
+                log("empty marker in read queue, exiting")
+                self.idle_add(self.close)
+                return
             self._read_parser_thread = make_thread(self._read_parse_thread_loop, "parse", daemon=True)
             self._read_parser_thread.start()
         self._read_queue.put(data)
+        #from now on, take shortcut:
+        if self._read_queue_put==self.read_queue_put:
+            self._read_queue_put = self._read_queue.put
 
     def _read_parse_thread_loop(self):
         log("read_parse_thread_loop starting")
@@ -702,7 +737,7 @@ class Protocol(object):
                     break
                 if payload_size<0:
                     if read_buffer[0] not in ("P", ord("P")):
-                        self._invalid_header(read_buffer)
+                        self._invalid_header(read_buffer, "invalid packet header byte %s" % read_buffer[0])
                         return
                     if bl<8:
                         break   #packet still too small
@@ -711,14 +746,14 @@ class Protocol(object):
 
                     #sanity check size (will often fail if not an xpra client):
                     if data_size>self.abs_max_packet_size:
-                        self._invalid_header(read_buffer)
+                        self._invalid_header(read_buffer, "invalid size in packet header: %s" % data_size)
                         return
 
                     bl = len(read_buffer)-8
                     if protocol_flags & FLAGS_CIPHER:
                         if self.cipher_in_block_size==0 or not self.cipher_in_name:
                             cryptolog.warn("received cipher block but we don't have a cipher to decrypt it with, not an xpra client?")
-                            self._invalid_header(read_buffer)
+                            self._invalid_header(read_buffer, "invalid encryption packet flag (no cipher configured)")
                             return
                         padding_size = self.cipher_in_block_size - (data_size % self.cipher_in_block_size)
                         payload_size = data_size + padding_size
@@ -851,7 +886,6 @@ class Protocol(object):
                 log("processing packet %s", packet_type)
                 self._process_packet_cb(self, packet)
                 packet = None
-                INJECT_FAULT(self)
 
     def flush_then_close(self, last_packet, done_callback=None):
         """ Note: this is best effort only
@@ -863,6 +897,7 @@ class Protocol(object):
             we wait again for the queue to flush,
             then no matter what, we close the connection and stop the threads.
         """
+        log("flush_then_close(%s, %s) closed=%s", last_packet, done_callback, self._closed)
         def done():
             log("flush_then_close: done, callback=%s", done_callback)
             if done_callback:
@@ -1002,12 +1037,26 @@ class Protocol(object):
         try:
             owq = self._write_queue
             self._write_queue = exit_queue
+            #discard all elements in the old queue and push the None marker:
+            try:
+                while owq.qsize()>0:
+                    owq.read(False)
+            except:
+                pass
             owq.put_nowait(None)
         except:
             pass
         try:
             orq = self._read_queue
             self._read_queue = exit_queue
+            #discard all elements in the old queue and push the None marker:
+            try:
+                while orq.qsize()>0:
+                    orq.read(False)
+            except:
+                pass
             orq.put_nowait(None)
         except:
             pass
+        #just in case the read thread is waiting again:
+        self._source_has_more.set()
