@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2015 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2015-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -10,12 +10,12 @@ import subprocess
 import binascii
 
 from xpra.gtk_common.gobject_compat import import_glib
-from xpra.util import repr_ellipsized
+from xpra.util import repr_ellipsized, envint, envbool
 
 from xpra.net.bytestreams import TwoFileConnection
-from xpra.net import ConnectionClosedException
+from xpra.net.common import ConnectionClosedException
 from xpra.net.protocol import Protocol
-from xpra.os_util import Queue, setbinarymode, SIGNAMES, bytestostr
+from xpra.os_util import Queue, setbinarymode, SIGNAMES, bytestostr, WIN32, POSIX
 from xpra.child_reaper import getChildReaper
 from xpra.log import Logger
 log = Logger("util")
@@ -30,15 +30,15 @@ log = Logger("util")
 #using the protocol for encoding the data
 
 
-DEBUG_WRAPPER = os.environ.get("XPRA_WRAPPER_DEBUG", "0")=="1"
+DEBUG_WRAPPER = envbool("XPRA_WRAPPER_DEBUG", False)
 #to make it possible to inspect files (more human readable):
-HEXLIFY_PACKETS = os.environ.get("XPRA_HEXLIFY_PACKETS", "0")=="1"
+HEXLIFY_PACKETS = envbool("XPRA_HEXLIFY_PACKETS", False)
 #avoids showing a new console window on win32:
-WIN32_SHOWWINDOW = os.environ.get("XPRA_WIN32_SHOWWINDOW", "0")=="1"
+WIN32_SHOWWINDOW = envbool("XPRA_WIN32_SHOWWINDOW", False)
 #this used to cause problems with py3k / gi bindings?
-HANDLE_SIGINT = os.environ.get("XPRA_WRAPPER_SIGING", "1")=="1"
+HANDLE_SIGINT = envbool("XPRA_WRAPPER_SIGINT", True)
 
-FAULT_RATE = int(os.environ.get("XPRA_WRAPPER_FAULT_INJECTION_RATE", "0"))
+FAULT_RATE = envint("XPRA_WRAPPER_FAULT_INJECTION_RATE")
 if FAULT_RATE>0:
     _counter = 0
     def INJECT_FAULT(p):
@@ -123,6 +123,7 @@ class subprocess_callee(object):
             self.run()
             return 0
         except KeyboardInterrupt as e:
+            log("start() KeyboardInterrupt %s", e)
             if str(e):
                 log.warn("%s", e)
             return 0
@@ -130,6 +131,7 @@ class subprocess_callee(object):
             log.error("error in main loop", exc_info=True)
             return 1
         finally:
+            log("run() ended, calling cleanup and protocol close")
             self.cleanup()
             if self.protocol:
                 self.protocol.close()
@@ -160,7 +162,7 @@ class subprocess_callee(object):
         else:
             self._output = open(self.output_filename, 'wb')
         #stdin and stdout wrapper:
-        conn = TwoFileConnection(self._output, self._input, abort_test=None, target=self.name, info=self.name, close_cb=self.net_stop)
+        conn = TwoFileConnection(self._output, self._input, abort_test=None, target=self.name, socktype=self.name, close_cb=self.net_stop)
         conn.timeout = 0
         protocol = Protocol(self, conn, self.process_packet, get_packet_cb=self.get_packet)
         setup_fastencoder_nocompression(protocol)
@@ -275,14 +277,23 @@ class subprocess_callee(object):
 
 
 def exec_kwargs():
-    if os.name=="posix":
-        return {"close_fds" : True}
-    elif sys.platform.startswith("win"):
+    kwargs = {}
+    stderr = sys.stderr.fileno()
+    if POSIX:
+        kwargs["close_fds"] = True
+    elif WIN32:
+        from xpra.platform.win32 import REDIRECT_OUTPUT
+        if REDIRECT_OUTPUT:
+            #stderr is not valid and would give us this error:
+            # WindowsError: [Errno 6] The handle is invalid
+            stderr = open(os.devnull, 'w')
         if not WIN32_SHOWWINDOW:
             startupinfo = subprocess.STARTUPINFO()
             startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
-            return {"startupinfo" : startupinfo}
-    return {}
+            startupinfo.wShowWindow = 0     #aka win32.con.SW_HIDE
+            kwargs["startupinfo"] = startupinfo
+    kwargs["stderr"] = stderr
+    return kwargs
 
 def exec_env(blacklist=["LS_COLORS", ]):
     env = os.environ.copy()
@@ -353,7 +364,7 @@ class subprocess_caller(object):
 
     def make_protocol(self):
         #make a connection using the process stdin / stdout
-        conn = TwoFileConnection(self.process.stdin, self.process.stdout, abort_test=self.abort_test, target=self.description, info=self.description, close_cb=self.subprocess_exit)
+        conn = TwoFileConnection(self.process.stdin, self.process.stdout, abort_test=self.abort_test, target=self.description, socktype=self.description, close_cb=self.subprocess_exit)
         conn.timeout = 0
         protocol = Protocol(self, conn, self.process_packet, get_packet_cb=self.get_packet)
         setup_fastencoder_nocompression(protocol)
@@ -365,13 +376,14 @@ class subprocess_caller(object):
         kwargs = exec_kwargs()
         env = self.get_env()
         log("exec_subprocess() command=%s, env=%s, kwargs=%s", self.command, env, kwargs)
-        proc = subprocess.Popen(self.command, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=sys.stderr.fileno(), env=env, **kwargs)
+        proc = subprocess.Popen(self.command, stdin=subprocess.PIPE, stdout=subprocess.PIPE, env=env, **kwargs)
         getChildReaper().add_process(proc, self.description, self.command, True, True, callback=self.subprocess_exit)
         return proc
 
     def get_env(self):
         env = exec_env()
         env["XPRA_LOG_PREFIX"] = "%s " % self.description
+        env["XPRA_FIX_UNICODE_OUT"] = "0"
         return env
 
     def cleanup(self):
