@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -8,6 +8,12 @@ import os as os
 import sys as sys
 
 
+try:
+    import encodings
+except ImportError:
+    pass
+
+
 _init_done = False
 def init(prgname=None, appname=None):
     """ do whatever is needed to prepare an application for running,
@@ -52,10 +58,12 @@ def set_default_name(prgname=None, appname=None):
 
 #platforms can override this
 def command_error(message):
-    print(message)
+    from xpra.scripts.main import error
+    error(message)
 
 def command_info(message):
-    print(message)
+    from xpra.scripts.main import info
+    info(message)
 
 
 _clean_done = False
@@ -131,11 +139,12 @@ def do_get_username():
 
 
 def platform_import(where, pm, required, *imports):
+    from xpra.os_util import OSX, POSIX
     if os.name == "nt":
         p = "win32"
-    elif sys.platform.startswith("darwin"):
+    elif OSX:
         p = "darwin"
-    elif os.name == "posix":
+    elif POSIX:
         p = "xposix"
     else:
         raise OSError("Unknown OS %s" % (os.name))
