@@ -3,8 +3,12 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+import sys
+
 def do_init():
-    pass
+    for x in list(sys.argv):
+        if x.startswith("-psn_"):
+            sys.argv.remove(x)
 
 def do_clean():
     pass
