@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -20,6 +20,45 @@ KEYS_TRANSLATION_OPTIONS = {
                     "Meta_R"        : ["Control_R", "Control_L"],
                     }
 
+#data extracted from:
+#https://support.apple.com/en-us/HT201794
+#"How to identify keyboard localizations"
+#maps Apple's names into standard X11 keyboard identifiers
+
+APPLE_LAYOUTS = {
+    "Arabic"    : "ar",
+    "Belgian"   : "be",
+    "Bulgarian" : "bg",
+    "Croatian"  : "cr",
+    "Czech"     : "cz",
+    "Danish"    : "dk",
+    "Dutch"     : "nl",
+    "British"   : "gb",
+    "US"        : "us",
+    "Finnish"   : "fi",
+    "Swedish"   : "se",
+    "French"    : "fr",
+    "German"    : "de",
+    "Greek"     : "gr",
+    "Hungarian" : "hu",
+    #"Icelandic" : "is",
+    "Israel"    : "il",
+    "Italian"   : "it",
+    "Japanese"  : "jp",
+    "Korean"    : "ko",
+    "Norwegian" : "no",
+    "Portugese" : "po",
+    "Romanian"  : "ro",
+    "Russian"   : "ru",
+    "Slovak"    : "sl",
+    "Spanish"   : "es",
+    #"Swiss"     : "ch",
+    "Taiwanese" : "tw",
+    "Thai"      : "th",
+    "Turkey"    : "tr",
+    }
+
+
 class Keyboard(KeyboardBase):
     """
         Switch Meta and Control
@@ -34,6 +73,40 @@ class Keyboard(KeyboardBase):
         self.num_lock_keycode = NUM_LOCK_KEYCODE
         self.key_translations = {}
 
+
+    def get_layout_spec(self):
+        layout = "us"
+        layouts = ["us"]
+        variant = ""
+        variants = []
+        try:
+            from AppKit import NSTextInputContext       #@UnresolvedImport
+            ic = NSTextInputContext.new()
+            current_keyboard = ic.selectedKeyboardInputSource()
+            code = APPLE_LAYOUTS.get(current_keyboard.split(".")[-1])
+            log("get_layout_spec() current_keyboard=%s, code=%s", current_keyboard, code)
+            all_keyboards = ic.keyboardInputSources()
+            log("get_layout_spec() other keyboards=%s", all_keyboards)
+            if code:
+                layout = code
+            if all_keyboards:
+                layouts = []
+                for k in all_keyboards:
+                    code = APPLE_LAYOUTS.get(k.split(".")[-1])
+                    if code:
+                        layouts.append(code)
+                if not layouts:
+                    layouts.append("us")
+            else:
+                if code not in layouts:
+                    layouts.insert(0, code)
+            log("get_layout_spec() layout=%s, layouts=%s", layout, layouts)
+        except Exception as e:
+            log("get_layout_spec()", exc_info=True)
+            log.error("Error querying keyboard layout:")
+            log.error(" %s", e)
+        return layout, layouts, variant, variants
+
     def get_keymap_modifiers(self):
         """
             Override superclass so we can tell the server
@@ -112,8 +185,11 @@ class Keyboard(KeyboardBase):
             if trans:
                 log("swap keys: translating key '%s' to %s", key_event, trans)
                 key_event.keycode, key_event.keyname = trans
-        if key_event.keycode==self.num_lock_keycode and not key_event.pressed:
-            log("toggling numlock")
-            self.num_lock_state = not self.num_lock_state
-            getOSXMenuHelper().update_numlock(self.num_lock_state)
+        if key_event.keycode==self.num_lock_keycode:
+            if not key_event.pressed:
+                log("toggling numlock")
+                self.num_lock_state = not self.num_lock_state
+                getOSXMenuHelper().update_numlock(self.num_lock_state)
+            #do not forward the "Escape" key that numlock usually comes up as
+            return
         send_key_action_cb(wid, key_event)
