@@ -1,9 +1,10 @@
 # This file is part of Xpra.
-# Copyright (C) 2011-2015 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2016 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os.path
+from xpra.util import envbool
 
 def debug(*msg):
     """ delay import of logger to prevent cycles """
@@ -13,12 +14,13 @@ def debug(*msg):
     return None
 
 
+_gtkosx_warning_ = False
 def do_get_resources_dir():
     rsc = None
     RESOURCES = "/Resources/"
     #FUGLY warning: importing gtkosx_application causes the dock to appear,
     #and in some cases we don't want that.. so use the env var XPRA_SKIP_UI as workaround for such cases:
-    if os.environ.get("XPRA_SKIP_UI", "0")=="0":
+    if not envbool("XPRA_SKIP_UI", False):
         try:
             import gtkosx_application        #@UnresolvedImport
             try:
@@ -28,10 +30,13 @@ def do_get_resources_dir():
                 #maybe we're not running from an app bundle?
                 pass
         except:
-            #delayed import to prevent cycles:
-            from xpra.log import Logger
-            log = Logger("util")
-            log.error("ERROR: gtkosx_application module is missing - trying to continue anyway")
+            global _gtkosx_warning_
+            if _gtkosx_warning_ is False:
+                _gtkosx_warning_ = True
+                #delayed import to prevent cycles:
+                from xpra.log import Logger
+                log = Logger("util")
+                log.error("ERROR: gtkosx_application module is missing - trying to continue anyway")
     else:
         debug("XPRA_SKIP_UI is set, not importing gtkosx_application")
     if rsc is None:
@@ -70,43 +75,52 @@ def do_get_default_conf_dirs():
     #the default config file we install into the Resources folder:
     #ie: /Volumes/Xpra/Xpra.app/Contents/Resources/etc
     from xpra.platform.paths import get_resources_dir
-    return [os.path.join(get_resources_dir(), "etc")]
+    return [os.path.join(get_resources_dir(), "etc", "xpra")]
 
 def do_get_system_conf_dirs():
     #the system wide configuration directory
+    dirs = []
     try:
         from Foundation import  NSSearchPathForDirectoriesInDomains, NSApplicationSupportDirectory, NSLocalDomainMask, NSSystemDomainMask  #@UnresolvedImport
-        dirs = NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSLocalDomainMask|NSSystemDomainMask, False)
-        return list(dirs)
+        sdirs = NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSLocalDomainMask|NSSystemDomainMask, False)
+        for x in sdirs:
+            #ie: "/Library/Application Support/Xpra"
+            dirs.append(os.path.join(x, "Xpra"))
     except:
-        pass
-    #fallback to hardcoded:
-    default_conf_dir = "/Library/Application Support/Xpra"
-    return [os.environ.get("XPRA_SYSCONF_DIR", default_conf_dir)]
+        #fallback to hardcoded:
+        default_conf_dir = "/Library/Application Support/Xpra"
+        dirs = [os.environ.get("XPRA_SYSCONF_DIR", default_conf_dir)]
+    dirs.append("/etc/xpra")
+    return dirs
 
 def do_get_user_conf_dirs():
     #the system wide configuration directory
+    dirs = []
     try:
         #when running sandboxed, it may look like this:
         #~/Library/Containers/<bundle_id>/Data/Library/Application Support/
         from Foundation import  NSSearchPathForDirectoriesInDomains, NSApplicationSupportDirectory, NSUserDomainMask    #@UnresolvedImport
-        dirs = list(NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSUserDomainMask, False))
+        udirs = NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSUserDomainMask, False)
+        for x in udirs:
+            dirs.append(os.path.join(x, "Xpra"))
     except:
         #fallback to hardcoded:
         dirs = ["/Library/Application Support/Xpra"]
     dirs.append("~/.xpra")
     return dirs
 
-def do_get_default_log_dir():
+def do_get_default_log_dirs():
+    dirs = []
     try:
         from Foundation import  NSSearchPathForDirectoriesInDomains, NSLibraryDirectory, NSUserDomainMask    #@UnresolvedImport
         udirs = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, False)
-        #ie: ~/Library/
-        library_dir = udirs[0]
-        return os.path.join(library_dir, "Xpra")
+        for x in udirs:
+            #ie: ~/Library/
+            dirs.append(os.path.join(x, "Logs", "Xpra"))
     except:
-        pass
-    return "~/Library/Logs"
+        dirs.append("~/Library/Logs/Xpra")
+    dirs.append("/tmp")
+    return dirs
 
 def do_get_socket_dirs():
     return ["/var/tmp/%s-Xpra" % os.getuid(), "~/.xpra"]
@@ -125,12 +139,27 @@ def do_get_download_dir():
     return d
 
 
-def do_get_sound_command():
+def do_get_sshpass_command():
+    from xpra.platform.paths import get_app_dir
+    base = get_app_dir()
+    p = os.path.join(base, "Resources", "bin", "sshpass")
+    if os.path.exists(p):
+        return p
+    return None
+
+
+def do_get_nodock_command():
     #try to use the subapp:
     from xpra.platform.paths import get_app_dir
     base = get_app_dir()
     subapp = os.path.join(base, "Xpra_NoDock.app", "Contents")
     if os.path.exists(subapp) and os.path.isdir(subapp):
         base = subapp
-    helper = os.path.join(subapp, "MacOS", "Xpra")
+    #appstore builds have script wrappers:
+    helper = os.path.join(base, "Resources", "scripts", "Xpra")
+    if not os.path.exists(helper):
+        helper = os.path.join(base, "Helpers", "Xpra")
     return [helper]
+
+def do_get_sound_command():
+    return do_get_nodock_command()
