@@ -1,10 +1,11 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2012-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+import os
 import sys
 
 _init_done = False
@@ -50,6 +51,9 @@ def get_native_notifier_classes():
     return []
 
 
+def get_session_type():
+    return ""
+
 
 def get_xdpi():
     return -1
@@ -78,42 +82,58 @@ def get_icon_size():
 def get_antialias_info():
     return {}
 
+def get_display_icc_info():
+    #per display info
+    return {}
+
 def get_icc_info():
     from xpra.log import Logger
     log = Logger("platform")
+    ENV_ICC_DATA = os.environ.get("XPRA_ICC_DATA")
+    if ENV_ICC_DATA:
+        import binascii
+        return {
+                "source"    : "environment-override",
+                "data"      : binascii.unhexlify(ENV_ICC_DATA),
+                }
     info = {}
     try:
-        from PIL.ImageCms import get_display_profile, \
-            getProfileName, getProfileInfo, getProfileCopyright, getProfileManufacturer, getProfileModel, getProfileDescription, getDefaultIntent, \
-            INTENT_PERCEPTUAL, INTENT_RELATIVE_COLORIMETRIC, INTENT_SATURATION, INTENT_ABSOLUTE_COLORIMETRIC
-        INTENT_STR = {
-                      INTENT_PERCEPTUAL             : "perceptual",
-                      INTENT_RELATIVE_COLORIMETRIC  : "relative-colorimetric",
-                      INTENT_SATURATION             : "saturation",
-                      INTENT_ABSOLUTE_COLORIMETRIC  : "absolute-colorimetric",
-                      }
+        from PIL import ImageCms
+        from PIL.ImageCms import get_display_profile, getDefaultIntent
+        INTENT_STR = {}
+        for x in ("PERCEPTUAL", "RELATIVE_COLORIMETRIC", "SATURATION", "ABSOLUTE_COLORIMETRIC"):
+            v = getattr(ImageCms, "INTENT_%s" % x, None)
+            if v:
+                INTENT_STR[v] = x.lower().replace("_", "-")
+        log("get_icc_info() intents=%s", INTENT_STR)
         def getDefaultIntentStr(_p):
             return INTENT_STR.get(getDefaultIntent(_p), "unknown")
         def getData(_p):
             return _p.tobytes()
         p = get_display_profile()
+        log("get_icc_info() display_profile=%s", p)
         if p:
             for (k, fn) in {
-                            "name"          : getProfileName,
-                            "info"          : getProfileInfo,
-                            "copyright"     : getProfileCopyright,
-                            "manufacturer"  : getProfileManufacturer,
-                            "model"         : getProfileModel,
-                            "description"   : getProfileDescription,
-                            "default-intent": getDefaultIntentStr,
-                            "data"          : getData,
+                            "name"          : "getProfileName",
+                            "info"          : "getProfileInfo",
+                            "copyright"     : "getProfileCopyright",
+                            "manufacturer"  : "getProfileManufacturer",
+                            "model"         : "getProfileModel",
+                            "description"   : "getProfileDescription",
+                            "default-intent": "getDefaultIntentStr",
+                            "data"          : "getData",
                             }.items():
+                m = getattr(ImageCms, fn, None)
+                if not m:
+                    log("%s lacks %s", ImageCms, fn)
+                    continue
                 try:
-                    v = fn(p)
+                    v = m(p)
                     info[k] = v
                 except Exception as e:
                     log("ICC profile error on %s using %s: %s", k, fn, e)
     except Exception as e:
+        log("get_icc_info()", exc_info=True)
         log.warn("Warning: cannot query ICC profiles:")
         log.warn(" %s", e)
     return info
@@ -208,6 +228,7 @@ def get_info_base():
             "workareas"                     : get_workareas(),
             "desktops"                      : get_number_of_desktops(),
             "desktop_names"                 : get_desktop_names(),
+            "session-type"                  : get_session_type(),
             "vertical-refresh"              : get_vrefresh(),
             "fixed_cursor_size"             : get_fixed_cursor_size(),
             "cursor_size"                   : get_cursor_size(),
@@ -223,6 +244,7 @@ def get_info_base():
                                                },
             "antialias"                     : get_antialias_info(),
             "icc"                           : get_icc_info(),
+            "display-icc"                   : get_display_icc_info(),
             "window_frame"                  : get_window_frame_sizes(),
             }
 
@@ -243,9 +265,10 @@ platform_import(globals(), "gui", False,
                 "get_native_tray_classes",
                 "get_native_system_tray_classes",
                 "get_native_notifier_classes",
+                "get_session_type",
                 "get_vrefresh", "get_workarea", "get_workareas",
                 "get_number_of_desktops", "get_desktop_names",
-                "get_antialias_info", "get_icc_info", "get_xdpi", "get_ydpi",
+                "get_antialias_info", "get_icc_info", "get_display_icc_info", "get_xdpi", "get_ydpi",
                 "get_icon_size",
                 "get_mouse_config",
                 "get_double_click_time", "get_double_click_distance",
@@ -258,6 +281,7 @@ platform_import(globals(), "gui", False,
 def main():
     from xpra.platform import program_context
     from xpra.util import print_nested_dict
+    from xpra.os_util import OSX, POSIX
     from xpra.log import enable_color
     with program_context("GUI-Properties"):
         enable_color()
@@ -269,14 +293,14 @@ def main():
                 x.enable_debug()
 
         #naughty, but how else can I hook this up?
-        import os
-        if os.name=="posix":
+        if POSIX and not OSX:
             try:
-                from xpra.x11.bindings import posix_display_source      #@UnusedImport
+                from xpra.x11.bindings.posix_display_source import init_posix_display_source    #@UnresolvedImport
+                init_posix_display_source()
             except:
                 pass    #maybe running on OSX? hope for the best..
         i = get_info()
-        print_nested_dict(i)
+        print_nested_dict(i, hex_keys=("data", "icc-data", "icc-profile"))
 
 
 if __name__ == "__main__":
