@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2015 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -51,7 +51,9 @@ def get_user_conf_dirs():
     return envaslist_or_delegate("XPRA_USER_CONF_DIRS", do_get_user_conf_dirs)
 def do_get_user_conf_dirs():
     #per-user configuration location:
-    return ["~/.xpra"]
+    if os.name!="posix" or os.getuid()>0:
+        return ["~/.xpra"]
+    return []
 
 def get_default_conf_dirs():
     return envaslist_or_delegate("XPRA_DEFAULT_CONF_DIRS", do_get_default_conf_dirs)
@@ -65,10 +67,10 @@ def get_socket_dirs():
 def do_get_socket_dirs():
     return ["~/.xpra"]
 
-def get_default_log_dir():
-    return env_or_delegate("XPRA_LOG_DIR", do_get_default_log_dir)
-def do_get_default_log_dir():
-    return "~/.xpra"
+def get_default_log_dirs():
+    return envaslist_or_delegate("XPRA_LOG_DIRS", do_get_default_log_dirs)
+def do_get_default_log_dirs():
+    return ["~/.xpra"]
 
 def get_download_dir():
     return env_or_delegate("XPRA_DOWNLOAD_DIR", do_get_download_dir)
@@ -86,7 +88,21 @@ def do_get_script_bin_dirs():
 def get_remote_run_xpra_scripts():
     return envaslist_or_delegate("XPRA_REMOTE_RUN_XPRA_SCRIPTS", do_get_remote_run_xpra_scripts)
 def do_get_remote_run_xpra_scripts():
-    return ["~/.xpra/run-xpra", "$XDG_RUNTIME_DIR/xpra/run-xpra", "xpra"]
+    return ["~/.xpra/run-xpra", "$XDG_RUNTIME_DIR/xpra/run-xpra", "xpra", "/usr/local/bin/xpra"]
+
+
+def get_sshpass_command():
+    return env_or_delegate("XPRA_SSHPASS", do_get_sshpass_command)
+def do_get_sshpass_command():
+    import os.path
+    def is_exe(fpath):
+        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)
+    for path in os.environ["PATH"].split(os.pathsep):
+        path = path.strip('"')
+        exe_file = os.path.join(path, "sshpass")
+        if is_exe(exe_file):
+            return exe_file
+    return None
 
 
 #overriden in platform code:
@@ -97,19 +113,27 @@ def do_get_app_dir():
 
 def default_get_app_dir():
     if os.name=="posix":
-        for prefix in [get_install_prefix(),
+        for prefix in [os.environ.get("RPM_BUILD_ROOT"),
+                       get_install_prefix(),
                        sys.exec_prefix,
                        "/usr",
                        "/usr/local"]:
+            if not prefix:
+                continue
             adir = os.path.join(prefix, "share", "xpra")
             if valid_dir(adir):
                 return adir
     adir = os.path.dirname(inspect.getfile(sys._getframe(1)))
+    def root_module(d):
+        pos = d.find("xpra%splatform" % os.path.sep)
+        if pos>=0:
+            return d[:pos]
+        return d
     if valid_dir(adir):
-        return adir
+        return root_module(adir)
     adir = os.path.dirname(sys.argv[0])
     if valid_dir(adir):
-        return adir
+        return root_module(adir)
     adir = os.getcwd()
     return adir       #tried our best, hope this works!
 
@@ -133,23 +157,6 @@ def do_get_icon_dir():
             return idir
     return adir     #better than nothing :(
 
-def get_icon_filename(name):
-    def err(*msg):
-        """ log an error message and return None """
-        from xpra.log import Logger
-        log = Logger("icon")
-        log.error(*msg)
-        return None
-    idir = get_icon_dir()
-    if not idir:
-        return err("cannot find icons directory!")
-    filename = os.path.join(idir, name)
-    if not os.path.exists(filename):
-        return err("icon file %s does not exist", filename)
-    if not os.path.isfile(filename):
-        return err("%s is not a file!", filename)
-    return filename
-
 def get_icon(name):
     filename = get_icon_filename(name)
     if not filename:
@@ -157,6 +164,20 @@ def get_icon(name):
     from xpra.gtk_common.gtk_util import get_icon_from_file
     return get_icon_from_file(filename)
 
+def get_icon_filename(basename=None, ext="png"):
+    if not basename:
+        return None
+    filename = basename
+    fext = os.path.splitext(filename)[1]
+    if not fext:
+        filename = "%s.%s" % (filename, ext)
+    if not os.path.isabs(filename):
+        icon_dir = get_icon_dir()
+        filename = os.path.join(icon_dir, filename)
+    if not os.path.exists(filename):
+        return None
+    return os.path.abspath(filename)
+
 
 LICENSE_TEXT = None
 def get_license_text(self):
@@ -166,7 +187,8 @@ def get_license_text(self):
     filename = os.path.join(get_resources_dir(), 'COPYING')
     if os.path.exists(filename):
         try:
-            if sys.version < '3':
+            from xpra.os_util import PYTHON2
+            if PYTHON2:
                 license_file = open(filename, mode='rb')
             else:
                 license_file = open(filename, mode='r', encoding='ascii')
@@ -178,6 +200,30 @@ def get_license_text(self):
     return LICENSE_TEXT
 
 
+
+def get_xpra_command():
+    envvalue = os.environ.get("XPRA_COMMAND")
+    if envvalue:
+        import shlex
+        return shlex.split(envvalue)
+    return do_get_xpra_command()
+def do_get_xpra_command():
+    #try to use the same "xpra" executable that launched this server:
+    if sys.argv and sys.argv[0].lower().endswith("/xpra"):
+        return [sys.argv[0]]
+    return ["xpra"]
+
+
+def get_nodock_command():
+    envvalue = os.environ.get("XPRA_NODOCK_COMMAND")
+    if envvalue:
+        import shlex
+        return shlex.split(envvalue)
+    return do_get_nodock_command()
+def do_get_nodock_command():
+    return get_xpra_command()
+
+
 def get_sound_command():
     envvalue = os.environ.get("XPRA_SOUND_COMMAND")
     if envvalue:
@@ -185,7 +231,7 @@ def get_sound_command():
         return shlex.split(envvalue)
     return do_get_sound_command()
 def do_get_sound_command():
-    return ["xpra"]
+    return get_xpra_command()
 
 
 from xpra.platform import platform_import
@@ -194,13 +240,16 @@ platform_import(globals(), "paths", True,
                 "do_get_app_dir",
                 "do_get_icon_dir")
 platform_import(globals(), "paths", False,
+                "do_get_sshpass_command",
+                "do_get_xpra_command",
                 "do_get_sound_command",
+                "do_get_nodock_command",
                 "do_get_install_prefix",
                 "do_get_default_conf_dirs",
                 "do_get_system_conf_dirs",
                 "do_get_user_conf_dirs",
                 "do_get_socket_dirs",
-                "do_get_default_log_dir",
+                "do_get_default_log_dirs",
                 "do_get_download_dir",
                 "do_get_script_bin_dirs")
 
@@ -211,14 +260,17 @@ def get_info():
             "system_conf"       : {"dirs"   : get_system_conf_dirs()},
             "user_conf"         : {"dirs"   : get_user_conf_dirs()},
             "socket"            : {"dirs"   : get_socket_dirs()},
-            "log"               : {"dir"    : get_default_log_dir()},
+            "log"               : {"dirs"   : get_default_log_dirs()},
             "download"          : {"dir"    : get_download_dir()},
             "app"               : {"dir"    : get_app_dir()},
             "app"               : {"default" : {"dir"   : default_get_app_dir()}},
             "resources"         : get_resources_dir(),
             "icons"             : get_icon_dir(),
             "home"              : os.path.expanduser("~"),
+            "xpra_command"      : get_xpra_command(),
+            "nodock_command"    : get_nodock_command(),
             "sound_command"     : get_sound_command(),
+            "sshpass_command"   : get_sshpass_command(),
             }
 
 
