@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2014, 2015 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2014-2016 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -11,20 +11,19 @@ import cups
 import time
 import subprocess
 import shlex
-import getpass
 import urllib
 from threading import Lock
 
-from xpra.util import engs
+from xpra.os_util import OSX, PYTHON3
+from xpra.util import engs, envint, envbool
 from xpra.log import Logger
 log = Logger("printing")
 
 
-SIMULATE_PRINT_FAILURE = os.environ.get("XPRA_SIMULATE_PRINT_FAILURE", "0")=="1"
+SIMULATE_PRINT_FAILURE = envint("XPRA_SIMULATE_PRINT_FAILURE")
 
-ALLOW = os.environ.get("XPRA_PRINTER_ALLOW", getpass.getuser())
-RAW_MODE = os.environ.get("XPRA_PRINTER_RAW", "0")=="1"
-GENERIC = os.environ.get("XPRA_PRINTERS_GENERIC", "1")=="1"
+RAW_MODE = envbool("XPRA_PRINTER_RAW", False)
+GENERIC = envbool("XPRA_PRINTERS_GENERIC", True)
 FORWARDER_TMPDIR = os.environ.get("XPRA_FORWARDER_TMPDIR", os.environ.get("TMPDIR", "/tmp"))
 #the mimetype to use for clients that do not specify one
 #(older clients just assumed postscript)
@@ -32,24 +31,26 @@ DEFAULT_MIMETYPE = os.environ.get("XPRA_PRINTER_DEFAULT_MIMETYPE", "application/
 
 LPADMIN = "lpadmin"
 LPINFO = "lpinfo"
+ADD_OPTIONS = ["-E", "-o printer-is-shared=false", "-u allow:$USER"]
+
 FORWARDER_BACKEND = "xpraforwarder"
 
 SKIPPED_PRINTERS = os.environ.get("XPRA_SKIPPED_PRINTERS", "Cups-PDF").split(",")
+CUPS_OPTIONS_WHITELIST = os.environ.get("XPRA_CUPS_OPTIONS_WHITELIST", "Resolution,PageSize").split(",")
 
 #PRINTER_PREFIX = "Xpra:"
-ADD_LOCAL_PRINTERS = os.environ.get("XPRA_ADD_LOCAL_PRINTERS", "0")=="1"
+ADD_LOCAL_PRINTERS = envbool("XPRA_ADD_LOCAL_PRINTERS", False)
 PRINTER_PREFIX = ""
 if ADD_LOCAL_PRINTERS:
     #this prevents problems where we end up deleting local printers!
     PRINTER_PREFIX = "Xpra:"
 PRINTER_PREFIX = os.environ.get("XPRA_PRINTER_PREFIX", PRINTER_PREFIX)
 
-DEFAULT_CUPS_DBUS = str(int(not sys.platform.startswith("darwin")))
-CUPS_DBUS = os.environ.get("XPRA_CUPS_DBUS", DEFAULT_CUPS_DBUS)=="1"
-POLLING_DELAY = int(os.environ.get("XPRA_CUPS_POLLING_DELAY", "60"))
+DEFAULT_CUPS_DBUS = int(not OSX)
+CUPS_DBUS = envint("XPRA_CUPS_DBUS", DEFAULT_CUPS_DBUS)
+POLLING_DELAY = envint("XPRA_CUPS_POLLING_DELAY", 60)
 log("pycups settings: DEFAULT_CUPS_DBUS=%s, CUPS_DBUS=%s, POLLING_DELAY=%s", DEFAULT_CUPS_DBUS, CUPS_DBUS, POLLING_DELAY)
 log("pycups settings: PRINTER_PREFIX=%s, ADD_LOCAL_PRINTERS=%s", PRINTER_PREFIX, ADD_LOCAL_PRINTERS)
-log("pycups settings: ALLOW=%s", ALLOW)
 log("pycups settings: FORWARDER_TMPDIR=%s", FORWARDER_TMPDIR)
 log("pycups settings: SKIPPED_PRINTERS=%s", SKIPPED_PRINTERS)
 
@@ -79,6 +80,10 @@ def set_lpadmin_command(lpadmin):
     global LPADMIN
     LPADMIN = lpadmin
 
+def set_add_printer_options(options):
+    global ADD_OPTIONS
+    ADD_OPTIONS = options
+
 def set_lpinfo_command(lpinfo):
     global LPINFO
     LPINFO = lpinfo
@@ -121,22 +126,30 @@ def get_lpinfo_drv(make_and_model):
     from xpra.util import nonl
     cr = getChildReaper()
     cr.add_process(proc, "lpinfo", command, ignore=True, forget=True)
-    from xpra.make_thread import make_thread
+    from xpra.make_thread import start_thread
     def watch_lpinfo():
         #give it 15 seconds to run:
         for _ in range(15):
             if proc.poll() is not None:
                 return      #finished already
             time.sleep(1)
-        log.warn("Warning: lpinfo command is taking too long")
-        proc.terminate()
-    make_thread(watch_lpinfo, "lpinfo watcher", daemon=True).start()
+        if proc.poll() is not None:
+            return
+        log.warn("Warning: lpinfo command is taking too long,")
+        log.warn(" is the cups server running?")
+        try:
+            proc.terminate()
+        except Exception as e:
+            log("%s.terminate()", proc, exc_info=True)
+            log.error("Error: failed to terminate lpinfo command")
+            log.error(" %s", e)
+    start_thread(watch_lpinfo, "lpinfo watcher", daemon=True)
     out, err = proc.communicate()
     if proc.wait()!=0:
         log.warn("Warning: lpinfo command failed and returned %s", proc.returncode)
         log.warn(" command used: '%s'", " ".join(command))
         return None
-    if sys.version_info[0]>=3:
+    if PYTHON3:
         try:
             out = out.decode()
         except:
@@ -209,29 +222,23 @@ def get_printer_definitions():
         log("pycups settings: PRINTER_DEF=%s", PRINTER_DEF)
     return PRINTER_DEF
 
+def get_printer_definition(mimetype):
+    v = get_printer_definitions().get("application/%s" % mimetype)
+    if not v:
+        return ""
+    if len(v)!=2:
+        return ""
+    if v[0] not in ("-m", "-P"):
+        return ""
+    return v[1]     #ie: /usr/share/ppd/cupsfilters/Generic-PDF_Printer-PDF.ppd
+
 
 def validate_setup():
     #very simple check: at least one ppd file exists
     defs = get_printer_definitions()
     if not defs:
-        log.warn("Warning: no printer definitions found, cannot enable printing")
         return False
-    #check for SELinux
-    try:
-        if os.path.exists("/sys/fs/selinux"):
-            log("SELinux is present")
-            from xpra.os_util import load_binary_file
-            enforce = load_binary_file("/sys/fs/selinux/enforce")
-            log("SELinux enforce=%s", enforce)
-            if enforce=="1":
-                log.warn("SELinux is running in enforcing mode")
-                log.warn(" printer forwarding is unlikely to work without a policy")
-            else:
-                log("SELinux is present but not in enforcing mode")
-    except Exception as e:
-        log.error("Error checking for the presence of SELinux:")
-        log.error(" %s", e)
-    return True
+    return defs
 
 
 def exec_lpadmin(args, success_cb=None):
@@ -251,6 +258,7 @@ def exec_lpadmin(args, success_cb=None):
             from xpra.platform import get_username
             log.warn(" verify that user '%s' has all the required permissions", get_username())
             log.warn(" for running: '%s'", LPADMIN)
+            log.warn(" full command: %s", b" ".join("'%s'" % x for x in command))
         elif success_cb:
             success_cb()
     cr.add_process(proc, "lpadmin", command, ignore=True, forget=True, callback=check_returncode)
@@ -285,13 +293,19 @@ def add_printer(name, options, info, location, attributes={}, success_cb=None):
         log.error("Error: cannot add printer '%s':", name)
         log.error(" the printing system does not support %s", " or ".join(mimetypes))
         return
-    command = ["-p", PRINTER_PREFIX+sanitize_name(name),
-               "-E",
+    command = [
+               "-p", PRINTER_PREFIX+sanitize_name(name),
                "-v", FORWARDER_BACKEND+":"+FORWARDER_TMPDIR+"?"+urllib.urlencode(attributes),
                "-D", info,
                "-L", location,
-               "-o", "printer-is-shared=false",
-               "-u", "allow:%s" % ALLOW]+printer_def
+               ]
+    if ADD_OPTIONS:
+        #ie: ["-E", "-o printer-is-shared=false", "-u allow:$USER"]
+        for opt in ADD_OPTIONS:
+            parts = shlex.split(opt)    #ie: "-u allow:$USER" -> ["-u", "allow:$USER"]
+            for part in parts:          #ie: "allow:$USER"
+                command.append(os.path.expandvars(part))
+    command += printer_def
     #add attributes:
     log("pycups_printing adding printer: %s", command)
     exec_lpadmin(command, success_cb=success_cb)
@@ -324,8 +338,13 @@ def init_dbus_listener():
             sig_match = system_bus.add_signal_receiver(handle_dbus_signal, path=DBUS_PATH, dbus_interface=DBUS_IFACE)
             log("system_bus.add_signal_receiver(..)=%s", sig_match)
             dbus_init = True
+        except ImportError as e:
+            log.warn("Warning: cannot watch for printer device changes,")
+            log.warn(" the dbus bindings seem to be missing:")
+            log.warn(" %s", e)
+            dbus_init = False
         except Exception:
-            if sys.platform.startswith("darwin"):
+            if OSX:
                 log("no dbus on osx")
             else:
                 log.error("failed to initialize dbus cups event listener", exc_info=True)
@@ -338,16 +357,30 @@ def check_printers():
     #fire the callback every time, relying in client_base
     #to notice that nothing has changed and avoid sending the same printers to the server
     log("check_printers() printers_modified_callback=%s", printers_modified_callback)
-    printers_modified_callback()
+    if printers_modified_callback:
+        printers_modified_callback()
     schedule_polling_timer()
 
 _polling_timer = None
 def schedule_polling_timer():
     #fallback to polling:
+    cancel_polling_timer()
     from threading import Timer
     global _polling_timer
     _polling_timer = Timer(POLLING_DELAY, check_printers)
     _polling_timer.start()
+    log("schedule_polling_timer() timer=%s", _polling_timer)
+
+def cancel_polling_timer():
+    global _polling_timer
+    pt = _polling_timer
+    log("cancel_polling_timer() timer=%s", pt)
+    if pt:
+        try:
+            _polling_timer = None
+            pt.cancel()
+        except:
+            pass
 
 def init_printing(callback=None):
     global printers_modified_callback
@@ -358,12 +391,7 @@ def init_printing(callback=None):
         schedule_polling_timer()
 
 def cleanup_printing():
-    global _polling_timer
-    try:
-        _polling_timer.cancel()
-        _polling_timer = None
-    except:
-        pass
+    cancel_polling_timer()
 
 
 def get_printers():
@@ -390,13 +418,18 @@ def print_files(printer, filenames, title, options):
         raise Exception("invalid printer: '%s'" % printer)
     log("pycups.print_files%s", (printer, filenames, title, options))
     actual_options = DEFAULT_CUPS_OPTIONS.copy()
-    actual_options.update(dict((str(k),str(v)) for k,v in options.items()))
+    used_options = dict((str(k),str(v)) for k,v in options.items() if str(k) in CUPS_OPTIONS_WHITELIST)
+    unused_options = dict((str(k),str(v)) for k,v in options.items() if str(k) not in CUPS_OPTIONS_WHITELIST)
+    log("used options=%s", used_options)
+    log("unused options=%s", unused_options)
+    actual_options.update(used_options)
     if SIMULATE_PRINT_FAILURE:
         log.warn("Warning: simulating print failure")
         conn = None
         printpid = -1
     else:
         conn = cups.Connection()
+        log("calling printFiles on %s", conn)
         printpid = conn.printFiles(printer, filenames, title, actual_options)
     if printpid<=0:
         log.error("Error: pycups printing on '%s' failed for file%s", printer, engs(filenames))
@@ -433,7 +466,6 @@ def get_info():
                                    "ppd"        : MIMETYPE_TO_PPD},
             "mimetype"          : {"default"    : DEFAULT_MIMETYPE},
             "simulate-failure"  : SIMULATE_PRINT_FAILURE,
-            "allow-user"        : ALLOW,
             "raw-mode"          : RAW_MODE,
             "generic"           : GENERIC,
             "tmpdir"            : FORWARDER_TMPDIR,
@@ -477,13 +509,18 @@ def main():
             log.info("* %-32s: %s", k, v)
         log.info("")
         log.info("local printers:")
-        printers = get_printers()
-        for k,d in get_all_printers().items():
-            log.info("* %s%s", k, [" (NOT EXPORTED)", ""][int(k in printers)])
-            for pk, pv in d.items():
-                if pk=="printer-state" and pv in PRINTER_STATE:
-                    pv = "%s (%s)" % (pv, PRINTER_STATE.get(pv))
-                log.info("    %-32s: %s", pk, pv)
+        try:
+            printers = get_printers()
+        except RuntimeError as e:
+            log.error("Error accessing the printing system")
+            log.error(" %s", e)
+        else:
+            for k,d in get_all_printers().items():
+                log.info("* %s%s", k, [" (NOT EXPORTED)", ""][int(k in printers)])
+                for pk, pv in d.items():
+                    if pk=="printer-state" and pv in PRINTER_STATE:
+                        pv = "%s (%s)" % (pv, PRINTER_STATE.get(pv))
+                    log.info("    %-32s: %s", pk, pv)
 
 
 if __name__ == "__main__":
