@@ -1,23 +1,24 @@
 # This file is part of Xpra.
-# Copyright (C) 2013, 2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2013-2016 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import os
-import time
 import threading
 from threading import Event
-from xpra.make_thread import make_thread
+
+from xpra.make_thread import start_thread
 from xpra.log import Logger
+from xpra.os_util import monotonic_time
+from xpra.util import envint
 log = Logger("util")
 
 from xpra.platform.features import UI_THREAD_POLLING
-FAKE_UI_LOCKUPS = int(os.environ.get("XPRA_FAKE_UI_LOCKUPS", "0"))
+FAKE_UI_LOCKUPS = envint("XPRA_FAKE_UI_LOCKUPS")
 if FAKE_UI_LOCKUPS>0 and UI_THREAD_POLLING<=0:
     #even if the platform normally disables UI thread polling,
     #we need it for testing:
     UI_THREAD_POLLING = 1000
-POLLING = int(os.environ.get("XPRA_UI_THREAD_POLLING", UI_THREAD_POLLING))
+POLLING = envint("XPRA_UI_THREAD_POLLING", UI_THREAD_POLLING)
 
 
 class UI_thread_watcher(object):
@@ -52,7 +53,7 @@ class UI_thread_watcher(object):
         #run once to initialize:
         self.UI_thread_wakeup()
         if self.polling_timeout>0:
-            make_thread(self.poll_UI_loop, "UI thread polling").start()
+            start_thread(self.poll_UI_loop, "UI thread polling")
         else:
             log("not starting an IO polling thread")
         if FAKE_UI_LOCKUPS>0:
@@ -60,6 +61,7 @@ class UI_thread_watcher(object):
             def sleep_in_ui_thread(*args):
                 t = threading.current_thread()
                 log.warn("sleep_in_ui_thread%s pausing %s for %ims", args, t, FAKE_UI_LOCKUPS)
+                import time
                 time.sleep(FAKE_UI_LOCKUPS/1000.0)
                 return True
             self.timeout_add(10*1000+FAKE_UI_LOCKUPS, sleep_in_ui_thread)
@@ -85,7 +87,7 @@ class UI_thread_watcher(object):
 
     def UI_thread_wakeup(self):
         log("UI_thread_wakeup()")
-        self.last_UI_thread_time = time.time()
+        self.last_UI_thread_time = monotonic_time()
         #UI thread was blocked?
         if self.UI_blocked:
             log.info("UI thread is running again, resuming")
@@ -96,7 +98,7 @@ class UI_thread_watcher(object):
     def poll_UI_loop(self):
         log("poll_UI_loop() running")
         while not self.exit.isSet():
-            delta = time.time()-self.last_UI_thread_time
+            delta = monotonic_time()-self.last_UI_thread_time
             log("poll_UI_loop() last_UI_thread_time was %.1f seconds ago (max %i), UI_blocked=%s", delta, self.max_delta/1000, self.UI_blocked)
             if delta>self.max_delta/1000.0:
                 #UI thread is (still?) blocked:
@@ -109,15 +111,20 @@ class UI_thread_watcher(object):
                 log("poll_UI_loop() ok, firing %s", self.alive_callbacks)
                 self.run_callbacks(self.alive_callbacks)
             self.timeout_add(0, self.UI_thread_wakeup)
-            wstart = time.time()
+            wstart = monotonic_time()
             wait_time = self.polling_timeout/1000.0     #convert to seconds
             self.exit.wait(wait_time)
             if not self.exit.isSet():
-                wdelta = time.time() - wstart
+                wdelta = monotonic_time() - wstart
                 log("wait(%.4f) actually waited %.4f", self.polling_timeout/1000.0, wdelta)
                 if wdelta>(wait_time+1):
+                    #this can be caused by an ntp update?
+                    #or just by suspend + resume
                     log.warn("Warning: long timer waiting time,")
                     log.warn(" UI thread polling waited %.1f seconds longer than intended (%.1f vs %.1f)", wdelta-wait_time, wdelta, wait_time)
+                    #force run resume (even if we never fired the fail callbacks)
+                    self.UI_blocked = False
+                    self.UI_thread_wakeup()
         self.init_vars()
         log("poll_UI_loop() ended")
 
