@@ -6,7 +6,7 @@
 
 # Platform-specific settings for Win32.
 CAN_DAEMONIZE = False
-MMAP_SUPPORTED = False
+MMAP_SUPPORTED = True
 SYSTEM_TRAY_SUPPORTED = True
 REINIT_WINDOWS = True
 
@@ -18,4 +18,7 @@ CLIPBOARD_NATIVE_CLASS = "xpra.clipboard.translated_clipboard.TranslatedClipboar
 DEFAULT_PULSEAUDIO_COMMAND = ""
 DEFAULT_PULSEAUDIO_CONFIGURE_COMMANDS = []
 PRINT_COMMAND = ""
-DEFAULT_SSH_COMMAND="plink -ssh -agent"
\ No newline at end of file
+DEFAULT_SSH_COMMAND="plink -ssh -agent"
+
+#not implemented:
+SYSTEM_PROXY_SOCKET = "xpra-proxy"
