@@ -1,34 +1,46 @@
 # This file is part of Xpra.
-# Copyright (C) 2014, 2015 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2014-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 from xpra.log import Logger
+from xpra.platform.paths import get_app_dir
 log = Logger("printing")
 
 import os.path
 import subprocess
-import win32print       #@UnresolvedImport
-import win32con         #@UnresolvedImport
-
-from xpra.util import csv
+from xpra.platform.win32 import constants as win32con
+from xpra.util import csv, envint, envbool
 
 
 #allows us to skip some printers we don't want to export
 SKIPPED_PRINTERS = os.environ.get("XPRA_SKIPPED_PRINTERS", "Microsoft XPS Document Writer,Fax").split(",")
+PDFIUM_PRINT = envbool("XPRA_PDFIUM_PRINT", True)
 
 
-PRINTER_ENUM_VALUES = {}
-PRINTER_ENUM_NAMES = {}
-for k in ("LOCAL", "NAME", "SHARED", "CONNECTIONS",
-          "NETWORK", "REMOTE", "CATEGORY_3D", "CATEGORY_ALL"):
-    v = getattr(win32print, "PRINTER_ENUM_%s" % k, None)
-    if v is not None:
-        PRINTER_ENUM_VALUES[k] = v
-        PRINTER_ENUM_NAMES[v] = k
+PRINTER_ENUM_VALUES = {
+    "DEFAULT"       : 1,
+    "LOCAL"         : 2,
+    "CONNECTIONS"   : 4,
+    "NAME"          : 8,
+    "REMOTE"        : 16,
+    "SHARED"        : 32,
+    "NETWORK"       : 64,
+    "EXPAND"        : 16384,
+    "CONTAINER"     : 32768,
+    "ICON1"         : 65536*1,
+    "ICON2"         : 65536*2,
+    "ICON3"         : 65536*3,
+    "ICON4"         : 65536*4,
+    "ICON5"         : 65536*5,
+    "ICON6"         : 65536*6,
+    "ICON7"         : 65536*7,
+    "ICON8"         : 65536*8,
+    }
+PRINTER_ENUM_NAMES = dict((v,k) for k,v in PRINTER_ENUM_VALUES.items())
 log("PRINTER_ENUM_VALUES: %s", PRINTER_ENUM_VALUES)
 
-PRINTER_LEVEL = int(os.environ.get("XPRA_WIN32_PRINTER_LEVEL", "1"))
+PRINTER_LEVEL = envint("XPRA_WIN32_PRINTER_LEVEL", 1)
 #DEFAULT_PRINTER_FLAGS = "LOCAL"
 DEFAULT_PRINTER_FLAGS = "LOCAL,SHARED+NETWORK+CONNECTIONS"
 PRINTER_FLAGS = [x.strip() for x in os.environ.get("XPRA_WIN32_PRINTER_FLAGS", DEFAULT_PRINTER_FLAGS).split(",")]
@@ -53,20 +65,22 @@ PROCESSES = {}
 
 GSVIEW_DIR = None
 GSPRINT_EXE = None
-GSWIN32C_EXE = None
+GSWINXXC_EXE = None
 printers_modified_callback = None
 def init_printing(callback=None):
-    global printers_modified_callback, GSVIEW_DIR, GSPRINT_EXE, GSWIN32C_EXE
+    global printers_modified_callback, GSVIEW_DIR, GSPRINT_EXE, GSWINXXC_EXE
     log("init_printing(%s) printers_modified_callback=%s", callback, printers_modified_callback)
     printers_modified_callback = callback
     #ensure we can find gsprint.exe in a subdirectory:
-    from xpra.platform.paths import get_app_dir
     GSVIEW_DIR = os.path.join(get_app_dir(), "gsview")
     assert os.path.exists(GSVIEW_DIR) and os.path.isdir(GSVIEW_DIR), "cannot find gsview directory in '%s'" % GSVIEW_DIR
     GSPRINT_EXE = os.path.join(GSVIEW_DIR, "gsprint.exe")
     assert os.path.exists(GSPRINT_EXE), "cannot find gsprint.exe in '%s'" % GSVIEW_DIR
-    GSWIN32C_EXE = os.path.join(GSVIEW_DIR, "gswin32c.exe")
-    assert os.path.exists(GSWIN32C_EXE), "cannot find gswin32c.exe in '%s'" % GSVIEW_DIR
+    for bits in (32, 64):
+        GSWINXXC_EXE = os.path.join(GSVIEW_DIR, "gswin%sc.exe" % bits)
+        if os.path.exists(GSWINXXC_EXE):
+            break
+    assert os.path.exists(GSWINXXC_EXE), "cannot find gswin??c.exe in '%s'" % GSVIEW_DIR
     try:
         init_winspool_listener()
     except Exception:
@@ -86,14 +100,70 @@ def on_devmodechange(wParam, lParam):
         printers_modified_callback()
 
 
+def EnumPrinters(flags, name=None, level=PRINTER_LEVEL):
+    import ctypes
+    from ctypes.wintypes import BYTE, DWORD, LPCWSTR
+
+    winspool = ctypes.WinDLL('winspool.drv', use_last_error=True)
+    msvcrt = ctypes.cdll.msvcrt
+
+    class PRINTER_INFO(ctypes.Structure):
+        _fields_ = [
+            ("Flags", DWORD),
+            ("pDescription", LPCWSTR),
+            ("pName", LPCWSTR),
+            ("pComment", LPCWSTR),
+        ]
+
+    # Invoke once with a NULL pointer to get buffer size.
+    info = ctypes.POINTER(BYTE)()
+    pcbNeeded = DWORD(0)
+    pcReturned = DWORD(0)  # the number of PRINTER_INFO_1 structures retrieved
+    r = winspool.EnumPrintersW(DWORD(flags), name, DWORD(level), ctypes.byref(info), DWORD(0), ctypes.byref(pcbNeeded), ctypes.byref(pcReturned))
+    log("EnumPrintersW(..)=%i pcbNeeded=%i", r, pcbNeeded.value)
+    if pcbNeeded.value<=0:
+        log("EnumPrinters probe failed for flags=%i, level=%i, pcbNeeded=%i", flags, level, pcbNeeded.value)
+        return []
+
+    bufsize = pcbNeeded.value
+    buf = msvcrt.malloc(bufsize)
+    if buf==0:
+        log.error("Error: cannot enumerate printers, malloc failed")
+        return []
+
+    r = winspool.EnumPrintersW(DWORD(flags), name, DWORD(level), buf, bufsize, ctypes.byref(pcbNeeded), ctypes.byref(pcReturned))
+    log("EnumPrintersW(..)=%i pcReturned=%i", r, pcReturned.value)
+    if r==0:
+        log.error("Error: EnumPrinters failed")
+        return []
+    info = ctypes.cast(buf, ctypes.POINTER(PRINTER_INFO))
+    printers = []
+    for i in range(pcReturned.value):
+        v = int(info[i].Flags), str(info[i].pDescription), str(info[i].pName), str(info[i].pComment)
+        log("EnumPrintersW(..) [%i]=%s", i, v)
+        printers.append(v)
+    msvcrt.free(buf)
+    return printers
+
+def get_info():
+    from xpra.platform.printing import default_get_info
+    i = default_get_info()
+    #win32 extras:
+    i.update({
+        "skipped-printers"      : SKIPPED_PRINTERS,
+        "printer-level"         : PRINTER_LEVEL,
+        "default-printer-flags" : DEFAULT_PRINTER_FLAGS,
+        "printer-flags"         : PRINTER_FLAGS,
+        "commands" : {
+            "gsview"    : GSPRINT_EXE or "",
+            "gswin"     : GSWINXXC_EXE or "",
+            },
+        })
+    return i
+
 def get_printers():
     global PRINTER_ENUMS, PRINTER_ENUM_VALUES, SKIPPED_PRINTERS, PRINTER_LEVEL, GSVIEW_DIR
     printers = {}
-    if not GSVIEW_DIR:
-        #without gsprint, we can't handle printing!
-        log("get_printers() gsview is missing, not querying any printers")
-        return printers
-    #default_printer = win32print.GetDefaultPrinter()
     for penum in PRINTER_ENUMS:
         try:
             eprinters = []
@@ -102,15 +172,15 @@ def get_printers():
             log("enum(%s)=%s=%s", penum, "+".join(str(x) for x in enum_values), enum_val)
             assert enum_val is not None, "invalid printer enum %s" % penum
             log("querying %s printers with level=%s", penum, PRINTER_LEVEL)
-            for p in win32print.EnumPrinters(enum_val, None, PRINTER_LEVEL):
+            for p in EnumPrinters(enum_val, None, PRINTER_LEVEL):
                 flags, desc, name, comment = p
                 if name in SKIPPED_PRINTERS:
-                    log("skipped printer: %s, %s, %s, %s", flags, desc, name, comment)
+                    log("skipped printer: %#x, %s, %s, %s", flags, desc, name, comment)
                     continue
                 if name in printers:
-                    log("skipped duplicate printer: %s, %s, %s, %s", flags, desc, name, comment)
+                    log("skipped duplicate printer: %#x, %s, %s, %s", flags, desc, name, comment)
                     continue
-                log("found printer: %s, %s, %s, %s", flags, desc, name, comment)
+                log("found printer: %#x, %s, %s, %s", flags, desc, name, comment)
                 #strip duplicated and empty strings from the description:
                 desc_els = []
                 [desc_els.append(x) for x in desc.split(",") if (x and not desc_els.count(x))]
@@ -129,23 +199,29 @@ def get_printers():
 
 def print_files(printer, filenames, title, options):
     log("win32.print_files%s", (printer, filenames, title, options))
-    global JOB_ID, PROCESSES, GSVIEW_DIR, GSPRINT_EXE, GSWIN32C_EXE
+    global JOB_ID, PROCESSES, GSVIEW_DIR, GSPRINT_EXE, GSWINXXC_EXE
     assert GSVIEW_DIR, "cannot print files without gsprint!"
     processes = []
     for filename in filenames:
         #command = ["C:\\Program Files\\Xpra\\gsview\\gsprint.exe"]
-        command = [GSPRINT_EXE, "-ghostscript", GSWIN32C_EXE, "-colour"]
-        if printer:
-            command += ["-printer", printer]
-        command += [filename]
+        if PDFIUM_PRINT:
+            cwd = get_app_dir()
+            command = ["PDFIUM_Print.exe", filename, printer, title]
+        else:
+            cwd = GSVIEW_DIR
+            command = [GSPRINT_EXE, "-ghostscript", GSWINXXC_EXE, "-colour"]
+            if printer:
+                command += ["-printer", printer]
+            command += [filename]
+            #add gsview directory
+            PATH = os.environ.get("PATH")
+            os.environ["PATH"] = str(PATH+";"+GSVIEW_DIR)
+            log("environment: %s", os.environ)
         log("print command: %s", command)
-        #add gsview directory
-        PATH = os.environ.get("PATH")
-        os.environ["PATH"] = str(PATH+";"+GSVIEW_DIR)
-        log("environment: %s", os.environ)
         startupinfo = subprocess.STARTUPINFO()
         startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
-        process = subprocess.Popen(command, stdin=None, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=GSVIEW_DIR, startupinfo=startupinfo)
+        startupinfo.wShowWindow = 0     #aka win32.con.SW_HIDE
+        process = subprocess.Popen(command, stdin=None, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=cwd, startupinfo=startupinfo)
         process.print_filename = filename
         #we just let it run, no need for reaping the process on win32
         processes.append(process)
