@@ -100,6 +100,8 @@ WNDPROC_EVENT_NAMES = {
     0x00ab: "WM_NCXBUTTONDOWN",
     0x00ac: "WM_NCXBUTTONUP",
     0x00ad: "WM_NCXBUTTONDBLCLK",
+    0x00ae: "WM_NCUAHDRAWCAPTION",
+    0x00af: "WM_NCUAHDRAWFRAME",
     0x00e0: "SBM_SETPOS",
     0x00e1: "SBM_GETPOS",
     0x00e2: "SBM_SETRANGE",
@@ -211,6 +213,7 @@ WNDPROC_EVENT_NAMES = {
     0x02a1: "WM_MOUSEHOVER",
     0x02a2: "WM_NCMOUSELEAVE",
     0x02a3: "WM_MOUSELEAVE",
+    0x02b1: "WM_WTSSESSION_CHANGE",
     0x0300: "WM_CUT",
     0x0301: "WM_COPY",
     0x0302: "WM_PASTE",
@@ -233,6 +236,7 @@ WNDPROC_EVENT_NAMES = {
     0x0317: "WM_PRINT",
     0x0318: "WM_PRINTCLIENT",
     0x0319: "WM_APPCOMMAND",
+    0x031E: "WM_DWMCOMPOSITIONCHANGED",
     0x0358: "WM_HANDHELDFIRST",
     0x035f: "WM_HANDHELDLAST",
     0x0360: "WM_AFXFIRST",
@@ -740,4 +744,4 @@ WNDPROC_EVENT_NAMES = {
     0x2210: "OCM_PARENTNOTIFY",
     0x8000: "WM_APP",
     0xcccd: "WM_RASDIALEVENT",
-}
\ No newline at end of file
+}
