@@ -3,17 +3,11 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import sys
-
 #don't bother trying to forward system tray with Ubuntu's "unity":
-from xpra.os_util import is_unity, is_Ubuntu, is_Fedora
+from xpra.os_util import is_unity, PYTHON2
 SYSTEM_TRAY_SUPPORTED = not is_unity()
-#this is only our best guess
-#there is more logic in setup.py, but it requires more effort too:
-XDUMMY = not is_Ubuntu() and not sys.platform.find("bsd")>=0
-#displayfd requires Xdummy, and we don't support servers with py3k:
-DISPLAYFD = XDUMMY and sys.version_info[0]<3
-XDUMMY_WRAPPER = is_Fedora()
+
+SHADOW_SUPPORTED = PYTHON2
 
 DEFAULT_ENV = [
              "#avoid Ubuntu's global menu, which is a mess and cannot be forwarded:",
@@ -29,3 +23,5 @@ DEFAULT_ENV = [
 
 DEFAULT_SSH_CMD = "ssh"
 CLIPBOARDS=["CLIPBOARD", "PRIMARY", "SECONDARY"]
+
+INPUT_DEVICES = ["auto", "xi"]
