@@ -1,35 +1,56 @@
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
+import sys
 import struct
 import binascii
+
 from xpra.log import Logger
 log = Logger("posix")
 eventlog = Logger("posix", "events")
 screenlog = Logger("posix", "screen")
 dbuslog = Logger("posix", "dbus")
-traylog = Logger("posix", "menu")
+traylog = Logger("posix", "tray")
 menulog = Logger("posix", "menu")
+mouselog = Logger("posix", "mouse")
 
-from xpra.util import iround
+from xpra.os_util import strtobytes, bytestostr
+from xpra.util import iround, envbool, csv
 from xpra.gtk_common.gobject_compat import get_xid, is_gtk3
 
+try:
+    from xpra.x11.bindings.window_bindings import X11WindowBindings
+    from xpra.x11.bindings.xi2_bindings import X11XI2Bindings   #@UnresolvedImport
+except Exception as e:
+    log.error("no X11 bindings", exc_info=True)
+    X11WindowBindings = None
+    X11XI2Bindings = None
+
 device_bell = None
-GTK_MENUS = os.environ.get("XPRA_GTK_MENUS", "0")=="1"
+GTK_MENUS = envbool("XPRA_GTK_MENUS", False)
+RANDR_DPI = envbool("XPRA_RANDR_DPI", True)
+XSETTINGS_DPI = envbool("XPRA_XSETTINGS_DPI", True)
+USE_NATIVE_TRAY = envbool("XPRA_USE_NATIVE_TRAY", True)
+
+
+def hexstr(v):
+    return binascii.hexlify(strtobytes(v))
 
 
 def get_native_system_tray_classes():
-    try:
-        from xpra.platform.xposix.appindicator_tray import AppindicatorTray, can_use_appindicator
-        if can_use_appindicator():
-            return [AppindicatorTray]
-    except Exception as e:
-        traylog("cannot load appindicator tray: %s", e)
-    return []
+    c = []
+    if USE_NATIVE_TRAY:
+        try:
+            from xpra.platform.xposix.appindicator_tray import AppindicatorTray, can_use_appindicator
+            if can_use_appindicator():
+                c.append(AppindicatorTray)
+        except Exception as e:
+            traylog("cannot load appindicator tray: %s", e)
+    return c
 
 def get_wm_name():
     wm_name = os.environ.get("XDG_CURRENT_DESKTOP", "")
@@ -72,28 +93,32 @@ def get_native_notifier_classes():
     return ncs
 
 
+def get_session_type():
+    return os.environ.get("XDG_SESSION_TYPE", "")
+
+
 #we duplicate some of the code found in gtk_x11.prop ...
 #which is still better than having dependencies on that GTK2 code
 def _get_X11_window_property(xid, name, req_type):
     try:
         from xpra.gtk_common.error import xsync
-        from xpra.x11.bindings.window_bindings import X11WindowBindings, PropertyError #@UnresolvedImport
-        window_bindings = X11WindowBindings()
+        from xpra.x11.bindings.window_bindings import PropertyError #@UnresolvedImport
         try:
+            X11Window = X11WindowBindings()
             with xsync:
-                prop = window_bindings.XGetWindowProperty(xid, name, req_type)
+                prop = X11Window.XGetWindowProperty(xid, name, req_type)
             log("_get_X11_window_property(%#x, %s, %s)=%s, len=%s", xid, name, req_type, type(prop), len(prop or []))
             return prop
         except PropertyError as e:
             log("_get_X11_window_property(%#x, %s, %s): %s", xid, name, req_type, e)
     except Exception as e:
-        log.warn("failed to get X11 window property %s on window %#x: %s", name, xid, e)
+        log.warn("Warning: failed to get X11 window property '%s' on window %#x: %s", name, xid, e)
+        log("get_X11_window_property%s", (xid, name, req_type), exc_info=True)
     return None
 def _get_X11_root_property(name, req_type):
     try:
-        from xpra.x11.bindings.window_bindings import X11WindowBindings #@UnresolvedImport
-        window_bindings = X11WindowBindings()
-        root_xid = window_bindings.getDefaultRootWindow()
+        X11Window = X11WindowBindings()
+        root_xid = X11Window.getDefaultRootWindow()
         return _get_X11_window_property(root_xid, name, req_type)
     except Exception as e:
         log.warn("Warning: failed to get X11 root property '%s'", name)
@@ -152,21 +177,22 @@ def get_menu_support_function():
 
 
 def _get_xsettings():
+    from xpra.gtk_common.error import xsync
     try:
-        from xpra.x11.bindings.window_bindings import X11WindowBindings #@UnresolvedImport
-        window_bindings = X11WindowBindings()
-        selection = "_XSETTINGS_S0"
-        owner = window_bindings.XGetSelectionOwner(selection)
-        if not owner:
-            return None
-        XSETTINGS = "_XSETTINGS_SETTINGS"
-        data = window_bindings.XGetWindowProperty(owner, XSETTINGS, XSETTINGS)
-        if not data:
-            return None
-        from xpra.x11.xsettings_prop import get_settings
-        return get_settings(window_bindings.get_display_name(), data)
+        X11Window = X11WindowBindings()
+        with xsync:
+            selection = "_XSETTINGS_S0"
+            owner = X11Window.XGetSelectionOwner(selection)
+            if not owner:
+                return None
+            XSETTINGS = "_XSETTINGS_SETTINGS"
+            data = X11Window.XGetWindowProperty(owner, XSETTINGS, XSETTINGS)
+            if not data:
+                return None
+            from xpra.x11.xsettings_prop import get_settings
+            return get_settings(X11Window.get_display_name(), data)
     except Exception as e:
-        log("_get_xsettings error: %s", e)
+        log("_get_xsettings()", exc_info=True)
     return None
 
 def _get_xsettings_dict():
@@ -175,37 +201,40 @@ def _get_xsettings_dict():
     if v:
         _, values = v
         for setting_type, prop_name, value, _ in values:
-            d[prop_name] = (setting_type, value)
+            d[bytestostr(prop_name)] = (setting_type, value)
     return d
 
 
 def _get_xsettings_dpi():
-    from xpra.x11.xsettings_prop import XSettingsTypeInteger
-    d = _get_xsettings_dict()
-    for k,div in {"Xft.dpi"         : 1,
-                  "Xft/DPI"         : 1024,
-                  "gnome.Xft/DPI"   : 1024,
-                  #"Gdk/UnscaledDPI" : 1024, ??
-                  }.items():
-        if k in d:
-            value_type, value = d.get(k)
-            if value_type==XSettingsTypeInteger:
-                screenlog("_get_xsettings_dpi() found %s=%s", k, value)
-                return max(10, min(1000, value/div))
+    if XSETTINGS_DPI:
+        from xpra.x11.xsettings_prop import XSettingsTypeInteger
+        d = _get_xsettings_dict()
+        for k,div in {
+            "Xft.dpi"         : 1,
+            "Xft/DPI"         : 1024,
+            "gnome.Xft/DPI"   : 1024,
+            #"Gdk/UnscaledDPI" : 1024, ??
+            }.items():
+            if k in d:
+                value_type, value = d.get(k)
+                if value_type==XSettingsTypeInteger:
+                    screenlog("_get_xsettings_dpi() found %s=%s", k, value)
+                    return max(10, min(1000, value/div))
     return -1
 
 def _get_randr_dpi():
-    try:
-        from xpra.x11.bindings.randr_bindings import RandRBindings  #@UnresolvedImport
-        randr_bindings = RandRBindings()
-        wmm, hmm = randr_bindings.get_screen_size_mm()
-        w, h =  randr_bindings.get_screen_size()
-        dpix = iround(w * 25.4 / wmm)
-        dpiy = iround(h * 25.4 / hmm)
-        screenlog("dpix=%s, dpiy=%s", dpix, dpiy)
-        return dpix, dpiy
-    except Exception as e:
-        screenlog.warn("failed to get dpi: %s", e)
+    if RANDR_DPI:
+        try:
+            from xpra.x11.bindings.randr_bindings import RandRBindings  #@UnresolvedImport
+            randr_bindings = RandRBindings()
+            wmm, hmm = randr_bindings.get_screen_size_mm()
+            w, h =  randr_bindings.get_screen_size()
+            dpix = iround(w * 25.4 / wmm)
+            dpiy = iround(h * 25.4 / hmm)
+            screenlog("xdpi=%s, ydpi=%s - size-mm=%ix%i, size=%ix%i", dpix, dpiy, wmm, hmm, w, h)
+            return dpix, dpiy
+        except Exception as e:
+            screenlog.warn("failed to get dpi: %s", e)
     return -1, -1
 
 def get_xdpi():
@@ -221,6 +250,31 @@ def get_ydpi():
     return _get_randr_dpi()[1]
 
 
+def get_icc_info():
+    try:
+        data = _get_X11_root_property("_ICC_PROFILE", "CARDINAL")
+        if data:
+            screenlog("_ICC_PROFILE=%s (%s)", type(data), len(data))
+            version = _get_X11_root_property("_ICC_PROFILE_IN_X_VERSION", "CARDINAL")
+            screenlog("get_icc_info() found _ICC_PROFILE_IN_X_VERSION=%s, _ICC_PROFILE=%s", hexstr(version or ""), hexstr(data))
+            icc = {
+                    "source"    : "_ICC_PROFILE",
+                    "data"      : data,
+                    }
+            if version:
+                try:
+                    version = ord(version)
+                except:
+                    pass
+                icc["version"] = version
+            return icc
+    except Exception as e:
+        screenlog.error("Error: cannot access _ICC_PROFILE X11 window property")
+        screenlog.error(" %s", e)
+        screenlog("get_icc_info()", exc_info=True)
+    return {}
+
+
 def get_antialias_info():
     info = {}
     try:
@@ -265,7 +319,7 @@ def get_current_desktop():
             v = struct.unpack("=I", d)[0]
     except Exception as e:
         log.warn("failed to get current desktop: %s", e)
-    log("get_current_desktop() %s=%s", binascii.hexlify(d or ""), v)
+    log("get_current_desktop() %s=%s", hexstr(d or ""), v)
     return v
 
 def get_workarea():
@@ -283,7 +337,7 @@ def get_workarea():
         else:
             cur_workarea = workarea[d*4*4:(d+1)*4*4]
             v = struct.unpack("=IIII", cur_workarea)
-            screenlog("get_workarea() %s=%s", binascii.hexlify(cur_workarea), v)
+            screenlog("get_workarea() %s=%s", hexstr(cur_workarea), v)
             return v
     except Exception as e:
         screenlog.warn("failed to get workarea: %s", e)
@@ -300,7 +354,7 @@ def get_number_of_desktops():
     except Exception as e:
         screenlog.warn("failed to get number of desktop: %s", e)
     v = max(1, v)
-    screenlog("get_number_of_desktops() %s=%s", binascii.hexlify(d or ""), v)
+    screenlog("get_number_of_desktops() %s=%s", hexstr(d or ""), v)
     return v
 
 def get_desktop_names():
@@ -314,7 +368,7 @@ def get_desktop_names():
                 v = v[:-1]
     except Exception as e:
         screenlog.warn("failed to get desktop names: %s", e)
-    screenlog("get_desktop_names() %s=%s", binascii.hexlify(d or ""), v)
+    screenlog("get_desktop_names() %s=%s", hexstr(d or ""), v)
     return v
 
 
@@ -414,9 +468,9 @@ def system_bell(window, device, percent, pitch, duration, bell_class, bell_id, b
 
 def _send_client_message(window, message_type, *values):
     try:
-        from xpra.x11.gtk2 import gdk_display_source
-        assert gdk_display_source
-        from xpra.x11.bindings.window_bindings import constants, X11WindowBindings  #@UnresolvedImport
+        from xpra.x11.gtk_x11.gdk_display_source import init_display_source
+        init_display_source()
+        from xpra.x11.bindings.window_bindings import constants #@UnresolvedImport
         X11Window = X11WindowBindings()
         root_xid = X11Window.getDefaultRootWindow()
         if window:
@@ -454,6 +508,22 @@ def set_shaded(window, shaded):
     _toggle_wm_state(window, "_NET_WM_STATE_SHADED", shaded)
 
 
+
+WINDOW_ADD_HOOKS = []
+def add_window_hooks(window):
+    global WINDOW_ADD_HOOKS
+    for x in WINDOW_ADD_HOOKS:
+        x(window)
+    log("add_window_hooks(%s) added %s", window, WINDOW_ADD_HOOKS)
+
+WINDOW_REMOVE_HOOKS = []
+def remove_window_hooks(window):
+    global WINDOW_REMOVE_HOOKS
+    for x in WINDOW_REMOVE_HOOKS:
+        x(window)
+    log("remove_window_hooks(%s) added %s", window, WINDOW_REMOVE_HOOKS)
+
+
 def get_info():
     from xpra.platform.gui import get_info_base
     i = get_info_base()
@@ -462,7 +532,7 @@ def get_info():
         serial, values = s
         xi = {"serial"  : serial}
         for _,name,value,_ in values:
-            xi[name] = value
+            xi[bytestostr(name)] = value
         i["xsettings"] = xi
     i.setdefault("dpi", {
                          "xsettings"    : _get_xsettings_dpi(),
@@ -471,6 +541,105 @@ def get_info():
     return i
 
 
+class XI2_Window(object):
+    def __init__(self, window):
+        log("XI2_Window(%s)", window)
+        self.XI2 = X11XI2Bindings()
+        self.X11Window = X11WindowBindings()
+        self.window = window
+        self.xid = window.get_window().xid
+        self.windows = ()
+        window.connect("configure-event", self.configured)
+        self.configured()
+        #replace event handlers with XI2 version:
+        self.do_motion_notify_event = window.do_motion_notify_event
+        window.do_motion_notify_event = self.noop
+        window.do_button_press_event = self.noop
+        window.do_button_release_event = self.noop
+        window.do_scroll_event = self.noop
+        window.connect("destroy", self.cleanup)
+
+    def noop(self, *args):
+        pass
+
+    def cleanup(self, *args):
+        for window in self.windows:
+            self.XI2.disconnect(window)
+        self.windows = []
+        self.window = None
+
+    def configured(self, *args):
+        self.windows = self.get_parent_windows(self.xid)
+        for window in self.windows:
+            self.XI2.connect(window, "XI_Motion", self.do_xi_motion)
+            self.XI2.connect(window, "XI_ButtonPress", self.do_xi_button)
+            self.XI2.connect(window, "XI_ButtonRelease", self.do_xi_button)
+
+    def get_parent_windows(self, oxid):
+        windows = [oxid]
+        root = self.X11Window.getDefaultRootWindow()
+        xid = oxid
+        while True:
+            xid = self.X11Window.getParent(xid)
+            if xid==0 or xid==root:
+                break
+            windows.append(xid)
+        log("get_parent_windows(%#x)=%s", oxid, csv(hex(x) for x in windows))
+        return windows
+
+
+    def do_xi_button(self, event):
+        window = self.window
+        client = window._client
+        if client.readonly:
+            return
+        if client.server_input_devices=="xi":
+            #skip synthetic scroll events for two-finger scroll,
+            #as the server should synthesize them from the motion events
+            #those have the same serial:
+            matching_motion = self.XI2.find_event("XI_Motion", event.serial)
+            #maybe we need more to distinguish?
+            if matching_motion:
+                return
+        button = event.detail
+        depressed = (event.name == "XI_ButtonPress")
+        args = self.get_pointer_extra_args(event)
+        window._button_action(button, event, depressed, *args)
+
+    def do_xi_motion(self, event):
+        window = self.window
+        if window.moveresize_event:
+            window.motion_moveresize(event)
+            self.do_motion_notify_event(event)
+            return
+        if window._client.readonly:
+            return
+        #find the motion events in the xi2 event list:
+        pointer, modifiers, buttons = window._pointer_modifiers(event)
+        wid = self.window.get_mouse_event_wid(*pointer)
+        mouselog("do_motion_notify_event(%s) wid=%s / focus=%s / window wid=%i, device=%s, pointer=%s, modifiers=%s, buttons=%s", event, wid, window._client._focused, self.window._id, event.device, pointer, modifiers, buttons)
+        packet = ["pointer-position", wid, pointer, modifiers, buttons] + self.get_pointer_extra_args(event)
+        window._client.send_mouse_position(packet)
+
+    def get_pointer_extra_args(self, event):
+        def intscaled(f):
+            return int(f*1000000), 1000000
+        def dictscaled(v):
+            return dict((k,intscaled(v)) for k,v in v.items())
+        raw_valuators = {}
+        raw_event_name = event.name.replace("XI_", "XI_Raw")    #ie: XI_Motion -> XI_RawMotion
+        raw = self.XI2.find_event(raw_event_name, event.serial)
+        #mouselog("raw(%s)=%s", raw_event_name, raw)
+        if raw:
+            raw_valuators = raw.raw_valuators
+        args = [event.device]
+        for x in ("x", "y", "x_root", "y_root"):
+            args.append(intscaled(getattr(event, x)))
+        for v in [event.valuators, raw_valuators]:
+            args.append(dictscaled(v))
+        return args
+
+
 class ClientExtras(object):
     def __init__(self, client, opts):
         self.client = client
@@ -480,15 +649,34 @@ class ClientExtras(object):
         self.upower_resuming_match = None
         self.upower_sleeping_match = None
         self.login1_match = None
+        self.x11_filter = None
         if client.xsettings_enabled:
             self.setup_xprops()
+        if client.input_devices=="xi":
+            #this would trigger warnings with our temporary opengl windows:
+            #only enable it after we have connected:
+            self.client.after_handshake(self.setup_xi)
         self.setup_dbus_signals()
 
     def ready(self):
         pass
 
+    def init_x11_filter(self):
+        if self.x11_filter:
+            return
+        try:
+            from xpra.x11.gtk2.gdk_bindings import init_x11_filter  #@UnresolvedImport
+            self.x11_filter = init_x11_filter()
+            log("x11_filter=%s", self.x11_filter)
+        except:
+            self.x11_filter = None
+
     def cleanup(self):
         log("cleanup() xsettings_watcher=%s, root_props_watcher=%s", self._xsettings_watcher, self._root_props_watcher)
+        if self.x11_filter:
+            from xpra.x11.gtk2.gdk_bindings import cleanup_x11_filter   #@UnresolvedImport
+            self.x11_filter = None
+            cleanup_x11_filter()
         if self._xsettings_watcher:
             self._xsettings_watcher.cleanup()
             self._xsettings_watcher = None
@@ -505,6 +693,8 @@ class ClientExtras(object):
                 bus._clean_up_signal_match(self.upower_sleeping_match)
             if self.login1_match:
                 bus._clean_up_signal_match(self.login1_match)
+        global WINDOW_METHOD_OVERRIDES
+        WINDOW_METHOD_OVERRIDES = {}
 
     def resuming_callback(self, *args):
         eventlog("resuming_callback%s", args)
@@ -516,12 +706,21 @@ class ClientExtras(object):
 
 
     def setup_dbus_signals(self):
+        try:
+            import xpra.dbus
+            assert xpra.dbus
+        except ImportError as e:
+            dbuslog("setup_dbus_signals()", exc_info=True)
+            dbuslog.info("dbus support is not installed")
+            dbuslog.info(" no support for power events")
+            return
         try:
             from xpra.dbus.common import init_system_bus
             bus = init_system_bus()
             self.system_bus = bus
             dbuslog("setup_dbus_signals() system bus=%s", bus)
         except Exception as e:
+            dbuslog("setup_dbus_signals()", exc_info=True)
             dbuslog.error("Error setting up dbus signals:")
             dbuslog.error(" %s", e)
             return
@@ -563,6 +762,7 @@ class ClientExtras(object):
             return
         ROOT_PROPS = ["RESOURCE_MANAGER", "_NET_WORKAREA", "_NET_CURRENT_DESKTOP"]
         try:
+            self.init_x11_filter()
             from xpra.x11.xsettings import XSettingsWatcher
             from xpra.x11.xroot_props import XRootPropWatcher
             if self._xsettings_watcher is None:
@@ -577,6 +777,44 @@ class ClientExtras(object):
         except ImportError as e:
             log.error("failed to load X11 properties/settings bindings: %s - root window properties will not be propagated", e)
 
+
+    def do_xi_devices_changed(self, event):
+        log("do_xi_devices_changed(%s)", event)
+        XI2 = X11XI2Bindings()
+        devices = XI2.get_devices()
+        if devices:
+            self.client.send_input_devices("xi", devices)
+
+    def setup_xi(self):
+        if self.client.server_input_devices!="xi":
+            log.info("server does not support xi input devices")
+        try:
+            from xpra.gtk_common.error import xsync
+            with xsync:
+                assert X11WindowBindings, "no X11 window bindings"
+                assert X11XI2Bindings, "no XI2 window bindings"
+                X11XI2Bindings().gdk_inject()
+                self.init_x11_filter()
+                XI2 = X11XI2Bindings()
+                XI2.select_xi2_events()
+                if self.client.server_input_devices:
+                    XI2.connect(0, "XI_HierarchyChanged", self.do_xi_devices_changed)
+                    devices = XI2.get_devices()
+                    if devices:
+                        self.client.send_input_devices("xi", devices)
+        except Exception as e:
+            log("enable_xi2()", exc_info=True)
+            log.error("Error: cannot enable XI2 events")
+            log.error(" %s", e)
+        else:
+            #register our enhanced event handlers:
+            self.add_xi2_method_overrides()
+
+    def add_xi2_method_overrides(self):
+        global WINDOW_ADD_HOOKS
+        WINDOW_ADD_HOOKS = [XI2_Window]
+
+
     def _get_xsettings(self):
         try:
             return self._xsettings_watcher.get_settings()
@@ -616,3 +854,17 @@ class ClientExtras(object):
             self.client.desktops_changed("from %s event" % self._root_props_watcher)
         else:
             log.error("unknown property %s", prop)
+
+
+def main():
+    try:
+        from xpra.x11.gtk_x11.gdk_display_source import init_display_source
+        init_display_source()
+    except:
+        pass
+    from xpra.platform.gui import main
+    main()
+
+
+if __name__ == "__main__":
+    sys.exit(main())
