@@ -11,22 +11,31 @@ from xpra.keyboard.layouts import xkbmap_query_tostring
 from xpra.log import Logger
 log = Logger("keyboard", "posix")
 
-try:
-    from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings   #@UnresolvedImport
-    keyboard_bindings = X11KeyboardBindings()
-except Exception as e:
-    log.warn("failed load posix keyboard bindings: %s", e)
-    keyboard_bindings = None
-
 
 class Keyboard(KeyboardBase):
 
+    def __init__(self):
+        KeyboardBase.__init__(self)
+        self.keymap_modifiers = None
+        try:
+            from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings   #@UnresolvedImport
+            self.keyboard_bindings = X11KeyboardBindings()
+        except Exception as e:
+            log.warn("failed load posix keyboard bindings: %s", e)
+            self.keyboard_bindings = None
+
+
     def get_keymap_modifiers(self):
-        if not keyboard_bindings:
+        if self.keymap_modifiers is None:
+            self.keymap_modifiers = self.do_get_keymap_modifiers()
+        return self.keymap_modifiers
+
+    def do_get_keymap_modifiers(self):
+        if not self.keyboard_bindings:
             log.warn("keyboard bindings are not available, expect keyboard mapping problems")
             return {}, [], []
         try:
-            mod_mappings = keyboard_bindings.get_modifier_mappings()
+            mod_mappings = self.keyboard_bindings.get_modifier_mappings()
             if mod_mappings:
                 #ie: {"shift" : ["Shift_L", "Shift_R"], "mod1" : "Meta_L", ...]}
                 log("modifier mappings=%s", mod_mappings)
@@ -44,19 +53,19 @@ class Keyboard(KeyboardBase):
             log.error("failed to use native get_modifier_mappings: %s", e, exc_info=True)
 
     def get_x11_keymap(self):
-        if not keyboard_bindings:
+        if not self.keyboard_bindings:
             return  {}
         try:
-            return keyboard_bindings.get_keycode_mappings()
+            return self.keyboard_bindings.get_keycode_mappings()
         except Exception as e:
             log.error("failed to use raw x11 keymap: %s", e)
 
 
     def get_keymap_spec(self):
-        log("get_keymap_spec() keyboard_bindings=%s", keyboard_bindings)
-        if not keyboard_bindings:
+        log("get_keymap_spec() keyboard_bindings=%s", self.keyboard_bindings)
+        if not self.keyboard_bindings:
             return None
-        _query_struct = keyboard_bindings.getXkbProperties()
+        _query_struct = self.keyboard_bindings.getXkbProperties()
         _query = xkbmap_query_tostring(_query_struct)
         log("get_keymap_spec() Xkb query tostring(%s)=%s", _query_struct, _query)
         #we no longer support servers via xkbmap_print:
@@ -84,8 +93,8 @@ class Keyboard(KeyboardBase):
         layout = ""
         layouts = []
         v = None
-        if keyboard_bindings:
-            v = keyboard_bindings.getXkbProperties().get("layout")
+        if self.keyboard_bindings:
+            v = self.keyboard_bindings.getXkbProperties().get("layout")
         if not v:
             #fallback:
             v = self.get_xkb_rules_names_property()
@@ -97,17 +106,17 @@ class Keyboard(KeyboardBase):
             layout = v
         def s(v):
             try:
-                return v.encode("ascii")
+                return v.encode("latin1")
             except:
                 return str(v)
-        return s(layout), [s(x) for x in layouts], b"", None
+        return s(layout), [s(x) for x in layouts], "", None
 
 
     def get_keyboard_repeat(self):
         v = None
-        if keyboard_bindings:
+        if self.keyboard_bindings:
             try:
-                v = keyboard_bindings.get_key_repeat_rate()
+                v = self.keyboard_bindings.get_key_repeat_rate()
                 if v:
                     assert len(v)==2
             except Exception as e:
@@ -123,8 +132,10 @@ class Keyboard(KeyboardBase):
             self.modifier_map = grok_modifier_map(display, xkbmap_mod_meanings)
         except ImportError:
             self.modifier_map = MODIFIER_MAP
+        #force re-query on next call:
+        self.keymap_modifiers = None
         try:
             dn = "%s %s" % (type(display).__name__, display.get_name())
-        except Exception as e:
+        except:
             dn = str(display)
         log("update_modifier_map(%s, %s) modifier_map=%s", dn, xkbmap_mod_meanings, self.modifier_map)
