@@ -8,19 +8,14 @@ from xpra.scripts.config import python_platform
 #only imported to make sure we can get hold of a reference to the real "platform" module
 assert python_platform
 from xpra.version_util import get_version_info, get_platform_info, get_host_info
-from xpra.util import nonl, pver
+from xpra.util import print_nested_dict
 
 
 def main():
-    def print_dict(d):
-        for k in sorted(d.keys()):
-            v = d[k]
-            print("* %s : %s" % (k.ljust(32), nonl(pver(v))))
-    from xpra.platform import init, clean
-    try:
-        init("Version-Info", "Version Info")
+    from xpra.platform import program_context
+    with program_context("Version-Info", "Version Info"):
         print("Build:")
-        print_dict(get_version_info())
+        print_nested_dict(get_version_info())
         print("")
         print("Platform:")
         pi = get_platform_info()
@@ -28,7 +23,7 @@ def main():
         if "" in pi:
             pi["sys"] = pi[""]
             del pi[""]
-        print_dict(pi)
+        print_nested_dict(pi)
         print("")
         print("Host:")
         d = get_host_info()
@@ -38,9 +33,7 @@ def main():
             d.update(pvinfo())
         except:
             pass
-        print_dict(d)
-    finally:
-        clean()
+        print_nested_dict(d)
 
 
 if __name__ == "__main__":
