@@ -1,11 +1,10 @@
 # This file is part of Xpra.
-# Copyright (C) 2016 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2016-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-from xpra.server.auth.sys_auth_base import SysAuthenticator, init
-
-assert init
+from xpra.server.auth.sys_auth_base import SysAuthenticator, init, log
+assert init and log #tests will disable logging from here
 
 
 class Authenticator(SysAuthenticator):
@@ -18,8 +17,5 @@ class Authenticator(SysAuthenticator):
     def __repr__(self):
         return "password"
 
-    def get_challenge(self):
-        return SysAuthenticator.get_challenge(self, mac="hmac")
-
     def get_password(self):
         return self.value
