@@ -1,6 +1,6 @@
 # coding=utf8
 # This file is part of Xpra.
-# Copyright (C) 2010-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2016 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -15,18 +15,17 @@ class KeyboardConfigBase(object):
     def __init__(self):
         self.enabled = True
         self.is_native_keymap = True
-        #FIXME: only defined here because the server code assumes this will exist...
-        self.modifier_client_keycodes = {}
+        self.owner = None
 
     def __repr__(self):
         return "KeyboardConfigBase"
 
     def get_info(self):
-        info = {"enabled"   : self.enabled,
+        return {
+                "enabled"   : self.enabled,
                 "native"    : self.is_native_keymap,
+                "owner"     : self.owner or "",
                 }
-        return info
-
 
     def parse_options(self, props):
         pass
@@ -34,7 +33,10 @@ class KeyboardConfigBase(object):
     def get_hash(self):
         return ""
 
-    def set_keymap(self):
+    def set_layout(self, layout, variant):
+        pass
+
+    def set_keymap(self, translate_only=False):
         pass
 
     def set_default_keymap(self):
