@@ -1,19 +1,16 @@
 # coding=utf8
 # This file is part of Xpra.
-# Copyright (C) 2010-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import time
-from math import sqrt
-
 from xpra.log import Logger
-log = Logger("window", "compress")
+log = Logger("window", "encoding")
 
 from xpra.net import compression
-from xpra.codecs.argb.argb import bgra_to_rgb, bgra_to_rgba, argb_to_rgb, argb_to_rgba  #@UnresolvedImport
+from xpra.codecs.argb.argb import bgra_to_rgb, bgra_to_rgba, argb_to_rgb, argb_to_rgba, r210_to_rgbx, r210_to_rgb, bgr565_to_rgbx, bgr565_to_rgb  #@UnresolvedImport
 from xpra.codecs.loader import get_codec
-from xpra.os_util import memoryview_to_bytes, _memoryview
+from xpra.os_util import memoryview_to_bytes, monotonic_time
 #"pixels_to_bytes" gets patched up by the OSX shadow server
 pixels_to_bytes = memoryview_to_bytes
 try:
@@ -31,53 +28,6 @@ def warn_encoding_once(key, message):
         encoding_warnings.add(key)
 
 
-def webp_encode(coding, image, rgb_formats, supports_transparency, quality, speed, options):
-    pixel_format = image.get_pixel_format()
-    #log("rgb_encode%s pixel_format=%s, rgb_formats=%s", (coding, image, rgb_formats, supports_transparency, speed, rgb_zlib, rgb_lz4), pixel_format, rgb_formats)
-    if pixel_format not in rgb_formats:
-        if not rgb_reformat(image, rgb_formats, supports_transparency):
-            raise Exception("cannot find compatible rgb format to use for %s! (supported: %s)" % (pixel_format, rgb_formats))
-        #get the new format:
-        pixel_format = image.get_pixel_format()
-    stride = image.get_rowstride()
-    enc_webp = get_codec("enc_webp")
-    #log("webp_encode%s stride=%s, enc_webp=%s", (coding, image, rgb_formats, supports_transparency, quality, speed, options), stride, enc_webp)
-    if enc_webp and stride>0 and stride%4==0 and image.get_pixel_format() in ("BGRA", "BGRX", "RGBA", "RGBX"):
-        #prefer Cython module:
-        alpha = supports_transparency and image.get_pixel_format().find("A")>=0
-        w = image.get_width()
-        h = image.get_height()
-        if quality==100:
-            #webp lossless is unbearibly slow for only marginal compression improvements,
-            #so force max speed:
-            speed = 100
-        else:
-            #normalize speed for webp: avoid low speeds!
-            speed = int(sqrt(speed) * 10)
-        speed = max(0, min(100, speed))
-        pixels = image.get_pixels()
-        assert pixels, "failed to get pixels from %s" % image
-        cdata = enc_webp.compress(pixels, w, h, stride=stride/4, quality=quality, speed=speed, has_alpha=alpha)
-        client_options = {"speed"       : speed,
-                          "rgb_format"  : pixel_format}
-        if quality>=0 and quality<=100:
-            client_options["quality"] = quality
-        if alpha:
-            client_options["has_alpha"] = True
-        return "webp", compression.Compressed("webp", cdata), client_options, image.get_width(), image.get_height(), 0, 24
-    #fallback to PIL
-    enc_pillow = get_codec("enc_pillow")
-    if enc_pillow:
-        log("using PIL fallback for webp: enc_webp=%s, stride=%s, pixel format=%s", enc_webp, stride, image.get_pixel_format())
-        for x in ("webp", "png"):
-            if x in enc_pillow.get_encodings():
-                return enc_pillow.encode(x, image, quality, speed, supports_transparency)
-    raise Exception("BUG: cannot use 'webp' encoding and none of the PIL fallbacks are available!")
-
-
-def roundup(n, m):
-    return (n + m - 1) & ~(m - 1)
-
 def rgb_encode(coding, image, rgb_formats, supports_transparency, speed, rgb_zlib=True, rgb_lz4=True, rgb_lzo=False):
     pixel_format = image.get_pixel_format()
     #log("rgb_encode%s pixel_format=%s, rgb_formats=%s", (coding, image, rgb_formats, supports_transparency, speed, rgb_zlib, rgb_lz4), pixel_format, rgb_formats)
@@ -96,7 +46,7 @@ def rgb_encode(coding, image, rgb_formats, supports_transparency, speed, rgb_zli
     options = {"rgb_format" : pixel_format}
 
     #we may want to re-stride:
-    image.restride()
+    image.may_restride()
 
     #compress here and return a wrapper so network code knows it is already zlib compressed:
     pixels = image.get_pixels()
@@ -108,14 +58,16 @@ def rgb_encode(coding, image, rgb_formats, supports_transparency, speed, rgb_zli
     #compression stage:
     level = 0
     algo = "not"
-    if len(pixels)>=256:
-        level = max(0, min(5, int(115-speed)/20))
-        if len(pixels)<1024:
-            #fewer pixels, make it more likely we won't bother compressing:
-            level = level // 2
+    l = len(pixels)
+    if l>=512 and speed<100:
+        if l>=32768:
+            level = 1+max(0, min(8, int(100-speed)//12))
+        else:
+            #fewer pixels, make it more likely we won't bother compressing (speed>90):
+            level = max(0, min(5, int(110-speed)//20))
     if level>0:
         if rgb_lz4 and compression.use_lz4:
-            cwrapper = compression.compressed_wrapper(coding, pixels, lz4=True)
+            cwrapper = compression.compressed_wrapper(coding, pixels, lz4=True, level=level)
             algo = "lz4"
             level = 1
         elif rgb_lzo and compression.use_lzo:
@@ -123,7 +75,7 @@ def rgb_encode(coding, image, rgb_formats, supports_transparency, speed, rgb_zli
             algo = "lzo"
             level = 1
         elif rgb_zlib and compression.use_zlib:
-            cwrapper = compression.compressed_wrapper(coding, pixels, zlib=True, level=level)
+            cwrapper = compression.compressed_wrapper(coding, pixels, zlib=True, level=level//2)
             algo = "zlib"
         else:
             cwrapper = None
@@ -146,7 +98,7 @@ def rgb_encode(coding, image, rgb_formats, supports_transparency, speed, rgb_zli
         bpp = 32
     else:
         bpp = 24
-    log("rgb_encode using level=%s, %s compressed %sx%s in %s/%s: %s bytes down to %s", level, algo, image.get_width(), image.get_height(), coding, pixel_format, len(pixels), len(cwrapper.data))
+    log("rgb_encode using level=%s for %5i pixels at %3i speed, %s compressed %4sx%-4s in %s/%s: %5s bytes down to %5s", level, l, speed, algo, image.get_width(), image.get_height(), coding, pixel_format, len(pixels), len(cwrapper.data))
     #wrap it using "Compressed" so the network layer receiving it
     #won't decompress it (leave it to the client's draw thread)
     return coding, cwrapper, options, width, height, stride, bpp
@@ -160,7 +112,38 @@ def argb_swap(image, rgb_formats, supports_transparency):
     pixels = image.get_pixels()
     assert pixels, "failed to get pixels from %s" % image
     rs = image.get_rowstride()
-    if pixel_format in ("BGRX", "BGRA"):
+    if pixel_format=="r210":
+        #r210 never contains any transparency at present
+        #if supports_transparency and "RGBA" in rgb_formats:
+        #    log("argb_swap: r210_to_rgba for %s on %s", pixel_format, type(pixels))
+        #    image.set_pixels(r210_to_rgba(pixels))
+        #    image.set_pixel_format("RGBA")
+        #    return True
+        if "RGB" in rgb_formats:
+            log("argb_swap: r210_to_rgb for %s on %s", pixel_format, type(pixels))
+            image.set_pixels(r210_to_rgb(pixels))
+            image.set_pixel_format("RGB")
+            image.set_rowstride(rs*3//4)
+            return True
+        if "RGBX" in rgb_formats:
+            log("argb_swap: r210_to_rgbx for %s on %s", pixel_format, type(pixels))
+            image.set_pixels(r210_to_rgbx(pixels))
+            image.set_pixel_format("RGBX")
+            return True
+    elif pixel_format=="BGR565":
+        if "RGB" in rgb_formats:
+            log("argb_swap: bgr565_to_rgb for %s on %s", pixel_format, type(pixels))
+            image.set_pixels(bgr565_to_rgb(pixels))
+            image.set_pixel_format("RGB")
+            image.set_rowstride(rs*3//2)
+            return True
+        if "RGBX" in rgb_formats:
+            log("argb_swap: bgr565_to_rgbx for %s on %s", pixel_format, type(pixels))
+            image.set_pixels(bgr565_to_rgbx(pixels))
+            image.set_pixel_format("RGBX")
+            image.set_rowstride(rs*2)
+            return True
+    elif pixel_format in ("BGRX", "BGRA"):
         if supports_transparency and "RGBA" in rgb_formats:
             log("argb_swap: bgra_to_rgba for %s on %s", pixel_format, type(pixels))
             image.set_pixels(bgra_to_rgba(pixels))
@@ -172,7 +155,7 @@ def argb_swap(image, rgb_formats, supports_transparency):
             image.set_pixel_format("RGB")
             image.set_rowstride(rs*3//4)
             return True
-    if pixel_format in ("XRGB", "ARGB"):
+    elif pixel_format in ("XRGB", "ARGB"):
         if supports_transparency and "RGBA" in rgb_formats:
             log("argb_swap: argb_to_rgba for %s on %s", pixel_format, type(pixels))
             image.set_pixels(argb_to_rgba(pixels))
@@ -213,14 +196,16 @@ def rgb_reformat(image, rgb_formats, supports_transparency):
     pixel_format = image.get_pixel_format()
     pixels = image.get_pixels()
     assert pixels, "failed to get pixels from %s" % image
-    if not PIL:
+    if not PIL or pixel_format in ("r210", "BGR565"):
         #try to fallback to argb module
+        #(required for r210 which is not handled by PIL directly)
         log("rgb_reformat: using argb_swap for %s", image)
         return argb_swap(image, rgb_formats, supports_transparency)
     if supports_transparency:
         modes = PIL_conv.get(pixel_format)
     else:
         modes = PIL_conv_noalpha.get(pixel_format)
+    assert modes, "no PIL conversion from %s" % (pixel_format)
     target_rgb = [(im,om) for (im,om) in modes if om in rgb_formats]
     if len(target_rgb)==0:
         log("rgb_reformat: no matching target modes for converting %s to %s", image, rgb_formats)
@@ -231,23 +216,21 @@ def rgb_reformat(image, rgb_formats, supports_transparency):
         warn_encoding_once(warning_key, "cannot convert %s to one of: %s" % (pixel_format, rgb_formats))
         return False
     input_format, target_format = target_rgb[0]
-    start = time.time()
+    start = monotonic_time()
     w = image.get_width()
     h = image.get_height()
     #PIL cannot use the memoryview directly:
-    if _memoryview and isinstance(pixels, _memoryview):
+    if isinstance(pixels, memoryview):
         pixels = pixels.tobytes()
     log("rgb_reformat: converting %s from %s to %s using PIL", image, input_format, target_format)
     img = PIL.Image.frombuffer(target_format, (w, h), pixels, "raw", input_format, image.get_rowstride())
     rowstride = w*len(target_format)    #number of characters is number of bytes per pixel!
-    #use tobytes() if present, fallback to tostring():
-    data_fn = getattr(img, "tobytes", getattr(img, "tostring", None))
-    data = data_fn("raw", target_format)
+    data = img.tobytes("raw", target_format)
     assert len(data)==rowstride*h, "expected %s bytes in %s format but got %s" % (rowstride*h, len(data))
     image.set_pixels(data)
     image.set_rowstride(rowstride)
     image.set_pixel_format(target_format)
-    end = time.time()
+    end = monotonic_time()
     log("rgb_reformat(%s, %s, %s) converted from %s (%s bytes) to %s (%s bytes) in %.1fms, rowstride=%s", image, rgb_formats, supports_transparency, pixel_format, len(pixels), target_format, len(data), (end-start)*1000.0, rowstride)
     return True
 
@@ -261,11 +244,11 @@ def mmap_send(mmap, mmap_size, image, rgb_formats, supports_transparency):
             warning_key = "mmap_send(%s)" % image.get_pixel_format()
             warn_encoding_once(warning_key, "cannot use mmap to send %s" % image.get_pixel_format())
             return None
-    start = time.time()
+    start = monotonic_time()
     data = image.get_pixels()
     assert data, "failed to get pixels from %s" % image
     mmap_data, mmap_free_size = mmap_write(mmap, mmap_size, data)
-    elapsed = time.time()-start+0.000000001 #make sure never zero!
+    elapsed = monotonic_time()-start+0.000000001 #make sure never zero!
     log("%s MBytes/s - %s bytes written to mmap in %.1f ms", int(len(data)/elapsed/1024/1024), len(data), 1000*elapsed)
     if mmap_data is None:
         return None
