@@ -1,14 +1,14 @@
 # This file is part of Xpra.
-# Copyright (C) 2013-2015 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2013-2017 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import time
 import socket
 import os
 import signal
 from threading import Timer
+from time import sleep
 
 from xpra.log import Logger
 log = Logger("proxy")
@@ -23,22 +23,28 @@ from xpra.net.protocol import Protocol, get_network_caps
 from xpra.codecs.loader import load_codecs, get_codec
 from xpra.codecs.image_wrapper import ImageWrapper
 from xpra.codecs.video_helper import getVideoHelper, PREFERRED_ENCODER_ORDER
-from xpra.os_util import Queue, SIGNAMES
-from xpra.util import flatten_dict, typedict, updict, repr_ellipsized, \
+from xpra.os_util import Queue, SIGNAMES, strtobytes, memoryview_to_bytes, getuid, getgid, monotonic_time, get_username_for_uid, setuidgid
+from xpra.util import flatten_dict, typedict, updict, repr_ellipsized, xor, std, envint, envbool, csv, \
     LOGIN_TIMEOUT, CONTROL_COMMAND_ERROR, AUTHENTICATION_ERROR, CLIENT_EXIT_TIMEOUT, SERVER_SHUTDOWN
-from xpra.version_util import local_version
-from xpra.make_thread import make_thread
+from xpra.version_util import XPRA_VERSION
+from xpra.make_thread import start_thread
 from xpra.scripts.config import parse_number, parse_bool
-from xpra.scripts.server import create_unix_domain_socket
-from xpra.scripts.main import SOCKET_TIMEOUT
+from xpra.scripts.main import full_version_str
+from xpra.server.socket_util import create_unix_domain_socket
 from xpra.platform.dotxpra import DotXpra
-from xpra.net.bytestreams import SocketConnection
+from xpra.net.bytestreams import SocketConnection, SOCKET_TIMEOUT
 from multiprocessing import Process
 
+try:
+    from xpra.codecs.xor.cyxor import xor_str           #@UnresolvedImport
+    xor = xor_str
+except:
+    pass
 
-PROXY_QUEUE_SIZE = int(os.environ.get("XPRA_PROXY_QUEUE_SIZE", "10"))
+
+PROXY_QUEUE_SIZE = envint("XPRA_PROXY_QUEUE_SIZE", 10)
 #for testing only: passthrough as RGB:
-PASSTHROUGH = os.environ.get("XPRA_PROXY_PASSTHROUGH", "0")=="1"
+PASSTHROUGH = envbool("XPRA_PROXY_PASSTHROUGH", False)
 MAX_CONCURRENT_CONNECTIONS = 20
 VIDEO_TIMEOUT = 5                  #destroy video encoder after N seconds of idle state
 
@@ -63,7 +69,7 @@ class ProxyInstanceProcess(Process):
         self.uid = uid
         self.gid = gid
         self.env_options = env_options
-        self.session_options = self.sanitize_session_options(session_options)
+        self.session_options = session_options
         self.socket_dir = socket_dir
         self.video_encoder_modules = video_encoder_modules
         self.csc_modules = csc_modules
@@ -75,8 +81,8 @@ class ProxyInstanceProcess(Process):
         self.caps = caps
         log("ProxyProcess%s", (uid, gid, env_options, session_options, socket_dir,
                                video_encoder_modules, csc_modules,
-                               client_conn, client_state, cipher, encryption_key, server_conn,
-                               "%s: %s.." % (type(caps), repr_ellipsized(caps)), message_queue))
+                               client_conn, repr_ellipsized(str(client_state)), cipher, encryption_key, server_conn,
+                               "%s: %s.." % (type(caps), repr_ellipsized(str(caps))), message_queue))
         self.client_protocol = None
         self.server_protocol = None
         self.exit = False
@@ -91,6 +97,7 @@ class ProxyInstanceProcess(Process):
         self.video_helper = None
         self.lost_windows = None
         #for handling the local unix domain socket:
+        self.control_socket_cleanup = None
         self.control_socket = None
         self.control_socket_thread = None
         self.control_socket_path = None
@@ -106,7 +113,7 @@ class ProxyInstanceProcess(Process):
                 self.stop("proxy server request")
                 return
             elif m=="socket-handover-complete":
-                log.info("setting sockets %s to blocking mode", (self.client_conn, self.server_conn))
+                log("setting sockets to blocking mode: %s", (self.client_conn, self.server_conn))
                 #set sockets to blocking mode:
                 set_blocking(self.client_conn)
                 set_blocking(self.server_conn)
@@ -139,14 +146,13 @@ class ProxyInstanceProcess(Process):
         Timer(timeout/1000.0, timer_exec).start()
 
     def run(self):
-        log("ProxyProcess.run() pid=%s, uid=%s, gid=%s", os.getpid(), os.getuid(), os.getgid())
-        #change uid and gid:
-        if os.getgid()!=self.gid:
-            os.setgid(self.gid)
-        if os.getuid()!=self.uid:
-            os.setuid(self.uid)
-        log("ProxyProcess.run() new uid=%s, gid=%s", os.getuid(), os.getgid())
-
+        log("ProxyProcess.run() pid=%s, uid=%s, gid=%s", os.getpid(), getuid(), getgid())
+        try:
+            import setproctitle
+            setproctitle.setproctitle("Xpra Proxy Instance for %s" % self.server_conn)
+        except ImportError as e:
+            log("setproctitle not installed: %s", e)
+        setuidgid(self.uid, self.gid)
         if self.env_options:
             #TODO: whitelist env update?
             os.environ.update(self.env_options)
@@ -160,13 +166,12 @@ class ProxyInstanceProcess(Process):
         signal.signal(signal.SIGINT, self.signal_quit)
         log("registered signal handler %s", self.signal_quit)
 
-        make_thread(self.server_message_queue, "server message queue").start()
+        start_thread(self.server_message_queue, "server message queue")
 
         if not self.create_control_socket():
             #TODO: should send a message to the client
             return
-        self.control_socket_thread = make_thread(self.control_socket_loop, "control")
-        self.control_socket_thread.start()
+        self.control_socket_thread = start_thread(self.control_socket_loop, "control")
 
         self.main_queue = Queue()
         #setup protocol wrappers:
@@ -181,22 +186,22 @@ class ProxyInstanceProcess(Process):
         self.server_protocol.large_packets.append("keymap-changed")
         self.server_protocol.large_packets.append("server-settings")
         if self.caps.boolget("file-transfer"):
+            self.client_protocol.large_packets.append("send-file")
+            self.client_protocol.large_packets.append("send-file-chunk")
             self.server_protocol.large_packets.append("send-file")
+            self.server_protocol.large_packets.append("send-file-chunk")
         self.server_protocol.set_compression_level(self.session_options.get("compression_level", 0))
         self.server_protocol.enable_default_encoder()
 
         self.lost_windows = set()
         self.encode_queue = Queue()
-        self.encode_thread = make_thread(self.encode_loop, "encode")
-        self.encode_thread.start()
+        self.encode_thread = start_thread(self.encode_loop, "encode")
 
         log("starting network threads")
         self.server_protocol.start()
         self.client_protocol.start()
 
-        #forward the hello packet:
-        hello_packet = ("hello", self.filter_client_caps(self.caps))
-        self.queue_server_packet(hello_packet)
+        self.send_hello()
         self.timeout_add(VIDEO_TIMEOUT*1000, self.timeout_video_encoders)
 
         try:
@@ -209,7 +214,7 @@ class ProxyInstanceProcess(Process):
     def video_init(self):
         enclog("video_init() loading codecs")
         load_codecs(decoders=False)
-        enclog("video_init() will try video encoders: %s", self.video_encoder_modules)
+        enclog("video_init() will try video encoders: %s", csv(self.video_encoder_modules) or "none")
         self.video_helper = getVideoHelper()
         #only use video encoders (no CSC supported in proxy)
         self.video_helper.set_modules(video_encoders=self.video_encoder_modules)
@@ -247,19 +252,30 @@ class ProxyInstanceProcess(Process):
             if x not in order:
                 order.append(x)
         self.video_encoder_types = [x for x in order if x in encoder_types]
-        enclog.info("proxy video encoders: %s", ", ".join(self.video_encoder_types))
+        enclog.info("proxy video encoders: %s", ", ".join(self.video_encoder_types or ["none",]))
 
 
     def create_control_socket(self):
         assert self.socket_dir
-        dotxpra = DotXpra(self.socket_dir)
-        sockpath = dotxpra.socket_path(":proxy-%s" % os.getpid())
+        username = get_username_for_uid(self.uid)
+        dotxpra = DotXpra(self.socket_dir, actual_username=username, uid=self.uid, gid=self.gid)
+        sockname = ":proxy-%s" % os.getpid()
+        sockpath = dotxpra.socket_path(sockname)
+        log("%s.socket_path(%s)=%s", dotxpra, sockname, sockpath)
         state = dotxpra.get_server_state(sockpath)
-        if state in (DotXpra.LIVE, DotXpra.UNKNOWN):
-            log.warn("You already have a proxy server running at %s, the control socket will not be created!", sockpath)
+        log("create_control_socket: socket path='%s', uid=%i, gid=%i, state=%s", sockpath, getuid(), getgid(), state)
+        if state in (DotXpra.LIVE, DotXpra.UNKNOWN, DotXpra.INACCESSIBLE):
+            log.error("Error: you already have a proxy server running at '%s'", sockpath)
+            log.error(" the control socket will not be created")
             return False
+        d = os.path.dirname(sockpath)
         try:
-            sock = create_unix_domain_socket(sockpath, None, 0o600)
+            dotxpra.mksockdir(d)
+        except Exception as e:
+            log.warn("Warning: failed to create socket directory '%s'", d)
+            log.warn(" %s", e)
+        try:
+            sock, self.control_socket_cleanup = create_unix_domain_socket(sockpath, None, 0o600)
             sock.listen(5)
         except Exception as e:
             log("create_unix_domain_socket failed for '%s'", sockpath, exc_info=True)
@@ -334,8 +350,10 @@ class ProxyInstanceProcess(Process):
                 self.stop("socket request", None)
                 return
             elif caps.get("version_request", False):
-                from xpra import __version__
-                proto.send_now(("hello", {"version" : __version__}))
+                version = XPRA_VERSION
+                if caps.boolget("full-version-request"):
+                    version = full_version_str()
+                proto.send_now(("hello", {"version" : version}))
                 self.timeout_add(5*1000, self.send_disconnect, proto, CLIENT_EXIT_TIMEOUT, "version sent")
                 return
         self.send_disconnect(proto, CONTROL_COMMAND_ERROR, "this socket only handles 'info', 'version' and 'stop' requests")
@@ -355,12 +373,22 @@ class ProxyInstanceProcess(Process):
         sinfo = {}
         sinfo.update(get_server_info())
         sinfo.update(get_thread_info(proto))
-        return {"proxy" : {
-                           "version"    : local_version,
-                           ""           : sinfo,
-                           },
-                "window" : self.get_window_info(),
-                }
+        return {
+            "proxy" : {
+                "version"    : XPRA_VERSION,
+                ""           : sinfo,
+                },
+            "window" : self.get_window_info(),
+            }
+
+    def send_hello(self, challenge_response=None, client_salt=None):
+        hello = self.filter_client_caps(self.caps)
+        if challenge_response:
+            hello.update({
+                          "challenge_response"      : challenge_response,
+                          "challenge_client_salt"   : client_salt,
+                          })
+        self.queue_server_packet(("hello", hello))
 
 
     def sanitize_session_options(self, options):
@@ -385,9 +413,9 @@ class ProxyInstanceProcess(Process):
         return d
 
     def filter_client_caps(self, caps):
-        fc = self.filter_caps(caps, ("cipher", "digest", "aliases", "compression", "lz4", "lz0", "zlib"))
+        fc = self.filter_caps(caps, ("cipher", "challenge", "digest", "aliases", "compression", "lz4", "lz0", "zlib"))
         #update with options provided via config if any:
-        fc.update(self.session_options)
+        fc.update(self.sanitize_session_options(self.session_options))
         #add video proxies if any:
         fc["encoding.proxy.video"] = len(self.video_encoding_defs)>0
         if self.video_encoding_defs:
@@ -413,7 +441,7 @@ class ProxyInstanceProcess(Process):
         #replace the network caps with the proxy's own:
         pcaps.update(flatten_dict(get_network_caps()))
         #then add the proxy info:
-        updict(pcaps, "proxy", get_server_info())
+        updict(pcaps, "proxy", get_server_info(), flatten_dicts=True)
         pcaps["proxy"] = True
         pcaps["proxy.hostname"] = socket.gethostname()
         return pcaps
@@ -446,22 +474,20 @@ class ProxyInstanceProcess(Process):
                 log.info("waiting for network connections to close")
             else:
                 log("still waiting %i/10 - client.closed=%s, server.closed=%s", i+1, self.client_protocol._closed, self.server_protocol._closed)
-            time.sleep(0.1)
+            sleep(0.1)
         log.info("proxy instance %s stopped", os.getpid())
 
     def stop(self, reason="proxy terminating", skip_proto=None):
         log.info("stop(%s, %s)", reason, skip_proto)
         self.exit = True
-        if self.control_socket_path:
-            try:
-                os.unlink(self.control_socket_path)
-            except:
-                pass
-            self.control_socket_path = None
         try:
             self.control_socket.close()
         except:
             pass
+        csc = self.control_socket_cleanup
+        if csc:
+            self.control_socket_cleanup = None
+            csc()
         self.main_queue.put(None)
         #empty the main queue:
         q = Queue()
@@ -506,7 +532,7 @@ class ProxyInstanceProcess(Process):
             self.client_protocol.source_has_more()
             return
         elif packet_type=="hello":
-            log.warn("invalid hello packet received after initial authentication (dropped)")
+            log.warn("Warning: invalid hello packet received after initial authentication (dropped)")
             return
         self.queue_server_packet(packet)
 
@@ -527,7 +553,7 @@ class ProxyInstanceProcess(Process):
         if len(packet)>index:
             data = packet[index]
             if len(data)<512:
-                packet[8] = str(data)
+                packet[index] = str(data)
                 return
             #FIXME: this is ugly and not generic!
             zlib = compression.use_zlib and self.caps.boolget("zlib", True)
@@ -593,10 +619,47 @@ class ProxyInstanceProcess(Process):
         elif packet_type=="cursor":
             #packet = ["cursor", x, y, width, height, xhot, yhot, serial, pixels, name]
             #or:
+            #packet = ["cursor", "png", x, y, width, height, xhot, yhot, serial, pixels, name]
+            #or:
             #packet = ["cursor", ""]
-            self._packet_recompress(packet, 8, "cursor")
+            if len(packet)>=8:
+                #hard to distinguish png cursors from normal cursors...
+                try:
+                    int(packet[1])
+                    self._packet_recompress(packet, 8, "cursor")
+                except:
+                    self._packet_recompress(packet, 9, "cursor")
         elif packet_type=="window-icon":
             self._packet_recompress(packet, 5, "icon")
+        elif packet_type=="send-file":
+            if packet[6]:
+                packet[6] = Compressed("file-data", packet[6])
+        elif packet_type=="send-file-chunk":
+            if packet[3]:
+                packet[3] = Compressed("file-chunk-data", packet[3])
+        elif packet_type=="challenge":
+            from xpra.net.crypto import get_salt, get_digest_module
+            #client may have already responded to the challenge,
+            #so we have to handle authentication from this end
+            salt = packet[1]
+            digest = packet[3]
+            client_salt = get_salt(len(salt))
+            salt = xor_str(salt, client_salt)
+            digestmod = get_digest_module(digest)
+            if not digestmod:
+                self.stop("digest mode '%s' not supported", std(digest))
+                return
+            password = self.session_options.get("password")
+            if not password:
+                self.stop("authentication requested by the server, but no password available for this session")
+                return
+            import hmac
+            password = strtobytes(password)
+            salt = memoryview_to_bytes(salt)
+            challenge_response = hmac.HMAC(password, salt, digestmod=digestmod).hexdigest()
+            log.info("sending %s challenge response", digest)
+            self.send_hello(challenge_response, client_salt)
+            return
         self.queue_client_packet(packet)
 
 
@@ -625,7 +688,7 @@ class ProxyInstanceProcess(Process):
                     #not a real packet, this is added by the timeout check:
                     wid = packet[1]
                     ve = self.video_encoders.get(wid)
-                    now = time.time()
+                    now = monotonic_time()
                     idle_time = now-self.video_encoders_last_used_time.get(wid)
                     if ve and idle_time>VIDEO_TIMEOUT:
                         enclog("timing out the video encoder context for window %s", wid)
@@ -643,7 +706,7 @@ class ProxyInstanceProcess(Process):
     def process_draw(self, packet):
         wid, x, y, width, height, encoding, pixels, _, rowstride, client_options = packet[1:11]
         #never modify mmap packets
-        if encoding=="mmap":
+        if encoding in ("mmap", "scroll"):
             return True
 
         #we have a proxy video packet:
@@ -750,7 +813,7 @@ class ProxyInstanceProcess(Process):
                 dst_formats = self.video_encoders_dst_formats
             ve.init_context(width, height, rgb_format, dst_formats, encoding, quality, speed, scaling, {})
             self.video_encoders[wid] = ve
-            self.video_encoders_last_used_time[wid] = time.time()       #just to make sure this is always set
+            self.video_encoders_last_used_time[wid] = monotonic_time()      #just to make sure this is always set
         #actual video compression:
         enclog("proxy compression using %s with quality=%s, speed=%s", ve, quality, speed)
         data, out_options = ve.compress_image(image, quality, speed, encoder_options)
@@ -759,14 +822,14 @@ class ProxyInstanceProcess(Process):
         for k in ("timestamp", "rgb_format", "depth", "csc"):
             if k not in out_options and k in client_options:
                 out_options[k] = client_options[k]
-        self.video_encoders_last_used_time[wid] = time.time()
+        self.video_encoders_last_used_time[wid] = monotonic_time()
         return send_updated(ve.get_encoding(), Compressed(encoding, data), out_options)
 
     def timeout_video_encoders(self):
         #have to be careful as another thread may come in...
         #so we just ask the encode thread (which deals with encoders already)
         #to do what may need to be done if we find a timeout:
-        now = time.time()
+        now = monotonic_time()
         for wid in list(self.video_encoders_last_used_time.keys()):
             idle_time = int(now-self.video_encoders_last_used_time.get(wid))
             if idle_time is None:
@@ -794,7 +857,7 @@ class ProxyInstanceProcess(Process):
 
     def get_window_info(self):
         info = {}
-        now = time.time()
+        now = monotonic_time()
         for wid, encoder in self.video_encoders.items():
             einfo = encoder.get_info()
             einfo["idle_time"] = int(now-self.video_encoders_last_used_time.get(wid, 0))
