@@ -1,15 +1,15 @@
 # coding=utf8
 # This file is part of Xpra.
-# Copyright (C) 2012-2015 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import time
 from math import log as mathlog, sqrt
 
 from xpra.log import Logger
 log = Logger("server", "stats")
 
+from xpra.os_util import monotonic_time
 from xpra.server.cystats import queue_inspect, logp, time_weighted_average, calculate_timesize_weighted_average_score   #@UnresolvedImport
 
 
@@ -64,7 +64,7 @@ def update_batch_delay(batch, factors):
         then combine it with the new factors.
     """
     current_delay = batch.delay
-    now = time.time()
+    now = monotonic_time()
     tv, tw = 0.0, 0.0
     decay = max(1, logp(current_delay/batch.min_delay)/5.0)
     max_delay = batch.max_delay
@@ -121,23 +121,28 @@ def get_target_speed(wid, window_dimensions, batch, global_statistics, statistic
     #abs: try to never go higher than 5 times reference latency:
     dam_lat_abs = max(0, ((statistics.avg_damage_in_latency or 0)-ref_damage_latency) / (ref_damage_latency * 4.0))
 
-    #calculate a target latency and try to get close to it
-    avg_delay = batch.delay
-    delays = list(batch.last_actual_delays)
-    if len(delays)>0:
-        #average recent actual delay:
-        avg_delay = time_weighted_average(delays)
-    #and average that with the current delay (which is lower or equal):
-    frame_delay = (avg_delay + batch.delay) / 2.0
-    #ensure we always spend at least as much time encoding as we spend batching:
-    #(one frame encoding whilst one frame is batching is our ideal result)
-    target_damage_latency = max(ref_damage_latency, frame_delay/1000.0)
-    #current speed:
-    speed = min_speed
-    if len(speed_data)>0:
-        speed = max(min_speed, time_weighted_average(speed_data))
-    #rel: do we need to increase or decrease speed to reach the target:
-    dam_lat_rel = speed/100.0 * statistics.avg_damage_in_latency / target_damage_latency
+    if batch.locked:
+        target_damage_latency = ref_damage_latency
+        dam_lat_rel = 0
+        frame_delay = 0
+    else:
+        #calculate a target latency and try to get close to it
+        avg_delay = batch.delay
+        delays = list(batch.last_actual_delays)
+        if len(delays)>0:
+            #average recent actual delay:
+            avg_delay = time_weighted_average(delays)
+        #and average that with the current delay (which is lower or equal):
+        frame_delay = (avg_delay + batch.delay) / 2.0
+        #ensure we always spend at least as much time encoding as we spend batching:
+        #(one frame encoding whilst one frame is batching is our ideal result)
+        target_damage_latency = max(ref_damage_latency, frame_delay/1000.0)
+        #current speed:
+        speed = min_speed
+        if len(speed_data)>0:
+            speed = max(min_speed, time_weighted_average(speed_data))
+        #rel: do we need to increase or decrease speed to reach the target:
+        dam_lat_rel = speed/100.0 * statistics.avg_damage_in_latency / target_damage_latency
 
     #ensure we decode at a reasonable speed (for slow / low-power clients)
     #maybe this should be configurable?
@@ -151,7 +156,7 @@ def get_target_speed(wid, window_dimensions, batch, global_statistics, statistic
     # may aggregate multiple damage requests into one packet - which may skip frames)
     #TODO: reconcile this with video regions
     #only count the last second's worth:
-    now = time.time()
+    now = monotonic_time()
     lim = now-1.0
     lde = [w*h for t,_,_,w,h in list(statistics.last_damage_events) if t>=lim]
     pixels = sum(lde)
@@ -191,6 +196,10 @@ def get_target_speed(wid, window_dimensions, batch, global_statistics, statistic
 
 
 def get_target_quality(wid, window_dimensions, batch, global_statistics, statistics, min_quality, min_speed):
+    info = {
+        "min_quality"   : min_quality,
+        "min_speed"     : min_speed,
+        }
     low_limit = get_low_limit(global_statistics.mmap_size>0, window_dimensions)
     #***********************************************************
     # quality:
@@ -199,43 +208,48 @@ def get_target_quality(wid, window_dimensions, batch, global_statistics, statist
     # here we try minimize client-latency, packet-backlog and batch.delay
     # the compression ratio tells us if we can increase the quality
     packets_backlog, pixels_backlog, _ = statistics.get_client_backlog()
-    pixels_bl = 1.0 - logp(pixels_backlog/low_limit)
+    pb_ratio = pixels_backlog/low_limit
+    pixels_bl = 1.0 - logp(pb_ratio//4)     #4 frames behind -> min quality
+    info["backlog_factor"] = packets_backlog, pixels_backlog, low_limit, pb_ratio, int(100.0*pixels_bl)
     target = pixels_bl
-    batch_q = -1
     if batch is not None:
         recs = len(batch.last_actual_delays)
-        if recs>0:
+        if recs>0 and not batch.locked:
             #weighted average between start delay and min_delay
             #so when we start and we don't have any records, we don't lower quality
             #just because the start delay is higher than min_delay
             ref_delay = (batch.START_DELAY*10.0/recs + batch.min_delay*recs) / (recs+10.0/recs)
-            #anything less than twice the minimum is good enough:
-            batch_q = ref_delay / max(1, batch.min_delay, batch.delay/2.0)
+            #anything less than N times the reference delay is good enough:
+            N = 4
+            batch_q = N * ref_delay / max(1, batch.min_delay, batch.delay)
+            info["batch-delay-ratio"] = int(100.0*batch_q)
             target = min(1.0, target, batch_q)
-    cratio_factor = None
     #from here on, the compression ratio integer value is in per-1000:
     es = [(t, pixels, 1000*compressed_size*bpp//pixels//32) for (t, _, pixels, bpp, compressed_size, _) in list(statistics.encoding_stats) if pixels>=4096]
     if len(es)>=2:
         #use the recent vs average compression ratio
-        #(add 10 to smooth things out a bit, so very low compression ratios don't skew things)
+        #(add value to smooth things out a bit, so very low compression ratios don't skew things)
         ascore, rscore = calculate_timesize_weighted_average_score(es)
         bump = 0
         if ascore>rscore:
             #raise the quality
             #only if there is no backlog:
             if packets_backlog==0:
-                bump = logp((float(10+ascore)/(10+rscore))-1.0)
+                smooth = 150
+                bump = logp((float(smooth+ascore)/(smooth+rscore)))-1.0
         else:
             #lower the quality
             #more so if the compression is not doing very well:
             mult = (1000 + rscore)/2000.0           #mult should be in the range 0.5 to ~1.0
-            bump = -logp((float(10+rscore)/(10+ascore))-1.0) * mult
+            smooth = 50
+            bump = -logp((float(smooth+rscore)/(smooth+ascore))-1.0) * mult
         target += bump
-        cratio_factor = ascore, rscore, int(100*bump)
-    latency_q = -1
+        info["compression-ratio"] = ascore, rscore, int(100*bump)
     if len(global_statistics.client_latency)>0 and global_statistics.recent_client_latency>0:
+        #if the latency is too high, lower quality target:
         latency_q = 3.0 * statistics.target_latency / global_statistics.recent_client_latency
         target = min(target, latency_q)
+        info["latency"] = int(100.0*latency_q)
     target = min(1.0, max(0.0, target))
     if min_speed>0:
         #discount the quality more aggressively if we have speed requirements to satisfy:
@@ -245,17 +259,20 @@ def get_target_quality(wid, window_dimensions, batch, global_statistics, statist
         #target=0.5   -> target=0.125
         #target=0     -> target=0
         target = target ** ((100.0 + 4*min_speed)/100.0)
+    #raise the quality when there are not many recent damage events:
+    ww, wh = window_dimensions
+    if ww>0 and wh>0:
+        now = monotonic_time()
+        damage_pixel_count = dict((lim, sum([w*h for t,_,_,w,h in list(statistics.last_damage_events) if t>=now-lim and t<now-lim+1])) for lim in range(1,11))
+        pixl5 = sum(v for lim,v in damage_pixel_count.items() if lim<=5)
+        pixn5 = sum(v for lim,v in damage_pixel_count.items() if lim>5)
+        pctpixdamaged = float(pixl5)/(ww*wh)
+        log("get_target_quality: target=%s (window %ix%i) pctpixdamaged=%i%%, dpc=%s", target, ww, wh, pctpixdamaged*100, damage_pixel_count)
+        if pctpixdamaged<=0.5:
+            target = min(1.0, target + (1.0-pctpixdamaged*2))
+        if pixl5<pixn5:
+            target = sqrt(target)
+    #apply min-quality:
     mq = min(100.0, max(min_quality, 0.0))
     target_quality = mq + (100.0-mq) * target
-    info = {
-            "min_quality"   : min_quality,
-            "min_speed"     : min_speed,
-            "backlog_factor": (packets_backlog, pixels_backlog, int(100.0*pixels_bl)),
-            }
-    if cratio_factor:
-        info["compression-ratio"] = cratio_factor
-    if batch_q>=0:
-        info["batch-delay-ratio"] = int(100.0*batch_q)
-    if latency_q>=0:
-        info["latency"] = int(100.0*latency_q)
     return info, target_quality
