@@ -1,27 +1,24 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2015 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2016 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import os
-import time
-
 from xpra.log import Logger
 log = Logger("sound")
 gstlog = Logger("gstreamer")
 
 #must be done before importing gobject!
-from xpra.sound.gstreamer_util import import_gst, gst_major_version
+from xpra.sound.gstreamer_util import import_gst
 gst = import_gst()
 MESSAGE_ELEMENT = getattr(gst, "MESSAGE_ELEMENT", None)
-from xpra.sound.gstreamer_util import gst_version       #must be done after import_gst()
 
-from xpra.util import csv
+from xpra.util import envint
+from xpra.os_util import monotonic_time
 from xpra.gtk_common.gobject_compat import import_glib
 from xpra.gtk_common.gobject_util import one_arg_signal, gobject
 
 
-FAULT_RATE = int(os.environ.get("XPRA_SOUND_FAULT_INJECTION_RATE", "0"))
+FAULT_RATE = envint("XPRA_SOUND_FAULT_INJECTION_RATE")
 _counter = 0
 def inject_fault():
     global FAULT_RATE
@@ -43,9 +40,11 @@ class SoundPipeline(gobject.GObject):
 
     def __init__(self, codec):
         gobject.GObject.__init__(self)
+        self.stream_compressor = None
         self.codec = codec
         self.codec_description = ""
         self.codec_mode = ""
+        self.container_format = ""
         self.container_description = ""
         self.bus = None
         self.bus_message_handler_id = None
@@ -65,12 +64,6 @@ class SoundPipeline(gobject.GObject):
         self.idle_add = glib.idle_add
         self.timeout_add = glib.timeout_add
         self.source_remove = glib.source_remove
-        if gst_version[0]<1:
-            #Gst 0.10: can handle both TAG and ELEMENT:
-            self.parse_message = self.parse_message0
-        else:
-            #Gst 1.0: (does not have MESSAGE_ELEMENT):
-            self.parse_message = self.parse_message1
 
     def idle_emit(self, sig, *args):
         self.idle_add(self.emit, sig, *args)
@@ -82,8 +75,11 @@ class SoundPipeline(gobject.GObject):
         def do_emit_info():
             self.emit_info_due = False
             if self.pipeline:
-                self.emit("info", self.get_info())
-        self.timeout_add(50, do_emit_info)
+                info = self.get_info()
+                #reset info:
+                self.info = {}
+                self.emit("info", info)
+        self.timeout_add(200, do_emit_info)
 
     def get_info(self):
         info = self.info.copy()
@@ -96,7 +92,7 @@ class SoundPipeline(gobject.GObject):
         gstlog("pipeline elements=%s", elements)
         self.pipeline_str = " ! ".join([x for x in elements if x is not None])
         gstlog("pipeline=%s", self.pipeline_str)
-        self.start_time = time.time()
+        self.start_time = monotonic_time()
         try:
             self.pipeline = gst.parse_launch(self.pipeline_str)
         except Exception as e:
@@ -133,6 +129,7 @@ class SoundPipeline(gobject.GObject):
         self.info["bitrate"] = new_bitrate
 
     def update_state(self, state):
+        log("update_state(%s)", state)
         self.state = state
         self.info["state"] = state
 
@@ -164,12 +161,20 @@ class SoundPipeline(gobject.GObject):
         self.idle_emit("new-stream", self.codec)
         self.update_state("active")
         self.pipeline.set_state(gst.STATE_PLAYING)
+        if self.stream_compressor:
+            self.info["stream-compressor"] = self.stream_compressor
         self.emit_info()
         #we may never get the stream start, synthesize codec event so we get logging:
         parts = self.codec.split("+")
         self.timeout_add(1000, self.new_codec_description, parts[0])
-        if len(parts)>1:
+        if len(parts)>1 and parts[1]!=self.stream_compressor:
             self.timeout_add(1000, self.new_container_description, parts[1])
+        elif self.container_format:
+            self.timeout_add(1000, self.new_container_description, self.container_format)
+        if self.stream_compressor:
+            def logsc():
+                self.gstloginfo("using stream compression %s", self.stream_compressor)
+            self.timeout_add(1000, logsc)
         log("SoundPipeline.start() done")
 
     def stop(self):
@@ -214,14 +219,28 @@ class SoundPipeline(gobject.GObject):
         log("SoundPipeline.cleanup() done")
 
 
+    def gstloginfo(self, msg, *args):
+        if self.state!="stopped":
+            gstlog.info(msg, *args)
+        else:
+            gstlog(msg, *args)
+
+    def gstlogwarn(self, msg, *args):
+        if self.state!="stopped":
+            gstlog.warn(msg, *args)
+        else:
+            gstlog(msg, *args)
+
     def new_codec_description(self, desc):
         log("new_codec_description(%s) current codec description=%s", desc, self.codec_description)
         if not desc:
             return
-        cdl = self.codec_description.lower()
         dl = desc.lower()
+        if dl=="wav" and self.codec_description:
+            return
+        cdl = self.codec_description.lower()
         if not cdl or (cdl!=dl and dl.find(cdl)<0 and cdl.find(dl)<0):
-            gstlog.info("using audio codec %s", dl)
+            self.gstloginfo("using audio codec %s", dl)
         self.codec_description = dl
         self.info["codec_description"]  = dl
 
@@ -230,9 +249,12 @@ class SoundPipeline(gobject.GObject):
         if not desc:
             return
         cdl = self.container_description.lower()
-        dl = desc.lower()
+        dl = {
+              "mka"         : "matroska",
+              "mpeg4"       : "iso fmp4",
+              }.get(desc.lower(), desc.lower())
         if not cdl or (cdl!=dl and dl.find(cdl)<0 and cdl.find(dl)<0):
-            gstlog.info("using container format %s", dl)
+            self.gstloginfo("using container format %s", dl)
         self.container_description = dl
         self.info["container_description"]  = dl
 
@@ -243,13 +265,15 @@ class SoundPipeline(gobject.GObject):
         t = message.type
         if t == gst.MESSAGE_EOS:
             self.pipeline.set_state(gst.STATE_NULL)
-            gstlog.info("EOS")
+            self.gstloginfo("EOS")
             self.update_state("stopped")
             self.idle_emit("state-changed", self.state)
         elif t == gst.MESSAGE_ERROR:
             self.pipeline.set_state(gst.STATE_NULL)
             err, details = message.parse_error()
             gstlog.error("pipeline error: %s", err)
+            if self.pipeline_str:
+                gstlog.error(" %s", self.pipeline_str)
             try:
                 #prettify (especially on win32):
                 p = details.find("\\Source\\")
@@ -257,7 +281,8 @@ class SoundPipeline(gobject.GObject):
                     details = details[p+len("\\Source\\"):]
                 for d in details.split(": "):
                     for dl in d.splitlines():
-                        gstlog.error(" %s", dl.strip())
+                        if dl.strip():
+                            gstlog.error(" %s", dl.strip())
             except:
                 gstlog.error(" %s", details)
             self.update_state("error")
@@ -268,8 +293,8 @@ class SoundPipeline(gobject.GObject):
             try:
                 self.parse_message(message)
             except Exception as e:
-                gstlog.warn("Warning: failed to parse gstreamer message:")
-                gstlog.warn(" %s: %s", type(e), e)
+                self.gstlogwarn("Warning: failed to parse gstreamer message:")
+                self.gstlogwarn(" %s: %s", type(e), e)
         elif t == gst.MESSAGE_STREAM_STATUS:
             gstlog("stream status: %s", message)
             try:
@@ -278,11 +303,10 @@ class SoundPipeline(gobject.GObject):
                 pass
         elif t == gst.MESSAGE_STREAM_START:
             log("stream start: %s", message)
-            if gst_major_version>0:
-                #with gstreamer 1.x, we don't always get the "audio-codec" message..
-                #so print the codec from here instead (and assume gstreamer is using what we told it to)
-                #after a delay, just in case we do get the real "audio-codec" message!
-                self.timeout_add(500, self.new_codec_description, self.codec.split("+")[0])
+            #with gstreamer 1.x, we don't always get the "audio-codec" message..
+            #so print the codec from here instead (and assume gstreamer is using what we told it to)
+            #after a delay, just in case we do get the real "audio-codec" message!
+            self.timeout_add(500, self.new_codec_description, self.codec.split("+")[0])
         elif t in (gst.MESSAGE_ASYNC_DONE, gst.MESSAGE_NEW_CLOCK):
             gstlog("%s", message)
         elif t == gst.MESSAGE_STATE_CHANGED:
@@ -293,73 +317,29 @@ class SoundPipeline(gobject.GObject):
                 self.update_state(state)
                 self.idle_emit("state-changed", state)
         elif t == gst.MESSAGE_DURATION:
-            if gst_major_version==0:
-                d = message.parse_duration()
-                try:
-                    v = d[1]
-                    if v>0:
-                        gstlog("duration changed: %s", v)
-                except:
-                    gstlog("duration changed: %s", d)
+            gstlog("duration changed: %s", message)
         elif t == gst.MESSAGE_LATENCY:
             gstlog("latency message from %s: %s", message.src, message)
         elif t == gst.MESSAGE_INFO:
-            log.info("pipeline message: %s", message)
+            self.gstloginfo("pipeline message: %s", message)
         elif t == gst.MESSAGE_WARNING:
             w = message.parse_warning()
-            gstlog.warn("pipeline warning: %s", w[0].message)
-            gstlog.info("pipeline warning: %s", w[1:])
+            self.gstlogwarn("pipeline warning: %s", w[0].message)
+            for x in w[1:]:
+                for l in x.split(":"):
+                    if l:
+                        if l.startswith("\n"):
+                            l = l.strip("\n")+" "
+                            for lp in l.split(". "):
+                                self.gstlogwarn(" %s", lp.strip())
+                        else:
+                            self.gstlogwarn("                  %s", l.strip("\n\r"))
         else:
-            gstlog.info("unhandled bus message type %s: %s", t, message)
+            self.gstlogwarn("unhandled bus message type %s: %s", t, message)
         self.emit_info()
         return 0
 
-    def parse_message0(self, message):
-        #message parsing code for GStreamer 0.10
-        structure = message.structure
-        found = False
-        if structure.has_field("bitrate"):
-            new_bitrate = int(structure["bitrate"])
-            self.update_bitrate(new_bitrate)
-            found = True
-        if structure.has_field("codec"):
-            desc = structure["codec"]
-            self.new_codec_description(desc)
-            found = True
-        if structure.has_field("audio-codec"):
-            desc = structure["audio-codec"]
-            self.new_codec_description(desc)
-            found = True
-        if structure.has_field("mode"):
-            mode = structure["mode"]
-            if self.codec_mode!=mode:
-                gstlog("mode: %s", mode)
-                self.codec_mode = mode
-                self.info["codec_mode"] = self.codec_mode
-            found = True
-        if structure.has_field("type"):
-            if structure["type"]=="volume-changed":
-                gstlog.info("volumes=%s", csv("%i%%" % (v*100/2**16) for v in structure["volumes"]))
-                found = True
-                self.info["volume"]  = self.get_volume()
-            else:
-                gstlog.info("type=%s", structure["type"])
-        if structure.has_field("container-format"):
-            v = structure["container-format"]
-            self.new_container_description(v)
-            found = True
-        if not found:
-            #these, we know about, so we just log them:
-            for x in ("minimum-bitrate", "maximum-bitrate", "channel-mode"):
-                if structure.has_field(x):
-                    v = structure[x]
-                    gstlog("tag message: %s = %s", x, v)
-                    return      #handled
-            gstlog.info("unknown sound pipeline message %s: %s", message, structure)
-            gstlog.info(" %s", structure.keys())
-
-
-    def parse_message1(self, message):
+    def parse_message(self, message):
         #message parsing code for GStreamer 1.x
         taglist = message.parse_tag()
         tags = [taglist.nth_tag_name(x) for x in range(taglist.n_tags())]
@@ -400,4 +380,4 @@ class SoundPipeline(gobject.GObject):
             #no match yet
             if len([x for x in tags if x in ("minimum-bitrate", "maximum-bitrate", "channel-mode")])==0:
                 structure = message.get_structure()
-                gstlog.info("unknown sound pipeline tag message: %s", structure.to_string())
+                self.gstloginfo("unknown sound pipeline tag message: %s", structure.to_string())
