@@ -5,14 +5,18 @@
 # later version. See the file COPYING for details.
 
 #tricky: use xpra.scripts.config to get to the python "platform" module
-from xpra.scripts.config import python_platform as pp
+from xpra.scripts.config import python_platform
 import sys
 import os
+
+import xpra
 from xpra.util import updict
-from xpra import __version__ as local_version
+from xpra.os_util import get_linux_distribution
 from xpra.log import Logger
 log = Logger("util")
 
+XPRA_VERSION = xpra.__version__     #@UndefinedVariable
+
 def version_as_numbers(version):
     return [int(x) for x in version.split(".")]
 
@@ -22,11 +26,11 @@ def version_compat_check(remote_version):
         log(msg)
         return msg
     rv = version_as_numbers(remote_version)
-    lv = version_as_numbers(local_version)
+    lv = version_as_numbers(XPRA_VERSION)
     if rv==lv:
         log("identical remote version: %s", remote_version)
         return None
-    if rv[0:3]<[0, 12, 6]:
+    if rv[0:3]<[0, 14, 10]:
         #this is the oldest version we support
         msg = "remote version %s is too old, sorry" % str(rv[:2])
         log(msg)
@@ -34,7 +38,7 @@ def version_compat_check(remote_version):
     if rv[0]>0:
         log("newer remote version %s may work, we'll see..", remote_version)
         return  None
-    log("local version %s should be compatible with remote version: %s", local_version, remote_version)
+    log("local version %s should be compatible with remote version: %s", XPRA_VERSION, remote_version)
     return None
 
 
@@ -42,18 +46,18 @@ def get_host_info():
     #this function is for non UI thread info
     info = {}
     try:
+        import struct
+        bits = struct.calcsize("P") * 8
         import socket
         info.update({
                     "pid"                   : os.getpid(),
                     "byteorder"             : sys.byteorder,
-                    "hostname"              : socket.gethostname()
-                    })
-        import struct
-        bits = struct.calcsize("P") * 8
-        updict(info, "python", {
-                    "bits"                  : bits,
-                    "full_version"          : sys.version,
-                    "version"               : ".".join(str(x) for x in sys.version_info[:3]),
+                    "hostname"              : socket.gethostname(),
+                    "python"                : {
+                                               "bits"                  : bits,
+                                               "full_version"          : sys.version,
+                                               "version"               : ".".join(str(x) for x in sys.version_info[:3]),
+                                               },
                     })
     except:
         pass
@@ -67,7 +71,7 @@ def get_host_info():
 
 def get_version_info():
     props = {
-             "version"  : local_version
+             "version"  : XPRA_VERSION
              }
     try:
         from xpra.src_info import LOCAL_MODIFICATIONS, REVISION
@@ -106,7 +110,8 @@ def get_version_info_full():
     return props
 
 def do_get_platform_info():
-    from xpra.os_util import platform_name
+    from xpra.os_util import platform_name, platform_release
+    pp = sys.modules.get("platform", python_platform)
     def get_processor_name():
         if pp.system() == "Windows":
             return pp.processor()
@@ -124,12 +129,15 @@ def do_get_platform_info():
                     return re.sub(".*model name.*:", "", line,1).strip()
         assert False
     info = {}
-    if sys.platform.startswith("linux") and hasattr(pp, "linux_distribution"):
-        info["linux_distribution"] = pp.linux_distribution()
+    ld = get_linux_distribution()
+    if ld:
+        info["linux_distribution"] = ld
+    release = platform_release(pp.release())
     info.update({
             ""          : sys.platform,
-            "name"      : platform_name(sys.platform, info.get("linux_distribution") or pp.release()),
+            "name"      : platform_name(sys.platform, info.get("linux_distribution") or release),
             "release"   : pp.release(),
+            "sysrelease": release,
             "platform"  : pp.platform(),
             "machine"   : pp.machine(),
             "processor" : pp.processor(),
