@@ -24,13 +24,19 @@ dbus_helper = None
 MENU_FORWARDING = os.environ.get("XPRA_MENU_FORWARDING", "1")=="1"
 if MENU_FORWARDING:
     try:
-        from xpra.dbus.helper import DBusHelper
-        dbus_helper = DBusHelper()
-        from xpra.dbus.gtk_menuactions import query_actions, query_menu, ACTIONS, MENUS
-    except Exception as e:
-        log.warn("Warning: menu forwarding is disabled:")
-        log.warn(" cannot load dbus helper: %s", e)
-        MENU_FORWARDING = False
+        from xpra import dbus
+        assert dbus
+    except ImportError as e:
+        log("this build does not include the dbus module, no menu forwarding")
+    else:
+        try:
+            from xpra.dbus.helper import DBusHelper
+            dbus_helper = DBusHelper()
+            from xpra.dbus.gtk_menuactions import query_actions, query_menu, ACTIONS, MENUS
+        except Exception as e:
+            log.warn("Warning: menu forwarding is disabled:")
+            log.warn(" cannot load dbus helper: %s", e)
+            MENU_FORWARDING = False
 
 
 X11Window = X11WindowBindings()
@@ -46,6 +52,10 @@ STATE_STRING = {
                 }
 IconicState = constants["IconicState"]
 NormalState = constants["NormalState"]
+ICONIC_STATE_STRING = {
+                       IconicState  : "Iconic",
+                       NormalState  : "Normal",
+                       }
 
 #add user friendly workspace logging:
 WORKSPACE_STR = {WORKSPACE_UNSET    : "UNSET",
@@ -164,13 +174,13 @@ class BaseWindowModel(CoreX11WindowModel):
                        gobject.PARAM_READWRITE),
         })
     _property_names = CoreX11WindowModel._property_names + [
-                      "transient-for", "fullscreen-monitors", "bypass-compositor", "group-leader", "window-type", "workspace", "strut",
+                      "transient-for", "fullscreen-monitors", "bypass-compositor", "group-leader", "window-type", "workspace", "strut", "opacity",
                       "menu",
                       #virtual attributes:
                       "fullscreen", "focused", "maximized", "above", "below", "shaded", "skip-taskbar", "skip-pager", "sticky"]
     _dynamic_property_names = CoreX11WindowModel._dynamic_property_names + [
                               "attention-requested",
-                              "menu", "workspace",
+                              "menu", "workspace", "opacity",
                               "fullscreen", "focused", "maximized", "above", "below", "shaded", "skip-taskbar", "skip-pager", "sticky"]
     _internal_property_names = CoreX11WindowModel._internal_property_names+["state"]
     _initial_x11_properties = CoreX11WindowModel._initial_x11_properties + [
@@ -201,8 +211,19 @@ class BaseWindowModel(CoreX11WindowModel):
 
     def __init__(self, client_window):
         super(BaseWindowModel, self).__init__(client_window)
+        self.last_unmap_serial = 0
         self._input_field = True            # The WM_HINTS input field
 
+    def serial_after_last_unmap(self, serial):
+        #"The serial member is set from the serial number reported in the protocol
+        # but expanded from the 16-bit least-significant bits to a full 32-bit value"
+        if serial>self.last_unmap_serial:
+            return True
+        #the serial can wrap around:
+        if self.last_unmap_serial-serial>=2**15:
+            return True
+        return False
+
 
     def _read_initial_X11_properties(self):
         metalog("%s.read_initial_X11_properties()", self._MODELTYPE)
@@ -636,16 +657,19 @@ class BaseWindowModel(CoreX11WindowModel):
                 log("ignoring 'HIDDEN' _NET_WM_STATE: %s", event)
                 #we don't honour those because they make little sense, see:
                 #https://mail.gnome.org/archives/wm-spec-list/2005-May/msg00004.html
-                pass
             elif atom1=="_NET_WM_STATE_MODAL":
                 update_wm_state("modal")
+            elif atom1=="_NET_WM_STATE_DEMANDS_ATTENTION":
+                update_wm_state("attention-requested")
             else:
-                log.info("process_client_message_event(%s) unhandled atom=%s", event, atom1)
+                log.info("Unhandled _NET_WM_STATE request: '%s'", event, atom1)
+                log.info(" event%s", event)
             return True
         elif event.message_type=="WM_CHANGE_STATE":
-            log("WM_CHANGE_STATE: %s", event.data[0])
-            if event.data[0]==IconicState and event.serial>self.last_unmap_serial and not self.is_OR() and not self.is_tray():
-                self._updateprop("iconic", True)
+            iconic = event.data[0]
+            log("WM_CHANGE_STATE: %s, serial=%s, last unmap serial=%#x", ICONIC_STATE_STRING.get(iconic, iconic), event.serial, self.last_unmap_serial)
+            if iconic in (IconicState, NormalState) and self.serial_after_last_unmap(event.serial) and not self.is_OR() and not self.is_tray():
+                self._updateprop("iconic", iconic==IconicState)
             return True
         elif event.message_type=="_NET_WM_MOVERESIZE":
             log("_NET_WM_MOVERESIZE: %s", event)
