@@ -10,13 +10,15 @@ import gobject
 from gtk import gdk
 import signal
 
-from xpra.x11.gtk2.models import Unmanageable
-from xpra.gtk_common.gobject_util import AutoPropGObjectMixin, one_arg_signal
+from xpra.util import envbool
+from xpra.x11.gtk2 import Unmanageable
+from xpra.gtk_common.gobject_util import one_arg_signal
 from xpra.gtk_common.error import XError, xsync, xswallow
 from xpra.x11.bindings.window_bindings import X11WindowBindings, constants, SHAPE_KIND #@UnresolvedImport
 from xpra.x11.gtk_x11.prop import prop_get, prop_set
 from xpra.x11.gtk_x11.send_wm import send_wm_delete_window
 from xpra.x11.gtk2.composite import CompositeHelper
+from xpra.x11.gtk2.models.model_stub import WindowModelStub
 from xpra.x11.gtk2.gdk_bindings import (
                 add_event_receiver,                         #@UnresolvedImport
                 remove_event_receiver,                      #@UnresolvedImport
@@ -32,9 +34,11 @@ geomlog = Logger("x11", "window", "geometry")
 
 
 X11Window = X11WindowBindings()
-ADDMASK = gdk.STRUCTURE_MASK | gdk.PROPERTY_CHANGE_MASK | gdk.FOCUS_CHANGE_MASK
-USE_XSHM = os.environ.get("XPRA_XSHM", "1")=="1"
-FORCE_QUIT = os.environ.get("XPRA_FORCE_QUIT", "1")=="1"
+ADDMASK = gdk.STRUCTURE_MASK | gdk.PROPERTY_CHANGE_MASK | gdk.FOCUS_CHANGE_MASK | gdk.POINTER_MOTION_MASK
+
+FORCE_QUIT = envbool("XPRA_FORCE_QUIT", True)
+XSHAPE = envbool("XPRA_XSHAPE", True)
+
 
 # grab stuff:
 NotifyNormal        = constants["NotifyNormal"]
@@ -69,7 +73,7 @@ def sanestr(s):
     return (s or "").strip("\0").replace("\0", " ")
 
 
-class CoreX11WindowModel(AutoPropGObjectMixin, gobject.GObject):
+class CoreX11WindowModel(WindowModelStub):
     """
         The utility superclass for all GTK2 / X11 window models,
         it wraps an X11 window (the "client-window").
@@ -92,6 +96,11 @@ class CoreX11WindowModel(AutoPropGObjectMixin, gobject.GObject):
         "geometry": (gobject.TYPE_PYOBJECT,
                 "current coordinates (x, y, w, h, border) for the window", "",
                 gobject.PARAM_READABLE),
+        #bits per pixel
+        "depth": (gobject.TYPE_INT,
+                "window bit depth", "",
+                -1, 64, -1,
+                gobject.PARAM_READABLE),
         #if the window depth is 32 bit
         "has-alpha": (gobject.TYPE_BOOLEAN,
                 "Does the window use transparency", "",
@@ -149,6 +158,7 @@ class CoreX11WindowModel(AutoPropGObjectMixin, gobject.GObject):
         "ungrab"                        : one_arg_signal,
         "bell"                          : one_arg_signal,
         "client-contents-changed"       : one_arg_signal,
+        "motion"                        : one_arg_signal,
         #x11 events we catch (and often re-emit as something else):
         "xpra-property-notify-event"    : one_arg_signal,
         "xpra-xkb-event"                : one_arg_signal,
@@ -158,10 +168,11 @@ class CoreX11WindowModel(AutoPropGObjectMixin, gobject.GObject):
         "xpra-client-message-event"     : one_arg_signal,
         "xpra-focus-in-event"           : one_arg_signal,
         "xpra-focus-out-event"          : one_arg_signal,
+        "xpra-motion-event"             : one_arg_signal,
         }
 
     #things that we expose:
-    _property_names         = ["xid", "has-alpha", "client-machine", "pid", "title", "role", "command", "shape", "class-instance", "protocols"]
+    _property_names         = ["xid", "depth", "has-alpha", "client-machine", "pid", "title", "role", "command", "shape", "class-instance", "protocols"]
     #exposed and changing (should be watched for notify signals):
     _dynamic_property_names = ["title", "command", "shape", "class-instance", "protocols"]
     #should not be exported to the clients:
@@ -182,30 +193,17 @@ class CoreX11WindowModel(AutoPropGObjectMixin, gobject.GObject):
         self.xid = client_window.xid
         self.client_window = client_window
         self.client_window_saved_events = self.client_window.get_events()
-        self._managed = False
-        self._managed_handlers = []
-        self._setup_done = False
         self._composite = None
         self._damage_forward_handle = None
+        self._setup_done = False
         self._kill_count = 0
         self._internal_set_property("client-window", client_window)
 
 
-    def __repr__(self):
-        try:
-            return "%s(%#x)" % (type(self).__name__, self.xid)
-        except:
-            return repr(self)
-
-
     #########################################
     # Setup and teardown
     #########################################
 
-    def is_managed(self):
-        return self._managed
-
-
     def call_setup(self):
         """
             Call this method to prepare the window:
@@ -220,6 +218,8 @@ class CoreX11WindowModel(AutoPropGObjectMixin, gobject.GObject):
         try:
             with xsync:
                 geom = X11Window.geometry_with_border(self.xid)
+                if geom is None:
+                    raise Unmanageable("window %#x disappeared already" % self.xid)
                 self._internal_set_property("geometry", geom[:4])
                 self._read_initial_X11_properties()
         except XError as e:
@@ -229,17 +229,19 @@ class CoreX11WindowModel(AutoPropGObjectMixin, gobject.GObject):
         # reparent, so I'm not sure doing this here in the superclass,
         # before we reparent, actually works... let's wait and see.
         try:
-            self._composite = CompositeHelper(self.client_window, False, USE_XSHM)
+            self._composite = CompositeHelper(self.client_window)
             with xsync:
                 self._composite.setup()
                 if X11Window.displayHasXShape():
                     X11Window.XShapeSelectInput(self.xid)
-        except XError as e:
+        except Exception as e:
             remove_event_receiver(self.client_window, self)
             log("%s %#x does not support compositing: %s", self._MODELTYPE, self.xid, e)
             with xswallow:
                 self._composite.destroy()
             self._composite = None
+            if isinstance(e, Unmanageable):
+                raise
             raise Unmanageable(e)
         #compositing is now enabled,
         #from now on we must call setup_failed to clean things up
@@ -287,6 +289,27 @@ class CoreX11WindowModel(AutoPropGObjectMixin, gobject.GObject):
             self._scrub_x11()
 
 
+    #########################################
+    # Damage / Composite
+    #########################################
+
+    def acknowledge_changes(self):
+        c = self._composite
+        assert c, "composite window destroyed outside the UI thread?"
+        c.acknowledge_changes()
+
+    def _forward_contents_changed(self, obj, event):
+        if self._managed:
+            self.emit("client-contents-changed", event)
+
+    def uses_XShm(self):
+        c = self._composite
+        return c and c.get_xshm_handle() is not None
+
+    def get_image(self, x, y, width, height, logger=log.debug):
+        return self._composite.get_image(x, y, width, height, logger)
+
+
     def _setup_property_sync(self):
         metalog("setup_property_sync()")
         #python properties which trigger an X11 property to be updated:
@@ -311,10 +334,11 @@ class CoreX11WindowModel(AutoPropGObjectMixin, gobject.GObject):
         """
         metalog("read_initial_X11_properties() core")
         #immutable ones:
-        has_alpha = X11Window.get_depth(self.xid)==32
-        metalog("initial X11 properties: xid=%#x, has-alpha=%s", self.xid, has_alpha)
+        depth = X11Window.get_depth(self.xid)
+        metalog("initial X11 properties: xid=%#x, depth=%i", self.xid, depth)
+        self._updateprop("depth", depth)
         self._updateprop("xid", self.xid)
-        self._updateprop("has-alpha", has_alpha)
+        self._updateprop("has-alpha", depth==32)
         self._updateprop("allowed-actions", self._DEFAULT_NET_WM_ALLOWED_ACTIONS)
         self._updateprop("shape", self._read_xshape())
         #note: some of those are technically mutable,
@@ -341,80 +365,30 @@ class CoreX11WindowModel(AutoPropGObjectMixin, gobject.GObject):
                 X11Window.XDeleteProperty(self.xid, prop)
 
 
-    #########################################
-    # Composite
-    #########################################
-
-    def _forward_contents_changed(self, obj, event):
-        if self._managed:
-            self.emit("client-contents-changed", event)
-
-    def acknowledge_changes(self):
-        c = self._composite
-        assert c, "composite window destroyed outside the UI thread?"
-        c.acknowledge_changes()
-
-    def uses_XShm(self):
-        c = self._composite
-        return c and c.get_shm_handle() is not None
-
-    def get_image(self, x, y, width, height, logger=log.debug):
-        handle = self._composite.get_contents_handle()
-        if handle is None:
-            logger("get_image(..) pixmap is None for window %#x", self.xid)
-            return  None
-
-        #try XShm:
-        try:
-            shm = self._composite.get_shm_handle()
-            #logger("get_image(..) XShm handle: %s, handle=%s, pixmap=%s", shm, handle, handle.get_pixmap())
-            if shm is not None:
-                with xsync:
-                    shm_image = shm.get_image(handle.get_pixmap(), x, y, width, height)
-                #logger("get_image(..) XShm image: %s", shm_image)
-                if shm_image:
-                    return shm_image
-        except Exception as e:
-            if type(e)==XError and e.msg=="BadMatch":
-                logger("get_image(%s, %s, %s, %s) get_image BadMatch ignored (window already gone?)", x, y, width, height)
-            else:
-                log.warn("get_image(%s, %s, %s, %s) get_image %s", x, y, width, height, e, exc_info=True)
-
-        try:
-            w = min(handle.get_width(), width)
-            h = min(handle.get_height(), height)
-            if w!=width or h!=height:
-                logger("get_image(%s, %s, %s, %s) clamped to pixmap dimensions: %sx%s", x, y, width, height, w, h)
-            with xsync:
-                return handle.get_image(x, y, w, h)
-        except Exception as e:
-            if type(e)==XError and e.msg=="BadMatch":
-                logger("get_image(%s, %s, %s, %s) get_image BadMatch ignored (window already gone?)", x, y, width, height)
-            else:
-                log.warn("get_image(%s, %s, %s, %s) get_image %s", x, y, width, height, e, exc_info=True)
-            return None
-
-
     #########################################
     # XShape
     #########################################
 
-    def _read_xshape(self):
-        if not X11Window.displayHasXShape():
+    def _read_xshape(self, x=0, y=0):
+        if not X11Window.displayHasXShape() or not XSHAPE:
             return {}
         extents = X11Window.XShapeQueryExtents(self.xid)
         if not extents:
             shapelog("read_shape for window %#x: no extents", self.xid)
             return {}
-        v = {}
         #w,h = X11Window.getGeometry(xid)[2:4]
+        shapelog("read_shape for window %#x: extents=%s", self.xid, extents)
         bextents = extents[0]
         cextents = extents[1]
         if bextents[0]==0 and cextents[0]==0:
             shapelog("read_shape for window %#x: none enabled", self.xid)
             return {}
-        v["Bounding.extents"] = bextents
-        v["Clip.extents"] = cextents
+        v = {
+             "x"                : x,
+             "y"                : y,
+             "Bounding.extents" : bextents,
+             "Clip.extents"     : cextents,
+             }
         for kind in SHAPE_KIND.keys():
             kind_name = SHAPE_KIND[kind]
             rectangles = X11Window.XShapeGetRectangles(self.xid, kind)
@@ -423,22 +397,6 @@ class CoreX11WindowModel(AutoPropGObjectMixin, gobject.GObject):
         return v
 
 
-    #########################################
-    # Connect to signals in a "managed" way
-    #########################################
-
-    def managed_connect(self, detailed_signal, handler, *args):
-        """ connects a signal handler and makes sure we will clean it up on unmanage() """
-        handler_id = self.connect(detailed_signal, handler, *args)
-        self._managed_handlers.append(handler_id)
-        return handler_id
-
-    def managed_disconnect(self):
-        for handler_id in self._managed_handlers:
-            self.disconnect(handler_id)
-        self._managed_handlers = []
-
-
     ################################
     # Property reading
     ################################
@@ -447,77 +405,8 @@ class CoreX11WindowModel(AutoPropGObjectMixin, gobject.GObject):
         #just extracts the size from the geometry:
         return self.get_property("geometry")[2:4]
 
-
-    #########################################
-    # Properties we choose to expose
-    #########################################
-
-    def get_property_names(self):
-        """ The properties that should be exposed to clients """
-        return self._property_names
-
-    def get_dynamic_property_names(self):
-        """ The properties that may change over time """
-        return self._dynamic_property_names
-
-    def get_internal_property_names(self):
-        """ The properties that should not be exposed to the client """
-        return self._internal_property_names
-
-    def get_logger(self, property_name):
-        if property_name in PROPERTIES_DEBUG:
-            return metalog.info
-        return metalog.debug
-
-    def _updateprop(self, name, value):
-        """ Updates the property and fires notify(),
-            but only if the value has changed
-            and if the window has finished setting up and it is still managed.
-            Can only be used for AutoPropGObjectMixin properties.
-        """
-        l = self.get_logger(name)
-        cur = self._gproperties.get(name, None)
-        if name not in self._gproperties or cur!=value:
-            l("updateprop(%s, %s) previous value=%s", name, value, cur)
-            self._gproperties[name] = value
-            if self._setup_done and self._managed:
-                self.notify(name)
-            else:
-                l("not sending notify(%s) (setup done=%s, managed=%s)", name, self._setup_done, self._managed)
-            return True
-        l("updateprop(%s, %s) unchanged", name, value)
-        return False
-
-    def get(self, name, default_value=None):
-        """ Allows us the avoid defining all the attributes we may ever query,
-            returns the default value if the property does not exist.
-        """
-        l = self.get_logger(name)
-        if name in self._property_names:
-            v = self.get_property(name)
-            l("get(%s, %s) using get_property=%s", name, default_value, v)
-        else:
-            v = default_value
-            l("get(%s, %s) returning default value=%s", name, default_value, v)
-        return v
-
-
-    #temporary? / convenience access methods:
-    def is_OR(self):
-        """ Is this an override-redirect window? """
-        return self.get("override-redirect", False)
-
-    def is_tray(self):
-        """ Is this a tray window? """
-        return self.get("tray", False)
-
-    def is_shadow(self):
-        """ Is this a shadow instead of a real window? """
-        return False
-
-    def has_alpha(self):
-        """ Does the pixel data have an alpha channel? """
-        return self.get("has-alpha", False)
+    def get_geometry(self):
+        return self.get_property("geometry")[:4]
 
 
     #########################################
@@ -604,8 +493,8 @@ class CoreX11WindowModel(AutoPropGObjectMixin, gobject.GObject):
         if name is None:
             name = self.prop_get("WM_NAME", "latin1", True)
             metalog("WM_NAME=%s", name)
-        self._updateprop("title", sanestr(name))
-        metalog("wm_name changed")
+        if self._updateprop("title", sanestr(name)):
+            metalog("wm_name changed")
 
     def _handle_role_change(self):
         role = self.prop_get("WM_WINDOW_ROLE", "latin1")
@@ -698,7 +587,7 @@ class CoreX11WindowModel(AutoPropGObjectMixin, gobject.GObject):
         shapelog("shape event: %s, kind=%s", event, SHAPE_KIND.get(event.kind, event.kind))
         cur_shape = self.get_property("shape")
         if cur_shape and cur_shape.get("serial", 0)>=event.serial:
-            shapelog("same or older xshape serial no: %#x", event.serial)
+            shapelog("same or older xshape serial no: %#x (current=%#x)", event.serial, cur_shape.get("serial", 0))
             return
         #remove serial before comparing dicts:
         try:
@@ -707,7 +596,12 @@ class CoreX11WindowModel(AutoPropGObjectMixin, gobject.GObject):
             pass
         #read new xshape:
         with xswallow:
-            v = self._read_xshape()
+            #should we pass the x and y offsets here?
+            #v = self._read_xshape(event.x, event.y)
+            if event.shaped:
+                v = self._read_xshape()
+            else:
+                v = {}
             if cur_shape==v:
                 shapelog("xshape unchanged")
                 return
@@ -719,7 +613,7 @@ class CoreX11WindowModel(AutoPropGObjectMixin, gobject.GObject):
     def do_xpra_xkb_event(self, event):
         #X11: XKBNotify
         log("WindowModel.do_xpra_xkb_event(%r)" % event)
-        if event.type!="bell":
+        if event.subtype!="bell":
             log.error("WindowModel.do_xpra_xkb_event(%r) unknown event type: %s" % (event, event.type))
             return
         event.window_model = self
@@ -759,6 +653,10 @@ class CoreX11WindowModel(AutoPropGObjectMixin, gobject.GObject):
             self.emit("ungrab", event)
 
 
+    def do_xpra_motion_event(self, event):
+        self.emit("motion", event)
+
+
     ################################
     # Actions
     ################################
@@ -787,8 +685,9 @@ class CoreX11WindowModel(AutoPropGObjectMixin, gobject.GObject):
                 # You don't wanna play ball?  Then no more Mr. Nice Guy!
                 self.force_quit()
             else:
-                log.warn("window %#x cannot be closed, it does not support WM_DELETE_WINDOW")
-                log.warn(" add FORCE_QUIT is disabled")
+                log.warn("window %#x ('%s') cannot be closed,", xid, title)
+                log.warn(" it does not support WM_DELETE_WINDOW")
+                log.warn(" and FORCE_QUIT is disabled")
 
     def force_quit(self):
         pid = self.get_property("pid")
