@@ -1,20 +1,24 @@
 # This file is part of Xpra.
 # Copyright (C) 2008, 2009 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 
 import gtk
+from gtk import gdk
 import cairo
 
-from xpra.gtk_common.gobject_util import one_arg_signal, non_none_list_accumulator
+from xpra.util import envint, envbool
+from xpra.gtk_common.gobject_util import one_arg_signal, non_none_list_accumulator, SIGNAL_RUN_LAST
 from xpra.gtk_common.error import XError
 from xpra.x11.gtk_x11.send_wm import send_wm_take_focus
-from xpra.x11.gtk_x11.prop import prop_set, prop_get, MotifWMHints
+from xpra.x11.gtk_x11.prop import prop_set, prop_get
+from xpra.x11.prop_conv import MotifWMHints
 from xpra.x11.bindings.window_bindings import X11WindowBindings #@UnresolvedImport
+from xpra.x11.gtk2 import Unmanageable
 from xpra.x11.gtk2.models.size_hints_util import sanitize_size_hints
-from xpra.x11.gtk2.models import Unmanageable, MAX_WINDOW_SIZE
+from xpra.x11.gtk2.models import MAX_WINDOW_SIZE
 from xpra.x11.gtk2.models.base import BaseWindowModel, constants
 from xpra.x11.gtk2.models.core import sanestr, gobject, xswallow, xsync
 from xpra.x11.gtk2.gdk_bindings import (
@@ -62,6 +66,11 @@ def configure_bits(value_mask):
     return "|".join((v for k,v in CW_MASK_TO_NAME.items() if (k&value_mask)))
 
 
+VALIDATE_CONFIGURE_REQUEST = envbool("XPRA_VALIDATE_CONFIGURE_REQUEST", False)
+CLAMP_OVERLAP = envint("XPRA_WINDOW_CLAMP_OVERLAP", 20)
+assert CLAMP_OVERLAP>=0
+
+
 class WindowModel(BaseWindowModel):
     """This represents a managed client window.  It allows one to produce
     widgets that view that client window in various ways."""
@@ -121,7 +130,7 @@ class WindowModel(BaseWindowModel):
         })
     __gsignals__ = dict(BaseWindowModel.__common_signals__)
     __gsignals__.update({
-        "ownership-election"            : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_PYOBJECT, (), non_none_list_accumulator),
+        "ownership-election"            : (SIGNAL_RUN_LAST, gobject.TYPE_PYOBJECT, (), non_none_list_accumulator),
         "child-map-request-event"       : one_arg_signal,
         "child-configure-request-event" : one_arg_signal,
         "xpra-destroy-event"            : one_arg_signal,
@@ -137,7 +146,7 @@ class WindowModel(BaseWindowModel):
                               "_NET_WM_STRUT", "_NET_WM_STRUT_PARTIAL"]
     _MODELTYPE = "Window"
 
-    def __init__(self, parking_window, client_window):
+    def __init__(self, parking_window, client_window, desktop_geometry):
         """Register a new client window with the WM.
 
         Raises an Unmanageable exception if this window should not be
@@ -147,10 +156,10 @@ class WindowModel(BaseWindowModel):
         super(WindowModel, self).__init__(client_window)
         self.parking_window = parking_window
         self.corral_window = None
+        self.desktop_geometry = desktop_geometry
         #extra state attributes so we can unmanage() the window cleanly:
         self.in_save_set = False
         self.client_reparented = False
-        self.last_unmap_serial = 0
         self.kill_count = 0
 
         self.call_setup()
@@ -165,12 +174,14 @@ class WindowModel(BaseWindowModel):
         x, y, w, h, _ = self.client_window.get_geometry()
         # We enable PROPERTY_CHANGE_MASK so that we can call
         # x11_get_server_time on this window.
-        self.corral_window = gtk.gdk.Window(self.parking_window,
-                                            x = x, y = y, width =w, height= h,
-                                            window_type=gtk.gdk.WINDOW_CHILD,
-                                            wclass=gtk.gdk.INPUT_OUTPUT,
-                                            event_mask=gtk.gdk.PROPERTY_CHANGE_MASK,
-                                            title = "CorralWindow-%#x" % self.xid)
+        # clamp this window to the desktop size:
+        x, y = self._clamp_to_desktop(x, y, w, h)
+        self.corral_window = gdk.Window(self.parking_window,
+                                        x=x, y=y, width=w, height=h,
+                                        window_type=gdk.WINDOW_CHILD,
+                                        wclass=gdk.INPUT_OUTPUT,
+                                        event_mask=gdk.PROPERTY_CHANGE_MASK,
+                                        title = "CorralWindow-%#x" % self.xid)
         log("setup() corral_window=%#x", self.corral_window.xid)
         prop_set(self.corral_window, "_NET_WM_NAME", "utf8", u"Xpra-CorralWindow-%#x" % self.xid)
         X11Window.substructureRedirect(self.corral_window.xid)
@@ -194,7 +205,10 @@ class WindowModel(BaseWindowModel):
         self.client_reparented = True
 
         geomlog("setup() geometry")
-        w, h = X11Window.geometry_with_border(self.xid)[2:4]
+        geom = X11Window.geometry_with_border(self.xid)
+        if geom is None:
+            raise Unmanageable("window %#x disappeared already" % self.xid)
+        w, h = geom[2:4]
         hints = self.get_property("size-hints")
         geomlog("setup() hints=%s size=%ix%i", hints, w, h)
         nw, nh = calc_constrained_size(w, h, hints)
@@ -211,6 +225,30 @@ class WindowModel(BaseWindowModel):
         self.client_window.get_geometry()
 
 
+    def _clamp_to_desktop(self, x, y, w, h):
+        if self.desktop_geometry:
+            dw, dh = self.desktop_geometry
+            if x+w<0:
+                x = min(0, CLAMP_OVERLAP-w)
+            elif x>=dw:
+                x = max(0, dw-CLAMP_OVERLAP)
+            if y+h<0:
+                y = min(0, CLAMP_OVERLAP-h)
+            elif y>dh:
+                y = max(0, dh-CLAMP_OVERLAP)
+        return x, y
+
+    def update_desktop_geometry(self, width, height):
+        if self.desktop_geometry==(width, height):
+            return  #no need to do anything
+        self.desktop_geometry = (width, height)
+        x, y, w, h = self.corral_window.get_geometry()[:4]
+        nx, ny = self._clamp_to_desktop(x, y, w, h)
+        if nx!=x or ny!=y:
+            log("update_desktop_geometry(%i, %i) adjusting corral window to new location: %i,%i", width, height, nx, ny)
+            self.corral_window.move(nx, ny)
+
+
     def _read_initial_X11_properties(self):
         metalog("read_initial_X11_properties() window")
         super(WindowModel, self)._read_initial_X11_properties()
@@ -267,7 +305,7 @@ class WindowModel(BaseWindowModel):
                 for prop in WindowModel.SCRUB_PROPERTIES:
                     X11Window.XDeleteProperty(self.xid, prop)
             if self.client_reparented:
-                self.client_window.reparent(gtk.gdk.get_default_root_window(), 0, 0)
+                self.client_window.reparent(gdk.get_default_root_window(), 0, 0)
                 self.client_reparented = False
             self.client_window.set_events(self.client_window_saved_events)
             #it is now safe to destroy the corral window:
@@ -301,7 +339,7 @@ class WindowModel(BaseWindowModel):
         with xsync:
             if X11Window.is_mapped(self.xid):
                 self.last_unmap_serial = X11Window.Unmap(self.xid)
-                log("client window %#x unmapped, serial=%s", self.xid, self.last_unmap_serial)
+                log("client window %#x unmapped, serial=%#x", self.xid, self.last_unmap_serial)
 
     def map(self):
         with xsync:
@@ -340,8 +378,8 @@ class WindowModel(BaseWindowModel):
         # Also, if we receive a *synthetic* UnmapNotify event, that always
         # means that the client has withdrawn the window (even if it was not
         # mapped in the first place) -- ICCCM section 4.1.4.
-        log("do_xpra_unmap_event(%s) client window unmapped", event)
-        if event.send_event or event.serial>self.last_unmap_serial:
+        log("do_xpra_unmap_event(%s) client window unmapped, last_unmap_serial=%#x", event, self.last_unmap_serial)
+        if event.send_event or self.serial_after_last_unmap(event.serial):
             self.unmanage()
 
     def do_xpra_destroy_event(self, event):
@@ -481,7 +519,8 @@ class WindowModel(BaseWindowModel):
             self._updateprop("geometry", (x, y, cw, ch))
 
     def do_child_configure_request_event(self, event):
-        geomlog("do_child_configure_request_event(%s) client=%#x, corral=%#x, value_mask=%s", event, self.xid, self.corral_window.xid, configure_bits(event.value_mask))
+        hints = self.get_property("size-hints")
+        geomlog("do_child_configure_request_event(%s) client=%#x, corral=%#x, value_mask=%s, size-hints=%s", event, self.xid, self.corral_window.xid, configure_bits(event.value_mask), hints)
         if event.value_mask & CWStackMode:
             geomlog(" restack above=%s, detail=%s", event.above, event.detail)
         # Also potentially update our record of what the app has requested:
@@ -496,7 +535,7 @@ class WindowModel(BaseWindowModel):
             ry = y
         if event.value_mask & CWX or event.value_mask & CWY:
             self._internal_set_property("set-initial-position", True)
-        self._updateprop("requested-position", (rx, ry))
+            self._updateprop("requested-position", (rx, ry))
 
         rw, rh = self.get_property("requested-size")
         if event.value_mask & CWWidth:
@@ -505,10 +544,11 @@ class WindowModel(BaseWindowModel):
         if event.value_mask & CWHeight:
             h = event.height
             rh = h
-        self._updateprop("requested-size", (rw, rh))
+        if event.value_mask & CWWidth or event.value_mask & CWHeight:
+            self._updateprop("requested-size", (rw, rh))
 
-        hints = self.get_property("size-hints")
-        w, h = calc_constrained_size(w, h, hints)
+        if VALIDATE_CONFIGURE_REQUEST:
+            w, h = calc_constrained_size(w, h, hints)
         #update the geometry now, as another request may come in
         #before we've had a chance to process the ConfigureNotify that the code below will generate
         self._updateprop("geometry", (x, y, w, h))
@@ -551,10 +591,10 @@ class WindowModel(BaseWindowModel):
 
     def _handle_icon_title_change(self):
         icon_name = self.prop_get("_NET_WM_ICON_NAME", "utf8", True)
-        iconlog("_NET_WM_ICON_NAME=%s", icon_name)
+        metalog("_NET_WM_ICON_NAME=%s", icon_name)
         if icon_name is None:
             icon_name = self.prop_get("WM_ICON_NAME", "latin1", True)
-            iconlog("WM_ICON_NAME=%s", icon_name)
+            metalog("WM_ICON_NAME=%s", icon_name)
         self._updateprop("icon-title", sanestr(icon_name))
 
     def _handle_motif_wm_hints_change(self):
@@ -600,16 +640,23 @@ class WindowModel(BaseWindowModel):
         if surf is not None:
             # FIXME: There is no Pixmap.new_for_display(), so this isn't
             # actually display-clean.  Oh well.
-            pixmap = gtk.gdk.Pixmap(None, surf.get_width(), surf.get_height(), 32)
+            pixmap = gdk.Pixmap(None, surf.get_width(), surf.get_height(), 32)
             screen = get_display_for(pixmap).get_default_screen()
-            pixmap.set_colormap(screen.get_rgba_colormap())
-            cr = pixmap.cairo_create()
-            cr.set_source_surface(surf)
-            # Important to use SOURCE, because a newly created Pixmap can have
-            # random trash as its contents, and otherwise that will show
-            # through any alpha in the icon:
-            cr.set_operator(cairo.OPERATOR_SOURCE)
-            cr.paint()
+            colormap = screen.get_rgba_colormap()
+            if not colormap:
+                colormap = screen.get_rgb_colormap()
+            if not colormap:
+                iconlog.warn("Warning: cannot find colormap for default screen")
+                pixmap = None
+            else:
+                pixmap.set_colormap(colormap)
+                cr = pixmap.cairo_create()
+                cr.set_source_surface(surf)
+                # Important to use SOURCE, because a newly created Pixmap can have
+                # random trash as its contents, and otherwise that will show
+                # through any alpha in the icon:
+                cr.set_operator(cairo.OPERATOR_SOURCE)
+                cr.paint()
         else:
             pixmap = None
         #FIXME: it would be more efficient to notify first,
@@ -654,7 +701,7 @@ class WindowModel(BaseWindowModel):
         #to make it consistent with the "icon" property,
         #return a cairo surface..
         surf = cairo.ImageSurface(cairo.FORMAT_ARGB32, p.get_width(), p.get_height())
-        gc = gtk.gdk.CairoContext(cairo.Context(surf))
+        gc = gdk.CairoContext(cairo.Context(surf))
         gc.set_source_pixbuf(p, 0, 0)
         gc.paint()
         iconlog("get_default_window_icon()=%s", surf)
@@ -690,7 +737,7 @@ class WindowModel(BaseWindowModel):
         # genuine race conditions here (e.g. suppose the client does not
         # actually get around to requesting the focus until after we have
         # already changed our mind and decided to give it to someone else).
-        now = gtk.gdk.x11_get_server_time(self.corral_window)
+        now = gdk.x11_get_server_time(self.corral_window)
         # ICCCM 4.1.7 *claims* to describe how we are supposed to give focus
         # to a window, but it is completely opaque.  From reading the
         # metacity, kwin, gtk+, and qt code, it appears that the actual rules
