@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2008, 2009 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2012-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -8,16 +8,19 @@ import gtk
 import gobject
 import os
 
+from xpra.util import envbool
 from xpra.gtk_common.error import xsync, xswallow
 from xpra.x11.gtk_x11.prop import prop_set, prop_get
+from xpra.x11.window_info import window_name, window_info
 from xpra.gtk_common.gobject_util import no_arg_signal, one_arg_signal
 
+from xpra.x11.gtk2 import Unmanageable
 from xpra.x11.gtk2.selection import ManagerSelection
 from xpra.x11.gtk2.world_window import WorldWindow
-from xpra.x11.gtk2.window import WindowModel, Unmanageable, configure_bits
+from xpra.x11.gtk2.window import WindowModel, configure_bits
 from xpra.x11.gtk2.gdk_bindings import (
                add_event_receiver,                              #@UnresolvedImport
-               add_catchall_receiver, remove_catchall_receiver, #@UnresolvedImport
+               add_fallback_receiver, remove_fallback_receiver, #@UnresolvedImport
                get_children,                                    #@UnresolvedImport
                )
 from xpra.x11.bindings.window_bindings import constants, X11WindowBindings #@UnresolvedImport
@@ -40,8 +43,8 @@ CWHeight        = constants["CWHeight"]
 NotifyPointerRoot   = constants["NotifyPointerRoot"]
 NotifyDetailNone    = constants["NotifyDetailNone"]
 
-FORCE_REPLACE_WM = os.environ.get("XPRA_FORCE_REPLACE_WM", "0")=="1"
-LOG_MANAGE_FAILURES = os.environ.get("XPRA_LOG_MANAGE_FAILURES", "0")=="1"
+FORCE_REPLACE_WM = envbool("XPRA_FORCE_REPLACE_WM", False)
+LOG_MANAGE_FAILURES = envbool("XPRA_LOG_MANAGE_FAILURES", False)
 
 NO_NET_SUPPORTED = os.environ.get("XPRA_NO_NET_SUPPORTED", "").split(",")
 
@@ -263,7 +266,7 @@ class Wm(gobject.GObject):
         # Okay, ready to select for SubstructureRedirect and then load in all
         # the existing clients.
         add_event_receiver(self._root, self)
-        add_catchall_receiver("xpra-client-message-event", self)
+        add_fallback_receiver("xpra-client-message-event", self)
         X11Window.substructureRedirect(self._root.xid)
 
         for w in get_children(self._root):
@@ -273,7 +276,7 @@ class Wm(gobject.GObject):
             if (w.get_window_type() == gtk.gdk.WINDOW_FOREIGN
                 and not X11Window.is_override_redirect(w.xid)
                 and X11Window.is_mapped(w.xid)):
-                log("Wm managing pre-existing child")
+                log("Wm managing pre-existing child window %#x", w.xid)
                 self._manage_client(w)
 
         # Also watch for focus change events on the root window
@@ -288,6 +291,9 @@ class Wm(gobject.GObject):
     def root_set(self, *args):
         prop_set(self._root, *args)
 
+    def root_get(self, *args):
+        return prop_get(self._root, *args)
+
     def set_dpi(self, xdpi, ydpi):
         #this is used by some newer versions of the dummy driver (xf86-driver-dummy)
         #(and will not be honoured by anything else..)
@@ -304,6 +310,9 @@ class Wm(gobject.GObject):
         v = [width, height]
         screenlog("_NET_DESKTOP_GEOMETRY=%s", v)
         self.root_set("_NET_DESKTOP_GEOMETRY", ["u32"], v)
+        #update all the windows:
+        for model in self._windows.values():
+            model.update_desktop_geometry(width, height)
 
     def set_default_frame_extents(self, v):
         framelog("set_default_frame_extents(%s)", v)
@@ -319,11 +328,6 @@ class Wm(gobject.GObject):
                 win._handle_frame_changed()
 
 
-    def enableCursors(self, on):
-        log("enableCursors(%s)" % on)
-        X11Keyboard.selectCursorChange(on)
-
-
     def do_get_property(self, pspec):
         if pspec.name == "windows":
             return frozenset(self._windows.itervalues())
@@ -340,26 +344,27 @@ class Wm(gobject.GObject):
             return
         try:
             with xsync:
-                self.do_manage_client(gdkwindow)
+                log("_manage_client(%s)", gdkwindow)
+                desktop_geometry = self.root_get("_NET_DESKTOP_GEOMETRY", ["u32"], True, False)
+                win = WindowModel(self._root, gdkwindow, desktop_geometry)
         except Exception as e:
             if LOG_MANAGE_FAILURES or type(e) not in (Unmanageable, ):
                 l = log.warn
             else:
                 l = log
-            l("failed to manage client %s: %s", gdkwindow, e, exc_info=True)
-
-    def do_manage_client(self, gdkwindow):
-        try:
-            win = WindowModel(self._root, gdkwindow)
-        except Unmanageable:
-            log("Window disappeared on us, never mind")
-            return
-        win.managed_connect("unmanaged", self._handle_client_unmanaged)
-        self._windows[gdkwindow] = win
-        self._windows_in_order.append(gdkwindow)
-        self.notify("windows")
-        self._update_window_list()
-        self.emit("new-window", win)
+            l("Warning: failed to manage client window %#x:", gdkwindow.xid)
+            l(" %s", e)
+            l("", exc_info=True)
+            with xswallow:
+                l(" window name: %s", window_name(gdkwindow))
+                l(" window info: %s", window_info(gdkwindow))
+        else:
+            win.managed_connect("unmanaged", self._handle_client_unmanaged)
+            self._windows[gdkwindow] = win
+            self._windows_in_order.append(gdkwindow)
+            self.notify("windows")
+            self._update_window_list()
+            self.emit("new-window", win)
 
     def _handle_client_unmanaged(self, window, wm_exiting):
         gdkwindow = window.get_property("client-window")
@@ -416,7 +421,7 @@ class Wm(gobject.GObject):
         self.cleanup()
 
     def cleanup(self):
-        remove_catchall_receiver("xpra-client-message-event", self)
+        remove_fallback_receiver("xpra-client-message-event", self)
         for win in list(self._windows.itervalues()):
             win.unmanage(True)
 
