@@ -1,7 +1,7 @@
 # coding=utf8
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2017 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -12,15 +12,16 @@ from xpra.log import Logger
 log = Logger("keyboard")
 
 
-from xpra.util import csv
+from xpra.util import csv, nonl
 from xpra.gtk_common.keymap import get_gtk_keymap
 from xpra.x11.gtk_x11.keys import grok_modifier_map
 from xpra.keyboard.mask import DEFAULT_MODIFIER_NUISANCE, DEFAULT_MODIFIER_NUISANCE_KEYNAMES, mask_to_names
 from xpra.server.keyboard_config_base import KeyboardConfigBase
-from xpra.x11.xkbhelper import do_set_keymap, set_all_keycodes, \
+from xpra.x11.xkbhelper import do_set_keymap, set_all_keycodes, set_keycode_translation, \
                            get_modifiers_from_meanings, get_modifiers_from_keycodes, \
                            clear_modifiers, set_modifiers, \
                            clean_keyboard_state
+from xpra.gtk_common.error import xsync
 from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings #@UnresolvedImport
 X11Keyboard = X11KeyboardBindings()
 
@@ -39,8 +40,10 @@ ALL_X11_MODIFIERS = {
 class KeyboardConfig(KeyboardConfigBase):
     def __init__(self):
         KeyboardConfigBase.__init__(self)
+        self.xkbmap_raw = False
         self.xkbmap_print = None
         self.xkbmap_query = None
+        self.xkbmap_query_struct = None
         self.xkbmap_mod_meanings = {}
         self.xkbmap_mod_managed = []
         self.xkbmap_mod_pointermissing = []
@@ -65,50 +68,47 @@ class KeyboardConfig(KeyboardConfigBase):
 
     def get_info(self):
         info = KeyboardConfigBase.get_info(self)
-        info["modifiers.filter"] = self.modifiers_filter
         #keycodes:
         if self.keycode_translation:
+            ksinfo = info.setdefault("keysym", {})
+            kcinfo = info.setdefault("keycode", {})
             for kc, keycode in self.keycode_translation.items():
                 if type(kc)==tuple:
                     client_keycode, keysym = kc
-                    info["keysym." + str(keysym)+"."+str(client_keycode)] = keycode
-                    info["keycode." + str(client_keycode)+"."+keysym] = keycode
+                    ksinfo.setdefault(keysym, {})[client_keycode] = keycode
+                    kcinfo.setdefault(client_keycode, {})[keysym] = keycode
                 else:
-                    info["keysym." + str(kc)] = keycode
-                    info["keycode." + str(kc)] = keycode
+                    kcinfo[kc] = keycode
         if self.xkbmap_keycodes:
             i = 0
+            kminfo = info.setdefault("keymap", {})
             for keyval, name, keycode, group, level in self.xkbmap_keycodes:
-                info["keymap.%s" % i] = (keyval, name, keycode, group, level)
+                kminfo[i] = (keyval, name, keycode, group, level)
                 i += 1
         #modifiers:
+        modinfo = {}
+        modsinfo = {}
+        modinfo["filter"] = self.modifiers_filter
         if self.modifier_client_keycodes:
             for mod, keys in self.modifier_client_keycodes.items():
-                info["modifier." + mod + ".client_keys"] = keys
+                modinfo.setdefault(mod, {})["client_keys"] = keys
         if self.keynames_for_mod:
             for mod, keys in self.keynames_for_mod.items():
-                info["modifier." + mod + ".keys"] = tuple(keys)
+                modinfo.setdefault(mod, {})["keys"] = tuple(keys)
         if self.keycodes_for_modifier_keynames:
             for mod, keys in self.keycodes_for_modifier_keynames.items():
-                info["modifier." + mod + ".keycodes"] = tuple(keys)
+                modinfo.setdefault(mod, {})["keycodes"] = tuple(keys)
         if self.xkbmap_mod_meanings:
             for mod, mod_name in self.xkbmap_mod_meanings.items():
-                info["modifier." + mod ] = mod_name
-        if self.xkbmap_x11_keycodes:
-            for keycode, keysyms in self.xkbmap_x11_keycodes.items():
-                info["x11_keycode." + str(keycode) ] = keysyms
-        for x in ("print", "layout", "variant"):
+                modinfo[mod] = mod_name
+        info["x11_keycode"] = self.xkbmap_x11_keycodes
+        for x in ("print", "layout", "variant", "mod_managed", "mod_pointermissing", "raw"):
             v = getattr(self, "xkbmap_"+x)
             if v:
                 info[x] = v
-        for x in ("nuisance", ):
-            v = getattr(self, "xkbmap_mod_"+x)
-            if v:
-                info["modifiers."+x] = list(v)
-        for x in ("managed", "pointermissing"):
-            v = getattr(self, "xkbmap_mod_"+x)
-            if v:
-                info["modifiers."+x] = v
+        modsinfo["nuisance"] = list(self.xkbmap_mod_nuisance or [])
+        info["modifier"] = modinfo
+        info["modifiers"] = modsinfo
         log("keyboard info: %s", info)
         return info
 
@@ -131,11 +131,12 @@ class KeyboardConfig(KeyboardConfigBase):
         #lists:
         parse_option("keycodes", props.listget)
         #dicts:
-        for x in ("mod_meanings", "x11_keycodes"):
+        for x in ("mod_meanings", "x11_keycodes", "query_struct"):
             parse_option(x, props.dictget)
         #lists of strings:
         for x in ("mod_managed", "mod_pointermissing"):
             parse_option(x, props.strlistget)
+        parse_option("raw", props.boolget)
         log("assign_keymap_options(..) modified %s", modded)
         return len(modded)>0
 
@@ -146,19 +147,28 @@ class KeyboardConfig(KeyboardConfigBase):
         """
         import hashlib
         m = hashlib.sha1()
+        def hashadd(v):
+            m.update(("/%s" % str(v)).encode("utf8"))
         m.update(KeyboardConfigBase.get_hash(self))
-        for x in (self.xkbmap_print, self.xkbmap_query, \
+        for x in (self.xkbmap_print, self.xkbmap_query, self.xkbmap_raw, \
                   self.xkbmap_mod_meanings, self.xkbmap_mod_pointermissing, \
                   self.xkbmap_keycodes, self.xkbmap_x11_keycodes):
-            m.update("/%s" % str(x))
+            hashadd(x)
+        if self.xkbmap_query_struct:
+            #flatten the dict in a predicatable order:
+            for k in sorted(self.xkbmap_query_struct.keys()):
+                hashadd(self.xkbmap_query_struct.get(k))
         return "%s/%s/%s" % (self.xkbmap_layout, self.xkbmap_variant, m.hexdigest())
 
     def compute_modifier_keynames(self):
         self.keycodes_for_modifier_keynames = {}
+        self.xkbmap_mod_nuisance = set(DEFAULT_MODIFIER_NUISANCE)
         keymap = gtk.gdk.keymap_get_default()
         if self.keynames_for_mod:
             for modifier, keynames in self.keynames_for_mod.items():
                 for keyname in keynames:
+                    if keyname in DEFAULT_MODIFIER_NUISANCE_KEYNAMES:
+                        self.xkbmap_mod_nuisance.add(modifier)
                     keyval = gtk.gdk.keyval_from_name(keyname)
                     if keyval==0:
                         log.error("no keyval found for keyname %s (modifier %s)", keyname, modifier)
@@ -166,7 +176,9 @@ class KeyboardConfig(KeyboardConfigBase):
                     entries = keymap.get_entries_for_keyval(keyval)
                     if entries:
                         for keycode, _, _ in entries:
-                            self.keycodes_for_modifier_keynames.setdefault(keyname, set()).add(keycode)
+                            l = self.keycodes_for_modifier_keynames.setdefault(keyname, [])
+                            if keycode not in l:
+                                l.append(keycode)
         log("compute_modifier_keynames: keycodes_for_modifier_keynames=%s", self.keycodes_for_modifier_keynames)
 
     def compute_client_modifier_keycodes(self):
@@ -179,7 +191,7 @@ class KeyboardConfig(KeyboardConfigBase):
             for k,v in self.keycode_translation.items():
                 reverse_trans[v] = k
             self.modifier_client_keycodes = {}
-            self.xkbmap_mod_nuisance = set()
+            self.xkbmap_mod_nuisance = set(DEFAULT_MODIFIER_NUISANCE)
             for modifier, keys in server_mappings.items():
                 client_keycodes = []
                 for keycode,keyname in keys:
@@ -192,7 +204,7 @@ class KeyboardConfig(KeyboardConfigBase):
             log("compute_client_modifier_keycodes() mappings=%s", self.modifier_client_keycodes)
             log("compute_client_modifier_keycodes() mod nuisance=%s", self.xkbmap_mod_nuisance)
         except Exception as e:
-            log.error("do_set_keymap: %s" % e, exc_info=True)
+            log.error("Error: compute_client_modifier_keycodes: %s" % e, exc_info=True)
 
     def compute_modifier_map(self):
         self.modifier_map = grok_modifier_map(gtk.gdk.display_get_default(), self.xkbmap_mod_meanings)
@@ -208,56 +220,96 @@ class KeyboardConfig(KeyboardConfigBase):
         return False
 
 
-    def set_keymap(self):
+    def set_layout(self, layout, variant):
+        log("set_layout(%s, %s)", layout, variant)
+        if layout!=self.xkbmap_layout or variant!=self.xkbmap_variant:
+            self.xkbmap_layout = layout
+            self.xkbmap_variant = variant
+            return True
+        return False
+
+
+    def set_keymap(self, translate_only=False):
         if not self.enabled:
             return
-        clean_keyboard_state()
-        try:
-            do_set_keymap(self.xkbmap_layout, self.xkbmap_variant,
-                          self.xkbmap_print, self.xkbmap_query)
-        except:
-            log.error("error setting new keymap", exc_info=True)
-        self.is_native_keymap = self.xkbmap_print!="" or self.xkbmap_query!=""
-        try:
-            #first clear all existing modifiers:
-            clean_keyboard_state()
-            clear_modifiers(ALL_X11_MODIFIERS.keys())       #just clear all of them (set or not)
-
-            #now set all the keycodes:
-            clean_keyboard_state()
-            self.keycode_translation = {}
-
-            has_keycodes = (self.xkbmap_x11_keycodes and len(self.xkbmap_x11_keycodes)>0) or \
-                            (self.xkbmap_keycodes and len(self.xkbmap_keycodes)>0)
-            assert has_keycodes, "client failed to provide any keycodes!"
-            #first compute the modifier maps as this may have an influence
-            #on the keycode mappings (at least for the from_keycodes case):
-            if self.xkbmap_mod_meanings:
-                #Unix-like OS provides modifier meanings:
-                self.keynames_for_mod = get_modifiers_from_meanings(self.xkbmap_mod_meanings)
-            elif self.xkbmap_keycodes:
-                #non-Unix-like OS provides just keycodes for now:
-                self.keynames_for_mod = get_modifiers_from_keycodes(self.xkbmap_keycodes)
-            else:
-                log.error("missing both xkbmap_mod_meanings and xkbmap_keycodes, modifiers will probably not work as expected!")
-                self.keynames_for_mod = {}
-            #if the client does not provide a full keymap,
-            #try to preserve the initial server keycodes
-            #(used by non X11 clients like osx,win32 or Android)
-            preserve_server_keycodes = not self.xkbmap_print and not self.xkbmap_query
-            self.keycode_translation = set_all_keycodes(self.xkbmap_x11_keycodes, self.xkbmap_keycodes, preserve_server_keycodes, self.keynames_for_mod)
+        log("set_keymap(%s) layout=%s, variant=%s, print=%s, query=%s", translate_only, self.xkbmap_layout, self.xkbmap_variant, nonl(self.xkbmap_print), nonl(self.xkbmap_query))
+        if translate_only:
+            self.keycode_translation = set_keycode_translation(self.xkbmap_x11_keycodes, self.xkbmap_keycodes)
             self.add_gtk_keynames()
-
-            #now set the new modifier mappings:
-            clean_keyboard_state()
-            log("going to set modifiers, xkbmap_mod_meanings=%s, len(xkbmap_keycodes)=%s", self.xkbmap_mod_meanings, len(self.xkbmap_keycodes or []))
-            if self.keynames_for_mod:
-                set_modifiers(self.keynames_for_mod)
             self.compute_modifier_keynames()
-            self.compute_client_modifier_keycodes()
-            log("keyname_for_mod=%s", self.keynames_for_mod)
+            return
+
+        try:
+            with xsync:
+                clean_keyboard_state()
+                do_set_keymap(self.xkbmap_layout, self.xkbmap_variant,
+                              self.xkbmap_print, self.xkbmap_query, self.xkbmap_query_struct)
         except:
-            log.error("error setting xmodmap", exc_info=True)
+            log.error("Error setting up new keymap", exc_info=True)
+        self.is_native_keymap = bool(self.xkbmap_print) or bool(self.xkbmap_query)
+        try:
+            with xsync:
+                #first clear all existing modifiers:
+                clean_keyboard_state()
+
+                if not self.xkbmap_raw:
+                    clear_modifiers(ALL_X11_MODIFIERS.keys())       #just clear all of them (set or not)
+
+                    #now set all the keycodes:
+                    clean_keyboard_state()
+
+                    has_keycodes = (self.xkbmap_x11_keycodes and len(self.xkbmap_x11_keycodes)>0) or \
+                                    (self.xkbmap_keycodes and len(self.xkbmap_keycodes)>0)
+                    assert has_keycodes, "client failed to provide any keycodes!"
+                    #first compute the modifier maps as this may have an influence
+                    #on the keycode mappings (at least for the from_keycodes case):
+                    if self.xkbmap_mod_meanings:
+                        #Unix-like OS provides modifier meanings:
+                        self.keynames_for_mod = get_modifiers_from_meanings(self.xkbmap_mod_meanings)
+                    elif self.xkbmap_keycodes:
+                        #non-Unix-like OS provides just keycodes for now:
+                        self.keynames_for_mod = get_modifiers_from_keycodes(self.xkbmap_keycodes)
+                    else:
+                        log.error("missing both xkbmap_mod_meanings and xkbmap_keycodes, modifiers will probably not work as expected!")
+                        self.keynames_for_mod = {}
+                    #if the client does not provide a full keymap,
+                    #try to preserve the initial server keycodes
+                    #(used by non X11 clients like osx,win32 or Android)
+                    preserve_server_keycodes = not self.xkbmap_print and not self.xkbmap_query
+                    self.keycode_translation = set_all_keycodes(self.xkbmap_x11_keycodes, self.xkbmap_keycodes, preserve_server_keycodes, self.keynames_for_mod)
+                    self.add_gtk_keynames()
+
+                    #now set the new modifier mappings:
+                    clean_keyboard_state()
+                    log("going to set modifiers, xkbmap_mod_meanings=%s, len(xkbmap_keycodes)=%s", self.xkbmap_mod_meanings, len(self.xkbmap_keycodes or []))
+                    if self.keynames_for_mod:
+                        set_modifiers(self.keynames_for_mod)
+                    log("keynames_for_mod=%s", self.keynames_for_mod)
+                    self.compute_modifier_keynames()
+                else:
+                    self.keycode_translation = {}
+                    log("keyboard raw mode, keycode translation left empty")
+                    log("keycode mappings=%s", X11Keyboard.get_keycode_mappings())
+                    mod_mappings = X11Keyboard.get_modifier_mappings()
+                    self.xkbmap_mod_meanings = {}
+                    for mod, mod_defs in mod_mappings.items():
+                        for mod_def in mod_defs:
+                            for v in mod_def:
+                                if type(v)==int:
+                                    l = self.keycodes_for_modifier_keynames.setdefault(mod, [])
+                                else:
+                                    self.xkbmap_mod_meanings[v] = mod
+                                    l = self.keynames_for_mod.setdefault(mod, [])
+                                if v not in l:
+                                    l.append(v)
+                    log("keynames_for_mod=%s", self.keynames_for_mod)
+                    log("keycodes_for_modifier_keynames=%s", self.keycodes_for_modifier_keynames)
+                    log("mod_meanings=%s", self.xkbmap_mod_meanings)
+                self.compute_client_modifier_keycodes()
+                log("keyname_for_mod=%s", self.keynames_for_mod)
+                clean_keyboard_state()
+        except:
+            log.error("Error setting X11 keyboard modifier map", exc_info=True)
 
     def add_gtk_keynames(self):
         #add the keynames we find via gtk
@@ -351,19 +403,18 @@ class KeyboardConfig(KeyboardConfigBase):
         if ignored_modifier_keynames is None:
             #this is not a keyboard event, ignore modifiers in "mod_pointermissing"
             def is_ignored(modifier, modifier_keynames):
-                m = modifier in (self.xkbmap_mod_pointermissing or [])
-                return m
+                return modifier in (self.xkbmap_mod_pointermissing or [])
         else:
             #keyboard event: ignore the keynames specified
             #(usually the modifier key being pressed/unpressed)
             def is_ignored(modifier, modifier_keynames):
-                m = set(modifier_keynames) & set(ignored_modifier_keynames)
-                return bool(m)
+                return len(set(modifier_keynames or []) & set(ignored_modifier_keynames or []))>0
 
         def filtered_modifiers_set(modifiers):
             m = set()
+            mm = self.xkbmap_mod_managed or ()
             for modifier in modifiers:
-                if self.xkbmap_mod_managed and modifier in self.xkbmap_mod_managed:
+                if modifier in mm:
                     log("modifier is server managed: %s", modifier)
                     continue
                 keynames = self.keynames_for_mod.get(modifier)
@@ -371,7 +422,7 @@ class KeyboardConfig(KeyboardConfigBase):
                     log("modifier %s ignored (in ignored keynames=%s)", modifier, keynames)
                     continue
                 m.add(modifier)
-            log("filtered_modifiers_set(%s)=%s", modifiers, csv(list(m)))
+            log("filtered_modifiers_set(%s)=%s", modifiers, m)
             return m
 
         def change_mask(modifiers, press, info):
@@ -379,7 +430,7 @@ class KeyboardConfig(KeyboardConfigBase):
             for modifier in modifiers:
                 keynames = self.keynames_for_mod.get(modifier)
                 if not keynames:
-                    log.error("unknown modifier: %s", modifier)
+                    log.error("Error: unknown modifier '%s'", modifier)
                     continue
                 #find the keycodes that match the keynames for this modifier
                 keycodes = []
@@ -402,7 +453,7 @@ class KeyboardConfig(KeyboardConfigBase):
                 #nuisance keys (lock, num, scroll) are toggled by a
                 #full key press + key release (so act accordingly in the loop below)
                 nuisance = modifier in self.xkbmap_mod_nuisance
-                log("keynames(%s)=%s, keycodes=%s, nuisance=%s", modifier, keynames, keycodes, nuisance)
+                log("keynames(%s)=%s, keycodes=%s, nuisance=%s, nuisance keys=%s", modifier, keynames, keycodes, nuisance, self.xkbmap_mod_nuisance)
                 modkeycode = None
                 if not press:
                     #since we want to unpress something,
@@ -445,9 +496,11 @@ class KeyboardConfig(KeyboardConfigBase):
         fr = change_mask(current.difference(wanted), False, "remove")
         fa = change_mask(wanted.difference(current), True, "add")
         if fr:
-            log.warn("Warning: failed to remove the following modifiers: %s", csv(fr))
+            log.warn("Warning: failed to remove the following modifiers:")
+            log.warn(" %s", csv(fr))
         elif fa:
-            log.warn("Warning: failed to add the following modifiers: %s", csv(fa))
+            log.warn("Warning: failed to add the following modifiers:")
+            log.warn(" %s", csv(fa))
         else:
             return  #all good!
         #this should never happen.. but if it does?
