@@ -1,16 +1,22 @@
 # coding=utf8
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2017 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
-import time
-import gtk.gdk
+
+try:
+    from gtk import gdk
+except Exception as e:
+    from xpra.scripts.main import InitException
+    raise InitException(e)
 
 #ensure that we use gtk as display source:
+from xpra.gtk_common.gobject_compat import want_gtk3
+want_gtk3(False)
 from xpra.x11.gtk2 import gdk_display_source
 assert gdk_display_source
 
@@ -18,11 +24,17 @@ from xpra.x11.bindings.randr_bindings import RandRBindings  #@UnresolvedImport
 RandR = RandRBindings()
 from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings #@UnresolvedImport
 X11Keyboard = X11KeyboardBindings()
-from xpra.x11.bindings.core_bindings import X11CoreBindings #@UnresolvedImport
+from xpra.x11.bindings.core_bindings import X11CoreBindings     #@UnresolvedImport
 X11Core = X11CoreBindings()
-from xpra.gtk_common.error import XError, xswallow, xsync
+from xpra.x11.bindings.window_bindings import X11WindowBindings #@UnresolvedImport
+X11Window = X11WindowBindings()
+from xpra.gtk_common.error import XError, xswallow, xsync, trap
+from xpra.gtk_common.gtk_util import get_xwindow
 from xpra.server.server_uuid import save_uuid, get_uuid
 from xpra.x11.fakeXinerama import find_libfakeXinerama, save_fakeXinerama_config, cleanup_fakeXinerama
+from xpra.os_util import StringIOClass, monotonic_time
+from xpra.net.compression import Compressed
+
 
 from xpra.log import Logger
 log = Logger("x11", "server")
@@ -31,34 +43,33 @@ mouselog = Logger("x11", "server", "mouse")
 grablog = Logger("server", "grab")
 cursorlog = Logger("server", "cursor")
 screenlog = Logger("server", "screen")
+xinputlog = Logger("xinput")
 gllog = Logger("screen", "opengl")
 
-from xpra.util import iround
+from xpra.util import iround, envbool, envint
 from xpra.server.gtk_server_base import GTKServerBase
 from xpra.x11.xkbhelper import clean_keyboard_state
 from xpra.x11.server_keyboard_config import KeyboardConfig
 
-MAX_CONCURRENT_CONNECTIONS = 20
+MAX_CONCURRENT_CONNECTIONS = envint("XPRA_MAX_CONCURRENT_CONNECTIONS", 20)
+ALWAYS_NOTIFY_MOTION = envbool("XPRA_ALWAYS_NOTIFY_MOTION", False)
 
 
-def window_name(window):
-    from xpra.x11.gtk_x11.prop import prop_get
-    return prop_get(window, "_NET_WM_NAME", "utf8", True) or "unknown"
+class XTestPointerDevice(object):
 
-def window_info(window):
-    from xpra.x11.gtk_x11.prop import prop_get
-    net_wm_name = prop_get(window, "_NET_WM_NAME", "utf8", True)
-    return "%s %s (%s / %s)" % (net_wm_name, window, window.get_geometry(), window.is_visible())
+    def __repr__(self):
+        return "XTestPointerDevice"
 
+    def move_pointer(self, screen_no, x, y, *args):
+        mouselog("xtest_fake_motion(%i, %s, %s)", screen_no, x, y)
+        X11Keyboard.xtest_fake_motion(screen_no, x, y)
 
-from xpra.x11.gtk2.gdk_bindings import get_children #@UnresolvedImport
-def dump_windows():
-    root = gtk.gdk.get_default_root_window()
-    log("root window: %s" % root)
-    children = get_children(root)
-    log("%s windows" % len(children))
-    for window in get_children(root):
-        log("found window: %s", window_info(window))
+    def click(self, button, pressed, *args):
+        mouselog("xtest_fake_button(%i, %s)", button, pressed)
+        X11Keyboard.xtest_fake_button(button, pressed)
+
+    def close(self):
+        pass
 
 
 class X11ServerBase(GTKServerBase):
@@ -68,18 +79,22 @@ class X11ServerBase(GTKServerBase):
         (see XpraServer or XpraX11ShadowServer for actual implementations)
     """
 
-    def __init__(self, clobber):
-        self.clobber = clobber
-        self.screen_number = gtk.gdk.display_get_default().get_default_screen().get_number()
-        self.root_window = gtk.gdk.get_default_root_window()
+    def __init__(self):
+        self.screen_number = gdk.display_get_default().get_default_screen().get_number()
+        self.root_window = gdk.get_default_root_window()
+        self.last_mouse_user = None
         GTKServerBase.__init__(self)
 
     def init(self, opts):
+        self.do_init(opts)
+        GTKServerBase.init(self, opts)
+
+    def do_init(self, opts):
         self.randr = opts.resize_display
         self.fake_xinerama = opts.fake_xinerama
         self.current_xinerama_config = None
         self.x11_init()
-        GTKServerBase.init(self, opts)
+        self.pointer_device = XTestPointerDevice()
 
     def x11_init(self):
         if self.fake_xinerama:
@@ -94,29 +109,51 @@ class X11ServerBase(GTKServerBase):
             log.error("Error: limited keyboard support")
         self.init_x11_atoms()
         if self.randr:
-            self.randr = RandR.has_randr()
-            if self.randr and len(RandR.get_screen_sizes())<=1:
-                #disable randr when we are dealing with a Xvfb
-                #with only one resolution available
-                #since we don't support adding them on the fly yet
-                self.randr = False
-            if self.randr:
-                display = gtk.gdk.display_get_default()
-                i=0
-                while i<display.get_n_screens():
-                    screen = display.get_screen(i)
-                    screen.connect("size-changed", self._screen_size_changed)
-                    i += 1
-            else:
-                log.warn("Warning: no X11 RandR support on %s", os.environ.get("DISPLAY"))
-        log("randr enabled: %s", self.randr)
+            self.init_randr()
+        self.init_cursor()
         self.query_opengl()
 
+    def init_randr(self):
+        self.randr = RandR.has_randr()
+        if self.randr and len(RandR.get_screen_sizes())<=1:
+            #disable randr when we are dealing with a Xvfb
+            #with only one resolution available
+            #since we don't support adding them on the fly yet
+            self.randr = False
+        if self.randr:
+            display = gdk.display_get_default()
+            i=0
+            while i<display.get_n_screens():
+                screen = display.get_screen(i)
+                screen.connect("size-changed", self._screen_size_changed)
+                i += 1
+            log("randr enabled: %s", self.randr)
+        else:
+            log.warn("Warning: no X11 RandR support on %s", os.environ.get("DISPLAY"))
+
+    def init_cursor(self):
+        #cursor:
+        self.default_cursor_data = None
+        self.last_cursor_serial = None
+        self.last_cursor_data = None
+        self.send_cursor_pending = False
+        def get_default_cursor():
+            self.default_cursor_data = X11Keyboard.get_cursor_image()
+            cursorlog("get_default_cursor=%s", self.default_cursor_data)
+        trap.swallow_synced(get_default_cursor)
+        X11Keyboard.selectCursorChange(True)
+
+    def get_display_bit_depth(self):
+        with xswallow:
+            return X11Window.get_depth(X11Window.getDefaultRootWindow())
+        return 0
+
     def query_opengl(self):
         self.opengl_props = {}
         try:
             import subprocess
-            cmd = self.get_full_child_command(["xpra", "opengl"])
+            from xpra.platform.paths import get_xpra_command
+            cmd = self.get_full_child_command(get_xpra_command()+["opengl"])
             env = self.get_child_env()
             proc = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=env, shell=False, close_fds=True)
             out,err = proc.communicate()
@@ -163,9 +200,19 @@ class X11ServerBase(GTKServerBase):
 
     def init_keyboard(self):
         GTKServerBase.init_keyboard(self)
+        self.current_keyboard_group = None
         #clear all modifiers
         clean_keyboard_state()
 
+    def set_keyboard_layout_group(self, grp):
+        if self.current_keyboard_group!=grp and X11Keyboard.hasXkb():
+            try:
+                with xsync:
+                    self.current_keyboard_group = X11Keyboard.set_layout_group(grp)
+            except Exception as e:
+                keylog("set_keyboard_layout_group(%s)", grp, exc_info=True)
+                keylog.error("Error: failed to set keyboard layout group '%s'", grp)
+                keylog.error(" %s", e)
 
     def init_packet_handlers(self):
         GTKServerBase.init_packet_handlers(self)
@@ -189,14 +236,14 @@ class X11ServerBase(GTKServerBase):
         return get_uuid()
 
     def save_uuid(self):
-        save_uuid(self.uuid)
+        save_uuid(unicode(self.uuid))
 
     def set_keyboard_repeat(self, key_repeat):
         if key_repeat:
             self.key_repeat_delay, self.key_repeat_interval = key_repeat
             if self.key_repeat_delay>0 and self.key_repeat_interval>0:
                 X11Keyboard.set_key_repeat_rate(self.key_repeat_delay, self.key_repeat_interval)
-                log.info("setting key repeat rate from client: %sms delay / %sms interval", self.key_repeat_delay, self.key_repeat_interval)
+                keylog.info("setting key repeat rate from client: %sms delay / %sms interval", self.key_repeat_delay, self.key_repeat_interval)
         else:
             #dont do any jitter compensation:
             self.key_repeat_delay = -1
@@ -214,45 +261,82 @@ class X11ServerBase(GTKServerBase):
                     "force_ungrab"              : True,
                     "keyboard.fast-switching"   : True,
                     })
+            if self.randr and len(RandR.get_screen_sizes())>1:
+                capabilities["screen-sizes"] = RandR.get_screen_sizes()
+            if self.default_cursor_data and source.wants_default_cursor:
+                capabilities["cursor.default"] = self.default_cursor_data
         return capabilities
 
     def do_get_info(self, proto, server_sources, window_ids):
-        start = time.time()
+        start = monotonic_time()
         info = GTKServerBase.do_get_info(self, proto, server_sources, window_ids)
         if self.opengl_props:
             info["opengl"] = self.opengl_props
-        #this is added here because the server keyboard config doesn't know about "keys_pressed"..
-        info.setdefault("keyboard", {}).update({
-                                                "state"           : {
-                                                                     "keys_pressed"   : list(self.keys_pressed.keys())
-                                                                     },
-                                                "fast-switching"  : True,
-                                                })
         sinfo = info.setdefault("server", {})
-        sinfo.update({"type"                : "Python/gtk/x11",
-                      "fakeXinerama"        : self.fake_xinerama and bool(self.libfakeXinerama_so),
-                      "libfakeXinerama"     : self.libfakeXinerama_so or "",
-                      "Xkb"                 : X11Keyboard.hasXkb(),
-                      "XTest"               : X11Keyboard.hasXTest(),
-                      })
+        sinfo.update({
+            "type"                  : "Python/gtk/x11",
+            "fakeXinerama"          : self.fake_xinerama and bool(self.libfakeXinerama_so),
+            "libfakeXinerama"       : self.libfakeXinerama_so or "",
+            })
+        log("X11ServerBase.do_get_info took %ims", (monotonic_time()-start)*1000)
+        return info
+
+    def get_ui_info(self, proto, wids=None, *args):
+        import threading
+        log("do_get_info thread=%s", threading.current_thread())
+        info = GTKServerBase.get_ui_info(self, proto, wids, *args)
+        #this is added here because the server keyboard config doesn't know about "keys_pressed"..
+        with xsync:
+            info.setdefault("keyboard", {}).update({
+                                                    "state"             : {
+                                                                           "keys_pressed"   : list(self.keys_pressed.keys())
+                                                                           },
+                                                    "fast-switching"    : True,
+                                                    "layout-group"      : X11Keyboard.get_layout_group(),
+                                                    })
+        sinfo = info.setdefault("server", {})
         try:
             from xpra.x11.gtk2.composite import CompositeHelper
             sinfo["XShm"] = CompositeHelper.XShmEnabled
         except:
             pass
+        #cursor:
+        log("do_get_info: adding cursor=%s", self.last_cursor_data)
+        info.setdefault("cursor", {}).update(self.get_cursor_info())
+        with xswallow:
+            sinfo.update({
+                "Xkb"                   : X11Keyboard.hasXkb(),
+                "XTest"                 : X11Keyboard.hasXTest(),
+                })
         #randr:
         try:
-            sizes = RandR.get_screen_sizes()
-            if self.randr and len(sizes)>=0:
-                sinfo["randr"] = {"options" : list(reversed(sorted(sizes)))}
+            with xsync:
+                sizes = RandR.get_screen_sizes()
+                if self.randr and len(sizes)>=0:
+                    sinfo["randr"] = {"options" : list(reversed(sorted(sizes)))}
         except:
             pass
-        log("X11ServerBase.do_get_info took %ims", (time.time()-start)*1000)
         return info
 
+
+    def get_cursor_info(self):
+        #(NOT from UI thread)
+        #copy to prevent race:
+        cd = self.last_cursor_data
+        if cd is None:
+            return {"" : "None"}
+        cinfo = {"is_default"   : bool(self.default_cursor_data and len(self.default_cursor_data)>=8 and len(cd)>=8 and cd[7]==cd[7])}
+        #all but pixels:
+        for i, x in enumerate(("x", "y", "width", "height", "xhot", "yhot", "serial", None, "name")):
+            if x:
+                v = cd[i] or ""
+                cinfo[x] = v
+        return cinfo
+
     def get_window_info(self, window):
         info = GTKServerBase.get_window_info(self, window)
         info["XShm"] = window.uses_XShm()
+        info["geometry"] = window.get_geometry()
         return info
 
 
@@ -267,12 +351,17 @@ class X11ServerBase(GTKServerBase):
 
 
     def set_keymap(self, server_source, force=False):
+        if self.readonly:
+            return
         try:
             #prevent _keys_changed() from firing:
             #(using a flag instead of keymap.disconnect(handler) as this did not seem to work!)
             self.keymap_changing = True
 
-            self.keyboard_config = server_source.set_keymap(self.keyboard_config, self.keys_pressed, force)
+            #if sharing, don't set the keymap, translate the existing one:
+            other_ui_clients = [s.uuid for s in self._server_sources.values() if s!=server_source and s.ui_client]
+            translate_only = len(other_ui_clients)>0
+            self.keyboard_config = server_source.set_keymap(self.keyboard_config, self.keys_pressed, force, translate_only)
         finally:
             # re-enable via idle_add to give all the pending
             # events a chance to run first (and get ignored)
@@ -289,7 +378,7 @@ class X11ServerBase(GTKServerBase):
         self.cancel_key_repeat_timer()
         #clear all the keys we know about:
         if len(self.keys_pressed)>0:
-            log("clearing keys pressed: %s", self.keys_pressed)
+            keylog("clearing keys pressed: %s", self.keys_pressed)
             for keycode in self.keys_pressed.keys():
                 X11Keyboard.xtest_fake_key(keycode, False)
             self.keys_pressed = {}
@@ -298,15 +387,23 @@ class X11ServerBase(GTKServerBase):
         X11Keyboard.unpress_all_keys()
 
 
-    def get_cursor_data(self):
+    def get_cursor_sizes(self):
+        display = gdk.display_get_default()
+        return display.get_default_cursor_size(), display.get_maximal_cursor_size()
+
+    def do_get_cursor_data(self):
         #must be called from the UI thread!
         try:
             with xsync:
-                cursor_data = X11Keyboard.get_cursor_image()
+                return X11Keyboard.get_cursor_image()
         except Exception as e:
             cursorlog.error("Error getting cursor data:")
             cursorlog.error(" %s", e)
-            return None, []
+            return None
+
+    def get_cursor_data(self):
+        #must be called from the UI thread!
+        cursor_data = self.do_get_cursor_data()
         if cursor_data is None:
             cursorlog("get_cursor_data() failed to get cursor image")
             return None, []
@@ -316,13 +413,12 @@ class X11ServerBase(GTKServerBase):
         if self.default_cursor_data is not None and str(pixels)==str(self.default_cursor_data[7]):
             cursorlog("get_cursor_data(): default cursor - clearing it")
             cursor_data = None
-        display = gtk.gdk.display_get_default()
-        cursor_sizes = display.get_default_cursor_size(), display.get_maximal_cursor_size()
+        cursor_sizes = self.get_cursor_sizes()
         return (cursor_data, cursor_sizes)
 
 
     def get_max_screen_size(self):
-        from xpra.x11.gtk2.window import MAX_WINDOW_SIZE
+        from xpra.x11.gtk2.models import MAX_WINDOW_SIZE
         max_w, max_h = self.root_window.get_size()
         sizes = RandR.get_screen_sizes()
         if self.randr and len(sizes)>=1:
@@ -335,7 +431,7 @@ class X11ServerBase(GTKServerBase):
         return max_w, max_h
 
 
-    def set_best_screen_size(self):
+    def configure_best_screen_size(self):
         #return ServerBase.set_best_screen_size(self)
         """ sets the screen size to use the largest width and height used by any of the clients """
         root_w, root_h = self.root_window.get_size()
@@ -357,11 +453,42 @@ class X11ServerBase(GTKServerBase):
             if len(sss)>1:
                 screenlog.info("* %s: %s", ss.uuid, size)
         screenlog("maximum client resolution is %sx%s (current server resolution is %sx%s)", max_w, max_h, root_w, root_h)
-        if max_w>0 and max_h>0:
-            return self.set_screen_size(max_w, max_h)
-        return  root_w, root_h
+        if max_w<=0 or max_h<=0:
+            #invalid - use fallback
+            return  root_w, root_h
+        return self.set_screen_size(max_w, max_h)
+
+    def get_best_screen_size(self, desired_w, desired_h, bigger=True):
+        return self.do_get_best_screen_size(desired_w, desired_h, bigger)
+
+    def do_get_best_screen_size(self, desired_w, desired_h, bigger=True):
+        #try to find the best screen size to resize to:
+        new_size = None
+        closest = {}
+        for w,h in RandR.get_screen_sizes():
+            if (w<desired_w)==bigger or (h<desired_h)==bigger:
+                distance = abs(w-desired_w)*abs(h-desired_h)
+                closest[distance] = (w, h)
+                continue            #size is too small/big for client
+            if new_size:
+                ew,eh = new_size
+                if (ew*eh<w*h)==bigger:
+                    continue        #we found a better (smaller/bigger) candidate already
+            new_size = w,h
+        if not new_size:
+            screenlog.warn("Warning: no matching resolution found for %sx%s", desired_w, desired_h)
+            if len(closest)>0:
+                new_size = sorted(closest.items())[0]
+                screenlog.warn(" using %sx%s instead", *new_size)
+            else:
+                root_w, root_h = self.root_window.get_size()
+                return  root_w, root_h
+        screenlog("best %s resolution for client(%sx%s) is: %s", ["smaller", "bigger"][bigger], desired_w, desired_h, new_size)
+        w, h = new_size
+        return w, h
 
     def set_screen_size(self, desired_w, desired_h):
+        screenlog("set_screen_size%s", (desired_w, desired_h))
         root_w, root_h = self.root_window.get_size()
         if not self.randr:
             return root_w,root_h
@@ -398,28 +525,8 @@ class X11ServerBase(GTKServerBase):
         self.set_dpi(xdpi, ydpi)
 
         #try to find the best screen size to resize to:
-        new_size = None
-        closest = {}
-        for w,h in RandR.get_screen_sizes():
-            if w<desired_w or h<desired_h:
-                distance = abs(w-desired_w)*abs(h-desired_h)
-                closest[distance] = (w, h)
-                continue            #size is too small for client
-            if new_size:
-                ew,eh = new_size
-                if ew*eh<w*h:
-                    continue        #we found a better (smaller) candidate already
-            new_size = w,h
-        if not new_size:
-            screenlog.warn("Warning: no matching resolution found for %sx%s", desired_w, desired_h)
-            if len(closest)>0:
-                new_size = sorted(closest.items())[0]
-                screenlog.warn(" using %sx%s instead", *new_size)
-            else:
-                return  root_w, root_h
-        screenlog("best resolution for client(%sx%s) is: %s", desired_w, desired_h, new_size)
-        #now actually apply the new settings:
-        w, h = new_size
+        w, h = self.get_best_screen_size(desired_w, desired_h)
+
         #fakeXinerama:
         ui_clients = [s for s in self._server_sources.values() if s.ui_client]
         source = None
@@ -493,9 +600,40 @@ class X11ServerBase(GTKServerBase):
         return  root_w, root_h
 
 
+    def set_icc_profile(self):
+        ui_clients = [s for s in self._server_sources.values() if s.ui_client]
+        if len(ui_clients)!=1:
+            screenlog("%i UI clients, not setting ICC profile")
+            self.reset_icc_profile()
+            return
+        icc = ui_clients[0].icc
+        data = None
+        for x in ("data", "icc-data", "icc-profile"):
+            if x in icc:
+                data = icc.get(x)
+                break
+        if not data:
+            screenlog("no icc data found in %s", icc)
+            self.reset_icc_profile()
+            return
+        import binascii
+        screenlog("set_icc_profile() icc data for %s: %s (%i bytes)", ui_clients[0], binascii.hexlify(data or ""), len(data or ""))
+        from xpra.x11.gtk_x11.prop import prop_set
+        #each CARD32 contains just one 8-bit value - don't ask me why
+        prop_set(self.root_window, "_ICC_PROFILE", ["u32"], [ord(x) for x in data])
+        prop_set(self.root_window, "_ICC_PROFILE_IN_X_VERSION", "u32", 0*100+4) #0.4 -> 0*100+4*1
+
+    def reset_icc_profile(self):
+        screenlog("reset_icc_profile()")
+        from xpra.x11.gtk_x11.prop import prop_del
+        prop_del(self.root_window, "_ICC_PROFILE")
+        prop_del(self.root_window, "_ICC_PROFILE_IN_X_VERSION")
+
+
     def do_cleanup(self, *args):
         GTKServerBase.do_cleanup(self)
         cleanup_fakeXinerama()
+        self.cleanup_input_devices()
 
 
     def _process_server_settings(self, proto, packet):
@@ -527,39 +665,175 @@ class X11ServerBase(GTKServerBase):
             X11Keyboard.xtest_fake_key(keycode, press)
 
 
-    def _move_pointer(self, wid, pos):
-        #(this is called within an xswallow context)
-        mouselog("move_pointer(%s, %s)", wid, pos)
-        x, y = pos
-        X11Keyboard.xtest_fake_motion(self.screen_number, x, y)
+    def do_xpra_cursor_event(self, event):
+        if not self.cursors:
+            return
+        if self.last_cursor_serial==event.cursor_serial:
+            cursorlog("ignoring cursor event %s with the same serial number %s", event, self.last_cursor_serial)
+            return
+        cursorlog("cursor_event: %s", event)
+        self.last_cursor_serial = event.cursor_serial
+        for ss in self._server_sources.values():
+            ss.send_cursor()
+        return False
 
-    def _process_mouse_common(self, proto, wid, pointer):
+
+    def _motion_signaled(self, model, event):
+        mouselog("motion_signaled(%s, %s) last mouse user=%s", model, event, self.last_mouse_user)
+        #find the window model for this gdk window:
+        wid = self._window_to_id.get(model)
+        if not wid:
+            return
+        for ss in self._server_sources.values():
+            if ALWAYS_NOTIFY_MOTION or self.last_mouse_user is None or self.last_mouse_user!=ss.uuid:
+                ss.update_mouse(wid, event.x_root, event.y_root, event.x, event.y)
+
+
+    def do_xpra_xkb_event(self, event):
+        #X11: XKBNotify
+        log("WindowModel.do_xpra_xkb_event(%r)" % event)
+        if event.subtype!="bell":
+            log.error("do_xpra_xkb_event(%r) unknown event type: %s" % (event, event.type))
+            return
+        #bell events on our windows will come through the bell signal,
+        #this method is a catch-all for events on windows we don't manage,
+        #so we use wid=0 for that:
+        wid = 0
+        for ss in self._server_sources.values():
+            ss.bell(wid, event.device, event.percent, event.pitch, event.duration, event.bell_class, event.bell_id, event.bell_name or "")
+
+
+    def _bell_signaled(self, wm, event):
+        log("bell signaled on window %#x", get_xwindow(event.window))
+        if not self.bell:
+            return
+        wid = 0
+        if event.window!=gdk.get_default_root_window() and event.window_model is not None:
+            try:
+                wid = self._window_to_id[event.window_model]
+            except:
+                pass
+        log("_bell_signaled(%s,%r) wid=%s", wm, event, wid)
+        for ss in self._server_sources.values():
+            ss.bell(wid, event.device, event.percent, event.pitch, event.duration, event.bell_class, event.bell_id, event.bell_name or "")
+
+
+    def get_screen_number(self, wid):
+        #maybe this should be in all cases (it is in desktop_server):
+        #model = self._id_to_window.get(wid)
+        #return model.client_window.get_screen().get_number()
+        #return gdk.display_get_default().get_default_screen().get_number()
+        #-1 uses the current screen
+        return -1
+
+
+    def cleanup_input_devices(self):
+        pass
+
+
+    def setup_input_devices(self):
+        xinputlog("setup_input_devices() format=%s, input_devices=%s", self.input_devices_format, self.input_devices)
+
+    def _move_pointer(self, wid, pos, deviceid, *args):
+        #(this is called within an xswallow context)
+        screen_no = self.get_screen_number(wid)
+        device = self.pointer_device
+        mouselog("move_pointer(%s, %s, %s) screen_no=%i, device=%s", wid, pos, deviceid, screen_no, device)
+        x, y = pos
+        try:
+            device.move_pointer(screen_no, x, y, *args)
+        except Exception as e:
+            mouselog.error("Error: failed to move the pointer to %sx%s using %s", x, y, device)
+            mouselog.error(" %s", e)
+
+    def do_process_mouse_common(self, proto, wid, pointer, deviceid=-1, *args):
+        log("do_process_mouse_common%s", tuple([proto, wid, pointer, deviceid]+list(args)))
+        if self.readonly:
+            return
+        if self.input_devices_data:
+            device_data = self.input_devices_data.get(deviceid)
+            if device_data:
+                mouselog("process_mouse_common from device=%s", device_data.get("name"))
         pos = self.root_window.get_pointer()[:2]
-        if pos!=pointer:
+        uuid = None
+        if proto:
+            ss = self._server_sources.get(proto)
+            if ss:
+                uuid = ss.uuid
+        if pos!=pointer or self.input_devices=="xi":
+            self.last_mouse_user = uuid
             with xswallow:
-                self._move_pointer(wid, pointer)
+                self._move_pointer(wid, pointer, deviceid, *args)
 
     def _update_modifiers(self, proto, wid, modifiers):
+        if self.readonly:
+            return
         ss = self._server_sources.get(proto)
         if ss:
+            if self.ui_driver and self.ui_driver!=ss.uuid:
+                return
             ss.make_keymask_match(modifiers)
             if wid==self.get_focus():
                 ss.user_event()
 
-    def _process_button_action(self, proto, packet):
-        ss = self._server_sources.get(proto)
-        if ss is None:
-            return
-        wid, button, pressed, pointer, modifiers = packet[1:6]
+    def do_process_button_action(self, proto, wid, button, pressed, pointer, modifiers, buttons=[], deviceid=-1, *args):
         self._update_modifiers(proto, wid, modifiers)
-        self._process_mouse_common(proto, wid, pointer)
-        ss.user_event()
-        mouselog("xtest_fake_button(%s, %s) at %s", button, pressed, pointer)
+        #TODO: pass extra args
+        self._process_mouse_common(proto, wid, pointer, deviceid)
+        self.button_action(button, pressed, deviceid)
+
+    def button_action(self, button, pressed, deviceid=-1, *args):
+        device = self.pointer_device
+        assert device, "pointer device %s not found" % deviceid
         try:
             with xsync:
-                X11Keyboard.xtest_fake_button(button, pressed)
+                device.click(button, pressed, *args)
         except XError:
-            err = "Failed to pass on (un)press of mouse button %s" % button
+            log.error("Error: failed (un)press mouse button %s", button)
             if button>=4:
-                err += " (perhaps your Xvfb does not support mousewheels?)"
-            log.warn(err)
+                log.error(" (perhaps your Xvfb does not support mousewheels?)")
+
+
+    def make_screenshot_packet_from_regions(self, regions):
+        #regions = array of (wid, x, y, PIL.Image)
+        if len(regions)==0:
+            log("screenshot: no regions found, returning empty 0x0 image!")
+            return ["screenshot", 0, 0, "png", -1, ""]
+        #in theory, we could run the rest in a non-UI thread since we're done with GTK..
+        minx = min([x for (_,x,_,_) in regions])
+        miny = min([y for (_,_,y,_) in regions])
+        maxx = max([(x+img.get_width()) for (_,x,_,img) in regions])
+        maxy = max([(y+img.get_height()) for (_,_,y,img) in regions])
+        width = maxx-minx
+        height = maxy-miny
+        log("screenshot: %sx%s, min x=%s y=%s", width, height, minx, miny)
+        from PIL import Image                           #@UnresolvedImport
+        screenshot = Image.new("RGBA", (width, height))
+        for wid, x, y, img in reversed(regions):
+            pixel_format = img.get_pixel_format()
+            target_format = {
+                     "XRGB"   : "RGB",
+                     "BGRX"   : "RGB",
+                     "BGRA"   : "RGBA"}.get(pixel_format, pixel_format)
+            pixels = img.get_pixels()
+            w = img.get_width()
+            h = img.get_height()
+            #PIL cannot use the memoryview directly:
+            if isinstance(pixels, memoryview):
+                pixels = pixels.tobytes()
+            try:
+                window_image = Image.frombuffer(target_format, (w, h), pixels, "raw", pixel_format, img.get_rowstride())
+            except:
+                log.error("Error parsing window pixels in %s format for window %i", pixel_format, wid, exc_info=True)
+                continue
+            tx = x-minx
+            ty = y-miny
+            screenshot.paste(window_image, (tx, ty))
+        buf = StringIOClass()
+        screenshot.save(buf, "png")
+        data = buf.getvalue()
+        buf.close()
+        packet = ["screenshot", width, height, "png", width*4, Compressed("png", data)]
+        log("screenshot: %sx%s %s", packet[1], packet[2], packet[-1])
+        return packet
+
