@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2011-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -7,10 +7,10 @@ import os
 import re
 
 #ensure that we use gtk as display source:
-from xpra.x11.gtk2 import gdk_display_source
-assert gdk_display_source
+from xpra.x11.gtk_x11.gdk_display_source import init_display_source
+init_display_source()
 
-from xpra.util import std
+from xpra.util import std, csv
 from xpra.keyboard.layouts import parse_xkbmap_query
 from xpra.gtk_common.error import xsync
 from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings #@UnresolvedImport
@@ -51,44 +51,56 @@ def clean_keyboard_state():
 # keyboard layouts
 
 def do_set_keymap(xkbmap_layout, xkbmap_variant,
-                  xkbmap_print, xkbmap_query):
+                  xkbmap_print, xkbmap_query, xkbmap_query_struct={}):
     """ xkbmap_layout is the generic layout name (used on non posix platforms)
         xkbmap_variant is the layout variant (may not be set)
         xkbmap_print is the output of "setxkbmap -print" on the client
         xkbmap_query is the output of "setxkbmap -query" on the client
+        xkbmap_query_struct is xkbmap_query parsed into a dictionary
         Use those to try to setup the correct keyboard map for the client
         so that all the keycodes sent will be mapped
     """
-    #First we try to use data from setxkbmap -query
-    if xkbmap_query:
-        log("do_set_keymap using xkbmap_query")
-        """ The xkbmap_query data will look something like this:
-        rules:      evdev
-        model:      evdev
-        layout:     gb
-        options:    grp:shift_caps_toggle
+    #First we try to use data from setxkbmap -query,
+    #preferably as structured data:
+    if xkbmap_query and not xkbmap_query_struct:
+        xkbmap_query_struct = parse_xkbmap_query(xkbmap_query)
+    if xkbmap_query_struct:
+        log("do_set_keymap using xkbmap_query struct=%s", xkbmap_query_struct)
+        """ The xkbmap_query_struct data will look something like this:
+            {
+            "rules"       : "evdev",
+            "model"       : "pc105",
+            "layout"      : "gb",
+            "options"     : "grp:shift_caps_toggle",
+            }
         (we execute the options separately in case that fails..)
         """
         #parse the data into a dict:
-        settings = parse_xkbmap_query(xkbmap_query)
-        rules = settings.get("rules")
-        model = settings.get("model")
-        layout = settings.get("layout")
-        variant = settings.get("variant")
-        options = settings.get("options")
+        rules = xkbmap_query_struct.get("rules")
+        model = xkbmap_query_struct.get("model")
+        layout = xkbmap_query_struct.get("layout")
+        variant = xkbmap_query_struct.get("variant")
+        options = xkbmap_query_struct.get("options")
         if layout:
-            log.info("setting keymap: %s", ", ".join(["%s=%s" % (std(k), std(v)) for k,v in settings.items() if k in ["rules", "model", "layout"] and v]))
+            log.info("setting keymap: %s", ", ".join(["%s=%s" % (std(k), std(v)) for k,v in xkbmap_query_struct.items() if k in ["rules", "model", "layout"] and v]))
             try:
                 X11Keyboard.setxkbmap(rules, model, layout, variant, options)
                 return
             except:
-                log.warn("failed to set exact keymap using %s", settings)
+                log.warn("failed to set exact keymap using %s", xkbmap_query_struct)
             if options:
                 #try again with no options:
                 try:
                     X11Keyboard.setxkbmap(rules, model, layout, variant, "")
+                    return
                 except:
                     log.error("failed to set exact keymap even without applying options")
+        else:
+            try:
+                X11Keyboard.setxkbmap(rules, model, "", "", "")
+                return
+            except:
+                log.warn("failed to set exact keymap using %s", xkbmap_query_struct)
     if xkbmap_print:
         log("do_set_keymap using xkbmap_print")
         #try to guess the layout by parsing "setxkbmap -print"
@@ -124,6 +136,56 @@ def apply_xmodmap(instructions):
         unset = instructions
     return unset
 
+
+def set_keycode_translation(xkbmap_x11_keycodes, xkbmap_keycodes):
+    """
+        Translate the given keycodes into the existing keymap
+    """
+    #get the list of keycodes (either from x11 keycodes or gtk keycodes):
+    if xkbmap_x11_keycodes and len(xkbmap_x11_keycodes)>0:
+        dump_dict(xkbmap_x11_keycodes)
+        keycodes = indexed_mappings(xkbmap_x11_keycodes)
+    else:
+        keycodes = gtk_keycodes_to_mappings(xkbmap_keycodes)
+    #keycodes = {
+    #     9: set([('', 1), ('Escape', 4), ('', 3), ('Escape', 0), ('Escape', 2)]),
+    #     10: set([('onesuperior', 4), ('onesuperior', 8), ('exclam', 1), ('1', 6), ('exclam', 3), ('1', 2), ('exclamdown', 9), ('exclamdown', 5), ('1', 0), ('exclam', 7)]),
+    x11_keycodes = X11Keyboard.get_keycode_mappings()
+    #x11_keycodes = {
+    #    8: ['Mode_switch', '', 'Mode_switch', '', 'Mode_switch'],
+    #    9: ['Escape', '', 'Escape', '', 'Escape'],
+    #for faster lookups:
+    x11_keycodes_for_keysym = {}
+    for keycode, keysyms in x11_keycodes.items():
+        for keysym in keysyms:
+            x11_keycodes_for_keysym.setdefault(keysym, []).append(keycode)
+    def find_keycode(kc, keysym, i):
+        keycodes = x11_keycodes_for_keysym.get(keysym)
+        if not keycodes:
+            return None
+        #no other option, use it:
+        if len(keycodes)==1:
+            return keycodes[0]
+        for keycode in keycodes:
+            defs = x11_keycodes.get(keycode)
+            assert defs, "bug: keycode %i not found in %s" % (keycode, x11_keycodes)
+            if (keysym, i) in defs:
+                return keycode
+        #if possible, use the same one:
+        if kc in keycodes:
+            return kc
+        return keycodes[0]
+    #generate the translation map:
+    trans = {}
+    for keycode, defs in keycodes.items():
+        for keysym,i in list(defs):             #ie: ('1', 0)
+            x11_keycode = find_keycode(keycode, keysym, i)
+            if x11_keycode:
+                trans[(keycode, keysym)] = x11_keycode
+                trans[keysym] = x11_keycode
+    log("set_keycode_translation(..)=%s", trans)
+    return trans
+
 def set_all_keycodes(xkbmap_x11_keycodes, xkbmap_keycodes, preserve_server_keycodes, modifiers):
     """
         Clients that have access to raw x11 keycodes should provide
@@ -170,11 +232,25 @@ def set_all_keycodes(xkbmap_x11_keycodes, xkbmap_keycodes, preserve_server_keyco
     def filter_mappings(mappings, drop_extra_keys=False):
         filtered = {}
         invalid_keysyms = set()
+        def estr(entries):
+            try:
+                return csv(list(set(x[0] for x in entries)))
+            except:
+                return csv(list(entries))
         for keycode, entries in mappings.items():
             mods = modifiers_for(entries)
             if len(mods)>1:
-                log.warn("keymapping removed invalid keycode entry %s pointing to more than one modifier (%s): %s", keycode, mods, entries)
-                continue
+                log.warn("Warning: keymapping changed:")
+                log.warn(" keycode %s points to %i modifiers: %s", keycode, len(mods), csv(list(mods)))
+                log.warn(" from definition: %s", estr(entries))
+                for mod in mods:
+                    emod = [entry for entry in entries if mod in modifiers_for([entry])]
+                    log.warn(" %s: %s", mod, estr(emod))
+                #keep just the first one:
+                mods = list(mods)[:1]
+                mod = mods[0]
+                entries = [entry for entry in entries if mod in modifiers_for([entry])]
+                log.warn(" keeping: %s for %s", estr(entries), mod)
             #now remove entries for keysyms we don't have:
             f_entries = set([(keysym, index) for keysym, index in entries if keysym and X11Keyboard.parse_keysym(keysym) is not None])
             for keysym, _ in entries:
@@ -190,7 +266,8 @@ def set_all_keycodes(xkbmap_x11_keycodes, xkbmap_keycodes, preserve_server_keyco
                     continue
             filtered[keycode] = f_entries
         if invalid_keysyms:
-            log.warn("the following keysyms are invalid and have not been mapped: %s", ", ".join((str(x) for x in invalid_keysyms)))
+            log.warn("Warning: the following keysyms are invalid and have not been mapped")
+            log.warn(" %s", csv(invalid_keysyms))
         return filtered
 
     #get the list of keycodes (either from x11 keycodes or gtk keycodes):
@@ -559,7 +636,9 @@ def get_modifiers_from_meanings(xkbmap_mod_meanings):
     #first generate a {modifier : [keynames]} dict:
     modifiers = {}
     for keyname, modifier in xkbmap_mod_meanings.items():
-        modifiers.setdefault(modifier, set()).add(keyname)
+        l = modifiers.setdefault(modifier, [])
+        if keyname not in l:
+            l.append(keyname)
     log("get_modifiers_from_meanings(%s) modifier dict=%s", xkbmap_mod_meanings, modifiers)
     return modifiers
 
