@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2017 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -46,9 +46,13 @@ class XSettingsManager(object):
         if type(settings)==list:
             settings = tuple(settings)
         elif type(settings)!=tuple:
-            log.warn("discarding xsettings because of incompatible format: %s", type(settings))
+            log.warn("Warning: discarding xsettings because of incompatible format: %s", type(settings))
             return
-        prop_set(self._window, XSETTINGS, XSETTINGS_TYPE, settings)
+        try:
+            prop_set(self._window, XSETTINGS, XSETTINGS_TYPE, settings)
+        except XError as e:
+            log.error("Error: XSettings not applied")
+            log.error(" %s", e)
 
 
 class XSettingsHelper(object):
@@ -61,12 +65,12 @@ class XSettingsHelper(object):
         self._clipboard = gtk.Clipboard(gtk.gdk.display_get_default(), self._selection)
 
     def xsettings_owner(self):
-        owner_x = X11Window.XGetSelectionOwner(self._selection)
-        log("XGetSelectionOwner(%s)=%s", self._selection, owner_x)
-        if owner_x == XNone:
-            return None
         try:
             with xsync:
+                owner_x = X11Window.XGetSelectionOwner(self._selection)
+                log("XGetSelectionOwner(%s)=%s", self._selection, owner_x)
+                if owner_x == XNone:
+                    return None
                 return get_pywindow(self._clipboard, owner_x)
         except XError:
             log("X error while fetching owner of XSettings data; ignored")
@@ -78,8 +82,7 @@ class XSettingsHelper(object):
         if owner is None:
             return None
         try:
-            with xsync:
-                return prop_get(owner, XSETTINGS, XSETTINGS_TYPE)
+            return prop_get(owner, XSETTINGS, XSETTINGS_TYPE)
         except XError:
             log("X error while fetching XSettings data; ignored")
             return None
