@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2012-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -14,16 +14,20 @@ and
 It is used by xpra.x11.gtk_x11.prop
 """
 
-import sys, os
+import sys
 import struct
+
 from xpra.log import Logger
 log = Logger("x11", "xsettings")
 
+from xpra.util import envbool
+from xpra.os_util import PYTHON3
 
-DEBUG_XSETTINGS = os.environ.get("XPRA_XSETTINGS_DEBUG", "0")=="1"
+DEBUG_XSETTINGS = envbool("XPRA_XSETTINGS_DEBUG", False)
 
 
-if sys.version > '3':
+if PYTHON3:
+    long = int              #@ReservedAssignment
     unicode = str           #@ReservedAssignment
 
 
@@ -47,9 +51,6 @@ XSettingsNames = {
                 XSettingsTypeColor      : "Color",
                 }
 
-if sys.version>='3':
-    long = int              #@ReservedAssignment
-
 
 XSETTINGS_CACHE = {}
 def get_settings(disp, d):
@@ -158,9 +159,10 @@ def set_settings(disp, d):
 
 def main():
     from xpra.platform.gui import init as gui_init
-    from xpra.platform import init as platform_init,clean
-    try:
-        platform_init("XSettings")
+    from xpra.os_util import POSIX
+    from xpra.platform import program_context
+    from xpra.gtk_common.error import xsync
+    with program_context("XSettings"):
         gui_init()
         verbose = "-v" in sys.argv or "--verbose" in sys.argv
         if verbose:
@@ -169,27 +171,27 @@ def main():
                 x.enable_debug()
 
         #naughty, but how else can I hook this up?
-        if os.name!="posix":
+        if not POSIX:
             print("xsettings require a posix OS")
             return 1
 
-        from xpra.x11.bindings import posix_display_source  #@UnresolvedImport - takes care of hooking up the display
-        assert posix_display_source
-        from xpra.x11.bindings.window_bindings import X11WindowBindings #@UnresolvedImport
-        window_bindings = X11WindowBindings()
-        selection = "_XSETTINGS_S0"
-        owner = window_bindings.XGetSelectionOwner(selection)
-        print("owner(%s)=%#x" % (selection, owner))
-        XSETTINGS = "_XSETTINGS_SETTINGS"
-        data = window_bindings.XGetWindowProperty(owner, XSETTINGS, XSETTINGS)
-        serial, settings = get_settings(None, data)
-        print("serial=%s" % serial)
-        print("%s settings:" % len(settings))
-        for s in settings:
-            print(s)
-        return 0
-    finally:
-        clean()
+        with xsync:
+            from xpra.x11.bindings.posix_display_source import init_posix_display_source    #@UnresolvedImport
+            init_posix_display_source()
+            from xpra.x11.bindings.window_bindings import X11WindowBindings #@UnresolvedImport
+            window_bindings = X11WindowBindings()
+            selection = "_XSETTINGS_S0"
+            owner = window_bindings.XGetSelectionOwner(selection)
+            print("owner(%s)=%#x" % (selection, owner))
+            XSETTINGS = "_XSETTINGS_SETTINGS"
+            if owner:
+                data = window_bindings.XGetWindowProperty(owner, XSETTINGS, XSETTINGS)
+                serial, settings = get_settings(None, data)
+                print("serial=%s" % serial)
+                print("%s settings:" % len(settings))
+                for s in settings:
+                    print(s)
+            return 0
 
 
 if __name__ == "__main__":
